#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Chassis class
#
#   Call probe() to create an instance based on platform-detect,
#   like this:
#       import cumulus.chassis
#       ...
#       c = chassis.probe()
#
#----------------------------------------------------------------------------

import subprocess
import imp

#
# Platform-to-module mapping
#
# The platform name is that returned by platform-detect,
# with the "," replaced by "_".
#
# Not all supported platforms need be listed here.
# We first try the platform name itself as the module name.
# If that fails, then we resort to the mapping.
#
model_to_module = {
    "cel_bigstone_g_lc1l": "cel_bigstone_g",
    "cel_bigstone_g_lc1r": "cel_bigstone_g",
    "cel_bigstone_g_fab1": "cel_bigstone_g",
    "accton_omp800_lca"  : "accton_omp800",
    "accton_omp800_lcb"  : "accton_omp800",
    "accton_omp800_fca"  : "accton_omp800",
    "accton_omp800_fcb"  : "accton_omp800",
}

def probe():
    """
    Create a chassis instance of the right type, based on
    platform-detect.

    This is the main module entry point.
    """

    # well, actually, we use cl-platform because that's all we need
    model = subprocess.check_output("/usr/lib/cumulus/cl-platform")
    model = model.replace(",", "_", 1)

    # first try the model name as a module
    # if that fails, go through the mapping and try again
    try:
        file, path, desc = imp.find_module(model, __path__)
    except ImportError:
        if model not in model_to_module:
            raise RuntimeError("Platform %s is not a supported chassis type" %
                               model)
        model = model_to_module[model]
        # if still not found, just let the exception happen
        file, path, desc = imp.find_module(model, __path__)
    assert(file is not None)

    # found module, load it, instantiate an instance of the chassis class
    try:
        m = imp.load_module(model, file, path, desc)
        return m.chassisImpl()
    finally:
        file.close()
