#!/usr/bin/python
# Copyright 2017 Cumulus Networks Inc, all rights reserved
#
# Mellanox ASIC monitor
#
#
#
#
#
#
#

import sys
import argparse
import getopt
import os
import subprocess
import json
import math
import time
import logging
import logging.handlers
import traceback
import datetime

from cumulus.asic_manager import StateDesc
from cumulus.asic_manager import AsicManager
from cumulus.asic_manager import ReadResults

from cumulus.mlx_status import sx_status_msg

import cumulus.porttab

global porttab
porttab = cumulus.porttab.porttab()

# initialize the global logger
global logger
logger  = logging.getLogger('mlx-asic-manager')
fmt     = logging.Formatter(fmt='%(name)s %(levelname)s: %(message)s')
handler = logging.handlers.SysLogHandler('/dev/log')
handler.setFormatter(fmt)
logger.setLevel(logging.INFO)
#logger.setLevel(logging.DEBUG)
logger.addHandler(handler)

# Add the path of the SDK
sys.path.append("/usr/lib/python2.7/site-packages/python_sdk_api")
sys.path.append("/usr/lib/python2.7/dist-packages/python_sdk_api")
from sx_api import *

# ----------------------------------------------------------
#
class MlxStateDesc(StateDesc):

    histogram_flag = 0x01
    buffer_flag    = 0x02
    packet_flag    = 0x04
    discard_flag   = packet_flag
    normal_flag    = 0x08
    priority_flag  = 0x10

    buffer_param_type_list = [SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                              SX_COS_INGRESS_PORT_ATTR_E,
                              SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E]

    def __init__(self, port_group_name, port_group, sdk_handle):
        super(MlxStateDesc,self).__init__(port_group)

        self.bytes_per_cell  = 96 # XXX replace this constant
        self.name            = port_group_name
        self.port_list       = []
        self.linux_port_dict = {}
        for linux_port_label in port_group.linux_port_list:
            # translate the linux port label to a logical port number
            # XXX this should be done by porttab
            # XXX this assumes the unit number == 0
            sdk_port_label = porttab.linux2bcm(linux_port_label)
            logical_port = (1 << 16) + (int(sdk_port_label[2:]) << 8)
            self.port_list.append(logical_port)
            self.linux_port_dict[logical_port] = linux_port_label

        # set the trigger thresholds
        self.unconditional_trigger_dict = {}
        self.queue_length_trigger_dict  = {}
        self.congestion_trigger_dict    = {}
        self.error_trigger_dict         = {}
        self._set_trigger_thresholds(self.snapshot_flag)
        self._set_trigger_thresholds(self.log_flag)
        self._set_trigger_thresholds(self.collect_flag)

        self.stat_flags = 0
        if port_group.stat_type == 'histogram':
            self.stat_flags |= self.histogram_flag
        elif port_group.stat_type == 'packet':
            self.stat_flags |= self.normal_flag
        elif port_group.stat_type == 'packet_extended':
            self.stat_flags |= (self.normal_flag | self.discard_flag)
        elif port_group.stat_type == 'packet_all':
            self.stat_flags |= (self.normal_flag | self.priority_flag | self.discard_flag)
        elif port_group.stat_type == 'buffer':
            self.stat_flags |= self.buffer_flag

        if self.stat_flags & self.histogram_flag:
            # translate the histogram boundary and sample configuration values for MLX
            self.min_boundary = port_group.histogram.minimum_bytes_boundary / self.bytes_per_cell

            histogram_cell_count = float(port_group.histogram.histogram_size_bytes / self.bytes_per_cell)
            self.histogram_cell_resolution = int(math.log(histogram_cell_count, 2))

            sample_time = float(port_group.histogram.sample_time_ns / 128) # replace hard-coded constant
            self.sample_time_resolution = int(math.log(sample_time, 2))

            # calculate the histogram bin sizes
            self.bin_min_queue_length = {}
            self.bin_label   = {}
            bin_size = int(histogram_cell_count / 8) * self.bytes_per_cell
            min_queue_length = 0
            max_queue_length = (self.min_boundary * self.bytes_per_cell) - 1
            logger.debug('bin size in bytes: %d' % bin_size)
            for bin_idx in range(SX_TELE_HIST_MAX_BINS) :
                self.bin_min_queue_length[bin_idx] = min_queue_length
                if bin_idx == SX_TELE_HIST_MAX_BINS - 1:
                    self.bin_label[bin_idx] = '%d:*' % min_queue_length
                else :
                    self.bin_label[bin_idx] = '%d:%d' % (min_queue_length, max_queue_length)
                logger.debug('bin %d: label %s' % (bin_idx, self.bin_label[bin_idx]))
                min_queue_length = (max_queue_length + 1)
                max_queue_length = min_queue_length + bin_size - 1

        if self.stat_flags & self.buffer_flag:
            # configure the parameters for each buffer type
            # XXX MLX SDK does not appear to handle multiple ports per request correctly,
            #     so we need one set of parameter objects per port XXX
            num_ports = len(self.port_list)
            self.num_buffer_param_types = len(self.buffer_param_type_list)
            self.buffer_param_list = {}
            for logical_port in self.port_list :
                buffer_param_list = new_sx_port_statistic_usage_params_t_arr(self.num_buffer_param_types)
                param_idx = 0
                for param_type in self.buffer_param_type_list:
                    buffer_param_list, m_items = self._set_port_buffer_param_item(buffer_param_list,
                                                                                  logical_port,
                                                                                  param_idx,
                                                                                  param_type)
                    param_idx += 1
                self.buffer_param_list[logical_port] = buffer_param_list

            # create the counter list
            num_buffer_counters   = 0
            max_buffer_counters  = 32
            self.num_buffer_counters_p = new_uint32_t_p()
            uint32_t_p_assign(self.num_buffer_counters_p, num_buffer_counters)
            self.buffer_counter_list = new_sx_port_occupancy_statistics_t_arr(max_buffer_counters)

            rc = sx_api_cos_port_buff_type_statistic_get(sdk_handle,
                                                         SX_ACCESS_CMD_READ,
                                                         self.buffer_param_list[self.port_list[0]],
                                                         self.num_buffer_param_types,
                                                         self.buffer_counter_list,
                                                         self.num_buffer_counters_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_cos_port_buff_type_statistist_get failed: %s' % sx_status_msg[rc])
            num_buffer_counters = uint32_t_p_value(self.num_buffer_counters_p)
            logger.debug('%d buffer stats counters per port, according to the SDK' % num_buffer_counters)
            if num_buffer_counters > max_buffer_counters:
                logger.error('%d buffer counters exceeds maximum %d counters' % (num_buffer_counters,
                                                                                 max_buffer_counters))

    def _set_trigger_thresholds(self, action_flag):
        if self.action_flags & action_flag:
            self.unconditional_trigger_dict[action_flag] = True
            if action_flag == self.snapshot_flag:
                action = self.snapshot
            elif action_flag == self.log_flag:
                action = self.log
            elif action_flag == self.collect_flag:
                action = self.collect
        else:
            return
        queue_bytes = getattr(action, 'queue_bytes', None)
        if queue_bytes != None:
            self.queue_length_trigger_dict[action_flag] = queue_bytes
            self.unconditional_trigger_dict[action_flag] = False
        congestion_drops = getattr(action, 'packet_congestion_drops', None)
        if congestion_drops != None:
            self.congestion_trigger_dict[action_flag] = congestion_drops
            self.unconditional_trigger_dict[action_flag] = False
        error_drops = getattr(action, 'packet_error_drops', None)
        if error_drops != None:
            self.error_trigger_dict[action_flag] = error_drops
            self.unconditional_trigger_dict[action_flag] = False

    def _set_port_buffer_param_item(self, param_list, logical_port, param_idx, param_type):

        param_item = sx_port_statistic_usage_params_t_arr_getitem(param_list, param_idx)
        param_item.port_cnt         = 1
        param_item.log_port_list_p  = new_sx_port_log_id_t_arr(1)
        sx_port_log_id_t_arr_setitem(param_item.log_port_list_p, 0, logical_port)
        param_item.sx_port_params.port_params_type = param_type
        m_items = {}

        if param_type == SX_COS_INGRESS_PORT_ATTR_E:
            param_item.sx_port_params.port_params_cnt  = 4
            pool_list = new_sx_cos_pool_id_t_arr(4)
            for pool_id in range(4) :
                sx_cos_pool_id_t_arr_setitem(pool_list, pool_id, pool_id)
            param_item.sx_port_params.port_param.ingress_port_pool_list_p = pool_list
            m_items[param_type] = pool_list

        elif param_type == SX_COS_EGRESS_PORT_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 4
            pool_list = new_sx_cos_pool_id_t_arr(4)
            for pool_id in range(4,8) :
                sx_cos_pool_id_t_arr_setitem(pool_list, pool_id, pool_id)
            param_item.sx_port_params.port_param.egress_port_pool_list_p  = pool_list
            m_items[param_type] = pool_list

        elif param_type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 8
            pg_list = new_sx_cos_priority_group_t_arr(8)
            for pg in range(8) :
                sx_cos_priority_group_t_arr_setitem(pg_list, pg, pg)
            param_item.sx_port_params.port_param.port_pg_list_p = pg_list
            m_items[param_type] = pg_list

        elif param_type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 8
            tc_list = new_sx_cos_traffic_class_t_arr(8)
            for tc in range(8) :
                sx_cos_traffic_class_t_arr_setitem(tc_list, tc, tc)
            param_item.sx_port_params.port_param.port_tc_list_p = tc_list
            m_items[param_type] = tc_list

        elif param_type == SX_COS_MULTICAST_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 16
            prio_list = new_sx_cos_priority_t_arr(16)
            for prio in range(16) :
                sx_cos_traffic_class_t_arr_setitem(prio_list, prio, prio)
            param_item.sx_port_params.port_param.mc_switch_prio = prio_list
            m_items[param_type] = prio_list

        # copy the item into the list
        sx_port_statistic_usage_params_t_arr_setitem(param_list, param_idx, param_item)
        return param_list, m_items

# ----------------------------------------------------------
#
class MlxAsicManager(AsicManager):

    asic_is_supported = True

    def __init__(self, config_manager, chip_name):
        super(MlxAsicManager,self).__init__(config_manager, chip_name)

        rc, self.sdk_handle = sx_api_open(None)
        if rc != SX_STATUS_SUCCESS:
            raise RuntimeError("Error connection to the MLX SDK: %s" % sx_status_msg[rc])

        self.discard_p = new_sx_port_cntr_discard_t_p()
        self.ieee_p    = new_sx_port_cntr_ieee_802_dot_3_t_p()
        self.cli_p     = new_sx_port_cntr_cli_t_p()
        self.perf_p    = new_sx_port_cntr_perf_t_p()
        self.tc_p      = new_sx_port_traffic_cntr_t_p()
        self.prio_p    = new_sx_port_cntr_prio_t_p()

        # create the pool occupancy list
        self.max_pool_count = 41 # XXX fixme: we should read the rm_resources_t.shared_buff_total_num_pools field when it is available in python
        self.pool_count_p   = new_uint32_t_p()
        self.pool_list_p    = new_sx_cos_pool_id_t_arr(self.max_pool_count)
        self.usage_list_p   = new_sx_cos_pool_occupancy_statistics_t_arr(self.max_pool_count)

        self.initialize_tele_module()

    def initialize_tele_module(self):
        init_params_p = new_sx_tele_init_params_t_p()
        rc = sx_api_tele_init_set(self.sdk_handle, init_params_p)
        if rc == SX_STATUS_ALREADY_INITIALIZED:
            self.clear_histograms()
        elif rc != SX_STATUS_SUCCESS:
            raise RuntimeError('sx_api_tele_init_set failed: %s' % sx_status_msg[rc])

    def create_state_desc(self, port_group_name, port_group):
        state_desc = MlxStateDesc(port_group_name, port_group, self.sdk_handle)
        if state_desc.stat_flags & state_desc.histogram_flag:
            self.configure_histogram(state_desc)
        return state_desc

    def configure_histogram(self, state_desc):

        # initialize histogram configuration
        bins      = new_uint64_t_arr(SX_TELE_HIST_MAX_BINS)
        histogram = sx_tele_histogram_data_t()
        histogram.bins     = bins
        histogram.bins_cnt = SX_TELE_HIST_MAX_BINS
        self.histogram_p   = new_sx_tele_histogram_data_t_p()
        sx_tele_histogram_data_t_p_assign(self.histogram_p, histogram)

        # configure individual histograms
        data      = sx_tele_histogram_attributes_data_t()
        data.type = SX_TELE_HISTOGRAM_TYPE_QUEUE_DEPTH_E
        q_data    = sx_tele_histogram_queue_depth_attributes_data_t()
        q_data.sample_time_resolution = state_desc.sample_time_resolution
        q_data.min_boundary           = state_desc.min_boundary
        q_data.bin_size_resolution    = state_desc.histogram_cell_resolution
        data.data.queue_depth         = q_data
        self.key_list  = []
        key_idx   = 0

        for switch_priority in state_desc.switch_priority_list:
            traffic_class = switch_priority  # XXX map from priority to tc
            for logical_port in state_desc.port_list:
                key = sx_tele_histogram_key_t()
                key.type = SX_TELE_HISTOGRAM_TYPE_QUEUE_DEPTH_E
                q_key = sx_tele_histogram_queue_depth_key_t()
                q_key.traffic_class = traffic_class
                q_key.network_port  = logical_port
                key.key.queue_depth = q_key
                self.key_list.append(key)
                rc = sx_api_tele_histogram_set(self.sdk_handle,
                                               SX_ACCESS_CMD_SET,
                                               key,
                                               data)
                if rc != SX_STATUS_SUCCESS:
                    raise RuntimeError('sx_api_tele_histogram_set failed: %s' % sx_status_msg[rc])

    def clear_histograms(self):

        # get the number of histograms configured in the SDK
        histogram_count_p = new_uint32_t_p()
        uint32_t_p_assign(histogram_count_p, 0)
        key_filter = sx_tele_histogram_filter_t()
        key_filter.filter_port_valid = SX_TELE_HISTOGRAM_KEY_FILTER_FIELD_NOT_VALID_E
        key_filter.filter_tc_valid   = SX_TELE_HISTOGRAM_KEY_FILTER_FIELD_NOT_VALID_E
        rc = sx_api_tele_histogram_iter_get(self.sdk_handle,
                                            SX_ACCESS_CMD_GET,
                                            None,
                                            key_filter,
                                            None,
                                            histogram_count_p)
        if rc != SX_STATUS_SUCCESS:
            logger.error('sx_api_tele_histogram_iter_get failed to get the histogram count: %s' % sx_status_msg[rc])
            return

        histogram_count = uint32_t_p_value(histogram_count_p)
        if histogram_count == 0:
            return

        # get the keys for the configured histograms
        histogram_list_p = new_sx_tele_histogram_key_t_arr(histogram_count)
        rc = sx_api_tele_histogram_iter_get(self.sdk_handle,
                                            SX_ACCESS_CMD_GET_FIRST,
                                            None,
                                            key_filter,
                                            histogram_list_p,
                                            histogram_count_p)

        if rc != SX_STATUS_SUCCESS:
            raise RuntimeError('sx_api_tele_histogram_iter_get failed: %s' % sx_status_msg[rc])

        # delete the histogram configurations
        histogram_count = uint32_t_p_value(histogram_count_p)
        attr = sx_tele_histogram_attributes_data_t()
        attr.type = SX_TELE_HISTOGRAM_TYPE_QUEUE_DEPTH_E
        q_data = sx_tele_histogram_queue_depth_attributes_data_t()
        q_data.sample_time_resolution = 0
        q_data.min_boundary = 0
        q_data.bin_size_resolution = 0
        attr.data.queue_depth = q_data
        for key_idx in range(histogram_count):
            histogram_key = sx_tele_histogram_key_t_arr_getitem(histogram_list_p, key_idx)
            rc = sx_api_tele_histogram_set(self.sdk_handle,
                                           SX_ACCESS_CMD_DESTROY,
                                           histogram_key,
                                           attr)
        if rc != SX_STATUS_SUCCESS:
            logger.error('sx_api_tele_histogram_set failed: %s' % sx_status_msg[rc])

    def clear_manager(self):

        self.clear_histograms()

        # de-initialize the telemetry module
        rc = sx_api_tele_deinit_set(self.sdk_handle)
        if rc != SX_STATUS_SUCCESS:
            logger.error('sx_api_tele_deinit_set failed: %s' % sx_status_msg[rc])

        sx_api_close(self.sdk_handle)

    # -------------------------------------
    #
    def read_packet_stats(self, logical_port, state_desc, packet_info):

        trigger_flags = 0

        if state_desc.stat_flags & state_desc.discard_flag:
            rc = sx_api_port_counter_discard_get(self.sdk_handle,
                                                SX_ACCESS_CMD_READ,
                                                logical_port,
                                                self.discard_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_port_counter_discard failed for port %d: %s' % (logical_port,
                                                                                           sx_status_msg[rc]))
            discard_data = sx_port_cntr_discard_t_p_value(self.discard_p)

        if (state_desc.stat_flags & (state_desc.discard_flag | state_desc.normal_flag)):
            rc = sx_api_port_counter_ieee_802_dot_3_get(self.sdk_handle,
                                                        SX_ACCESS_CMD_READ,
                                                        logical_port,
                                                        self.ieee_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_port_counter_ieee_get failed: %s' % sx_status_msg[rc])
            ieee_data = sx_port_cntr_ieee_802_dot_3_t_p_value(self.ieee_p)

            rc = sx_api_port_counter_cli_get(self.sdk_handle,
                                             SX_ACCESS_CMD_READ,
                                             logical_port,
                                             self.cli_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_port_counter_cli_get failed: %s' % sx_status_msg[rc])
            cli_data = sx_port_cntr_cli_t_p_value(self.cli_p)

            rc = sx_api_port_counter_perf_get(self.sdk_handle,
                                              SX_ACCESS_CMD_READ,
                                              logical_port,
                                              SX_PORT_PRIO_ID_0,
                                              self.perf_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_port_counter_perf_get failed: prio value %d, rc %s' % (prio_value,
                                                                                                  sx_status_msg[rc]))
            perf_data = sx_port_cntr_perf_t_p_value(self.perf_p)

            #TC8-15 are not valid for MC_AWARE Ports
            #As per MLX they are mapped to TC0-7
            #Hence the check, if its MC_AWARE PORT, then fetch for TC0-7 only
            sx_port_max_tc_val = SX_PORT_TC_ID_MAX
            mc_aware_p = new_boolean_t_p()
            rc = sx_api_cos_port_tc_mcaware_get(self.sdk_handle, logical_port, mc_aware_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('Unable to fetch MC_AWARE Status while'
                                    'fetching TC Stats for portid : %d |'
                                    'Error Code : %d\n' % (logical_port, rc))
            mc_aware = boolean_t_p_value(mc_aware_p)
            if (True == mc_aware):
                sx_port_max_tc_val = SX_PORT_TC_ID_7

        if state_desc.stat_flags & state_desc.priority_flag:
            tc_data = []
            for tc_value in range(SX_PORT_TC_ID_MIN, sx_port_max_tc_val, 1):
                rc = sx_api_port_counter_tc_get(self.sdk_handle,
                                                SX_ACCESS_CMD_READ,
                                                logical_port,
                                                tc_value,
                                                self.tc_p)
                if rc != SX_STATUS_SUCCESS:
                    raise RuntimeError('sx_api_port_counter_tc_get failed: %s' % sx_status_msg[rc])
                tc_data.append(sx_port_traffic_cntr_t_p_value(self.tc_p))

            prio_data = []
            prio_id_boundary = (SX_PORT_PRIO_ID_MAX+1)
            for prio_value in range(SX_PORT_PRIO_ID_0, prio_id_boundary, 1):
                rc = sx_api_port_counter_prio_get(self.sdk_handle,
                                                  SX_ACCESS_CMD_READ,
                                                  logical_port,
                                                  prio_value,
                                                  self.prio_p)
                if rc != SX_STATUS_SUCCESS:
                    raise RuntimeError('sx_api_port_counter_prio_get failed: %s' % sx_status_msg[rc])
                prio_data.append(sx_port_cntr_prio_t_p_value(self.prio_p))

        if state_desc.stat_flags & state_desc.discard_flag :
            packet_info['discard'] = {}
            packet_info['discard']['ingress'] = {}
            packet_info['discard']['egress']  = {}
            packet_info['discard']['general'] = {}
            packet_info['discard']['ingress']['general']         = discard_data.ingress_general
            packet_info['discard']['ingress']['policy']          = discard_data.ingress_policy_engine
            packet_info['discard']['ingress']['tag_frame_type']  = discard_data.ingress_tag_frame_type
            packet_info['discard']['ingress']['vlan_membership'] = discard_data.ingress_vlan_membership
            packet_info['discard']['egress']['vlan_membership']  = discard_data.egress_vlan_membership
            packet_info['discard']['egress']['hoq']              = discard_data.egress_hoq
            packet_info['discard']['egress']['general']          = discard_data.egress_general
            packet_info['discard']['egress']['link_down']        = discard_data.egress_link_down
            packet_info['discard']['egress']['policy']           = discard_data.egress_policy_engine
            packet_info['discard']['general']['port_isolation']  = discard_data.port_isolation
            packet_info['discard']['general']['loopback_filter'] = discard_data.loopback_filter
            packet_info['discard']['general']['fcs']             = ieee_data.a_frame_check_sequence_errors
            packet_info['discard']['general']['alignment']       = ieee_data.a_alignment_errors
            packet_info['discard']['general']['in_range_length'] = ieee_data.a_in_range_length_errors
            packet_info['discard']['general']['out_of_range_length'] = ieee_data.a_out_of_range_length_field
            packet_info['discard']['general']['too_long']        = ieee_data.a_frame_too_long_errors
            packet_info['discard']['general']['symbol_error']    = ieee_data.a_symbol_error_during_carrier
            packet_info['discard']['ingress']['opcodes']         = ieee_data.a_unsupported_opcodes_received
            packet_info['discard']['egress']['other']            = cli_data.port_tx_errors
            packet_info['discard']['ingress']['buffer']          = cli_data.port_rx_no_buffer
            packet_info['discard']['ingress']['runt']            = cli_data.port_rx_runt
            packet_info['discard']['ingress']['other']           = cli_data.port_rx_other_errors
            packet_info['discard']['egress']['mc_buffer']        = perf_data.no_buffer_discard_mc

            # check for thresholds
            for action_flag, congestion_threshold in state_desc.congestion_trigger_dict.items():
                if trigger_flags & action_flag:
                    continue
                if packet_info['discard']['ingress']['buffer'] > congestion_threshold:
                    trigger_flags |= action_flag
                elif packet_info['discard']['egress']['mc_buffer'] > congestion_threshold:
                    trigger_flags |= action_flag
            for action_flag, error_threshold in state_desc.error_trigger_dict.items():
                if trigger_flags & action_flag:
                    continue
                for error_type, error_count in packet_info['discard']['ingress'].items():
                    if error_count >= error_threshold:
                        trigger_flags |= action_flag
                if trigger_flags & action_flag:
                    continue
                for error_type, error_count in packet_info['discard']['egress'].items():
                    if error_count >= error_threshold:
                        trigger_flags |= action_flag

        if state_desc.stat_flags & state_desc.normal_flag:
            packet_info['good'] = {}
            packet_info['good']['ingress'] = {}
            packet_info['good']['egress'] = {}
            packet_info['good']['ingress']['frames']  = ieee_data.a_frames_received_ok
            packet_info['good']['ingress']['bytes']   = ieee_data.a_octets_received_ok
            packet_info['good']['ingress']['mc']      = ieee_data.a_multicast_frames_received_ok
            packet_info['good']['ingress']['bc']      = ieee_data.a_broadcast_frames_recieved_ok
            packet_info['good']['ingress']['mac_control']    = ieee_data.a_mac_control_frames_received
            packet_info['good']['ingress']['pause_mac_ctrl'] = ieee_data.a_pause_mac_ctrl_frames_received
            packet_info['good']['egress']['frames']   = ieee_data.a_frames_transmitted_ok
            packet_info['good']['egress']['bytes']    = ieee_data.a_octets_transmitted_ok
            packet_info['good']['egress']['mc']       = ieee_data.a_multicast_frames_xmitted_ok
            packet_info['good']['egress']['bc']       = ieee_data.a_broadcast_frames_xmitted_ok
            packet_info['good']['egress']['mac_control']     = ieee_data.a_mac_control_frames_transmitted
            packet_info['good']['egress']['pause_mac_ctrl']  = ieee_data.a_pause_mac_ctrl_frames_transmitted

            packet_info['total'] = {}
            packet_info['total']['ingress'] = {}
            packet_info['total']['egress']  = {}
            packet_info['total']['ingress']['frames']    = cli_data.port_rx_frames
            packet_info['total']['ingress']['bytes']     = cli_data.port_rx_octets
            packet_info['total']['ingress']['uc']        = cli_data.port_rx_unicast
            packet_info['total']['ingress']['mc']        = cli_data.port_rx_multicast
            packet_info['total']['ingress']['bc']        = cli_data.port_rx_broadcast
            packet_info['total']['ingress']['jumbo']     = cli_data.port_rx_jumbo
            packet_info['total']['egress']['frames']     = cli_data.port_tx_frames
            packet_info['total']['egress']['bytes']      = cli_data.port_tx_octets
            packet_info['total']['egress']['uc']         = cli_data.port_tx_unicast
            packet_info['total']['egress']['mc']         = cli_data.port_tx_multicast
            packet_info['total']['egress']['bc']         = cli_data.port_tx_broadcast
            packet_info['total']['egress']['jumbo']      = cli_data.port_tx_jumbo
            packet_info['total']['egress']['wait']       = perf_data.tx_wait
            packet_info['total']['egress']['ecn_marked'] = perf_data.ecn_marked

        if state_desc.stat_flags & state_desc.priority_flag:
            packet_info['prio'] = {}
            for prio_idx in range(len(prio_data)) :
                packet_info['prio'][prio_idx] = {}
                packet_info['prio'][prio_idx]['ingress'] = {}
                packet_info['prio'][prio_idx]['egress']  = {}
                packet_info['prio'][prio_idx]['ingress']['frames']         = prio_data[prio_idx].rx_frames
                packet_info['prio'][prio_idx]['ingress']['pause_frames']   = prio_data[prio_idx].rx_pause
                packet_info['prio'][prio_idx]['ingress']['pause_duration'] = prio_data[prio_idx].rx_pause_duration
                packet_info['prio'][prio_idx]['egress']['pause_frames']    = prio_data[prio_idx].tx_pause
                packet_info['prio'][prio_idx]['egress']['pause_duration']  = prio_data[prio_idx].tx_pause_duration
                packet_info['prio'][prio_idx]['ingress']['discard']        = prio_data[prio_idx].rx_discard

            packet_info['queue'] = {}
            for queue_idx in range(len(tc_data)) :
                packet_info['queue'][queue_idx] = {}
                packet_info['queue'][queue_idx]['egress'] = {}
                packet_info['queue'][queue_idx]['egress']['frames'] = tc_data[queue_idx].tx_frames
                packet_info['queue'][queue_idx]['egress']['bytes']  = tc_data[queue_idx].tx_octet
                packet_info['queue'][queue_idx]['egress']['uc']     = tc_data[queue_idx].tx_uc_frames
                packet_info['queue'][queue_idx]['egress']['mc']     = tc_data[queue_idx].tx_mc_frames
                packet_info['queue'][queue_idx]['egress']['bc']     = tc_data[queue_idx].tx_bc_frames
                packet_info['queue'][queue_idx]['egress']['length'] = tc_data[queue_idx].tx_queue
                packet_info['queue'][queue_idx]['egress']['uc_buffer'] = tc_data[queue_idx].tx_no_buffer_discard_uc

                # check for thresholds
                for action_flag, congestion_threshold in state_desc.congestion_trigger_dict.items():
                    if trigger_flags & action_flag:
                        continue
                    if packet_info['queue'][queue_idx]['egress']['uc_buffer'] > congestion_threshold:
                        trigger_flags |= action_flag

        return trigger_flags

    # -------------------------------------
    #
    def read_buffer_stats(self, state_desc, buffer_info):

        param_type_str = {SX_COS_INGRESS_PORT_ATTR_E : 'ingress port',
                          SX_COS_EGRESS_PORT_ATTR_E   : 'egress port',
                          SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E   : 'egress queue',
                          SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E : 'ingress priority'}

        for logical_port in state_desc.port_list:
            linux_port_label = state_desc.linux_port_dict[logical_port]
            buffer_info[linux_port_label] = {}

            rc = sx_api_cos_port_buff_type_statistic_get(self.sdk_handle,
                                                         SX_ACCESS_CMD_READ_CLEAR,
                                                         state_desc.buffer_param_list[logical_port],
                                                         state_desc.num_buffer_param_types,
                                                         state_desc.buffer_counter_list,
                                                         state_desc.num_buffer_counters_p)
            if rc != SX_STATUS_SUCCESS:
                raise RuntimeError('sx_api_cos_port_buff_type_statistic_get failed: %s' % sx_status_msg[rc])

            # write the buffer stats into a data structure
            num_buffer_counters = uint32_t_p_value(state_desc.num_buffer_counters_p)
            for idx in range(num_buffer_counters) :
                stat = sx_port_occupancy_statistics_t_arr_getitem(state_desc.buffer_counter_list, idx)
                stat_type = int(stat.sx_port_params.port_params_type)
                if stat_type == SX_COS_INGRESS_PORT_ATTR_E:
                    elem_idx = int(stat.sx_port_params.port_param.ingress_port_pool)
                elif stat_type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                    elem_idx = int(stat.sx_port_params.port_param.port_pg)
                elif stat_type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
                    elem_idx = int(stat.sx_port_params.port_param.port_tc)
                elif stat_type == SX_COS_EGRESS_PORT_ATTR_E:
                    elem_idx = int(stat.sx_port_params.port_param.egress_port_pool)
                buffer_type = param_type_str.get(int(stat.sx_port_params.port_params_type), 'no type')
                if buffer_type not in buffer_info[linux_port_label]:
                    buffer_info[linux_port_label][buffer_type] = {}
                buffer_info[linux_port_label][buffer_type][elem_idx] = \
                  { 'current_value'   : int(stat.statistics.curr_occupancy) * state_desc.bytes_per_cell,
                    'watermark_value' : int(stat.statistics.watermark)  * state_desc.bytes_per_cell }

        # collect pool occupancy stats
        uint32_t_p_assign(self.pool_count_p, 0)
        rc = sx_api_cos_pools_list_get(self.sdk_handle,
                                       self.pool_count_p,
                                       None)
        if rc != SX_STATUS_SUCCESS:
            raise RuntimeError('sx_api_cos_pools_list_get failed to get count: %s' % sx_status_msg[rc])
        pool_count = uint32_t_p_value(self.pool_count_p)
        if pool_count > self.max_pool_count:
            raise RuntimeError('sx_api_cos_pools_list_get returned a pool count %d greater than our maximum array size %d' % (pool_count, self.max_pool_count))
        rc = sx_api_cos_pools_list_get(self.sdk_handle,
                                       self.pool_count_p,
                                       self.pool_list_p)
        if rc != SX_STATUS_SUCCESS:
            raise RuntimeError('sx_api_cos_pools_list_get failed to get %d pools: %s' % (pool_count, sx_status_msg[rc]))
        rc = sx_api_cos_pool_statistic_get(self.sdk_handle,
                                           SX_ACCESS_CMD_READ_CLEAR,
                                           self.pool_list_p,
                                           pool_count,
                                           self.usage_list_p)
        if rc != SX_STATUS_SUCCESS:
            raise RuntimeError('sx_api_cos_pool_statistic_get failed to get %d pools: %s' % (pool_count, sx_status_msg[rc]))
        buffer_info['pools'] = {}
        for pool_idx in range(pool_count):
            pool_id = sx_cos_pool_id_t_arr_getitem(self.pool_list_p, pool_idx)
            stat = sx_cos_pool_occupancy_statistics_t_arr_getitem(self.usage_list_p, pool_idx)
            buffer_info['pools'][pool_id] = \
              { 'current_value'   : int(stat.statistics.curr_occupancy) * state_desc.bytes_per_cell,
                'watermark_value' : int(stat.statistics.watermark)  * state_desc.bytes_per_cell }

    # -------------------------------------
    #
    def read_histogram_stats(self, state_desc, histogram_info):
        trigger_flags = 0
        for key in self.key_list:
            rc = sx_api_tele_histogram_data_get(self.sdk_handle,
                                                SX_ACCESS_CMD_READ_CLEAR,
                                                key,
                                                self.histogram_p)
            if rc != SX_STATUS_SUCCESS :
                raise RuntimeError('sx_api_tele_histogram_data_get failed: %s' % sx_status_msg[rc])

            histogram = sx_tele_histogram_data_t_p_value(self.histogram_p)
            bins = histogram.bins
            bin_count = histogram.bins_cnt

            for bin_idx in range(bin_count) :
                bin_value = uint64_t_arr_getitem(bins, bin_idx)
                if bin_value > 0:
                    logical_port = key.key.queue_depth.network_port
                    linux_port_label = state_desc.linux_port_dict[logical_port]
                    if linux_port_label not in histogram_info.keys():
                        histogram_info[linux_port_label] = {}
                    histogram_info[linux_port_label][state_desc.bin_label[bin_idx]] = bin_value
                    for action_flag, length_threshold in state_desc.queue_length_trigger_dict.items():
                        if trigger_flags & action_flag:
                            continue
                        if  state_desc.bin_min_queue_length[bin_idx] >= length_threshold:
                            trigger_flags |= action_flag
        return trigger_flags

    # -------------------------------------
    #
    def read_state(self, state_desc):

        start_time = datetime.datetime.now()
        histogram_info = {}
        buffer_info    = {}
        packet_info    = {}
        json_output    = []

        results = ReadResults()
        results.trigger_flags = 0

        for action_flag, trigger in state_desc.unconditional_trigger_dict.items():
            if trigger:
                results.trigger_flags |= action_flag

        if state_desc.stat_flags & state_desc.histogram_flag:
            histogram_start_time = datetime.datetime.now()
            histogram_trigger_flags = self.read_histogram_stats(state_desc, histogram_info)
            histogram_end_time = datetime.datetime.now()
            histogram_time = histogram_end_time - histogram_start_time
            results.trigger_flags |= histogram_trigger_flags

        if state_desc.stat_flags & state_desc.buffer_flag:
            buffer_start_time = datetime.datetime.now()
            self.read_buffer_stats(state_desc, buffer_info)
            buffer_end_time = datetime.datetime.now()
            buffer_time     = buffer_end_time - buffer_start_time

        if state_desc.stat_flags >= state_desc.packet_flag:
            packet_start_time = datetime.datetime.now()
            for logical_port in state_desc.port_list:
                linux_port_label = state_desc.linux_port_dict[logical_port]
                packet_info[linux_port_label] = {}
                packet_trigger_flags = self.read_packet_stats(logical_port,
                                                              state_desc,
                                                              packet_info[linux_port_label])
                results.trigger_flags |= packet_trigger_flags

            packet_end_time = datetime.datetime.now()
            packet_time = packet_end_time - packet_start_time

        results.histogram_info = histogram_info
        results.buffer_info    = buffer_info
        results.packet_info    = packet_info
        results.end_time       = datetime.datetime.now()

        if state_desc.report_time :
            num_ports = len(state_desc.port_list)
            if state_desc.stat_flags & state_desc.histogram_flag:
                delta_in_us = histogram_time.total_seconds() * 1000000
                print '%d ports: histogram collection elapsed time in microseconds:' % num_ports,
                print delta_in_us

            if state_desc.stat_flags & state_desc.buffer_flag:
                delta_in_us = buffer_time.total_seconds() * 1000000
                print '%d ports: buffer data collection elapsed time in microseconds:' % num_ports,
                print delta_in_us

            if state_desc.stat_flags >= state_desc.packet_flag:
                delta_in_us = packet_time.total_seconds() * 1000000
                print '%d ports: packet data collection elapsed time in microseconds:' % num_ports,
                print delta_in_us

            time_delta = results.end_time - start_time
            delta_in_us = time_delta.total_seconds() * 1000000
            print '%d ports: total elapsed time in microseconds:' % num_ports,
            print delta_in_us

        return results
