#
# Copyright (C) 2017 Cumulus Networks, Inc.
#
# multicast.py --
#
#     UDP Multicast
#

from socket import *
import struct
import os
import fcntl


class Multicast:
    """
    UDP multicast

    How to use:
    Create an instance with with address, port number, and
    interface name.  Then call send() and recv().
    IPv4 and IPv6 addresses are supported.
    """

    max_size = 32768	# maximum datagram size, see comment in send()

    # sockopts not defined in the socket module
    SO_BINDTODEVICE = 25
    MCAST_JOIN_GROUP = 42

    def __init__(self, addr, port, ifname,
                 nonblock = False, loopback = False, hops = 1):
        assert type(addr) is str
        assert type(port) is int
        assert type(ifname) is str

        self.addr = addr
        self.port = port
        self.ifname = ifname
        self.ifindex = self.if_nametoindex(ifname)
        #print "interface %s %d" % (ifname, self.ifindex)

        try:
            ai = getaddrinfo(addr, port, 0, SOCK_DGRAM)
            if len(ai) == 0:
                raise RuntimeError(
                        "getaddrinfo(%s, %d, 0, SOCK_DGRAM) failed" %
                        (addr, port))
            family, socktype, proto, canonname, sockaddr = ai[0]
            #print "family %d socktype %d proto %d sockaddr %s\n" % \
            #      (family, socktype, proto, sockaddr)
            s = socket(family, socktype, proto)
            s.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
            s.setsockopt(SOL_SOCKET, SO_REUSEPORT, 1)
            if family == AF_INET6:
                s.setsockopt(IPPROTO_IPV6, IPV6_MULTICAST_LOOP, loopback)
                s.setsockopt(IPPROTO_IPV6, IPV6_MULTICAST_HOPS, hops)
                ifi = struct.pack("I", self.ifindex)
                s.setsockopt(IPPROTO_IPV6, IPV6_MULTICAST_IF, ifi)
                # struct ipv6_mreq
                mreq = inet_pton(family, sockaddr[0]) + ifi
                s.setsockopt(IPPROTO_IPV6, IPV6_JOIN_GROUP, mreq)
            elif family == AF_INET:
                s.setsockopt(IPPROTO_IP, IP_MULTICAST_LOOP, loopback)
                s.setsockopt(IPPROTO_IP, IP_MULTICAST_TTL, hops)
                s.setsockopt(SOL_SOCKET, multicast.SO_BINDTODEVICE, ifname)
                # struct group_req, including struct sockaddr_in and padding
                greq = struct.pack("I4xHH4s120x",
                                   self.ifindex, family, htons(port),
                                   inet_aton(sockaddr[0]))
                s.setsockopt(IPPROTO_IP, multicast.MCAST_JOIN_GROUP, greq)
            else:
                raise RuntimeError("Address family %d not supported" % family)
            s.bind(("", port))
            if nonblock:
		fcntl.fcntl(s, fcntl.F_SETFL, os.O_NONBLOCK)
        except Exception as e:
            raise RuntimeError(
                    "Failed to create socket for addr %s port %d: %s" %
                    (addr, port, str(e)))
        self.sock = s

    def if_nametoindex(self, ifname):
        """
        Map interface name to index, just like the libc function.
        """
        try:
            with open("/sys/class/net/%s/ifindex" % ifname) as f:
                return int(f.readline().strip(), 0)
        except Exception as e:
            raise RuntimeError(
                "Failed to convert interface name \"%s\" to index: %s" %
                (ifname, str(e)))

    def fileno(self):
        """
        Return the file descriptor for the underlying socket,
        suitable for passing to select().
        """
        return self.sock.fileno()

    def send(self, data):
        """
        Send data, as a single packet.
        Number of bytes sent is returned.
        """

	#
	# We want to send all the data in one go, so that they
	# can be received as a unit, using the datagram boundary
	# as the message boundary.
	# max_size can be as large as the maximum UDP data size,
	# which is the maximum IP packet size (64k) minus IP
	# and UDP headers. However, there's a penalty for large
	# sizes.  See comment at recv().
	#
	if len(data) > self.max_size:
            raise RuntimeError("Data size (%d) > maximum size (%d)" % \
                               (len(data), self.max_size))

        return self.sock.sendto(data, (self.addr, self.port))

    def recv(self):
        """
        Receive one packet.
        Return values are the packet data (as a string) and
        the sender address (as a sockaddr tuple).
        """

	#
	# A buffer of max_size is preallocated in recvfrom().
	# When the packet is much smaller, which is proably most
	# of the time, most of that memory is wasted, if only for
	# a short time.  C'est la vie.
	#
        return self.sock.recvfrom(self.max_size)
