#
# Copyright (c) 2012, 2019 Cumulus Networks, Inc.  All Rights Reserved.
#
# platform.py --
#     Shared platform-specific objects.  Typically used by specific switch
#     platforms defined in cumulus.platforms.
#

import os
import cumulus.platforms

from platform_bcm import BCMPort, MaverickPort, TomahawkPort, TomahawkTwoPort
from platform_bcm import TridentThreePort, TridentTwoPort, TridentTwoPlusPort
from platform_bcm import TridentPort, TriumphPort, ApolloPort, Helix4Port
from platform_bcm import Hurricane2Port, Apollo1GPort, Triumph1GPort
from platform_bcm import Helix41GPort, Hurricane21GPort, Apollo10GPort
from platform_bcm import Triumph10GPort, Helix410GPort, Hurricane210GPort
from platform_bcm import Hurricane213GPort, Hurricane213GFabricPort
from platform_bcm import Trident10GPort, Trident10GPort, TridentTwo10GPort
from platform_bcm import TridentTwo10GFabricPort, TridentTwoPlus10GPort
from platform_bcm import Trident40GPort, TridentTwo40GPort, TridentTwoPlus40GPort
from platform_bcm import TridentThree100GPort, TridentThree25GPort, TridentThree10GPort
from platform_bcm import TridentThreeMGPort, Tomahawk100GPort, Tomahawk100GFabricPort
from platform_bcm import Tomahawk25GPort, TomahawkMgmt10GPort, TomahawkTwo100GPort
from platform_bcm import Maverick100GPort, Maverick40GPort, Maverick40GFabricPort
from platform_bcm import Maverick10GPort, Maverick10GFabricPort, BCMChip
from platform_bcm import TridentPlusChip, TridentChip, TridentTwoChip
from platform_bcm import TridentTwo_56850_Chip, TridentTwo_56854_Chip
from platform_bcm import TridentTwoPlusChip, TridentTwoPlus_56860_Chip
from platform_bcm import TridentTwoPlus_56864_Chip, TridentThreeX5Chip
from platform_bcm import TridentThreeX5_56771_Chip, TridentThreeX7Chip
from platform_bcm import TridentThreeX7_56870_Chip, TridentThreeX7_56873_Chip
from platform_bcm import TomahawkChip, TomahawkPlusChip, TomahawkPlus_56967_Chip
from platform_bcm import TomahawkPlus_56965_Chip, TomahawkTwoChip, TomahawkTwo_56970_Chip
from platform_bcm import MaverickChip, Maverick_56762_Chip, Maverick_56768_Chip, TriumphChip
from platform_bcm import ApolloChip, Helix4Chip, Hurricane2Chip

class NoSuchPlatform(RuntimeError):
    pass
class NoSuchChip(RuntimeError):
    pass

class Platform:
    cpld = None
    switch = None
    bmc = None
    chassis = None
    use_dynamic_sensors = False

class SDKSwitch:
    '''
    SDK platform common attributes

    has_phy_ucode_spi_rom - true if platform supports SPI-ROM for phy microcode
    has_phy_ucode_mdio    - true if platform supports MDIO for phy microcode
    led_controllers       - Specify the set of led controllers to populate.
    '''

    has_phy_ucode_spi_rom = None
    has_phy_ucode_mdio = None
    led_controllers = None

    def __init__(self, chip) :
        self.chip = chip

class BCMSwitch(SDKSwitch):
    '''
    Broadcom switch platform
    '''
    def __init__(self, chip) :
        SDKSwitch.__init__(self, chip)

class NoSuchMode(RuntimeError):
    pass

class Port(object):
    '''
    A physical switch port.  Typically on the front panel with one physical
    text label per port.

    label - the text written next to the physical port
    '''
    is_fabric = False
    create_fabric = False
    def __init__(self, label):
        self.label = label
        self.sfp = None

class SDKPort(Port):
    '''
    SDK port data.

    modes - tuple of valid modes (e.g. 4x10G, 40G/4)
    default_mode - the default mode
    gang_modes - a tuple of supported ganged port modes (e.g. 40G/4 or 100G/4)
    max_gang_size - maximum number of ports in a ganged group
    pbmp_map - a map from hardware interface name to SDK pbmp position (e.g. "hg0 -> 1")
    phy_init - a map of register to values used to initalize PHYs
    '''
    modes = ()
    default_mode = None
    gang_modes = ()
    max_gang_size = 0
    pbmp_map = {}
    phy_init = {}
    _speed_map = {}
    # phys which are automatically detectable by some SDK
    _autodetect_phys = ('8727',
                        '8747',
                        '8754',
                        '54282',
                        '54680',
                        '54280',
                        '84328',
                        '84758',
                        '84848',
                        '84858')
    disable_prefix_xlate = None
    # Does the platform require a serdes_fiber_pref line to configure autoneg correctly?
    set_fiber_pref = False

    # Serdes Interface map is dictionary of (Port speed, Interface Mode)
    # which is platform specific
    # Ex: 10:'SFI', 40: 'XLAUI'
    _serdes_intf_map = {}

    # Should we use autoneg=on by default for this port?
    autoneg_pref = None

    # Which group of ports is this port a part of?
    sdk_group = None

    # Override the Linux interface name with this
    name_override = None

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
                 leds, unit, rx_polarity_flip, tx_polarity_flip,
                 rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
                 pre_drv_current, oversub, phy_id1, sys_interface):
        '''
        hw_ports - An ordered list of hardware interfaces associated with the port.
                   In this context, ports are identified by their unmapped
                   interface name as in the SDK chip datasheets (For example,
                   the "Ports Summary" on page 143 of 56840-PR101-R).  This usually
                   means 10G interfaces are specified using their "hg" prefix.

        phy_types - An ordered list of PHY device types associated with the port.
        phy_addrs - An ordered list of PHY addresses associated with the port.
        phy_layer - A description of the PHY layer, one of: '10GBaseT',
                      'SFP+', 'QSSFP+', '10BaseT'
        leds - An ordered list of led chain addresses associated with the port.
               (controller, chain position)
        unit - In multi-chip systems, the chip number, "unit" in SDK terms.
        rx_polarity_flip - flip the rx polarity in hardware
                               (True/False for single serdes ports, SDK-style
                                4-digit hex number for multi-serdes ports)
        tx_polarity_flip - flip the tx polarity in hardware
        rx_lane_map - controls lane swapping
        tx_lane_map - controls lane swapping
        mdi_pair_map - mapping of physical pairs in an MDI interface
        pre_emphasis - per-lane pre-emphasis (Ref: Table7 56840-AN603-R)
        drv_current - per-lane driver current (Ref: Table7 56840-AN603-R)
        pre_drv_current - per-lane pre-driver current (Ref: Table7 56840-AN603-R)
        oversub  - a bool whether the port bandwidth is over-subscribed
        phy_id1 - Second part of a uniqe PHY identifier
        sys_interface - Specify the sytem side interface to be configured for the PHY
        '''
        Port.__init__(self, label)
        self.hw_ports = hw_ports
        self.prim_offset = prim_offset
        self.phy_types = phy_types
        self.phy_addrs = phy_addrs
        self.phy_layer = phy_layer
        self.leds = leds
        self.unit = unit
        self.rx_polarity_flip = rx_polarity_flip
        self.tx_polarity_flip = tx_polarity_flip
        self.rx_lane_map = rx_lane_map
        self.tx_lane_map = tx_lane_map
        self.mdi_pair_map = mdi_pair_map
        self.pre_emphasis = pre_emphasis
        self.drv_current = drv_current
        self.pre_drv_current = pre_drv_current
        self.oversub = oversub
        self.phy_id1 = phy_id1
        self.sys_interface = sys_interface

    def port_mangler(self, mode):
        '''
        Allow a port to return a different set of ports, based on the mode.
        The default is to return a list with just the current port and mode.
        '''
        return ((self, mode),)

    def complete_mode(config, self, mode):
        '''
        We want to support setting the breakout configuration without an
        associated speed.  Therefore, complete the shorthand '1x', '/1' etc.
        into a fully qualified breakout + speed setting.
        '''
        sff_speeds = { 'SFP+'   : 10,
                       'SFP28'  : 25,
                       'QSFP+'  : 40,
                       'QSFP28' : 100,
                     }

        if mode not in ('1x', '2x', '4x', '/1', '/2', '/4'):
            return mode

        cage_type = self.phy_layer
        max_speed = sff_speeds[cage_type]

        if mode == '1x':
            mode = '%dG' % max_speed
        elif mode == '2x':
            mode = '2x%dG' % (max_speed / 2)
        elif mode == '4x':
            mode = '4x%dG' % (max_speed / 4)
        elif mode == '/1':
            mode = '%dG' % max_speed
        elif mode == '/2':
            mode = '%dG/2' % (max_speed * 2)
        elif mode == '/4':
            mode = '%dG/4' % (max_speed * 4)

        return mode

    def configurable(self, config_map):
        '''
        A ports is SDK configurable if it's not marked as 'disable'.
        Some platforms may add further qualifications by overriding this method.
        '''
        label = "Fabric-" + self.label if self.is_fabric else self.label
        mode = config_map.get(label, self.default_mode)
        return not mode.lower().startswith('disable')

    def phy_autodetect(self, mode, sub):
        return self.phy_types[sub] in self._autodetect_phys

    @classmethod
    def _check_mode(self, mode):
        '''
        Verify the mode specified in the config file is valid.
        '''
        if mode not in self.modes:
            shorthand_modes = set()
            for m in self.modes:
                if 'x' in m:
                    shorthand_modes.add('1x')
                    shorthand_modes.add('/1')
                    shorthand_modes.add(m[:2])

                if '/' in m:
                    shorthand_modes.add('1x')
                    shorthand_modes.add('/1')
                    shorthand_modes.add(m[-2:])
            raise NoSuchMode('error: mode not found: %s, must be one of: %s' %
                             (mode, str(list(shorthand_modes) + list(self.modes))))

    @classmethod
    def speed(self, mode):
        '''
        Return the port speed in gigabits per second for a given mode.
        '''
        self._check_mode(mode)
        return self._speed_map[mode]

    @classmethod
    def num_lanes(self, mode):
        speed = self.speed(mode)
        if speed in (40, 100):
            return 4
        elif speed == 50:
            return 2
        elif speed in (1, 10, 25):
            return 1
        else:
            raise RuntimeError('invalid speed: %d' % speed)

    def num_physical_ports(self, mode):
        '''
        Return the number of physical (front panel holes) for a given port.
        '''
        return self.num_logical_ports(mode)

    def num_logical_ports(self, mode):
        '''
        Return the number of logical (Linux interfaces) for a given port.
        '''
        raise NotImplementedError

    @property
    def max_logical_ports(self):
        '''
        Return the maximum number of logical (Linux interfaces) for a given port.
        '''
        raise NotImplementedError

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port would contribute to the
        system total.
        '''
        if self.configurable(config_map) and (not self.is_fabric or self.create_fabric):
            return self.max_logical_ports
        return 0

    @property
    def num_phys(self):
        '''
        Return the number of phys for a given port.
        '''
        if self.phy_addrs is not None:
            return len(filter(lambda x: x is not None, self.phy_addrs))
        else:
            return 0

    def hw_intf_prefix(self, mode, sub):
        '''
        Return the physical port interface prefix (always unmapped by portmap)
        '''
        return self.hw_ports[sub][:2]

    def hw_intf_num(self, mode, sub):
        '''
        Return the physical port interface number (always unmapped by portmap)
        '''
        if self.num_logical_ports(mode) == 2:
            return int(self.hw_ports[sub * 2][2:])
        else:
            return int(self.hw_ports[sub][2:])

    def hw_intf_num_portmap(self, mode, sub):
        '''
        Return the physical port interface number to be used by portmap
        '''
        return(self.hw_intf_num(mode, sub))

    def base_hw_intf_num(self, mode, sub):
        '''
        Return the lowest numbered hw_intf_num that makes up a logical port.
        '''
        lports = self.num_logical_ports(mode)
        hports = len(self.hw_ports)
        if hports > lports:
            # the interface uses of multiple serdes
            return min([ self.hw_intf_num(mode, x) for x in range(hports) ])
        else:
            return self.hw_intf_num(mode, sub)

    def hw_intf(self, mode, sub):
        '''
        Return the physical hardware port for the sub-interface (prefix
        concatenated with number).
        '''
        return '%s%u' % (self.hw_intf_prefix(mode, sub), self.hw_intf_num(mode, sub))

    def hw_pbmp(self, mode, sub):
        '''
        Return the hardware port bitmap mask for the sub-interface.
        '''
        return 1 << self.pbmp_map[self.hw_ports[sub]]

    def hw_port_num(self, mode, sub):
        '''
        Return the hardware port number (pbmp bit position)
        '''
        return self.pbmp_map[self.hw_ports[sub]]

    def port_mac_index(self, mode, mac_index, sub):
        '''
        Return the MAC address index for the sub-interface
        '''
        return mac_index + sub

    def led_port_num(self, mode, sub):
        '''
        Return the led port number (identical to the hw_port_num on most platforms)
        '''
        return self.pbmp_map[self.hw_ports[sub]]

    def set_leds(self, mode, led_port, sub, led_index_list):
        return {}

    def polarity_flip_val(self, mode, sub, polarity_flip):
        '''
        Return the generic TX/RX polarity_flip parameter for the
        interface as an integer.
        '''
        raise NotImplementedError

    def rx_logical_polarity_flip(self, mode, sub):
        '''
        Return the rx_polarity_flip parameter for the interface
        as an integer.
        '''
        return self.polarity_flip_val(mode, sub, self.rx_polarity_flip)

    def tx_logical_polarity_flip(self, mode, sub):
        '''
        Return the rx_polarity_flip parameter for the interface
        as an integer.
        '''
        return self.polarity_flip_val(mode, sub, self.tx_polarity_flip)

    def serdes_if_type(self, speed):
        '''
        Return the serdes interface type based on the port speed
        '''
        return self._serdes_intf_map.get(speed, None)

    def an_c37(self, speed):
        return None

    def an_c73(self, speed):
        return None

    def logical_port_num(self, mode, sub):
        '''
        Return the logical port number for the sub-interface
        '''
        raise NotImplementedError


class MLXSpeedModeInfo:
    '''
    Speed mode information

    speed       - speed in Gbps
    speed_bmap  - SDK speed bitmap value
    width       - number of serdes lanes
    num_ports   - number of logical ports

    '''
    def __init__(self, speed=None, speed_bmap=None,
                 width=None, num_ports=None):
        self.speed = speed
        self.speed_bmap = speed_bmap
        self.width = width
        self.num_ports = num_ports

class MLXSwitch(SDKSwitch):
    '''
    Mellanox switch platform
    '''
    def __init__(self, chip) :
        SDKSwitch.__init__(self, chip)

class SwitchXTwoSwitch(MLXSwitch):
    '''
    Mellanox switch with Switch-X2 ASIC
    '''
    def __init__(self, chip) :
        MLXSwitch.__init__(self, chip)

class SpectrumSwitch(MLXSwitch):
    '''
    Mellanox switch with Spectrum ASIC
    '''
    def __init__(self, chip) :
        MLXSwitch.__init__(self, chip)

class MLXPort(SDKPort):
    '''
    Mellanox specific port data.
    '''
    autoneg_pref = True

    _autodetect_phys = ()
    _mode_info_map = {}

    def __init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
             leds, unit, rx_polarity_flip, tx_polarity_flip,
             rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
             pre_drv_current, oversub, phy_id1, sys_interface):
        SDKPort.__init__(self, label, hw_ports, phy_types, phy_addrs, phy_layer, prim_offset,
             leds, unit, rx_polarity_flip, tx_polarity_flip,
             rx_lane_map, tx_lane_map, mdi_pair_map, pre_emphasis, drv_current,
             pre_drv_current, oversub, phy_id1, sys_interface)

    def phy_autodetect(self, mode, sub):
        raise NotImplementedError

    def speed(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].speed

    def speed_bmap(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].speed_bmap

    def num_logical_ports(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].num_ports

    def serdes_width(self, mode):
        self._check_mode(mode)
        return self._mode_info_map[mode].width

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError
        return str(p)

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError
        return str(self.phy_addrs[p])

class SwitchXTwoPort(MLXPort):
    def __init__(self, label, adj_label, hw_ports, phy_addrs, phy_layer,
                 blocked_adj_split=None):
        '''
        Addition port attributes:

        adj_label is front panel port label of the adjacent port (because we
            don't want to automatically assume the adjacent label is label+1),
        blocked_adj_split Port is blocked when its adjacent port is split into
            this number of ports (or more)
        '''
        self.adj_label = adj_label
        self.blocked_adj_split = blocked_adj_split

        MLXPort.__init__(self, label, hw_ports=hw_ports, phy_types=None,
                         phy_addrs=phy_addrs, phy_layer=phy_layer,
                         prim_offset=None, leds=None, unit=0,
                         rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

    def blocked_adj_split_num(self):
        if not self.blocked_adj_split is None:
            return self.blocked_adj_split

        return 0

    def configurable(self, config_map):
        '''
        On Mellanox platforms, a port may be blocked (i.e., left unused)
        when an adjacent port is split. As such, it would not appear
        in any config file (such as ports.conf).

        Alternatively, the blocked port might have been marked as 'disable'.
        '''
        if config_map.get(self.label, None) is not None:
            mode = config_map.get(self.label, self.default_mode)
            return not mode.lower().startswith('disable')
        return False


class SwitchXTwo10GPort(SwitchXTwoPort):
    modes = ('10G')
    default_mode = '10G'
    _mode_info_map = { '10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4,  width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SwitchXTwoPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                                phy_addrs=(phy_addr,), phy_layer='SFP+')

    @property
    def max_logical_ports(self):
        return 1

class SwitchXTwo40GPort(SwitchXTwoPort):
    modes = ('40G', '4x10G', '2x10G')
    default_mode = '40G'
    _mode_info_map = { '40G'   : MLXSpeedModeInfo(speed=40, speed_bmap=1<<6, width=4, num_ports=1),
                       '4x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4, width=1, num_ports=4),
                       '2x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=1<<4, width=1, num_ports=2)}

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        SwitchXTwoPort.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                                phy_addrs=phy_addrs, phy_layer='QSFP+',
                                blocked_adj_split=blocked_adj_split)

    @property
    def max_logical_ports(self):
        return len(self.hw_ports)

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port would contribute to the
        system total. It contributes zero if it could be blocked by a split
        adjacent port, in which case, the contribution would come from the
        adjacent port when everything is eventually summed up.
        '''
        if self.blocked_adj_split_num() > 0:
            return 0

        return self.max_logical_ports

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        _xml_lanes_map = { '40G'  : '0,1,2,3',
                           '4x10G' : str(p),
                           '2x10G' : str(p)}

        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        return _xml_lanes_map[mode]

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        # return phy_addrs tuple as a string without any parens or spaces
        #
        if mode == '40G':
            return str(self.phy_addrs).strip('()').replace(' ', '')

        return str(self.phy_addrs[p])

class SpectrumPort(MLXPort):
    def __init__(self, label, adj_label, hw_ports, phy_addrs, phy_layer,
                 blocked_adj_split=None):
        '''
        Addition port attributes:

        adj_label is front panel port label of the adjacent port (because we
            do not want to automatically assume the adjacent label is label+1),
        blocked_adj_split Port is blocked when its adjacent port is split into
            this number of ports (or more)
        '''
        self.adj_label = adj_label
        self.blocked_adj_split = blocked_adj_split

        MLXPort.__init__(self, label, hw_ports=hw_ports, phy_types=None,
                         phy_addrs=phy_addrs, phy_layer=phy_layer,
                         prim_offset=None, leds=None, unit=0,
                         rx_polarity_flip=None, tx_polarity_flip=None,
                         rx_lane_map=None, tx_lane_map=None, mdi_pair_map=None,
                         pre_emphasis=None, drv_current=None,
                         pre_drv_current=None, oversub=False,
                         phy_id1=None, sys_interface=None)

    def blocked_adj_split_num(self):
        if not self.blocked_adj_split is None:
            return self.blocked_adj_split

        return 0

    def configurable(self, config_map):
        '''
        On Mellanox platforms, a port may be blocked (i.e., left unused)
        when an adjacent port is split. As such, it would not appear
        in any config file (such as ports.conf).

        Alternatively, the blocked port might have been marked as 'disable'.
        '''
        if config_map.get(self.label, None) is not None:
            mode = config_map.get(self.label, self.default_mode)
            return not mode.lower().startswith('disable')
        return False

    def port_mac_index(self, mode, mac_index, sub):
        '''
        Return the MAC address index for the sub-interface
        '''
        return self.hw_intf_num(mode, sub) - 1

class Spectrum25GPort(SpectrumPort):
    modes = ('10G', '25G')
    default_mode = '25G'
    _mode_info_map = { '10G'   : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=1),
                       '25G'   : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                              phy_addrs=(phy_addr,), phy_layer='SFP28')

    @property
    def max_logical_ports(self):
        return 1

class Spectrum10GPort(SpectrumPort):
    modes = ('10G',)
    default_mode = '10G'
    _mode_info_map = { '10G'   : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=1)}

    def __init__(self, label, adj_label, hw_port, phy_addr):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=(hw_port,),
                              phy_addrs=(phy_addr,), phy_layer='SFP+')

    @property
    def max_logical_ports(self):
        return 1

class Spectrum28Port(SpectrumPort):
    modes = ( '40G',
              '50G',
              '100G',
              '2x50G',
              '4x10G',
              '4x25G')

    # 40G always uses 4 lanes per CR4/SR4 standard, hence
    # splitting into 2x40G is not possible. Split types other than
    # listed below are supported in hardware but for now we exclude
    # them to simplify testing.
    #
    _mode_info_map = { '40G'   : MLXSpeedModeInfo(speed=40, speed_bmap=0x2801F041, width=4, num_ports=1),
                       '50G'   : MLXSpeedModeInfo(speed=50, speed_bmap=0x68007001, width=2, num_ports=1),
                       '100G'  : MLXSpeedModeInfo(speed=100,speed_bmap=0x68B1F041, width=4, num_ports=1),
                       '2x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=2),
                       '2x25G' : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=2),
                       '2x50G' : MLXSpeedModeInfo(speed=50, speed_bmap=0x68007001, width=2, num_ports=2),
                       '4x10G' : MLXSpeedModeInfo(speed=10, speed_bmap=0x00007001, width=1, num_ports=4),
                       '4x25G' : MLXSpeedModeInfo(speed=25, speed_bmap=0x28007001, width=1, num_ports=4)}

    def __init__(self, label, adj_label, hw_ports, phy_addrs, phy_layer, blocked_adj_split=None):
        SpectrumPort.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                              phy_addrs=phy_addrs, phy_layer=phy_layer,
                              blocked_adj_split=blocked_adj_split)

    @property
    def max_logical_ports(self):
        return len(self.hw_ports)

    def mac_addrs_contrib_total(self, config_map):
        '''
        Return the number of MAC addresses this port contributes to the system
        total. If there is an adjacent port, then it contributes 4 minus the
        number of macs consumed by the adjacent port, up to the number of ports.
        If there is no adjacent port but the port is not configurable, it
        contributes zero macs: otherwise the port is configurable and it
        it contributes the number of ports, with a minimum of 2.
        '''
        adj_mode = config_map.get(self.adj_label)
        if adj_mode is not None:
            adj_modeInfo = self._mode_info_map.get(adj_mode)
            if adj_modeInfo is not None:
                return min(4-adj_modeInfo.num_ports, self.max_logical_ports)
        elif not self.configurable(config_map):
            # this port has been disabled
            return 0
        return max(2, self.num_logical_ports(config_map.get(self.label)))

    # Convert (mode, p) to Mellanox XML (serdes) lanes string
    #
    def xml_lanes(self, mode, p):
        _xml_2x50G_lanes_map = { 0 : '0,1',
                                 1 : '2,3'}

        _xml_lanes_map = { '1G'    : str(p),
                           '10G'   : str(p),
                           '25G'   : str(p),
                           '40G'   : '0,1,2,3',
                           '50G'   : '0,1',
                           '100G'  : '0,1,2,3',
                           '2x1G'  : str(p),
                           '2x10G' : str(p),
                           '2x25G' : str(p),
                           '2x50G' : None,
                           '4x1G'  : str(p),
                           '4x10G' : str(p),
                           '4x25G' : str(p)}
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        if mode == '2x50G':
            return _xml_2x50G_lanes_map[p]
        return _xml_lanes_map[mode]

    # Convert (mode, p, phy_addrs) to Mellanox XML lane-to-module string
    #
    def xml_modules(self, mode, p):
        self._check_mode(mode)
        if p >= self.num_logical_ports(mode):
            raise IndexError

        width = self.serdes_width(mode)
        addr_idx = p * width
        mod_list = []
        for w in range(width):
            mod_list.append(self.phy_addrs[addr_idx])
            addr_idx = addr_idx + 1

        # Return list as a string without any brackets or spaces,
        # e.g., [1, 2] would be returned as '1,2'
        #
        return str(mod_list).strip('[]').replace(' ', '')

    def hw_intf_num(self, mode, sub):
        '''
        Return the physical port interface number (always unmapped by portmap)
        '''
        return int(self.hw_ports[sub][2:])

class Spectrum40GPort(Spectrum28Port):
    modes = ('40G', '4x10G')
    default_mode = '40G'

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        Spectrum28Port.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                                phy_addrs=phy_addrs, phy_layer='QSFP+',
                                blocked_adj_split=blocked_adj_split)

class Spectrum100GPort(Spectrum28Port):
    default_mode = '100G'

    def __init__(self, label, adj_label, hw_ports, phy_addrs, blocked_adj_split=None):
        Spectrum28Port.__init__(self, label, adj_label=adj_label, hw_ports=hw_ports,
                                phy_addrs=phy_addrs, phy_layer='QSFP28',
                                blocked_adj_split=blocked_adj_split)


class VXSwitch():
    '''
    Virtual Switch platform for Cumulus VX
    '''
    has_phy_ucode_spi_rom = False

    def __init__(self, chip):
        self.chip = chip
        self.ports = ()
        self.led_controllers = ()

class SDKChip:
    '''
    SDK chip common attributes
    '''
    sw_base         = None # SDK
    pci_vendor      = None # PCI vendor ID
    pci_device      = None # PCI device ID
    file_list       = None # datapath configuration file list
    port_channel_type_map = {} # maps number of channels to a port class

    # XXX - These belong in BCMChip
    portmap_capable = None # supports Broadcom portmap remapping
    dport_capable   = None # supports Broadcom dport mapping
    bcm_pn          = None # Broadcom part number
    hsp_bw          = None # chip bandwidth
    base_pbmp       = None # defaut/base port bitmap, typically enables ports
                           # to the CPU and "load balancer"

    def __init__(self, initial_path) :
        self.chip_dir_list = [initial_path]

    @classmethod
    def is_acpi(self):
        return False

    @property
    def chip_config_path(self):
            return os.path.dirname(__file__) + '/__chip_config/'

    @property
    def bcm_config_path(self):
            return os.path.dirname(__file__) + '/__bcm_config/'

    def get_config(self, file_name) :
        if self.file_list == None :
            return None
        if file_name in self.file_list :
            for prefix in self.chip_dir_list :
                file_path = self.chip_config_path + prefix + '/' + file_name
                if os.path.exists(file_path) :
                    with open(file_path, 'r') as config_file :
                        config = config_file.read()
                        return config
        return None


class MLXChip(SDKChip):
    '''
    Mellanox chip common attributes
    '''
    sw_base = 'mlx'
    ipv4_host_table_size = 32*1024
    file_list = 'datapath.conf'

    def __init__(self) :
        SDKChip.__init__(self, '/mlx')

class SwitchXTwoChip(MLXChip):
    '''
    Mellanox Switch-X2 ASIC
    '''
    def __init__(self) :
        MLXChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Switch-X2')

class SpectrumChip(MLXChip):
    '''
    Mellanox Spectrum ASIC
    '''
    def __init__(self) :
        MLXChip.__init__(self)
        self.chip_dir_list.insert(0,self.chip_dir_list[0] + '/Spectrum')

class VXChip(SDKChip):
    '''
    Virtual Chip for Cumulus VX
    '''
    sw_base = 'vx'
    file_list = None
    # This number is max of all current chips (if default forwarding table profile
    # is used)
    ipv4_host_table_size = 40*1024

    def __init__(self) :
        SDKChip.__init__(self, '/vx')

chips = (TriumphChip,
         ApolloChip,
         TridentChip,
         Helix4Chip,
         Hurricane2Chip,
         TridentTwo_56850_Chip,
         TridentTwo_56854_Chip,
         TridentThreeX5_56771_Chip,
         TridentThreeX7_56870_Chip,
         TridentThreeX7_56873_Chip,
         TomahawkChip,
         TomahawkPlus_56965_Chip,
         TomahawkPlus_56967_Chip,
         TomahawkTwoChip,
         TomahawkTwo_56970_Chip,
         MaverickChip,
         Maverick_56762_Chip,
         Maverick_56768_Chip)

class BMC:
    '''
    Information related to the baseboard management controller.
    These are the members so far:
        name (string): a short, printable name
            It should be unique among all the entities
            on this switch.  For a chassis, there may be
            many BMCs as well as switch processors.
        type (string): a the BMC type
            Known values are
                openbmc: OpenBMC
        ipv6_addr (string): the fixed, well-known, internal ipv6
                            address, including the zone ("%...")
            This should ideally be an address on an internal
            management network.  The processor should be able
            to reach the BMC at this address.
        ipv4_addr (string): the fixed, well-known, internal ipv4
                            address
            This should ideally be an address on an internal
            management network.  The processor should be able
            to reach the BMC at this address.
    '''
    name = None
    type = None
    ipv6_addr = None
    ipv4_addr = None
    ignores = []
    renames = {}
