#
# Copyright 2016, 2019.  Cumulus Networks, Inc.
#

import os

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS4610_54SFPp(cumulus.platform.Helix410GPort):
    '''
;;; Accton AS4610-54
;;;
;;; There are two LEDs for each of the four SFP+ and two Stack ports.
;;; The Stack port LED data is first, followed by the SFP+ port LED data.
;;;
;;; The two LED bits behave as follows:
;;;
;;;       1st bit       2nd bit
;;;      +-------+     +-------+
;;;    0 | green |     | amber |
;;;      +-------+     +-------+
;;;    1 |  off  |     |  off  |
;;;      +-------+     +-------+
;;;
'''
    _set_led = '''
'''

    _set_led_10G = '''
;;; AcctonAS4610_54SFP_10G
set_led:
    port    1
    pushst  PORTSTATUS_ONE
    pack

    port    a
    pushst  PORTSTATUS_RX
    pushst  PORTSTATUS_TX
    tor
    push    (BLINK_STATE_P)
    tand
    tinv                    ; flash off with activity
    pushst  PORTSTATUS_LINKEN
    tand
    tinv
    pack

    ret
'''
    _set_led_1G = '''
;;; AcctonAS4610_54_1G
set_led:
    port    1
    pushst  PORTSTATUS_ONE
    pack

    port    a
    pushst  PORTSTATUS_RX
    pushst  PORTSTATUS_TX
    tor
    push    (BLINK_STATE_P)
    tand
    tinv                    ; flash off with activity
    pushst  PORTSTATUS_LINKEN
    tand
    tinv
    pack

    ret
'''
    _set_led_stack = '''
;;; AcctonAS4610_54_Stack
set_led:
    port    1
    pushst  PORTSTATUS_ONE
    pack

    port    1
    pushst  PORTSTATUS_ONE
    pack

    port    1
    pushst  PORTSTATUS_ONE
    pack

    port    1
    pushst  PORTSTATUS_ONE
    pack

    ret
'''
    def __init__(self, label, hw_port, phy_addr, leds, rx_polarity_flip=None,
                 tx_polarity_flip=None, tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="84758",
                                                phy_addr=phy_addr,
                                                phy_layer="SFP+",
                                                leds=leds,
                                                rx_lane_map=rx_lane_map,
                                                tx_lane_map=tx_lane_map,
                                                rx_polarity_flip=rx_polarity_flip,
                                                tx_polarity_flip=tx_polarity_flip,
                                                unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led_10G, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None

        return set_calls

class AcctonAS4610_54GigE(cumulus.platform.Helix41GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Helix41GPort.__init__(self,
                                               label=label,
                                               hw_port=hw_port,
                                               phy_type="54282",
                                               phy_addr=phy_addr,
                                               phy_layer="1GBaseT",
                                               prim_offset=prim_offset,
                                               leds=None,
                                               unit=0)

class AcctonAS4610_54LED(cumulus.ledup.Helix4LEDController):
    num_leds = 12


#------------------------------------------------------------------------------
# Accton AS4610 Switch Ports

class AcctonAS4610_54Switch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS4610_54LED
    _gige = AcctonAS4610_54GigE
    _sfp = AcctonAS4610_54SFPp
    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _gige(label= "1", hw_port="ge25", phy_addr=0x22, prim_offset=0x1901),
        _gige(label= "2", hw_port="ge24", phy_addr=0x21, prim_offset=0x1900),
        _gige(label= "3", hw_port="ge26", phy_addr=0x23, prim_offset=0x1902),
        _gige(label= "4", hw_port="ge27", phy_addr=0x24, prim_offset=0x1903),
        _gige(label= "5", hw_port="ge29", phy_addr=0x26, prim_offset=0x1905),
        _gige(label= "6", hw_port="ge28", phy_addr=0x25, prim_offset=0x1904),
        _gige(label= "7", hw_port="ge30", phy_addr=0x27, prim_offset=0x1906),
        _gige(label= "8", hw_port="ge31", phy_addr=0x28, prim_offset=0x1907),
        _gige(label= "9", hw_port="ge33", phy_addr=0x2c, prim_offset=0x2101),
        _gige(label= "10", hw_port="ge32", phy_addr=0x2b, prim_offset=0x2100),
        _gige(label= "11", hw_port="ge34", phy_addr=0x2d, prim_offset=0x2102),
        _gige(label= "12", hw_port="ge35", phy_addr=0x2e, prim_offset=0x2103),
        _gige(label= "13", hw_port="ge37", phy_addr=0x30, prim_offset=0x2105),
        _gige(label= "14", hw_port="ge36", phy_addr=0x2f, prim_offset=0x2104),
        _gige(label= "15", hw_port="ge38", phy_addr=0x31, prim_offset=0x2106),
        _gige(label= "16", hw_port="ge39", phy_addr=0x32, prim_offset=0x2107),
        _gige(label= "17", hw_port="ge41", phy_addr=0x36, prim_offset=0x2901),
        _gige(label= "18", hw_port="ge40", phy_addr=0x35, prim_offset=0x2900),
        _gige(label= "19", hw_port="ge42", phy_addr=0x37, prim_offset=0x2902),
        _gige(label= "20", hw_port="ge43", phy_addr=0x38, prim_offset=0x2903),
        _gige(label= "21", hw_port="ge45", phy_addr=0x3a, prim_offset=0x2905),
        _gige(label= "22", hw_port="ge44", phy_addr=0x39, prim_offset=0x2904),
        _gige(label= "23", hw_port="ge46", phy_addr=0x3b, prim_offset=0x2906),
        _gige(label= "24", hw_port="ge47", phy_addr=0x3c, prim_offset=0x2907),
        _gige(label= "25",  hw_port="ge1",  phy_addr=0x2,  prim_offset=0x0101),
        _gige(label= "26",  hw_port="ge0",  phy_addr=0x1,  prim_offset=0x0100),
        _gige(label= "27",  hw_port="ge2",  phy_addr=0x3,  prim_offset=0x0102),
        _gige(label= "28",  hw_port="ge3",  phy_addr=0x4,  prim_offset=0x0103),
        _gige(label= "29",  hw_port="ge5",  phy_addr=0x6,  prim_offset=0x0105),
        _gige(label= "30",  hw_port="ge4",  phy_addr=0x5,  prim_offset=0x0104),
        _gige(label= "31",  hw_port="ge6",  phy_addr=0x7,  prim_offset=0x0106),
        _gige(label= "32",  hw_port="ge7",  phy_addr=0x8,  prim_offset=0x0107),
        _gige(label= "33",  hw_port="ge9",  phy_addr=0xc,  prim_offset=0x0901),
        _gige(label= "34", hw_port="ge8",  phy_addr=0xb,  prim_offset=0x0900),
        _gige(label= "35", hw_port="ge10", phy_addr=0xd,  prim_offset=0x0902),
        _gige(label= "36", hw_port="ge11", phy_addr=0xe,  prim_offset=0x0903),
        _gige(label= "37", hw_port="ge13", phy_addr=0x10, prim_offset=0x0905),
        _gige(label= "38", hw_port="ge12", phy_addr=0xf,  prim_offset=0x0904),
        _gige(label= "39", hw_port="ge14", phy_addr=0x11, prim_offset=0x0906),
        _gige(label= "40", hw_port="ge15", phy_addr=0x12, prim_offset=0x0907),
        _gige(label= "41", hw_port="ge17", phy_addr=0x16, prim_offset=0x1101),
        _gige(label= "42", hw_port="ge16", phy_addr=0x15, prim_offset=0x1100),
        _gige(label= "43", hw_port="ge18", phy_addr=0x17, prim_offset=0x1102),
        _gige(label= "44", hw_port="ge19", phy_addr=0x18, prim_offset=0x1103),
        _gige(label= "45", hw_port="ge21", phy_addr=0x1a, prim_offset=0x1105),
        _gige(label= "46", hw_port="ge20", phy_addr=0x19, prim_offset=0x1104),
        _gige(label= "47", hw_port="ge22", phy_addr=0x1b, prim_offset=0x1106),
        _gige(label= "48", hw_port="ge23", phy_addr=0x1c, prim_offset=0x1107),

        _sfp(label="49",   hw_port="hg0",  phy_addr=0x40, leds=(0, 10)),
        _sfp(label="50",   hw_port="hg1",  phy_addr=0x41, leds=(0, 8)),
        _sfp(label="51",   hw_port="hg2",  phy_addr=0x42, leds=(0, 6)),
        _sfp(label="52",   hw_port="hg3",  phy_addr=0x43, leds=(0, 4)),
    )


#==============================================================================
#
# Accton AS4610 CPLD
#

class AcctonAS4610_54CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0030'


#==============================================================================
#
# Accton AS4610 Sensors
#

#------------------------------------------------------------------------------
# Accton AS4610 Power Supplies

class AcctonAS4610_54PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = AcctonAS4610_54CPLD.path
    driver_hwmon = ['psu_pwr1']
    hwmon_search = False

class AcctonAS4610_54PSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = AcctonAS4610_54CPLD.path
    driver_hwmon = ['psu_pwr2']
    hwmon_search = False


#------------------------------------------------------------------------------
# Accton AS4610 Temperature Sensors

# Helix-4 has three onboard temp sensors:
#
#    TOP_PVTMON_RESULT_0 - Near center of the die
#    TOP_PVTMON_RESULT_1 - Lower left corner of the die, near WC0
#    TOP_PVTMON_RESULT_2 - Upper right edge of the die in iProc block
#
# These are monitored with registered reads of internal registers in the TOP
# block (TOP_PVTMON_RESULT_0, TOP_PVTMON_RESULT_1, and TOP_PVTMON_RESULT_2).
# Not sure how to incorporate these into the python sensors code.  Probably
# need to write a driver for the Helix4...

### Accton will provide temperature settings later.

class AcctonAS4610_54Temp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Ambient Temperature'
    driver_path = '/sys/bus/i2c/devices/9-0048'
    driver_hwmon = ['temp1']
    temp_crit = 75
    temp_crit_hyst = 70
    temp_max = 60
    temp_min = 10
    temp_fan_max = 55


#------------------------------------------------------------------------------
# Accton AS4610 PSU Temp Sensors

### Accton will provide PSU spec later.


#------------------------------------------------------------------------------
# Accton AS4610 Fans

# maximum fan speed = 17,000 RPM
# fan_max = 75.0% (Fan Speed Control = 0x06)
# fan_min = 12.5% (Fan Speed Control = 0x01)

class AcctonAS4610_54Fan(cumulus.sensors.Fan_Unit):
    pwm_path = AcctonAS4610_54CPLD.path
    driver_path = AcctonAS4610_54CPLD.path
    pwm_hwmon = ['pwm1']
    cpld_path = AcctonAS4610_54CPLD.path
    # min of 32 translates to fan1_target = 2
    minpwm = 32
    # max of 111 translates to fan1_target = 6
    maxpwm = 111
    hwmon_search = False

class AcctonAS4610_54Fan1(AcctonAS4610_54Fan):
    name = 'Fan1'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']
    fan_max = 9750
    fan_min = 2350

class AcctonAS4610_54BFan(AcctonAS4610_54Fan):
    fan_max = 9750
    fan_min = 2500

class AcctonAS4610_54BFan1(AcctonAS4610_54BFan):
    name    = 'Fan1'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']

class AcctonAS4610_54BFan2(AcctonAS4610_54BFan):
    name = 'Fan2'
    cpld_hwmon = ['fan2']
    driver_hwmon = ['fan2']

#------------------------------------------------------------------------------
# Accton AS4610-54 PSU Fans

# Each PSU also has a fan that can be read from I2C.
# The fan speed is at address 0x90.
# fan_max = 100%
# fan_min = 40%
#

def AcctonAS4610_54POE_Support():
    # Quick hack to prevent other platforms from running this.
    # Need to fix this elegantly.
    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return ()
    if plat.strip() != 'accton,as4610_54':
        return ()

    cpld_path = '/sys/bus/i2c/devices/0-0030/'
    cpld_reg = 'poe_support'
    fn = os.path.join(cpld_path, cpld_reg)
    try:
        value = open(fn, 'r').read()
        return value.strip().rstrip('\n')
    except:
        # Unable to determine the value of POE Support, assuming non POE
        print 'Unable to determine the value of POE Support, assuming non POE'
        return '0'

def getProductId():
    cpld_reg = '/sys/bus/i2c/devices/0-0030/product_id'
    try:
	val = open(cpld_reg, 'r').read()
    except:
	return 'unknown'
        
    return val.strip().rstrip('\n')

def AcctonAS4610_54FindFans():
    as4610_54p_fans = (AcctonAS4610_54Fan1(),
                       )
    as4610_54t_fans = ()
    as4610_54tb_fans = (AcctonAS4610_54BFan1(), AcctonAS4610_54BFan2(),)

    if AcctonAS4610_54POE_Support() == '1':
        return as4610_54p_fans
    # Two versions of non poe switches
    elif getProductId() == 'AS4610-48-B':
	return as4610_54tb_fans
    elif getProductId() == 'AS4610-48':
        return as4610_54t_fans
    else:
	return ()
	

class AcctonAS4610_54Sensors():
    units = (AcctonAS4610_54PSU1(),
             AcctonAS4610_54PSU2(),
             AcctonAS4610_54Temp1(),
             )
    fans = AcctonAS4610_54FindFans()
    units = units + fans


#==============================================================================
#
# Accton AS4610 PWM Control
#

class AcctonAS4610_54PWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = AcctonAS4610_54FindFans()
    temps = (AcctonAS4610_54Temp1(),
             )

class AcctonAS4610_54PWMs():
    units = (AcctonAS4610_54PWM(),
             )


#==============================================================================
#
# Accton AS4610 Status LEDs
#

'''
    Accton AS4610-54 LEDs

        System:
          green - normal operation
          amber - diagnostics failed or fan failed
        PSU1/2:
          green - psu ok
          amber - psu failure or psu missing
        Fan:
          green - fan ok
          amber - fan failure or fan missing
        PoE:
          green - PoE ok
          amber - failure or psu missing

    Not used

        PRI:
          To indicate if this is the primary (or secondary or other) unit
        Stack1/2:
          To indicate if the ports are in stacking mode or standalone mode.
        Display1/2:
          7-segment display.
        Dot1/2:
          Dot in the 7-segment display.
'''

class AcctonAS4610_54StatusLED(cumulus.sensors.StatusLED_Unit):
    # if no sources are specified, assumes all sources
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'yellow'

class AcctonAS4610_54PSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1 LED'
    sources = 'PSU1'
    led_location = 'led_psu1'
    good_led_color = 'green'
    bad_led_color = 'yellow'

class AcctonAS4610_54PSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2 LED'
    sources = 'PSU2'
    led_location = 'led_psu2'
    good_led_color = 'green'
    bad_led_color = 'yellow'

class AcctonAS4610_54FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan LED'
    sources = 'Fan'
    led_location = 'led_fan'
    good_led_color = 'green'
    bad_led_color = 'yellow'

class AcctonAS4610_54PoELED(cumulus.sensors.StatusLED_Unit):
    name = 'PoE LED'
    sources = ['POE']
    led_location = 'led_poe'
    good_led_color = 'green'
    bad_led_color = 'yellow'

    def led_find_color(self, status):
        """
        For POE led we will default to bad state and only
        indicate good if we get a OK status
        """
        color = self.bad_led_color
        for source in self.sources:
            if status.has_key(source):
                state = status[source]
                if state in self.good_states:
                    color = self.good_led_color
                    break
        return color

def AcctonAS4610_54FindLEDs():
    as4610_54t_units = (AcctonAS4610_54StatusLED(),
                        AcctonAS4610_54PSU1LED(),
                        AcctonAS4610_54PSU2LED(),
                        )
    as4610_54p_units = (AcctonAS4610_54StatusLED(),
                        AcctonAS4610_54PSU1LED(),
                        AcctonAS4610_54PSU2LED(),
                        AcctonAS4610_54FanLED(),
                        AcctonAS4610_54PoELED(),
                        )

    if AcctonAS4610_54POE_Support() == '1':
        return as4610_54p_units
    else:
        return as4610_54t_units

class AcctonAS4610_54LEDs():
    units = AcctonAS4610_54FindLEDs()


#==============================================================================
#
# Accton AS4610 Class
#

class AcctonAS4610_54(cumulus.platform.Platform):
    name = 'accton,as4610_54'
    chip = cumulus.platform.Helix4Chip()
    switch = AcctonAS4610_54Switch(chip)
    sensors = AcctonAS4610_54Sensors()
    pwms = AcctonAS4610_54PWMs()
    cpld = AcctonAS4610_54CPLD()
    statusleds = AcctonAS4610_54LEDs()

Platform = AcctonAS4610_54
