#
# Copyright (c) 2019, 2020 Cumulus Networks, Inc.  All rights reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS5835SFP(cumulus.platform.TridentThreeMGPort):
    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
        cumulus.platform.TridentThreeMGPort.__init__(self,
                                                     label=label,
                                                     serdes=serdes,
                                                     phy_types=None,
                                                     phy_addrs=None,
                                                     phy_layer='SFP+',
                                                     leds=None,
                                                     tx_lane_map=tx_lane_map,
                                                     rx_lane_map=rx_lane_map,
                                                     tx_polarity_flip=tx_polarity_flip,
                                                     rx_polarity_flip=rx_polarity_flip,
                                                     unit=0,
                                                     oversub=True)
        self.sfp = sfp

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    # Normally we just use the physical port number as the logical port number.
    # In this case we must remap that physical port number (on the left) to
    # a new logical port number (on the right):
    # physical port numbers 1 to 40 must map to logical port numbers 0 to 32.
    # physical port numbers 41 to 80 must map to logical port numbers 33 to 64.
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 53: 33,
                  54: 34,
                  55: 35,
                  56: 36,
                  57: 37,
                  58: 38,
                  59: 39,
                  60: 40,
                  61: 41,
                  62: 42,
                  63: 43,
                  64: 44,
                  65: 45,
                  66: 46,
                  67: 47,
                  68: 48,
                  69: 49,
                  70: 50,
                  71: 51,
                  72: 52,
                  73: 53,
                  74: 54,
                  75: 55,
                  76: 56,
                  }
        lport = reMap.get(lport, lport)
        return lport

class AcctonAS5835QSFP28(cumulus.platform.TridentThree100GPort):
    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                       label=label,
                                                       serdes=serdes,
                                                       leds=None,
                                                       phy_types=None,
                                                       phy_addrs=None,
                                                       phy_layer=sfp.type.upper(),
                                                       tx_lane_map=tx_lane_map,
                                                       rx_lane_map=rx_lane_map,
                                                       tx_polarity_flip=tx_polarity_flip,
                                                       rx_polarity_flip=rx_polarity_flip,
                                                       pre_emphasis=pre_emphasis,
                                                       unit=0,
                                                       oversub=True)
        self.sfp = sfp

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    # Normally we just use the physical port number as the logical port number.
    # In this case we must remap that physical port number (on the left) to
    # a new logical port number (on the right):
    # physical port numbers 1 to 40 must map to logical port numbers 0 to 32.
    # physical port numbers 41 to 80 must map to logical port numbers 33 to 64.
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 37: 25,
                  39: 26,
                  29: 27,
                  31: 28,
                  33: 29,
                  35: 30,
                  49: 57,
                  51: 58,
                  45: 59,
                  47: 60,
                  41: 61,
                  43: 62,
            }
        lport = reMap.get(lport, lport)
        return lport

class AcctonAS5835_54XSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = AcctonAS5835SFP
    _qsfp28 = AcctonAS5835QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False
    ports = (
        _sfp28(label="1",  serdes=(0,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="2",  serdes=(1,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="3",  serdes=(2,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="4",  serdes=(3,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True),

        _sfp28(label="5",  serdes=(4,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="6",  serdes=(5,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="7",  serdes=(6,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="8",  serdes=(7,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="9",  serdes=(8,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10", serdes=(9,),  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="11", serdes=(10,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="12", serdes=(11,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="13", serdes=(12,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="14", serdes=(13,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="15", serdes=(14,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="16", serdes=(15,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="17", serdes=(16,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="18", serdes=(17,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="19", serdes=(18,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="20", serdes=(19,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True),

        _sfp28(label="21", serdes=(20,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="22", serdes=(21,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="23", serdes=(22,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="24", serdes=(23,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="25", serdes=(52,), tx_lane_map=0x3102, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="26", serdes=(53,), tx_lane_map=0x3102, rx_lane_map=0x3120, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="27", serdes=(54,), tx_lane_map=0x3102, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="28", serdes=(55,), tx_lane_map=0x3102, rx_lane_map=0x3120, tx_polarity_flip=True,  rx_polarity_flip=True),

        _sfp28(label="29", serdes=(56,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="30", serdes=(57,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="31", serdes=(58,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="32", serdes=(59,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="33", serdes=(60,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="34", serdes=(61,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="35", serdes=(62,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="36", serdes=(63,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True),

        _sfp28(label="37", serdes=(64,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="38", serdes=(65,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="39", serdes=(66,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="40", serdes=(67,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True),

        _sfp28(label="41", serdes=(68,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="42", serdes=(69,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="43", serdes=(70,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="44", serdes=(71,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),

        _sfp28(label="45", serdes=(72,), tx_lane_map=0x0231, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="46", serdes=(73,), tx_lane_map=0x0231, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="47", serdes=(74,), tx_lane_map=0x0231, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="48", serdes=(75,), tx_lane_map=0x0231, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        _qsfp28(label="49", serdes=( 36,  37,  38,  39), tx_lane_map=0x3210, rx_lane_map=0x2130, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="50", serdes=( 28,  29,  30,  31), tx_lane_map=0x2031, rx_lane_map=0x1302, tx_polarity_flip=0x000f, rx_polarity_flip=0x0005),
        _qsfp28(label="51", serdes=( 32,  33,  34,  35), tx_lane_map=0x3120, rx_lane_map=0x1203, tx_polarity_flip=0x0004, rx_polarity_flip=0x000a),
        _qsfp28(label="52", serdes=( 48,  49,  50,  51), tx_lane_map=0x3210, rx_lane_map=0x0312, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="53", serdes=( 44,  45,  46,  47), tx_lane_map=0x3210, rx_lane_map=0x3201, tx_polarity_flip=0x0000, rx_polarity_flip=0x0003),
        _qsfp28(label="54", serdes=( 40,  41,  42,  43), tx_lane_map=0x3210, rx_lane_map=0x1320, tx_polarity_flip=0x0000, rx_polarity_flip=0x0008),
    )

# CPLD paths

class AcctonAS5835_54XCPUCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/1-0065'

class AcctonAS5835_54XCPLD1(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/11-0060'

class AcctonAS5835_54XCPLD2(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/11-0061'

class AcctonAS5835_54XCPLD3(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/11-0062'

class AcctonAS5835_54XCPLD4(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/11-0063'

# PSUs

class AcctonAS5835_54XPSU(cumulus.sensors.PSU_Unit):
    cpld_path = AcctonAS5835_54XCPLD1.path

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.psu_present) == '1':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0

        if ((self.get_cpld_value(self.psu_ac_ok) == '1') and
            (self.get_cpld_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0

        return 0

class AcctonAS5835_54XPSU1(AcctonAS5835_54XPSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class AcctonAS5835_54XPSU2(AcctonAS5835_54XPSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

# Intel CPU temp sensors

class AcctonAS5835_54XCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_crit = 91

class AcctonAS5835_54XCPUTemp1(AcctonAS5835_54XCoreTemp):
    name = 'CPUTemp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class AcctonAS5835_54XCPUTemp2(AcctonAS5835_54XCoreTemp):
    name = 'CPUTemp2'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class AcctonAS5835_54XCPUTemp3(AcctonAS5835_54XCoreTemp):
    name = 'CPUTemp3'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class AcctonAS5835_54XCPUTemp4(AcctonAS5835_54XCoreTemp):
    name = 'CPUTemp4'
    description = 'Core 8'
    driver_hwmon = ['temp10']

class AcctonAS5835_54XCPUTemp5(AcctonAS5835_54XCoreTemp):
    name = 'CPUTemp5'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Trident3 internal temp sensors

class AcctonAS5835_54XASICTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class AcctonAS5835_54XASICTemp1(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp1'
    driver_hwmon = ['temp1']

class AcctonAS5835_54XASICTemp2(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp2'
    driver_hwmon = ['temp2']

class AcctonAS5835_54XASICTemp3(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp3'
    driver_hwmon = ['temp3']

class AcctonAS5835_54XASICTemp4(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp4'
    driver_hwmon = ['temp4']

class AcctonAS5835_54XASICTemp5(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp5'
    driver_hwmon = ['temp5']

class AcctonAS5835_54XASICTemp6(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp6'
    driver_hwmon = ['temp6']

class AcctonAS5835_54XASICTemp7(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp7'
    driver_hwmon = ['temp7']

class AcctonAS5835_54XASICTemp8(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp8'
    driver_hwmon = ['temp8']

class AcctonAS5835_54XASICTemp9(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp9'
    driver_hwmon = ['temp9']

class AcctonAS5835_54XASICTemp10(AcctonAS5835_54XASICTemp):
    name = 'ASICTemp10'
    driver_hwmon = ['temp10']

# Board temp sensors

class AcctonAS5835_54XTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

# U23 (rear right)
class AcctonAS5835_54XTemp1(AcctonAS5835_54XTemp):
    driver_path = '/sys/bus/i2c/devices/37-004a'
    name = 'Temp1'
    description = 'Rear right'
#    temp_fan_max = XXX
#    temp_fan_min = XXX

# U50 (front middle)
class AcctonAS5835_54XTemp2(AcctonAS5835_54XTemp):
    driver_path = '/sys/bus/i2c/devices/35-004c'
    name = 'Temp2'
    description = 'Front middle'
    temp_fan_max = 57
    temp_fan_min = 43

# U20 (rear left)
class AcctonAS5835_54XTemp3(AcctonAS5835_54XTemp):
    driver_path = '/sys/bus/i2c/devices/36-0049'
    name = 'Temp3'
    description = 'Rear left'
    temp_fan_max = 57
    temp_fan_min = 43

# U20 (on CPU board)
class AcctonAS5835_54XTemp4(AcctonAS5835_54XTemp):
    driver_path = '/sys/bus/i2c/devices/34-004b'
    name = 'Temp4'
    description = 'CPU board'
#    temp_fan_max = XXX
#    temp_fan_min = XXX

# Fans

class AcctonAS5835_54XFanUnit(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    cpld_path = AcctonAS5835_54XCPLD4.path
    driver_path = cpld_path
    maxpwm = 20
    minpwm = 8

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            if self.get_cpld_value(self.fan_fault) == '0':
                self.attrs['fan_0'] = 'installed, ok'
            else:
                self.attrs['fan_0'] = 'not_ok'
        else:
            self.attrs['fan_0'] = 'not_installed'

        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
                                                    
class AcctonAS5835_54XFan(AcctonAS5835_54XFanUnit):
    fan_max = 21500
    fan_min = 4300

class AcctonAS5835_54XFanR(AcctonAS5835_54XFanUnit):
    fan_max = 18000
    fan_min = 3600

class AcctonAS5835_54XFan1(AcctonAS5835_54XFan):
    name = 'Fan1'
    description = 'Fan1 Front'
    cpld_hwmon = ['fan1']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan1_present'
    fan_fault = 'fan1_fault'

class AcctonAS5835_54XFanR1(AcctonAS5835_54XFanR):
    name = 'Fan6'
    description = 'Fan1 Rear'
    cpld_hwmon = ['fanr1']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan1_present'
    fan_fault = 'fanr1_fault'

class AcctonAS5835_54XFan2(AcctonAS5835_54XFan):
    name = 'Fan2'
    description = 'Fan2 Front'
    cpld_hwmon = ['fan2']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan2_present'
    fan_fault = 'fan2_fault'

class AcctonAS5835_54XFanR2(AcctonAS5835_54XFanR):
    name = 'Fan7'
    description = 'Fan2 Rear'
    cpld_hwmon = ['fanr2']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan2_present'
    fan_fault = 'fanr2_fault'

class AcctonAS5835_54XFan3(AcctonAS5835_54XFan):
    name = 'Fan3'
    description = 'Fan3 Front'
    cpld_hwmon = ['fan3']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan3_present'
    fan_fault = 'fan3_fault'

class AcctonAS5835_54XFanR3(AcctonAS5835_54XFanR):
    name = 'Fan8'
    description = 'Fan3 Rear'
    cpld_hwmon = ['fanr3']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan3_present'
    fan_fault = 'fanr3_fault'

class AcctonAS5835_54XFan4(AcctonAS5835_54XFan):
    name = 'Fan4'
    description = 'Fan4 Front'
    cpld_hwmon = ['fan4']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan4_present'
    fan_fault = 'fan4_fault'

class AcctonAS5835_54XFanR4(AcctonAS5835_54XFanR):
    name = 'Fan9'
    description = 'Fan4 Rear'
    cpld_hwmon = ['fanr4']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan4_present'
    fan_fault = 'fanr4_fault'

class AcctonAS5835_54XFan5(AcctonAS5835_54XFan):
    name = 'Fan5'
    description = 'Fan5 Front'
    cpld_hwmon = ['fan5']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan5_present'
    fan_fault = 'fan5_fault'

class AcctonAS5835_54XFanR5(AcctonAS5835_54XFanR):
    name = 'Fan10'
    description = 'Fan5 Rear'
    cpld_hwmon = ['fanr5']
    pwm_hwmon = ['fan_pwm']
    fan_present = 'fan5_present'
    fan_fault = 'fanr5_fault'

# LEDs

'''
    Accton AS5835-54X LEDs

    PSU1/PSU2:
        green            - PSU OK
        amber            - power supply failure
    Diag:
        off              - all systems OK
        amber            - service required
    Fan:
        green            - all fans OK
        amber            - fan failure
    Loc:
        off              - not used
'''

class AcctonAS5835_54XLED(cumulus.sensors.StatusLED_Unit):
    led_path = AcctonAS5835_54XCPLD1.path

class AcctonAS5835_54XPSU1LED(AcctonAS5835_54XLED):
    name = 'PSU1 LED'
    led_location = 'led_psu1'
    sources = ['PSU1']
    good_led_color = 'green'
    bad_led_color = 'amber'

class AcctonAS5835_54XPSU2LED(AcctonAS5835_54XLED):
    name = 'PSU2 LED'
    led_location = 'led_psu2'
    sources = ['PSU2']
    good_led_color = 'green'
    bad_led_color = 'amber'

class AcctonAS5835_54XDiagLED(AcctonAS5835_54XLED):
    name = 'Diag LED'
    led_location = 'led_diag'
    good_led_color = 'green'
    bad_led_color = 'amber'

class AcctonAS5835_54XFanLED(AcctonAS5835_54XLED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6', 'Fan7', 'Fan8', 'Fan9', 'Fan10']
    good_led_color = 'green'
    bad_led_color = 'amber'

class AcctonAS5835_54XSensors():
    units = (AcctonAS5835_54XPSU1(),
             AcctonAS5835_54XPSU2(),
             AcctonAS5835_54XCPUTemp1(),
             AcctonAS5835_54XCPUTemp2(),
             AcctonAS5835_54XCPUTemp3(),
             AcctonAS5835_54XCPUTemp4(),
             AcctonAS5835_54XCPUTemp5(),
             AcctonAS5835_54XASICTemp1(),
             AcctonAS5835_54XASICTemp2(),
             AcctonAS5835_54XASICTemp3(),
             AcctonAS5835_54XASICTemp4(),
             AcctonAS5835_54XASICTemp5(),
             AcctonAS5835_54XASICTemp6(),
             AcctonAS5835_54XASICTemp7(),
             AcctonAS5835_54XASICTemp8(),
             AcctonAS5835_54XASICTemp9(),
             AcctonAS5835_54XASICTemp10(),
             AcctonAS5835_54XTemp1(),
             AcctonAS5835_54XTemp2(),
             AcctonAS5835_54XTemp3(),
             AcctonAS5835_54XTemp4(),
             AcctonAS5835_54XFan1(),
             AcctonAS5835_54XFanR1(),
             AcctonAS5835_54XFan2(),
             AcctonAS5835_54XFanR2(),
             AcctonAS5835_54XFan3(),
             AcctonAS5835_54XFanR3(),
             AcctonAS5835_54XFan4(),
             AcctonAS5835_54XFanR4(),
             AcctonAS5835_54XFan5(),
             AcctonAS5835_54XFanR5(),
             )

class AcctonAS5835_54XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS5835_54XFan1(),
            AcctonAS5835_54XFanR1(),
            AcctonAS5835_54XFan2(),
            AcctonAS5835_54XFanR2(),
            AcctonAS5835_54XFan3(),
            AcctonAS5835_54XFanR3(),
            AcctonAS5835_54XFan4(),
            AcctonAS5835_54XFanR4(),
            AcctonAS5835_54XFan5(),
            AcctonAS5835_54XFanR5(),
            )

    temps = (AcctonAS5835_54XPSU1(),
             AcctonAS5835_54XPSU2(),
             AcctonAS5835_54XCPUTemp1(),
             AcctonAS5835_54XCPUTemp2(),
             AcctonAS5835_54XCPUTemp3(),
             AcctonAS5835_54XCPUTemp4(),
             AcctonAS5835_54XCPUTemp5(),
             AcctonAS5835_54XASICTemp1(),
             AcctonAS5835_54XASICTemp2(),
             AcctonAS5835_54XASICTemp3(),
             AcctonAS5835_54XASICTemp4(),
             AcctonAS5835_54XASICTemp5(),
             AcctonAS5835_54XASICTemp6(),
             AcctonAS5835_54XASICTemp7(),
             AcctonAS5835_54XASICTemp8(),
             AcctonAS5835_54XASICTemp9(),
             AcctonAS5835_54XASICTemp10(),
             AcctonAS5835_54XTemp1(),
             AcctonAS5835_54XTemp2(),
             AcctonAS5835_54XTemp3(),
             AcctonAS5835_54XTemp4(),
             )

class AcctonAS5835_54XPWMs():
    units = (AcctonAS5835_54XPWM1(),
             )

class AcctonAS5835_54XLEDs():
    units = (AcctonAS5835_54XPSU1LED(),
             AcctonAS5835_54XPSU2LED(),
             AcctonAS5835_54XDiagLED(),
             AcctonAS5835_54XFanLED(),
             )

class AcctonAS5835_54X(cumulus.platform.Platform):
    name = 'accton,as5835_54x'
    chip = cumulus.platform.TridentThreeX5_56771_Chip()
    switch = AcctonAS5835_54XSwitch(chip)
    cpld = None
    sensors = AcctonAS5835_54XSensors()
    statusleds = AcctonAS5835_54XLEDs()
    pwms = AcctonAS5835_54XPWMs()

Platform = AcctonAS5835_54X
