#
# Copyright 2016.  Cumulus Networks, Inc.
#

import os

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelE1031SFPp(cumulus.platform.Helix410GPort):
    _set_led = '''
;;; CelE1031SFPp
;;;
;;; Two bits per port:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 |   on  |  off  |
;;;  2nd bit   +-------+-------+
;;;          1 | blink |  off  |
;;;            +-------+-------+
;;;
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack                    ; push 1st bit (active high)


        pushst  PORTSTATUS_ZERO
        pack                    ; push second bit (always 0)

        ret

'''
    def __init__(self, label, hw_port, leds, rx_polarity_flip = None, tx_polarity_flip=None,
                        tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 tx_lane_map=tx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None

        return set_calls

class CelE1031GigE(cumulus.platform.Helix41GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=None,
                                                unit=0)

class CelE1031LED(cumulus.ledup.Helix4LEDController):
    num_leds = 8

class CelE1031Switch(cumulus.platform.BCMSwitch):
    '''
    led chain starting from bit 1 and ending on bit 40....
     1 : sfp_45_act, sfp_45_link_g, sfp_45_link_y,
     4 : sfp_46_act, sfp_46_link_g, sfp_46_link_y,
     7 : sfp_47_act, sfp_47_link_g, sfp_47_link_y,
    10 : sfp_48_act, sfp_48_link_g, sfp_48_link_y,
    13 : rj_45_act, 1, rj_45_link_g, rj_45_link_y,
    17 : rj_46_act, 1, rj_46_link_g, rj_46_link_y,
    21 : rj_47_act, 1, rj_47_link_g, rj_47_link_y,
    25 : rj_48_act, 1, rj_48_link_g, rj_48_link_y,
    29 : mod_a_0, mod_a_1, mod_a_2,
    32 : mod_b_0, mod_b_1, mod_b_2,
    35 : 1, 1, 1,
    38 : 1, 1, 1
    '''
    ledup = CelE1031LED
    _gige = CelE1031GigE
    _sfp = CelE1031SFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True

    ports = (
        _gige(label="1",  hw_port='ge1',  phy_addr=0x2,  prim_offset=0x0001),
        _gige(label="2",  hw_port='ge0',  phy_addr=0x1,  prim_offset=0x0000),
        _gige(label="3",  hw_port='ge3',  phy_addr=0x4,  prim_offset=0x0003),
        _gige(label="4",  hw_port='ge2',  phy_addr=0x3,  prim_offset=0x0002),
        _gige(label="5",  hw_port='ge5',  phy_addr=0x6,  prim_offset=0x0005),
        _gige(label="6",  hw_port='ge4',  phy_addr=0x5,  prim_offset=0x0004),
        _gige(label="7",  hw_port='ge7',  phy_addr=0x8,  prim_offset=0x0007),
        _gige(label="8",  hw_port='ge6',  phy_addr=0x7,  prim_offset=0x0006),
        _gige(label="9",  hw_port='ge9',  phy_addr=0xb,  prim_offset=0x0801),
        _gige(label="10", hw_port='ge8' , phy_addr=0xa,  prim_offset=0x0800),
        _gige(label="11", hw_port='ge11', phy_addr=0xd,  prim_offset=0x0803),
        _gige(label="12", hw_port='ge10', phy_addr=0xc,  prim_offset=0x0802),
        _gige(label="13", hw_port='ge13', phy_addr=0xf,  prim_offset=0x0805),
        _gige(label="14", hw_port='ge12', phy_addr=0xe,  prim_offset=0x0804),
        _gige(label="15", hw_port='ge15', phy_addr=0x11, prim_offset=0x0807),
        _gige(label="16", hw_port='ge14', phy_addr=0x10, prim_offset=0x0806),
        _gige(label="17", hw_port='ge17', phy_addr=0x14, prim_offset=0x1001),
        _gige(label="18", hw_port='ge16', phy_addr=0x13, prim_offset=0x1000),
        _gige(label="19", hw_port='ge19', phy_addr=0x16, prim_offset=0x1003),
        _gige(label="20", hw_port='ge18', phy_addr=0x15, prim_offset=0x1002),
        _gige(label="21", hw_port='ge21', phy_addr=0x18, prim_offset=0x1005),
        _gige(label="22", hw_port='ge20', phy_addr=0x17, prim_offset=0x1004),
        _gige(label="23", hw_port='ge23', phy_addr=0x1a, prim_offset=0x1007),
        _gige(label="24", hw_port='ge22', phy_addr=0x19, prim_offset=0x1006),
        _gige(label="25", hw_port='ge25', phy_addr=0x22, prim_offset=0x1801),
        _gige(label="26", hw_port='ge24', phy_addr=0x21, prim_offset=0x1800),
        _gige(label="27", hw_port='ge27', phy_addr=0x24, prim_offset=0x1803),
        _gige(label="28", hw_port='ge26', phy_addr=0x23, prim_offset=0x1802),
        _gige(label="29", hw_port='ge29', phy_addr=0x26, prim_offset=0x1805),
        _gige(label="30", hw_port='ge28', phy_addr=0x25, prim_offset=0x1804),
        _gige(label="31", hw_port='ge31', phy_addr=0x28, prim_offset=0x1807),
        _gige(label="32", hw_port='ge30', phy_addr=0x27, prim_offset=0x1806),
        _gige(label="33", hw_port='ge33', phy_addr=0x2b, prim_offset=0x2001),
        _gige(label="34", hw_port='ge32', phy_addr=0x2a, prim_offset=0x2000),
        _gige(label="35", hw_port='ge35', phy_addr=0x2d, prim_offset=0x2003),
        _gige(label="36", hw_port='ge34', phy_addr=0x2c, prim_offset=0x2002),
        _gige(label="37", hw_port='ge37', phy_addr=0x2f, prim_offset=0x2005),
        _gige(label="38", hw_port='ge36', phy_addr=0x2e, prim_offset=0x2004),
        _gige(label="39", hw_port='ge39', phy_addr=0x31, prim_offset=0x2007),
        _gige(label="40", hw_port='ge38', phy_addr=0x30, prim_offset=0x2006),
        _gige(label="41", hw_port='ge41', phy_addr=0x34, prim_offset=0x2801),
        _gige(label="42", hw_port='ge40', phy_addr=0x33, prim_offset=0x2800),
        _gige(label="43", hw_port='ge43', phy_addr=0x36, prim_offset=0x2803),
        _gige(label="44", hw_port='ge42', phy_addr=0x35, prim_offset=0x2802),
        _gige(label="45", hw_port='ge45', phy_addr=0x38, prim_offset=0x2805),
        _gige(label="46", hw_port='ge44', phy_addr=0x37, prim_offset=0x2804),
        _gige(label="47", hw_port='ge47', phy_addr=0x3a, prim_offset=0x2807),
        _gige(label="48", hw_port='ge46', phy_addr=0x39, prim_offset=0x2806),
        _sfp(label="49",  hw_port='hg1',  leds=(0,2)),
        _sfp(label="50",  hw_port='hg0',  leds=(0,0),
                tx_lane_map=0x2301, rx_lane_map=0x1032,
                rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="51",  hw_port='hg3',  leds=(0,6)),
        _sfp(label="52",  hw_port='hg2',  leds=(0,4),
                rx_polarity_flip=True, tx_polarity_flip=True),
    )

CelE1031_i2c_path = '/sys/bus/i2c/devices/'

''' max6699 temp sensor 1 '''
CelE1031_i2c_11_1a_path = CelE1031_i2c_path + '11-001a'

''' max6699 temp sensor 2 '''
CelE1031_i2c_21_1a_path = CelE1031_i2c_path + '21-001a'

CelE1031_i2c_10_4d_path = CelE1031_i2c_path + '10-004d'

''' emc2305: Fan Tray 1, 2, 3 Controller '''
CelE1031_i2c_35_4d_path = CelE1031_i2c_path + '35-004d'

CelE1031_cpld_path =  '/sys/bus/platform/devices/cel_e1031_cpld.0'

class CelE1031CPLD(cumulus.cpld.SysFS):
    path = CelE1031_cpld_path

'''
   Cel E1031 LEDS
     System:
       green - normal operation
       amber blinking - Fault
     Master:
       green - switch is in stacking mode
       amber - switch is in slave mode (normal operation)
       (we will set it to green because that is a better choice than amber)
     FanX (1 - 3):
       green - fan X installed and working
       yellow - fan X malfunction
     PSU1 & PSU2:
       are controlled by the PSU hardware
'''

class CelE1031SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system_status'
    bad_led_color = 'green_blinking'
    led_path =  CelE1031_cpld_path

class CelE1031MasterLED(cumulus.sensors.StatusLED_Unit):
    name = 'Master'
    led_location = 'led_system_master'
    good_led_color = 'green'
    bad_led_color = 'green'
    led_path =  CelE1031_cpld_path

class CelE1031Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1']
    led_location = 'led_fan1'
    bad_led_color = 'yellow_blinking'
    led_path =   CelE1031_cpld_path

class CelE1031Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan2']
    led_location = 'led_fan2'
    bad_led_color = 'yellow_blinking'
    led_path =   CelE1031_cpld_path

class CelE1031Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan3']
    led_location = 'led_fan3'
    bad_led_color = 'yellow_blinking'
    led_path =   CelE1031_cpld_path

class CelE1031CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

class CelE1031InletTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 58
    temp_max = 49
    temp_fan_min = 31

class CelE1031MacTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 90
    temp_max = 65
    temp_fan_min = 50

class CelE1031Temp1(CelE1031CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class CelE1031Temp2(CelE1031CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class CelE1031Temp3(CelE1031CoreTemp):
    name = 'Temp3'
    description = 'CPU sensor F2R'
    driver_path = CelE1031_i2c_11_1a_path
    driver_hwmon = ['temp2']

class CelE1031Temp4(CelE1031CoreTemp):
    name = 'Temp4'
    description = 'CPU sensor R2F'
    driver_path = CelE1031_i2c_11_1a_path
    driver_hwmon = ['temp3']

class CelE1031Temp5(CelE1031InletTemp):
    name = 'Temp5'
    description = 'Inlet ambient sensor'
    driver_path = CelE1031_i2c_21_1a_path
    driver_hwmon = ['temp1']

class CelE1031Temp6(CelE1031MacTemp):
    name = 'Temp6'
    description =  cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = CelE1031_i2c_21_1a_path
    driver_hwmon = ['temp2']

class CelE1031Temp7(CelE1031InletTemp):
    name = 'Temp7'
    description = 'Right Inlet ambient sensor'
    driver_path = CelE1031_i2c_21_1a_path
    driver_hwmon = ['temp3']

class CelE1031Temp8(CelE1031MacTemp):
    name = 'Temp8'
    description =  cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = CelE1031_i2c_21_1a_path
    driver_hwmon = ['temp4']

class CelE1031Temp9(CelE1031InletTemp):
    name = 'Temp9'
    description = 'Left Inlet ambient sensor'
    driver_path = CelE1031_i2c_21_1a_path
    driver_hwmon = ['temp5']

class CelE1031Fan(cumulus.sensors.Fan_Unit):
    fan_max = 19000
    fan_min = 8200
    minpwm = 102
    maxpwm = 255
    hwmon_search = False	

class CelE1031Fan1(CelE1031Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = CelE1031_i2c_35_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_path =   CelE1031_cpld_path
    cpld_hwmon = ['fan_1']

class CelE1031Fan2(CelE1031Fan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_path = CelE1031_i2c_35_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_path =   CelE1031_cpld_path
    cpld_hwmon = ['fan_2']

class CelE1031Fan3(CelE1031Fan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_path = CelE1031_i2c_35_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_path =   CelE1031_cpld_path
    cpld_hwmon = ['fan_3']

class CelE1031PSU(cumulus.sensors.PSU_Unit):

    def find_and_add_attrs(self):
        attr_dict = { ''       : '_present',
                      '_ac'    : '_ac_ok',
                      '_power' : '_dc_ok',
                    }
        err = 0
        ok_cnt = 0
        for p, n in attr_dict.iteritems():
            fname = os.path.join(self.cpld_path, self.cpld_hwmon[0] + p)
            with open(fname) as fp:
                try:
                    val = fp.read().rsplit('\n')[0]
                    # In case of failure, it would be not_installed or power_bad.
                    # In case of good, it will be installed or ok.
                    if (val == 'installed') or (val == 'ok'):
                        self.attrs[self.cpld_hwmon[0] + n] = 1
                        ok_cnt += 1
                    else:
                        self.attrs[self.cpld_hwmon[0] + n] = 0
                except:
                    self.update_msg('Unable to read device')
                    err |= -1
        if ok_cnt == len(attr_dict):
            self.attrs[self.cpld_hwmon[0] + '_all_ok'] = 1
        else:
            self.attrs[self.cpld_hwmon[0] + '_all_ok'] = 0
        return err

class CelE1031PSU1(CelE1031PSU):
    name = 'PSU1'
    cpld_path =   CelE1031_cpld_path
    cpld_hwmon = ['psu_pwr1']

class CelE1031PSU2(CelE1031PSU):
    name = 'PSU2'
    cpld_path =   CelE1031_cpld_path
    cpld_hwmon = ['psu_pwr2']

class CelE1031PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 50
    temp_crit = 70

class CelE1031PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']        

class CelE1031PSU1Fan(CelE1031PSUFan):
    name = 'PSU1Fan'
    driver_path = '/sys/bus/i2c/devices/23-005b'
    description = 'PSU1 Fan'

class CelE1031PSU2Fan(CelE1031PSUFan):
    name = 'PSU2Fan'
    driver_path = '/sys/bus/i2c/devices/22-005a'
    description = 'PSU2 Fan'
            
class CelE1031PSU1Temp1(CelE1031PSUTemp):
    name = 'PSU1Temp1'
    driver_path = '/sys/bus/i2c/devices/23-005b'
    driver_hwmon = ['temp1']
    description = 'PSU1 Temp Sensor'        

class CelE1031PSU1Temp2(CelE1031PSUTemp):
    name = 'PSU1Temp2'
    driver_path = '/sys/bus/i2c/devices/23-005b'
    driver_hwmon = ['temp2']
    description = 'PSU1 Temp Sensor'        

class CelE1031PSU2Temp1(CelE1031PSUTemp):
    name = 'PSU2Temp1'
    driver_path = '/sys/bus/i2c/devices/22-005a'
    driver_hwmon = ['temp1']
    description = 'PSU2 Temp Sensor'        

class CelE1031PSU2Temp2(CelE1031PSUTemp):
    name = 'PSU2Temp2'
    driver_path = '/sys/bus/i2c/devices/22-005a'
    driver_hwmon = ['temp2']
    description = 'PSU2 Temp Sensor'
            
class CelE1031PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CelE1031Fan1(),
            CelE1031Fan2(),
            CelE1031Fan3(),
            CelE1031PSU1Fan(),
            CelE1031PSU2Fan(),
        )

    temps = (CelE1031Temp1(),
             CelE1031Temp2(),
             CelE1031Temp3(),
             CelE1031Temp4(),
             CelE1031Temp5(),
             CelE1031Temp6(),
             CelE1031Temp7(),
             CelE1031Temp8(),
             CelE1031Temp9(),
             CelE1031PSU1Temp1(),
             CelE1031PSU1Temp2(),
             CelE1031PSU2Temp1(),
             CelE1031PSU2Temp2(),
           )

class CelE1031Sensors():
    units = (CelE1031PSU1(),
             CelE1031PSU2(),
             CelE1031Temp1(),
             CelE1031Temp2(),
             CelE1031Temp3(),
             CelE1031Temp4(),
             CelE1031Temp5(),
             CelE1031Temp6(),
             CelE1031Temp7(),
             CelE1031Temp8(),
             CelE1031Temp9(),
             CelE1031Fan1(),
             CelE1031Fan2(),
             CelE1031Fan3(),
             CelE1031PSU1Fan(),
             CelE1031PSU2Fan(),
             CelE1031PSU1Temp1(),
             CelE1031PSU1Temp2(),
             CelE1031PSU2Temp1(),
             CelE1031PSU2Temp2(),
        )

class CelE1031PWMs():
    units = (CelE1031PWM1(),
             )

class CelE1031LEDs():
    units = (
             CelE1031SystemLED(),
             CelE1031MasterLED(),
             CelE1031Fan1LED(),
             CelE1031Fan2LED(),
             CelE1031Fan3LED(),
        )

class CelE1031(cumulus.platform.Platform):
    name = 'cel,e1031'
    chip = cumulus.platform.Helix4Chip()
    switch = CelE1031Switch(chip)
    statusleds = CelE1031LEDs()
    pwms = CelE1031PWMs()
    sensors = CelE1031Sensors()

Platform = CelE1031
