#
# Copyright (c) 2017, 2018, 2019  Cumulus Networks, Inc.
#
import cumulus.platform
import cumulus.ledup
import cumulus.sfp
import cumulus.sensors
import re

class CelQuestoneSFP28(cumulus.platform.Tomahawk25GPort):

    _set_led = '''
;;; CelQuestoneSFP28
;;
set_led:
set_any_led:
;;; The first bit will indicate link/activity (low active)
;;; The second bit will always be 0
;;;
;;;        ld      b, (ID_ENABLE_P)
;;;        tst     b, ID_ENABLE_BIT
;;;        jc      id_mode
        
        port    a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack
        ret

id_mode:
        push    1
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
        tinv
pack_it:
        pop
        push    cy
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None,
                 rx_polarity_flip=None, tx_polarity_flip=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_lane_map=tx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  oversub=True,
                                                  unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)
        set_calls = {}
        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class CelQuestoneQSFP28(cumulus.platform.Tomahawk100GPort):
    _set_led = '''
;;; CelQuestoneQSFP28
;;;
;;
set_led:
        call    set_any_led    ; set the first led green/off

        pushst    PORTSTATUS_ONE ; 1, 1 => led 2 off
        pack
        pushst    PORTSTATUS_ONE
        pack

        pushst    PORTSTATUS_ONE ; 1, 1 => led 3 off
        pack
        pushst    PORTSTATUS_ONE
        pack

        pushst    PORTSTATUS_ONE ; 1, 1 => led 4 off
        pack
        pushst    PORTSTATUS_ONE
        pack

        ret
'''
    _set_led_4x = '''
;;; CelQuestoneQSFP28_4x
;;
;;  For 4x mode, we use an LED for each port.
;;
set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)
        set_calls = {}
        if mode.startswith("4x") and (sub != 0):
            # For 4x mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode.startswith("4x"):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        else:
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class CelQuestoneLED(cumulus.ledup.TomahawkLEDController):
    num_leds = 72
    has_phys_id = True

class CelQuestoneSwitch(cumulus.platform.BCMSwitch):
    ledup = CelQuestoneLED
    _sfp28 = CelQuestoneSFP28
    _qsfp28 = CelQuestoneQSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False
    ports = (

        # FalconCore 12
        _sfp28(label="1",  serdes=(48,), tx_lane_map=0x1032, rx_lane_map=0x3201, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 0)),
        _sfp28(label="2",  serdes=(49,), tx_lane_map=0x1032, rx_lane_map=0x3201, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 2)),
        _sfp28(label="3",  serdes=(51,), tx_lane_map=0x1032, rx_lane_map=0x3201, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 4)),
        _sfp28(label="4",  serdes=(50,), tx_lane_map=0x1032, rx_lane_map=0x3201, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 6)),

        # FalconCore 13
        _sfp28(label="5",  serdes=(53,), tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 8)),
        _sfp28(label="6",  serdes=(55,), tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 10)),
        _sfp28(label="7",  serdes=(52,), tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 12)),
        _sfp28(label="8",  serdes=(54,), tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 14)),

        # FalconCore 16
        _sfp28(label="9",  serdes=(64,), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  leds=(1, 16)),
        _sfp28(label="10", serdes=(65,), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 18)),
        _sfp28(label="11", serdes=(66,), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  leds=(1, 20)),
        _sfp28(label="12", serdes=(67,), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  leds=(1, 22)),

        # FalconCore 17
        _sfp28(label="13", serdes=(68,), tx_lane_map=0x1023, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 24)),
        _sfp28(label="14", serdes=(69,), tx_lane_map=0x1023, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 26)),
        _sfp28(label="15", serdes=(70,), tx_lane_map=0x1023, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 28)),
        _sfp28(label="16", serdes=(71,), tx_lane_map=0x1023, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 30)),

        # FalconCore 8
        _sfp28(label="17", serdes=(35,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 32)),
        _sfp28(label="18", serdes=(34,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 34)),
        _sfp28(label="19", serdes=(33,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 36)),
        _sfp28(label="20", serdes=(32,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True,  leds=(1, 38)),

        # FalconCore 9
        _sfp28(label="21", serdes=(36,), tx_lane_map=0x1023, rx_lane_map=0x0132, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 40)),
        _sfp28(label="22", serdes=(37,), tx_lane_map=0x1023, rx_lane_map=0x0132, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 42)),
        _sfp28(label="23", serdes=(39,), tx_lane_map=0x1023, rx_lane_map=0x0132, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 44)),
        _sfp28(label="24", serdes=(38,), tx_lane_map=0x1023, rx_lane_map=0x0132, tx_polarity_flip=True,  rx_polarity_flip=False, leds=(1, 46)),

        # FalconCore 10
        _sfp28(label="25", serdes=(40,), tx_lane_map=0x3021, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=True,  leds=(1, 48)),
        _sfp28(label="26", serdes=(43,), tx_lane_map=0x3021, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=True,  leds=(1, 50)),
        _sfp28(label="27", serdes=(42,), tx_lane_map=0x3021, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 52)),
        _sfp28(label="28", serdes=(41,), tx_lane_map=0x3021, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 54)),

        # FalconCore 21
        _sfp28(label="29", serdes=(85,), tx_lane_map=0x1203, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 56)),
        _sfp28(label="30", serdes=(87,), tx_lane_map=0x1203, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 58)),
        _sfp28(label="31", serdes=(86,), tx_lane_map=0x1203, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 60)),
        _sfp28(label="32", serdes=(84,), tx_lane_map=0x1203, rx_lane_map=0x1203, tx_polarity_flip=False, rx_polarity_flip=False, leds=(1, 62)),

        # FalconCore 20
        _sfp28(label="33", serdes=(83,), tx_lane_map=0x3021, rx_lane_map=0x3021, tx_polarity_flip=True, rx_polarity_flip=False, leds=(1, 64)),
        _sfp28(label="34", serdes=(81,), tx_lane_map=0x3021, rx_lane_map=0x3021, tx_polarity_flip=True, rx_polarity_flip=False, leds=(1, 66)),
        _sfp28(label="35", serdes=(80,), tx_lane_map=0x3021, rx_lane_map=0x3021, tx_polarity_flip=True, rx_polarity_flip=False, leds=(1, 68)),
        _sfp28(label="36", serdes=(82,), tx_lane_map=0x3021, rx_lane_map=0x3021, tx_polarity_flip=True, rx_polarity_flip=False, leds=(1, 70)),

        # FalconCore 24
        _sfp28(label="37", serdes=(99,), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True, leds=(0, 0)),
        _sfp28(label="38", serdes=(97,), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True, leds=(0, 2)),
        _sfp28(label="39", serdes=(98,), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True, leds=(0, 4)),
        _sfp28(label="40", serdes=(96,), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=True, rx_polarity_flip=True, leds=(0, 6)),

        # FalconCore 25
        _sfp28(label="41", serdes=(100,), tx_lane_map=0x2130, rx_lane_map=0x2130, tx_polarity_flip=False, rx_polarity_flip=False, leds=(0, 8)),
        _sfp28(label="42", serdes=(102,), tx_lane_map=0x2130, rx_lane_map=0x2130, tx_polarity_flip=False, rx_polarity_flip=False, leds=(0, 10)),
        _sfp28(label="43", serdes=(103,), tx_lane_map=0x2130, rx_lane_map=0x2130, tx_polarity_flip=False, rx_polarity_flip=False, leds=(0, 12)),
        _sfp28(label="44", serdes=(101,), tx_lane_map=0x2130, rx_lane_map=0x2130, tx_polarity_flip=False, rx_polarity_flip=False, leds=(0, 14)),

        # FalconCore 26
        _sfp28(label="45", serdes=(104,), tx_lane_map=0x0213, rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=False, leds=(0, 16)),
        _sfp28(label="46", serdes=(106,), tx_lane_map=0x0213, rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=False, leds=(0, 18)),
        _sfp28(label="47", serdes=(105,), tx_lane_map=0x0213, rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=False, leds=(0, 20)),
        _sfp28(label="48", serdes=(107,), tx_lane_map=0x0213, rx_lane_map=0x0213, tx_polarity_flip=True, rx_polarity_flip=False, leds=(0, 22)),

        # QSFP28_1 = FalconCore 1
        _qsfp28(label="49", serdes=(4, 5, 6, 7), leds=(0, 24), tx_polarity_flip=0x000b, rx_polarity_flip=0x000d, tx_lane_map=0x1302, rx_lane_map=0x1302),
        # QSFP28_2 = FalconCore
        _qsfp28(label="50", serdes=(108, 109, 110, 111), leds=(0, 32), tx_polarity_flip=0x000e, rx_polarity_flip=0x0007,
            tx_lane_map=0x1230, rx_lane_map=0x3210),
        # QSFP28_3 = FalconCore
        _qsfp28(label="51", serdes=(8, 9, 10, 11), leds=(0, 40),
            tx_polarity_flip=0x0001, rx_polarity_flip=0x0006,
            tx_lane_map=0x1302, rx_lane_map=0x1302),
        # QSFP28_4 = FalconCore
        _qsfp28(label="52", serdes=(116, 117, 118, 119), leds=(0, 48),
            tx_polarity_flip=0x000e, rx_polarity_flip=0x0002,
            tx_lane_map=0x0213, rx_lane_map=0x0213),
        # QSFP28_5 = FalconCore
        _qsfp28(label="53", serdes=(20, 21, 22, 23), leds=(0, 56),
            tx_polarity_flip=0x000b, rx_polarity_flip=0x000d,
            tx_lane_map=0x2103, rx_lane_map=0x1032),
        # QSFP28_6 = FalconCore
        _qsfp28(label="54", serdes=(0, 1, 2, 3), leds=(0, 64),
            tx_polarity_flip=0x0001, rx_polarity_flip=0x0008,
            tx_lane_map=0x0213, rx_lane_map=0x0213),
    )

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

# Intel CPU temp sensors

class CelQuestoneCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 82
    temp_crit = 104
    no_shutdown = True

class CelQuestoneCPUTemp1(CelQuestoneCoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class CelQuestoneCPUTemp2(CelQuestoneCoreTemp):
    name = 'Temp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class CelQuestoneCPUTemp3(CelQuestoneCoreTemp):
    name = 'Temp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

# Tomahawk internal temp sensors

class CelQuestoneTomahawkTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    no_shutdown = True

class CelQuestoneTomahawkTemp1(CelQuestoneTomahawkTemp):
    name = 'Temp4'
    driver_hwmon = ['temp1']

class CelQuestoneTomahawkTemp2(CelQuestoneTomahawkTemp):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class CelQuestoneTomahawkTemp3(CelQuestoneTomahawkTemp):
    name = 'Temp6'
    driver_hwmon = ['temp3']

class CelQuestoneTomahawkTemp4(CelQuestoneTomahawkTemp):
    name = 'Temp7'
    driver_hwmon = ['temp4']

class CelQuestoneTomahawkTemp5(CelQuestoneTomahawkTemp):
    name = 'Temp8'
    driver_hwmon = ['temp5']

class CelQuestoneTomahawkTemp6(CelQuestoneTomahawkTemp):
    name = 'Temp9'
    driver_hwmon = ['temp6']

class CelQuestoneTomahawkTemp7(CelQuestoneTomahawkTemp):
    name = 'Temp10'
    driver_hwmon = ['temp7']

class CelQuestoneTomahawkTemp8(CelQuestoneTomahawkTemp):
    name = 'Temp11'
    driver_hwmon = ['temp8']

# BMC temp sensors

class CelQuestoneBMCTemp(cumulus.sensors.Temp_Unit):
    no_shutdown = True
    driver_path = vhwmon_path + 'temp'

class CelQuestoneBMCTemp1(CelQuestoneBMCTemp):
    name = 'Temp12'
    description = 'Base_Temp_U5'
    driver_hwmon = ['temp1']

class CelQuestoneBMCTemp2(CelQuestoneBMCTemp):
    name = 'Temp13'
    description = 'Base_Temp_U7'
    driver_hwmon = ['temp2']

class CelQuestoneBMCTemp3(CelQuestoneBMCTemp):
    name = 'Temp14'
    description = 'Switch_Temp_U2'
    driver_hwmon = ['temp3']

class CelQuestoneBMCTemp4(CelQuestoneBMCTemp):
    name = 'Temp15'
    description = 'Switch_Temp_U4'
    driver_hwmon = ['temp4']

class CelQuestoneBMCTemp5(CelQuestoneBMCTemp):
    name = 'Temp16'
    description = 'Switch_Temp_U38'
    driver_hwmon = ['temp5']

class CelQuestoneBMCTemp6(CelQuestoneBMCTemp):
    name = 'Temp17'
    description = 'Switch_Temp_U44'
    driver_hwmon = ['temp6']

class CelQuestoneBMCTemp7(CelQuestoneBMCTemp):
    name = 'Temp18'
    description = 'TempCOM-E'
    driver_hwmon = ['temp7']

class CelQuestoneBMCTemp8(CelQuestoneBMCTemp):
    name = 'Temp19'
    description = 'Switch_U11_temp'
    driver_hwmon = ['temp8']

class CelQuestoneBMCTemp9(CelQuestoneBMCTemp):
    name = 'Temp20'
    description = 'Switch_U15_temp'
    driver_hwmon = ['temp9']

class CelQuestoneBMCTemp10(CelQuestoneBMCTemp):
    name = 'Temp21'
    description = 'Switch_U26_temp'
    driver_hwmon = ['temp10']

# Fans

class CelQuestoneFan(cumulus.sensors.Fan_Unit):
    fan_min = 1950
    fan_max = 28050
    driver_path = vhwmon_path + 'fan'
    no_pwm = True

class CelQuestoneFan1(CelQuestoneFan):
    name = 'Fan1'
    description = 'Fan 1 Rear'
    driver_hwmon = ['fan5']

class CelQuestoneFan2(CelQuestoneFan):
    name = 'Fan2'
    description = 'Fan 1 Front'
    driver_hwmon = ['fan6']

class CelQuestoneFan3(CelQuestoneFan):
    name = 'Fan3'
    description = 'Fan 2 Rear'
    driver_hwmon = ['fan7']

class CelQuestoneFan4(CelQuestoneFan):
    name = 'Fan4'
    description = 'Fan 2 Front'
    driver_hwmon = ['fan8']

class CelQuestoneFan5(CelQuestoneFan):
    name = 'Fan5'
    description = 'Fan 4 Rear'
    driver_hwmon = ['fan9']

class CelQuestoneFan6(CelQuestoneFan):
    name = 'Fan6'
    description = 'Fan 4 Front'
    driver_hwmon = ['fan10']

class CelQuestoneFan7(CelQuestoneFan):
    name = 'Fan7'
    description = 'Fan 5 Rear'
    driver_hwmon = ['fan11']

class CelQuestoneFan8(CelQuestoneFan):
    name = 'Fan8'
    description = 'Fan 5 Front'
    driver_hwmon = ['fan12']

# PSUs

class CelQuestonePSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class CelQuestonePSU1(CelQuestonePSU):
    name = 'PSU1'
    driver_path = vhwmon_path + 'psul'
    description = 'PSU1'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        CelQuestonePSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelQuestonePSU2(CelQuestonePSU):
    name = 'PSU2'
    driver_path = vhwmon_path + 'psur'
    description = 'PSU2'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr2_all_ok'] = val
        CelQuestonePSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

# PSU temps

class CelQuestonePSUTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)

class CelQuestonePSU1Temp1(CelQuestonePSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    psu = CelQuestonePSU1()
    driver_path = psu.driver_path

class CelQuestonePSU2Temp1(CelQuestonePSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    psu = CelQuestonePSU2()
    driver_path = psu.driver_path

# PSU fans

class CelQuestonePSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_ok')
        return cumulus.sensors.Fan_Unit.get_status(self)

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Fan_Unit.detect_state_plat(self)

class CelQuestonePSU1Fan1(CelQuestonePSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = CelQuestonePSU1()
    driver_path = psu.driver_path

class CelQuestonePSU2Fan1(CelQuestonePSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = CelQuestonePSU2()
    driver_path = psu.driver_path

# Sensors

class CelQuestoneSensors():
    units = (
        CelQuestoneCPUTemp1(),
        CelQuestoneCPUTemp2(),
        CelQuestoneCPUTemp3(),
        CelQuestoneTomahawkTemp1(),
        CelQuestoneTomahawkTemp2(),
        CelQuestoneTomahawkTemp3(),
        CelQuestoneTomahawkTemp4(),
        CelQuestoneTomahawkTemp5(),
        CelQuestoneTomahawkTemp6(),
        CelQuestoneTomahawkTemp7(),
        CelQuestoneTomahawkTemp8(),
        CelQuestoneBMCTemp1(),
        CelQuestoneBMCTemp2(),
        CelQuestoneBMCTemp3(),
        CelQuestoneBMCTemp4(),
        CelQuestoneBMCTemp5(),
        CelQuestoneBMCTemp6(),
        CelQuestoneBMCTemp7(),
        CelQuestoneBMCTemp8(),
        CelQuestoneBMCTemp9(),
        CelQuestoneBMCTemp10(),
        CelQuestonePSU1Temp1(),
        CelQuestonePSU2Temp1(),
        CelQuestoneFan1(),
        CelQuestoneFan2(),
        CelQuestoneFan3(),
        CelQuestoneFan4(),
        CelQuestoneFan5(),
        CelQuestoneFan6(),
        CelQuestoneFan7(),
        CelQuestoneFan8(),
        CelQuestonePSU1Fan1(),
        CelQuestonePSU2Fan1(),
        CelQuestonePSU1(),
        CelQuestonePSU2(),
        )

class CelQuestoneBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class CelQuestone(cumulus.platform.Platform):
    name = 'cel,questone'
    chip = cumulus.platform.TomahawkPlus_56967_Chip()
    switch = CelQuestoneSwitch(chip)
    cpld = None
    statusleds = None
    pwms = None
    sensors = CelQuestoneSensors()
    bmc = CelQuestoneBMC()

Platform = CelQuestone
