#
# Copyright (c) 2016 Cumulus Networks, Inc.  All rights reserved.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp

class CelSmallstoneQSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; CelSmallstoneQSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY       equ     0x0  ; activity
SUB_INTF_LINKEN         equ     0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; CelSmallstoneQSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active high), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        ;;; initialize B
        ld      b,0

        ;;; load sub-0 info
        call    sub_intf_info

        ;;; load sub-1 info
        inc     a
        call    sub_intf_info

        ;;; load sub-2 info
        inc     a
        call    sub_intf_info

        ;;; load sub-3 info
        inc     a
        call    sub_intf_info

        ;;; push accumlated link activity onto stack
        tst     B, SUB_INTF_ACTIVITY
        push    cy

        ;;; combine with BLINK state
        push    (BLINK_STATE_P)
        tand
        tinv    ; flash off with activity

        ;;; push accumlated link enable onto stack
        tst     B, SUB_INTF_LINKEN
        push    cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        ;;; LEDs are active low
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    rx_lane_map=rx_lane_map,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    tx_lane_map=tx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        # Only one LED per QSFP on this platform
        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class CelSmallstoneLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 32

class CelSmallstoneSwitch(cumulus.platform.BCMSwitch):
    ledup = CelSmallstoneLED
    _qsfp = CelSmallstoneQSFPp

    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _qsfp(label="1", serdes=(  0,   1,   2,   3), leds=(0, 0),
              rx_lane_map=0x3120, tx_lane_map=0x2031,
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F),
        _qsfp(label="2", serdes=(  4,   5,   6,   7), leds=(0, 2),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="3", serdes=(  8,   9,  10,  11), leds=(0, 4),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000D, rx_polarity_flip=0x000F),
        _qsfp(label="4", serdes=( 12,  13,  14,  15), leds=(0, 6),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="5", serdes=( 16,  17,  18,  19), leds=(0, 8),
              rx_lane_map=0x3120, tx_lane_map=0x2031,
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F),
        _qsfp(label="6", serdes=( 20,  21,  22,  23), leds=(0, 10),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="7", serdes=( 24,  25,  26,  27), leds=(0, 12),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000D, rx_polarity_flip=0x000F),
        _qsfp(label="8", serdes=( 28,  29,  30,  31), leds=(0, 14),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="9", serdes=( 32,  33,  34,  35), leds=(0, 16),
              rx_lane_map=0x3120, tx_lane_map=0x2031,
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F),
        _qsfp(label="10", serdes=( 36,  37,  38,  39), leds=(0, 18),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="11", serdes=( 40,  41,  42,  43), leds=(0, 20),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000D, rx_polarity_flip=0x000F),
        _qsfp(label="12", serdes=( 44,  45,  46,  47), leds=(0, 22),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 24),
              rx_lane_map=0x3120, tx_lane_map=0x2031,
              tx_polarity_flip=0x000F, rx_polarity_flip=0x000F),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 26),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 28),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000D, rx_polarity_flip=0x000F),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 30),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x000C, rx_polarity_flip=0x000F),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 0),
              rx_lane_map=0x3120, tx_lane_map=0x2031,
              tx_polarity_flip=None, rx_polarity_flip=None),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 2),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 4),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x0002, rx_polarity_flip=None),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 6),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="21", serdes=( 80,  81,  82,  83), leds=(1, 8),
              rx_lane_map=0x3120, tx_lane_map=0x3120,
              tx_polarity_flip=None, rx_polarity_flip=None),
        _qsfp(label="22", serdes=( 84,  85,  86,  87), leds=(1, 10),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="23", serdes=( 88,  89,  90,  91), leds=(1, 12),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0002, rx_polarity_flip=None),
        _qsfp(label="24", serdes=( 92,  93,  94,  95), leds=(1, 14),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="25", serdes=( 96,  97,  98,  99), leds=(1, 16),
              rx_lane_map=0x3120, tx_lane_map=0x3120,
              tx_polarity_flip=None, rx_polarity_flip=None),
        _qsfp(label="26", serdes=(100, 101, 102, 103), leds=(1, 18),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="27", serdes=(104, 105, 106, 107), leds=(1, 20),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0002, rx_polarity_flip=None),
        _qsfp(label="28", serdes=(108, 109, 110, 111), leds=(1, 22),
              rx_lane_map=0x2031, tx_lane_map=0x1302,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="29", serdes=(112, 113, 114, 115), leds=(1, 24),
              rx_lane_map=0x3120, tx_lane_map=0x3120,
              tx_polarity_flip=None, rx_polarity_flip=None),
        _qsfp(label="30", serdes=(116, 117, 118, 119), leds=(1, 26),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
        _qsfp(label="31", serdes=(120, 121, 122, 123), leds=(1, 28),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x0002, rx_polarity_flip=None),
        _qsfp(label="32", serdes=(124, 125, 126, 127), leds=(1, 30),
              rx_lane_map=0x2031, tx_lane_map=0x0213,
              tx_polarity_flip=0x0003, rx_polarity_flip=None),
    )

class CelSmallstoneCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ffe05000.localbus/ffb00000.CPLD1'

class CelSmallstonePSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class CelSmallstonePSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class CelSmallstoneTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Front Right Corner Ambient Temp Sensor'
    driver_path = '/sys/bus/i2c/drivers/lm75/0-004b'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelSmallstoneTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'P2020 Temp Sensor'
    driver_path = '/sys/bus/i2c/drivers/lm75/1-0048'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 73

class CelSmallstoneTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'System Rear Outlet Temp Sensor'
    driver_path = '/sys/bus/i2c/drivers/lm75/1-0049'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelSmallstoneTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/bus/i2c/drivers/lm75/1-004e'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 80

class CelSmallstoneTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Front Left Corner Ambient Temp Sensor'
    driver_path = '/sys/bus/i2c/drivers/lm75/1-004f'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelSmallstoneFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/bus/i2c/drivers/emc2305/1-004d'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_min = 2000
    fan_max = 18000

class CelSmallstoneFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/bus/i2c/drivers/emc2305/1-004d'
    cpld_hwmon = ['fan2']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_min = 2000
    fan_max = 18000

class CelSmallstoneFan3(cumulus.sensors.Fan_Unit):
    name = 'Fan3'
    driver_path = '/sys/bus/i2c/drivers/emc2305/1-004d'
    cpld_hwmon = ['fan3']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_min = 2000
    fan_max = 18000

class CelSmallstoneFan4(cumulus.sensors.Fan_Unit):
    name = 'Fan4'
    driver_path = '/sys/bus/i2c/drivers/emc2305/1-004d'
    cpld_hwmon = ['fan4']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_min = 2000
    fan_max = 18000

class CelSmallstoneFan5(cumulus.sensors.Fan_Unit):
    name = 'Fan5'
    driver_path = '/sys/bus/i2c/drivers/emc2305/1-004d'
    cpld_hwmon = ['fan5']
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    fan_min = 2000
    fan_max = 18000

class CelSmallstonePWM1(cumulus.sensors.PWM_Unit):
    fans = (CelSmallstoneFan1(),
            CelSmallstoneFan2(),
            CelSmallstoneFan3(),
            CelSmallstoneFan4(),
            CelSmallstoneFan5(),
            )
    temps = (CelSmallstoneTemp1(),
             CelSmallstoneTemp2(),
             CelSmallstoneTemp3(),
             CelSmallstoneTemp4(),
             CelSmallstoneTemp5(),
             )

class CelSmallstoneSensors():
    units = ( CelSmallstonePSU1(),
              CelSmallstonePSU2(),
              CelSmallstoneFan1(),
              CelSmallstoneFan2(),
              CelSmallstoneFan3(),
              CelSmallstoneFan4(),
              CelSmallstoneFan5(),
              CelSmallstoneTemp1(),
              CelSmallstoneTemp2(),
              CelSmallstoneTemp3(),
              CelSmallstoneTemp4(),
              CelSmallstoneTemp5(),
              )

class CelSmallstonePWMs():
    units = (CelSmallstonePWM1(),
             )

class CelSmallstoneSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class CelSmallstoneLEDs():
    units = (CelSmallstoneSystemLED(),
             )

class CelSmallstone(cumulus.platform.Platform):
    name = 'cel,smallstone'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = CelSmallstoneSwitch(chip)
    cpld = CelSmallstoneCPLD()
    statusleds = CelSmallstoneLEDs()
    pwms = CelSmallstonePWMs()
    sensors = CelSmallstoneSensors()

Platform = CelSmallstone
