#
# Copyright 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class VoyagerLaser(cumulus.platform.SDKPort):
    '''
    A Voyager Laser port is a port on the front panel labeled L1-L4. These are
    the network ports of an AC400.
    '''
    gang_modes = ('3/2',)
    modes = ('100G','2x100G',) + gang_modes
    default_mode = modes[1]
    max_gang_size = 2

    def __init__(self, label, index, hostifs):
        cumulus.platform.Port.__init__(self, label=label)
        self.index = index
        self.hostifs = hostifs
        self.phy_layer = None

    def port_mangler(self, mode):
        '''
        Return a list of physical ports which are "behind" the laser ports
        based upon the mode.
        '''
        if mode == '100G':
            self.hostifs[0].name_override = "swp%s" % (self.label,)
            return ((self.hostifs[0], mode),)
        elif mode == '2x100G':
            self.hostifs[0].name_override = "swp%ss0" % (self.label,)
            self.hostifs[1].name_override = "swp%ss1" % (self.label,)
            return ((self.hostifs[0], '100G'), (self.hostifs[1], '100G'))
        elif mode == '3/2' and not (self.index % 2):
            self.hostifs[0].name_override = "swp%ss0" % (self.label,)
            self.hostifs[1].name_override = "swp%ss1" % (self.label,)
            self.hostifs[2].name_override = "swp%ss2" % (self.label,)
            return ((self.hostifs[0], '100G'), (self.hostifs[1], '100G'), (self.hostifs[2], '100G'))
        return ()

    def complete_mode(config, self, mode):
        '''
        We want to support setting the breakout configuration without an
        associated speed.  Therefore, complete the shorthand '1x', '/1' etc.
        into a fully qualified breakout + speed setting.
        '''
        if mode == '1x':
            return '100G'
        elif mode == '2x':
            return '2x100G'
        return mode

    def num_logical_ports(self, mode):
        if mode == '2x100G':
            return 2
        elif mode == '3/2':
            return 3
        return 1


class VoyagerQSFP28(cumulus.platform.Tomahawk100GPort):
    '''
    QSFP28 ports have 8 LED bits per port.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_port_led:
        port    a                   ; The port upon which we are operating

        pushst  PORTSTATUS_RX       ; Any RX or TX activity recently?
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)     ; If we are blinking, turn it off
        tand
        tinv
        pushst  PORTSTATUS_LINKEN   ; Is there any link on this port?
        tand
        tinv                        ; Invert for correct color.
        pack                        ; Blue = 1, Off = 0

        push    1                   ; Second bit, (always on)
        pack

        ret

set_led_off:
        push    1
        pack
        push    1
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the first 2 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; The rest are ignored
        call    set_led_off
        call    set_led_off
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, the first and third 2 bits of the LED chain set the LED color.
;;;
set_led:
        call    set_port_led        ; Set the first LED in the chain
        call    set_led_off         ; These two bits are ignored
        add     a, 2
        call    set_port_led        ; Set the third LED in the chain
        call    set_led_off         ; These two bits are ignored
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, all 2-bit LED values in the LED chain must be set.
;;; indicate link/activity.
;;;
set_led:
        ld      b, 4
set_next_led:
        call    set_port_led        ; Set the 2-bit port LED
        inc     a
        dec     b
        jnz     set_next_led
        ret
'''

    def __init__(self, label, serdes, leds, tx_polarity_flip, rx_polarity_flip,
                 tx_lane_map, rx_lane_map):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   leds=leds,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        for led in led_index_list:
            set_calls[led] = call
            for x in range(1,8):
                set_calls[led + x] = None

        return set_calls

class VoyagerLED(cumulus.ledup.TomahawkLEDController):
    led_functions = (VoyagerQSFP28._set_led_common)
    num_leds = 128

class VoyagerSwitch(cumulus.platform.BCMSwitch):
    ledup = VoyagerLED
    led_controllers = set([0, 1])
    _qsfp28 = VoyagerQSFP28
    _fab = cumulus.platform.Tomahawk100GFabricPort
    _laser = VoyagerLaser
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # AC400 Line side ports
    # FC11 - AC400 #1 TX/RX0
    fp0 = _fab(label="0", serdes=( 44,  45,  46,  47),
               tx_polarity_flip=0x0003, rx_polarity_flip=0x000c,
               tx_lane_map=0x0231, rx_lane_map=0x2013, sdk_group=1)

    # FC12 - AC400 #1 TX/RX1
    fp1 = _fab(label="1", serdes=( 48,  49,  50,  51),
               tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
               tx_lane_map=0x0123, rx_lane_map=0x3210, sdk_group=1)

    # FC10 - AC400 #1 TX/RX2
    fp2 = _fab(label="2", serdes=( 40,  41,  42,  43),
               tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
               tx_lane_map=0x0123, rx_lane_map=0x0123, sdk_group=1)

    # FC9 - AC400 #1 TX/RX3
    fp3 = _fab(label="3", serdes=( 36,  37,  38,  39),
               tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
               tx_lane_map=0x3210, rx_lane_map=0x3210, sdk_group=1)

    # FC19 - AC400 #2 TX/RX0
    fp4 = _fab(label="4", serdes=( 76,  77,  78,  79),
               tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
               tx_lane_map=0x3102, rx_lane_map=0x0213, sdk_group=2)

    # FC18 - AC400 #2 TX/RX1
    fp5 = _fab(label="5", serdes=( 72,  73,  74,  75),
               tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
               tx_lane_map=0x3210, rx_lane_map=0x0123, sdk_group=2)

    # FC17 - AC400 #2 TX/RX2
    fp6 = _fab(label="6", serdes=( 68,  69,  70,  71),
               tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
               tx_lane_map=0x3210, rx_lane_map=0x0123, sdk_group=2)

    # FC16 - AC400 #2 TX/RX3
    fp7 = _fab(label="7", serdes=( 64,  65,  66,  67),
               tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
               tx_lane_map=0x0123, rx_lane_map=0x3210, sdk_group=2)

    ports = (
        # Front panel "Client" ports
        # FC21
        _qsfp28(label="1",  serdes=(84,  85,  86,  87 ), leds=(1, 112),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x1302),
        # FC24
        _qsfp28(label="2",  serdes=(96,  97,  98,  99 ), leds=(0, 48),
                tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
                tx_lane_map=0x2031, rx_lane_map=0x2031),

        # FC26
        _qsfp28(label="3",  serdes=(104, 105, 106, 107), leds=(0, 56),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x2031, rx_lane_map=0x2031),

        # FC27
        _qsfp28(label="4",  serdes=(108, 109, 110, 111), leds=(0, 64),
                tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
                tx_lane_map=0x1302, rx_lane_map=0x1302),

        # FC28
        _qsfp28(label="5",  serdes=(112, 113, 114, 115), leds=(0, 72),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x0321, rx_lane_map=0x3012),

        # FC29
        _qsfp28(label="6",  serdes=(116, 117, 118, 119), leds=(0, 80),
                tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x3120),

        # FC0
        _qsfp28(label="7",  serdes=(  0,   1,   2,   3), leds=(0, 88),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x3120),

        # FC2
        _qsfp28(label="8",  serdes=(  8,   9,  10,  11), leds=(0, 96),
                tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x3120),

        # FC3
        _qsfp28(label="9",  serdes=( 12,  13,  14,  15), leds=(0, 104),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x3120),

        # FC4
        _qsfp28(label="10", serdes=( 16,  17,  18,  19), leds=(0, 112),
                tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
                tx_lane_map=0x1230, rx_lane_map=0x1203),

        # FC5
        _qsfp28(label="11", serdes=( 20,  21,  22,  23), leds=(0, 120),
                tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
                tx_lane_map=0x2031, rx_lane_map=0x3120),

        # FC8
        _qsfp28(label="12", serdes=( 32,  33,  34,  35), leds=(1, 120),
                tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
                tx_lane_map=0x1302, rx_lane_map=0x0213),

        _laser(label="L1", index=0, hostifs=(fp4, fp5, fp6)),
        _laser(label="L2", index=1, hostifs=(fp6, fp7)),
        _laser(label="L3", index=2, hostifs=(fp0, fp1, fp2)),
        _laser(label="L4", index=3, hostifs=(fp2, fp3)),
    )

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class VoyagerCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/syscpld'

class VoyagerCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 90
    temp_crit = 95
    no_shutdown = True

class VoyagerTemp1(VoyagerCoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class VoyagerTemp2(VoyagerCoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class VoyagerTemp3(VoyagerCoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class VoyagerTemp4(VoyagerCoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

class VoyagerTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'bmc: CPU Temp'
    driver_path = vhwmon_path + 'come_ec_i2c_0_33'
    driver_hwmon = ['temp1']

class VoyagerTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'bmc: Inlet Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_48'
    driver_hwmon = ['temp1']

class VoyagerTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'bmc: Outlet Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_4b'
    driver_hwmon = ['temp1']

class VoyagerTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    description = 'bmc: DIMM Temp'
    driver_path = vhwmon_path + 'come_ec_i2c_0_33'
    driver_hwmon = ['temp2']

class VoyagerFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + 'fancpld_i2c_8_33'
    hwmon_search = False
    no_pwm = True
    # using defaults for min, max, and var

class VoyagerFan1(VoyagerFan):
    name = 'Fan1'
    description = 'bmc: Fan 1 front'
    driver_hwmon = ['fan1']

class VoyagerFan2(VoyagerFan):
    name = 'Fan2'
    description = 'bmc: Fan 1 rear'
    driver_hwmon = ['fan2']

class VoyagerFan3(VoyagerFan):
    name = 'Fan3'
    description = 'bmc: Fan 2 front'
    driver_hwmon = ['fan3']

class VoyagerFan4(VoyagerFan):
    name = 'Fan4'
    description = 'bmc: Fan 2 rear'
    driver_hwmon = ['fan4']

class VoyagerFan5(VoyagerFan):
    name = 'Fan5'
    description = 'bmc: Fan 3 front'
    driver_hwmon = ['fan5']

class VoyagerFan6(VoyagerFan):
    name = 'Fan6'
    description = 'bmc: Fan 3 rear'
    driver_hwmon = ['fan6']

class VoyagerFan7(VoyagerFan):
    name = 'Fan7'
    description = 'bmc: Fan 4 front'
    driver_hwmon = ['fan7']

class VoyagerFan8(VoyagerFan):
    name = 'Fan8'
    description = 'bmc: Fan 4 rear'
    driver_hwmon = ['fan8']

class VoyagerFan9(VoyagerFan):
    name = 'Fan9'
    description = 'bmc: Fan 5 front'
    driver_hwmon = ['fan9']

class VoyagerFan10(VoyagerFan):
    name = 'Fan10'
    description = 'bmc: Fan 5 rear'
    driver_hwmon = ['fan10']

class VoyagerPSU(cumulus.sensors.PSU_Unit):
    cpld_path = VoyagerCPLD.path

    def update_attrs(self, path, hwmon):
        '''
        the Voyager CPLD does not provide a psu_pwrX_all_ok which smond
         expects to see, we will construct one using the available data.
        '''
        ret = cumulus.sensors.PSU_Unit.update_attrs(self, path, hwmon)
        oks = ['psu_pwr%u_dc_ok' % self.psu_num,
               'psu_pwr%u_ac_ok' % self.psu_num,]
        status = '1'
        for ok in oks:
            if int(self.attrs.get(ok)) != 1:
                status = '0'
        self.attrs['psu_pwr%u_all_ok' % self.psu_num] = status
        return ret

class VoyagerPSU1(VoyagerPSU):
    name = 'PSU1'
    psu_num = 1
    cpld_hwmon = ['psu_pwr1']

class VoyagerPSU2(VoyagerPSU):
    name = 'PSU2'
    psu_num = 2
    cpld_hwmon = ['psu_pwr2']

class VoyagerSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = VoyagerCPLD.path
    led_location = 'system_led1'
    bad_led_color = 'yellow_blinking'

class VoyagerPSUsLED(cumulus.sensors.StatusLED_Unit):
    name = 'PSUs'
    led_path = VoyagerCPLD.path
    led_location = 'system_led2'
    bad_led_color = 'yellow_blinking'
    sources = ['PSU1', 'PSU2']

class VoyagerLEDs():
    units = (
        VoyagerSystemLED(),
        VoyagerPSUsLED(),
    )

class VoyagerSensors():
    units = (VoyagerTemp1(),
             VoyagerTemp2(),
             VoyagerTemp3(),
             VoyagerTemp4(),
             VoyagerTemp5(),
             VoyagerTemp6(),
             VoyagerTemp7(),
             VoyagerTemp8(),
             VoyagerFan1(),
             VoyagerFan2(),
             VoyagerFan3(),
             VoyagerFan4(),
             VoyagerFan5(),
             VoyagerFan6(),
             VoyagerFan7(),
             VoyagerFan8(),
             VoyagerFan9(),
             VoyagerFan10(),
             VoyagerPSU1(),
             VoyagerPSU2(),
            )

class VoyagerBMC(cumulus.platform.BMC):
    name = "bmc"
    type = "openbmc"
    ipv6_addr = "fe80::1:1%eth0.4088"

class Voyager(cumulus.platform.Platform):
    name = 'celestica,voyager'
    chip = cumulus.platform.TomahawkChip()
    switch = VoyagerSwitch(chip)
    cpld = VoyagerCPLD()
    statusleds = VoyagerLEDs()
    pwms = None
    sensors = VoyagerSensors()
    bmc = VoyagerBMC()

Platform = Voyager

