#
# Copyright 2014.  Cumulus Networks, Inc.
#
# acpi.py --
#     Firmware Defined Platform Data.
#

import cumulus.platform
import cumulus.sfp
import cumulus.acpi
import os

class ACPI(cumulus.platform.Platform):
    name = 'cumulus,acpi'
    _switch = None
    _chip = None
    _device = None

    @classmethod
    def is_acpi(self):
        return True

    @classmethod
    def probe(self):
        return cumulus.acpi.validate()

    @classmethod
    def find_chipdev(self):
        for chip in cumulus.platform.chips:
            dev = cumulus.acpi.find_pci_device(chip.pci_vendor, chip.pci_device)
            if dev:
                return (chip, dev)

        raise cumulus.platform.NoSuchChip('unable to find ACPI switch ASIC')

    @property
    def chip(self):
        if self._chip is None:
            (self._chip, self._device) = self.find_chipdev()

        return self._chip

    @property
    def device(self):
        if self._device is None:
            (self._chip, self._device) = self.find_chipdev()

        return self._device

    @property
    def switch(self):
        if self._switch is None:
            self._switch = Switch(self.chip(), self.device)

        return self._switch

Platform = ACPI

class MetaPort(type):
    '''
    A metaclass to control the creation of Single (sfp+/sfp28) and Quad
    (qsfp+/qsfp28) classes.  The metaclass uses ACPI to look up the relevant
    chip/port types at class creation time.
    '''
    def __new__(cls, name, bases, dct):
        try:
            (chip, dev) = ACPI.find_chipdev()
        except:
            return type.__new__(cls, name, bases, dct)
        port_base = chip.port_channel_type_map[dct['channels']]
        dct['_port_base'] = port_base
        return type.__new__(cls, name, (port_base,), dct)

class SinglePort(object):
    metaclass_= MetaPort
    channels = 1

    def __init__(self, label, serdes):
        sfp = cumulus.sfp.SFPROMACPI(label)
        self._port_base.__init__(self,
                                 label=label,
                                 serdes=serdes,
                                 phy_types=None,
                                 phy_addrs=None,
                                 phy_layer=sfp.type.upper(),
                                 leds=None,
                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        # XXX - not implemented
        return {}

class QuadPort(object):
    metaclass_= MetaPort
    channels = 4

    def __init__(self, label, serdes):
        sfp = cumulus.sfp.SFPROMACPI(label)
        self._port_base.__init__(self,
                                 label=label,
                                 serdes=serdes,
                                 phy_types=None,
                                 phy_addrs=None,
                                 phy_layer=sfp.type.upper(),
                                 leds=None,
                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        # XXX - not implemented
        return {}

class Switch(cumulus.platform.BCMSwitch):
    def __init__(self, chip, device):
        cumulus.platform.BCMSwitch.__init__(self, chip)
        self._xrs = cumulus.acpi.BCMXR.get_xrs(device)
        self._swps = cumulus.acpi.SWP.get_swps(device)
        self.ports = self._get_ports()

    def _get_ports(self):
        ports = []
        for swp in self._swps:
            xrs = swp.find_xrs(self._xrs)
            serdes = [ xr.index for xr in xrs ]
            port_base = self.chip.port_channel_type_map[swp.channels]
            if swp.channels == 1:
                port_class = SinglePort
            elif swp.channels == 4:
                port_class = QuadPort
            ports.append(port_class(swp.label, serdes=tuple(serdes)))
        return tuple(sorted(ports, key=lambda x: int(x.label)))
