#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS4000SFPp(cumulus.platform.TridentTwo10GPort):

    _set_led = '''
;;; DellS4000SFPp
;;
set_led:
;;; Each 10G port has two LEDs. One for link and one for activity.
;;; The link LED is dual-color green/yellow
;;; The activity LED is just green
;;; Three bits to pack all active high:
;;;   0: link_green
;;;   1: link_yellow
;;;   2: activity_green
;;; We use:
;;;   link_green on with link
;;;   link_yellow always off
;;;   activity_green flashes on with activity
;;;
        port a

        pushst  PORTSTATUS_LINKEN   ; link_green
        pack                        ;  - on with link

        pushst  PORTSTATUS_ZERO     ; link_yellow
        pack                        ;  - off

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand                        ; activity_green
        pack                        ;  - flash on with activity

        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.TridentTwo10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 tx_lane_map=tx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls

class DellS4000QSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; DellS4000QSFPp
;;;
;;; 40G ports have 4 green/yellow LEDs, two bits per LED.
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;;  We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ld      a, 7
qsfp_pack_loop:
        port 1
        pushst  PORTSTATUS_ZERO
        pack
        sub     a, 1
        jnz     qsfp_pack_loop

        ret
'''
    _set_led_4x10 = '''
;;; DellS4000QSFPp_4x10
;;;
;;;  For 4x10G mode, we use an LED for each port. Each port has
;;;  a single green/yellow LED:
;;;    0: link_green   (active high)
;;;    1: link_yellow  (active high)
;;;  We use the link_green led for link and activity
;;;  link_yellow is always off
;;
set_led:
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led

        ret

set_qsfp_4x10_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO ; link_yellow off
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer=sfp.type.upper(),
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_lane_map=tx_lane_map,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls

class DellS4000LED_1_36(cumulus.ledup.TridentTwoLEDController):
    num_leds = 112

class DellS4000LED_37_54(cumulus.ledup.TridentTwoLEDController):
    num_leds = 84

class DellS4000CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dell_s4000_cpld.0'


'''
   Dell S4000 LEDS
     System:
       green - all systems ok
       green blinking - fan missing or broken, psu missing or broken
     Fan:
       green - all 3 fans installed and working
       yellow blinking - fan missing or malfunction
     Fan_Tray_X (0 - 2):
       green - fan X installed and working
       yellow - fan X malfunction
'''

DellS4000_i2c_path = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 1 '''
DellS4000_i2c_11_4c_path = DellS4000_i2c_path + '11-004c'

''' tmp75: temp sensor 2 '''
DellS4000_i2c_10_4d_path = DellS4000_i2c_path + '10-004d'

''' tmp75: temp sensor 3 '''
DellS4000_i2c_11_4d_path = DellS4000_i2c_path + '11-004d'

''' tmp75: temp sensor 4 '''
DellS4000_i2c_11_4e_path = DellS4000_i2c_path + '11-004e'

''' jc42: temp sensor 5 '''
DellS4000_i2c_00_1a_path = DellS4000_i2c_path + '0-001a'

''' dps460: PSU1 Temp/Fan '''
DellS4000_i2c_14_58_path = DellS4000_i2c_path + '14-0058'

''' dps460: PSU2 Temp/Fan '''
DellS4000_i2c_14_59_path = DellS4000_i2c_path + '14-0059'

''' max6620: Fan Tray 1, 2 Controller '''
DellS4000_i2c_11_2a_path = DellS4000_i2c_path + '11-002a'

''' max6620: Fan Tray 3, 4, 5, 6 Controller '''
DellS4000_i2c_11_29_path = DellS4000_i2c_path + '11-0029'

''' ltc4215: PSU1 '''
DellS4000_i2c_11_40_path = DellS4000_i2c_path + '11-0040'

''' ltc4215: PSU2 '''
DellS4000_i2c_11_42_path = DellS4000_i2c_path + '11-0042'

class DellS4000SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_health'
    bad_led_color = 'amber_blinking'

class DellS4000PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'amber_blinking'
    
class DellS4000FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fans'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'

class DellS4000Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'yellow'

class DellS4000Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray_2'
    bad_led_color = 'yellow'

class DellS4000Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan_tray_3'
    bad_led_color = 'yellow'

class DellS4000Temp1(cumulus.sensors.Temp_Unit):
    # U54, Hardware shutdown, Fan Controlled
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS4000_i2c_11_4c_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4000Temp1F2B(DellS4000Temp1):
    temp_max = 66
    temp_crit = 77
    temp_fan_min = 40
    temp_fan_max = 65

class DellS4000Temp1B2F(DellS4000Temp1):
    temp_max = 63
    temp_crit = 70
    temp_fan_min = 33
    temp_fan_max = 57

class DellS4000Temp2(cumulus.sensors.Temp_Unit):
    # U245, Hardware shutdown, Fan Controlled
    name = 'Temp2'
    description = 'Board Sensor near QSFP+ cage'
    driver_path = DellS4000_i2c_11_4e_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4000Temp2F2B(DellS4000Temp2):
    temp_max = 52
    temp_crit = 60
    temp_fan_min = 23
    temp_fan_max = 48
        
class DellS4000Temp2B2F(DellS4000Temp2):
    temp_max = 51
    temp_crit = 60
    temp_fan_min = 24
    temp_fan_max = 49
        
class DellS4000Temp3(cumulus.sensors.Temp_Unit):
    # U244, Hardware shutdown, Fan Controlled
    name = 'Temp3'
    description = 'Board Sensor near SFP+ cage'
    driver_path = DellS4000_i2c_11_4d_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
    
class DellS4000Temp3F2B(DellS4000Temp3):
    temp_max = 60
    temp_crit = 71
    temp_fan_min = 34
    temp_fan_max = 58    
    
class DellS4000Temp3B2F(DellS4000Temp3):
    temp_max = 62
    temp_crit = 69
    temp_fan_min = 32
    temp_fan_max = 56    
            
class DellS4000Temp4(cumulus.sensors.Temp_Unit):
    # U57, Software shutdown
    name = 'Temp4'
    description = 'Board Sensor near CPU'
    driver_path = DellS4000_i2c_10_4d_path
    
class DellS4000Temp4F2B(DellS4000Temp4):
    temp_max = 54
    # Dell recommends 64 C in "S4048-ON System Thermal Management Design Guide Rev0.8.pdf"
    # Increasing this limit to Temp1 Critical to avoid software shutdown related issue. CM-7318.
    temp_crit = 77
    temp_fan_min = 35
    temp_fan_max = 50        

class DellS4000Temp4B2F(DellS4000Temp4):
    temp_max = 49
    # Dell recommends 61 C in "S4048-ON System Thermal Management Design Guide Rev0.8.pdf"
    # Increasing this limit to Temp1 Critical to avoid software shutdown related issue. CM-7318.
    temp_crit = 70
    temp_fan_min = 35
    temp_fan_max = 45        
        
class DellS4000CT(cumulus.sensors.Temp_Unit):
    # U16
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98

class DellS4000Temp5(DellS4000CT):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class DellS4000Temp6(DellS4000CT):
    name = 'Temp6'
    driver_hwmon = ['temp3']
        
class DellS4000PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    # Dell recommends 61 C in "S4048-ON System Thermal Management Design Guide Rev0.8.pdf"
    # Increasing this limit to 65 C (PSU OTP) to avoid software shutdown related issue. CM-7318.
    temp_crit = 65
    psu = None
    temp_otp_hw = True

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS4000Temp7(DellS4000PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS4000_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'
    
class DellS4000Temp8(DellS4000PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS4000_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'
        
class DellS4000T2Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 97
    temp_crit    = 104
    temp_fan_min = 56
    temp_fan_max = 80

class DellS4000T2Temp1(DellS4000T2Temp):
    name = 'Temp7'
    driver_hwmon = ['temp1']

class DellS4000T2Temp2(DellS4000T2Temp):
    name = 'Temp8'
    driver_hwmon = ['temp2']

class DellS4000T2Temp3(DellS4000T2Temp):
    name = 'Temp9'
    driver_hwmon = ['temp3']

class DellS4000T2Temp4(DellS4000T2Temp):
    name = 'Temp10'
    driver_hwmon = ['temp4']

class DellS4000T2Temp5(DellS4000T2Temp):
    name = 'Temp11'
    driver_hwmon = ['temp5']

class DellS4000T2Temp6(DellS4000T2Temp):
    name = 'Temp12'
    driver_hwmon = ['temp6']

class DellS4000T2Temp7(DellS4000T2Temp):
    name = 'Temp13'
    driver_hwmon = ['temp7']

class DellS4000T2Temp8(DellS4000T2Temp):
    name = 'Temp14'
    driver_hwmon = ['temp8']

class DellS4000T2Temp9(DellS4000T2Temp):
    name = 'Temp15'
    driver_hwmon = ['temp9']
        
class DellS4000Fan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_max = 19000
    fan_min = 7000
    hwmon_search = False

class DellS4000PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 102 # Corresponds to duty 40 % and RPM 7200
    fan_max = 18000
    fan_min = 6000
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS4000Fan1(DellS4000Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = DellS4000_i2c_11_2a_path
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_1']

class DellS4000Fan2(DellS4000Fan):
    name = 'Fan2'
    description = 'Fan Tray 1'
    driver_path = DellS4000_i2c_11_2a_path
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']

class DellS4000Fan3(DellS4000Fan):
    name = 'Fan3'
    description = 'Fan Tray 2'
    driver_path = DellS4000_i2c_11_29_path
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_2']

class DellS4000Fan4(DellS4000Fan):
    name = 'Fan4'
    description = 'Fan Tray 2'
    driver_path = DellS4000_i2c_11_29_path
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_2']

class DellS4000Fan5(DellS4000Fan):
    name = 'Fan5'
    description = 'Fan Tray 3'
    driver_path = DellS4000_i2c_11_29_path
    driver_hwmon = ['fan1']

class DellS4000Fan6(DellS4000Fan):
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_path = DellS4000_i2c_11_29_path
    driver_hwmon = ['fan2']

class DellS4000Fan7(DellS4000PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS4000_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'

class DellS4000Fan8(DellS4000PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS4000_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']    
    psu = 'PSU2'

class DellS4000PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS4000PSU1(DellS4000PSU):
    name = 'PSU1'
    driver_path = DellS4000_i2c_11_42_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DellS4000PSU2(DellS4000PSU):
    name = 'PSU2'
    driver_path = DellS4000_i2c_11_40_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']


def dell_s4000_airflow():
    f2b_part_nums = ['0J09D3', '0KMJ74', '0M68YC']
    b2f_part_nums = ['04NM4C', '0RM2JW', '099TJK']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dell,s4000_c2338':
        return None

    try:
        b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3    
    part_num = ''
    for l in b_info.split('\n'):
        if 'Part Number' in l:
            part_num = l.split()[4]
            break
    if part_num in b2f_part_nums:
        return 'B2F'        
    if part_num in f2b_part_nums:
        return 'F2B'        
    return None

class DellS4000PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DellS4000Fan1(),
            DellS4000Fan2(),
            DellS4000Fan3(),
            DellS4000Fan4(),
            DellS4000Fan5(),
            DellS4000Fan6(),
            DellS4000Fan7(),
            DellS4000Fan8(),
            )

    temps = (DellS4000Temp5(),
             DellS4000Temp6(),
             DellS4000Temp7(),
             DellS4000Temp8(),
             DellS4000T2Temp1(),
             DellS4000T2Temp2(),
             DellS4000T2Temp3(),
             DellS4000T2Temp4(),
             DellS4000T2Temp5(),
             DellS4000T2Temp6(),
             DellS4000T2Temp7(),
             DellS4000T2Temp8(),
             DellS4000T2Temp9(),
             )
    f2b_temps = (DellS4000Temp1F2B(),
                 DellS4000Temp2F2B(),
                 DellS4000Temp3F2B(),
                 DellS4000Temp4F2B(),
                )
    b2f_temps = (DellS4000Temp1B2F(),
                 DellS4000Temp2B2F(),
                 DellS4000Temp3B2F(),
                 DellS4000Temp4B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4000_airflow() == 'F2B':
        temps = temps + f2b_temps
    else:
        temps = temps + b2f_temps
        
class DellS4000Sensors():
    units = (DellS4000PSU1(),
             DellS4000PSU2(),
             DellS4000Temp5(),
             DellS4000Temp6(),
             DellS4000Temp7(),
             DellS4000Temp8(),
             DellS4000Fan1(),
             DellS4000Fan2(),
             DellS4000Fan3(),
             DellS4000Fan4(),
             DellS4000Fan5(),
             DellS4000Fan6(),
             DellS4000Fan7(),
             DellS4000Fan8(),
             DellS4000T2Temp1(),
             DellS4000T2Temp2(),
             DellS4000T2Temp3(),
             DellS4000T2Temp4(),
             DellS4000T2Temp5(),
             DellS4000T2Temp6(),
             DellS4000T2Temp7(),
             DellS4000T2Temp8(),
             DellS4000T2Temp9(),
             )

    f2b_temps = (DellS4000Temp1F2B(),
                 DellS4000Temp2F2B(),
                 DellS4000Temp3F2B(),
                 DellS4000Temp4F2B(),
                )
    b2f_temps = (DellS4000Temp1B2F(),
                 DellS4000Temp2B2F(),
                 DellS4000Temp3B2F(),
                 DellS4000Temp4B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4000_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps
        
class DellS4000PWMs():
    units = (DellS4000PWM1(),
             )

class DellS4000LEDs():
    units = (
             DellS4000SystemLED(),
             DellS4000PowerLED(),
             DellS4000FanLED(),
             DellS4000Fan1LED(),
             DellS4000Fan2LED(),
             DellS4000Fan3LED(),
        )

class DellS4000Switch(cumulus.platform.BCMSwitch):
    ledup = (DellS4000LED_1_36, DellS4000LED_37_54)
    _sfp = DellS4000SFPp
    _qsfp = DellS4000QSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=(44,), leds=(0,  0),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="2",  serdes=(45,), leds=(0,  3),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="3",  serdes=(46,), leds=(0,  6),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="4",  serdes=(47,), leds=(0,  9),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="5",  serdes=(48,), leds=(0, 12),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="6",  serdes=(49,), leds=(0, 15),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="7",  serdes=(50,), leds=(0, 18),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="8",  serdes=(51,), leds=(0, 21),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="9",  serdes=(12,), leds=(0, 24),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="10", serdes=(13,), leds=(0, 27),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="11", serdes=(14,), leds=(0, 30),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="12", serdes=(15,), leds=(0, 33),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="13", serdes=(20,), leds=(0, 36),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="14", serdes=(21,), leds=(0, 39),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="15", serdes=(22,), leds=(0, 42),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="16", serdes=(23,), leds=(0, 45),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="17", serdes=(24,), leds=(0, 48),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="18", serdes=(25,), leds=(0, 51),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="19", serdes=(26,), leds=(0, 54),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="20", serdes=(27,), leds=(0, 57),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="21", serdes=(28,), leds=(0, 60),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="22", serdes=(29,), leds=(0, 63),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="23", serdes=(30,), leds=(0, 66),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="24", serdes=(31,), leds=(0, 69),
             rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="25", serdes=(52,), leds=(0, 72),
             rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="26", serdes=(53,), leds=(0, 75),
             rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="27", serdes=(54,), leds=(0, 78),
             rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="28", serdes=(55,), leds=(0, 81),
             rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="29", serdes=(56,), leds=(0, 84),
             rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="30", serdes=(57,), leds=(0, 87),
             rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="31", serdes=(58,), leds=(0, 90),
             rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="32", serdes=(59,), leds=(0, 93),
             rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="33", serdes=(60,), leds=(0, 96),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="34", serdes=(61,), leds=(0, 99),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="35", serdes=(62,), leds=(0,102),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="36", serdes=(63,), leds=(0,105),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="37", serdes=(64,), leds=(1,  0),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="38", serdes=(65,), leds=(1,  3),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="39", serdes=(66,), leds=(1,  6),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="40", serdes=(67,), leds=(1,  9),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="41", serdes=(68,), leds=(1, 12),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="42", serdes=(69,), leds=(1, 15),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="43", serdes=(70,), leds=(1, 18),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="44", serdes=(71,), leds=(1, 21),
             rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="45", serdes=(72,), leds=(1, 24),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="46", serdes=(73,), leds=(1, 27),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="47", serdes=(74,), leds=(1, 30),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="48", serdes=(75,), leds=(1, 33),
             rx_lane_map=0x2301, tx_lane_map=0x1032),
        _qsfp(label="49", serdes=(100, 101, 102, 103), leds=(1, 36),
                tx_lane_map=0x2031, rx_lane_map=0x0132,
                rx_polarity_flip=0x5, tx_polarity_flip=0x2
         ),
        _qsfp(label="50", serdes=(96, 97, 98, 99), leds=(1, 44),
                tx_lane_map=0x3021, rx_lane_map=0x1320,
                rx_polarity_flip=0xd, tx_polarity_flip=0xb
         ),
        _qsfp(label="51", serdes=(108, 109, 110, 111), leds=(1, 52),
                tx_lane_map=0x0213, rx_lane_map=0x0132,
                rx_polarity_flip=0x5, tx_polarity_flip=0x8
        ),
        _qsfp(label="52", serdes=(104, 105, 106, 107), leds=(1, 60),
                tx_lane_map=0x2031, rx_lane_map=0x0132,
                rx_polarity_flip=0x3, tx_polarity_flip=0x3
        ),
        _qsfp(label="53", serdes=( 80, 81, 82, 83), leds=(1, 68),
                tx_lane_map=0x0132, rx_lane_map=0x0132,
        rx_polarity_flip=0xb, tx_polarity_flip=0x0
        ),
        _qsfp(label="54", serdes=( 76,  77,  78,  79), leds=(1, 76),
                tx_lane_map=0x2031, rx_lane_map=0x1320,
                rx_polarity_flip=0xd, tx_polarity_flip=0x3
        ),
    )

class DellS4000(cumulus.platform.Platform):
    name = 'dell,s4000_c2338'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = DellS4000Switch(chip)
    cpld = DellS4000CPLD()
    pwms = DellS4000PWMs()
    sensors = DellS4000Sensors()
    statusleds = DellS4000LEDs()

Platform = DellS4000
