#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
from json import loads
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS6000QSFPp(cumulus.platform.TridentTwo40GPort):
    # _set_led_common defines LED functions common to both 40G and
    # 4x10G LED programs.
    _set_led_common = '''
;;; this is a common entry point used by all port types to set a single led
set_any_led:
        port    a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        ret
'''
    _set_led = '''
;;; DellS6000QSFPp
;;; - one led: on for link and flash with activity (low active)
set_led:
        call    set_any_led
        ret
'''
    _set_led_4x10_even = '''
;;;  handle the four ports on this QSFP
;;;   on even ports the layout is 2 1 4 3
set_led:
        inc     a
        call    set_any_led
        dec     a
        call    set_any_led
        add     a,3
        call    set_any_led
        dec     a
        call    set_any_led
        ret
'''
    _set_led_4x10_odd = '''
;;;  handle the four ports on this QSFP
;;;   on odd ports the layout is 1, 2, 3, 4
set_led:
set_loop:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip=None, tx_polarity_flip=None,
                 pre_emphasis=None, drv_current=None, pre_drv_current=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    rx_lane_map=rx_lane_map,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    tx_lane_map=tx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    pre_emphasis=pre_emphasis,
                                                    drv_current=drv_current,
                                                    pre_drv_current=pre_drv_current,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        ''' - 4 leds per QSFP
            - in 40G mode, the first led indicates link/activity
            - in 4x10G mode, all four leds are used - one for each breakout port
            - led layouts are different for odd and even ports
        '''
        self._check_mode(mode)
        set_calls = {}

        if mode == "4x10G" and sub != 0:
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        remap = 0
        if mode == "4x10G":
            for i in range(4):
                set_calls[self.leds[1] + i] = None
            if self._is_even_port():
                call = (fn_name + "_4x10_e", self._set_led_4x10_even, led_port)
            else:
                call = (fn_name + "_4x10_odd", self._set_led_4x10_odd, led_port)
        elif mode == "40G":
            if self._is_even_port():
                remap = 1
            call = (fn_name, self._set_led, led_port)
        set_calls[self.leds[1] + remap] = call

        return set_calls

    def _is_even_port(self):
        return (self.leds[1] / 4) & 1

class DellS6000LED(cumulus.ledup.TridentTwoLEDController):
    led_functions = (DellS6000QSFPp._set_led_common)
    num_leds = 64

class DellS6000CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dell_s6000_cpld.0'


'''
   Dell S6000 LEDS
     System:
       green - all systems ok
       green blinking - fan missing or broken, psu missing or broken
     Power:
       green - both psu installed and working
       yellow blinking - psu missing or malfunction
     Fan:
       green - all 3 fans installed and working
       yellow blinking - fan missing or malfunction
     Fan_Tray_X (0 - 2):
       green - fan X installed and working
       yellow - fan X malfunction
     Master:
       off
'''

def DellS6000_get_paths():
    # Quick hack to prevent other platforms from running this. CM-2908
    # Need to fix this elegantly.
    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return '', ''
    if plat.strip() != 'dell,s6000_s1220':
        return '', ''

    isch_path = '/sys/bus/pci/devices/0000:00:1f.0/isch_smbus.3168'
    if os.path.exists(isch_path):
        for x in os.listdir(isch_path):
            if x.startswith('i2c-'):
                isch_path = os.path.join(isch_path, x)
                break

    ismt_0_13_1_path = '/sys/bus/pci/devices/0000:00:13.1'
    if os.path.exists(ismt_0_13_1_path):
        for x in os.listdir(ismt_0_13_1_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                ismt_0_13_1_path = os.path.join(ismt_0_13_1_path, x) + '/' + b
                break

    return isch_path, ismt_0_13_1_path

DellS6000_isch_path, DellS6000_ismt_0_13_1_path = DellS6000_get_paths()
DellS6000_i2c_11_2a_path = DellS6000_isch_path + '/i2c-11/11-002a'
DellS6000_i2c_11_29_path = DellS6000_isch_path + '/i2c-11/11-0029'
DellS6000_i2c_10_4d_path = DellS6000_isch_path + '/i2c-10/10-004d'
DellS6000_i2c_2_58_path = DellS6000_ismt_0_13_1_path + '-0058'
DellS6000_i2c_2_59_path = DellS6000_ismt_0_13_1_path + '-0059'
DellS6000_w83627ehf_path = '/sys/devices/platform/w83627ehf.2560'

class DellS6000PSULED(cumulus.sensors.StatusLED_Unit):
    name = 'PSUs'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'yellow_blinking'

class DellS6000SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'green_blinking'

class DellS6000FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fans'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'

class DellS6000Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray_0'
    bad_led_color = 'yellow'

class DellS6000Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'yellow'

class DellS6000Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan_tray_2'
    bad_led_color = 'yellow'

class DellS6000TempCPUF2B(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 47
    temp_fan_max = 76
    temp_max = 90
    temp_crit = 100

class DellS6000TempCPUB2F(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 32
    temp_fan_max = 61
    temp_max = 90
    temp_crit = 100    

class DellS6000Temp1F2B(DellS6000TempCPUF2B):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DellS6000Temp2F2B(DellS6000TempCPUF2B):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class DellS6000Temp1B2F(DellS6000TempCPUB2F):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DellS6000Temp2B2F(DellS6000TempCPUB2F):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class DellS6000Temp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'CPU0 Temp Sensor'
    driver_path = DellS6000_i2c_10_4d_path
    driver_hwmon = ['temp1']
    temp_max = 75
    temp_crit = 85

class DellS6000Temp3F2B(DellS6000Temp3):
    temp_fan_min = 42
    temp_fan_max = 71

class DellS6000Temp3B2F(DellS6000Temp3):
    temp_fan_min = 27
    temp_fan_max = 56

class DellS6000Temp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'CPU1 Temp Sensor'
    driver_path = DellS6000_i2c_10_4d_path
    driver_hwmon = ['temp2']
    temp_max = 75
    temp_crit = 85
    
class DellS6000Temp4F2B(DellS6000Temp4):
    temp_fan_min = 42
    temp_fan_max = 71

class DellS6000Temp4B2F(DellS6000Temp4):
    temp_fan_min = 28
    temp_fan_max = 57

class DellS6000Temp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS6000_isch_path + '/i2c-11/11-004c'
    driver_hwmon = ['temp1']
    temp_max = 77
    temp_crit = 85
    temp_otp_hw = True

class DellS6000Temp5F2B(DellS6000Temp5):
    temp_fan_min = 47
    temp_fan_max = 76

class DellS6000Temp5B2F(DellS6000Temp5):
    temp_fan_min = 38
    temp_fan_max = 67

class DellS6000Temp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Board Sensor near NIC'
    driver_path = DellS6000_isch_path + '/i2c-11/11-004d'
    driver_hwmon = ['temp1']
    temp_max = 77
    temp_crit = 85
    temp_otp_hw = True
    
class DellS6000Temp6F2B(DellS6000Temp6):
    temp_fan_min = 27
    temp_fan_max = 56

class DellS6000Temp6B2F(DellS6000Temp6):
    temp_fan_min = 28
    temp_fan_max = 57

class DellS6000Temp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Ambient Temp Sensor'
    driver_path = DellS6000_isch_path + '/i2c-11/11-004e'
    driver_hwmon = ['temp1']
    temp_max = 77
    temp_crit = 85
    temp_otp_hw = True
      
class DellS6000Temp7F2B(DellS6000Temp7):
    temp_fan_min = 31
    temp_fan_max = 60

class DellS6000Temp7B2F(DellS6000Temp7):
    temp_fan_min = 32
    temp_fan_max = 61
      
class DellS6000PSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 85
    temp_max = 77
    temp_fan_min = 31
    temp_fan_max = 60
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS6000Temp8(DellS6000PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS6000_i2c_2_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'
        
class DellS6000Temp9(DellS6000PSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS6000_i2c_2_59_path
    driver_hwmon = ['temp2']
    psu = 'PSU1'

class DellS6000Temp10(DellS6000PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS6000_i2c_2_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'

class DellS6000Temp11(DellS6000PSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS6000_i2c_2_58_path
    driver_hwmon = ['temp2']
    psu = 'PSU2'

class DellS6000T2Temp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 97
    temp_crit    = 104
    temp_fan_min = 56
    temp_fan_max = 80

class DellS6000T2Temp1(DellS6000T2Temp):
    name = 'Temp8'
    driver_hwmon = ['temp1']

class DellS6000T2Temp2(DellS6000T2Temp):
    name = 'Temp9'
    driver_hwmon = ['temp2']

class DellS6000T2Temp3(DellS6000T2Temp):
    name = 'Temp10'
    driver_hwmon = ['temp3']

class DellS6000T2Temp4(DellS6000T2Temp):
    name = 'Temp11'
    driver_hwmon = ['temp4']

class DellS6000T2Temp5(DellS6000T2Temp):
    name = 'Temp12'
    driver_hwmon = ['temp5']

class DellS6000T2Temp6(DellS6000T2Temp):
    name = 'Temp13'
    driver_hwmon = ['temp6']

class DellS6000T2Temp7(DellS6000T2Temp):
    name = 'Temp14'
    driver_hwmon = ['temp7']

class DellS6000T2Temp8(DellS6000T2Temp):
    name = 'Temp15'
    driver_hwmon = ['temp8']

class DellS6000T2Temp9(DellS6000T2Temp):
    name = 'Temp16'
    driver_hwmon = ['temp9']

class DellS6000Fan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_max = 19000
    fan_min = 7200
    hwmon_search = False

class DellS6000PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 102  # Corresponds to duty 40 % and RPM 7200
    fan_max = 18000
    fan_min = 7000
    psu = None
    
    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS6000Fan1(DellS6000Fan):
    name = 'Fan1'
    description = 'Fan Tray 0'
    driver_path = DellS6000_i2c_11_2a_path
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_0']

class DellS6000Fan2(DellS6000Fan):
    name = 'Fan2'
    description = 'Fan Tray 0'
    driver_path = DellS6000_i2c_11_2a_path
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_0']

class DellS6000Fan3(DellS6000Fan):
    name = 'Fan3'
    description = 'Fan Tray 1'
    driver_path = DellS6000_i2c_11_29_path
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_1']

class DellS6000Fan4(DellS6000Fan):
    name = 'Fan4'
    description = 'Fan Tray 1'
    driver_path = DellS6000_i2c_11_29_path
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_1']

class DellS6000Fan5(DellS6000Fan):
    name = 'Fan5'
    description = 'Fan Tray 2'
    driver_path = DellS6000_i2c_11_29_path
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_2']

class DellS6000Fan6(DellS6000Fan):
    name = 'Fan6'
    description = 'Fan Tray 2'
    driver_path = DellS6000_i2c_11_29_path
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_2']

class DellS6000Fan7(DellS6000PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS6000_i2c_2_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'

class DellS6000Fan8(DellS6000PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS6000_i2c_2_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'

class DellS6000PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS6000PSU1(DellS6000PSU):
    name = 'PSU1'
    driver_path = DellS6000_isch_path + '/i2c-11/11-0042'
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DellS6000PSU2(DellS6000PSU):
    name = 'PSU2'
    driver_path = DellS6000_isch_path + '/i2c-11/11-0040'
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']

# The range for the voltage sensors in +/- 15% of the base value seen.
# This is an approximation, as range is not provided.
class DellS6000Volt1(cumulus.sensors.Volt_Unit):
    name = 'Volt1'
    description = 'CPU VCore'
    volt_min = 0.89
    volt_max = 1.21
    driver_path = DellS6000_w83627ehf_path
    driver_hwmon = ['in0']
    hwmon_search = False
    
class DellS6000Volt2(cumulus.sensors.Volt_Unit):
    name = 'Volt2'
    description = 'DDR VTT'
    volt_min = 1.00
    volt_max = 1.36
    driver_path = DellS6000_w83627ehf_path
    driver_hwmon = ['in4']
    hwmon_search = False
    
class DellS6000Volt3(cumulus.sensors.Volt_Unit):
    name = 'Volt3'
    description = '+3.3V'
    volt_min = 2.82
    volt_max = 3.82
    driver_path = DellS6000_w83627ehf_path
    driver_hwmon = ['in3']
    hwmon_search = False
    
def dell_s6000_airflow():
    '''
    Attempt to determine airflow direction from part number in syseeprom.

    Returns:
    'F2B' : Front-to-back airflow
    'B2F' : Back-to-front aiflow
    None  : Unable to determine airflow direction
    '''

    b2f_part_nums = ['07VJDK',]
    f2b_part_nums = ['09CX17',]

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dell,s6000_s1220':
        return None

    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3    
    try:
        part_num = loads(subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom', '--json']))['tlv']['Part Number']['value']
    except:
        return None

    if part_num in b2f_part_nums:
        return 'B2F'        
    if part_num in f2b_part_nums:
        return 'F2B'        
    return None
    
class DellS6000PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DellS6000Fan1(),
            DellS6000Fan2(),
            DellS6000Fan3(),
            DellS6000Fan4(),
            DellS6000Fan5(),
            DellS6000Fan6(),
            DellS6000Fan7(),
            DellS6000Fan8(),
            )
    temps = (DellS6000Temp8(),
             DellS6000Temp9(),
             DellS6000Temp10(),
             DellS6000Temp11(),
	     DellS6000T2Temp1(),
	     DellS6000T2Temp2(),
	     DellS6000T2Temp3(),
	     DellS6000T2Temp4(),
	     DellS6000T2Temp5(),
	     DellS6000T2Temp6(),
	     DellS6000T2Temp7(),
	     DellS6000T2Temp8(),
	     DellS6000T2Temp9(),
             )
    f2b_temps = (DellS6000Temp1F2B(),
                 DellS6000Temp2F2B(),
                 DellS6000Temp3F2B(),
                 DellS6000Temp4F2B(),
                 DellS6000Temp5F2B(),
                 DellS6000Temp6F2B(),
                 DellS6000Temp7F2B(),
                 )

    b2f_temps = (DellS6000Temp1B2F(),
                 DellS6000Temp2B2F(),
                 DellS6000Temp3B2F(),
                 DellS6000Temp4B2F(),
                 DellS6000Temp5B2F(),
                 DellS6000Temp6B2F(),
                 DellS6000Temp7B2F(),
                 )
    # Default to F2B if airflow cant be figured out.
    if dell_s6000_airflow() == 'B2F':
        temps = temps + b2f_temps
    else:
        temps = temps + f2b_temps
                
class DellS6000Sensors():
    units = (DellS6000PSU1(),
             DellS6000PSU2(),
             DellS6000Volt1(),
             DellS6000Volt2(),
             DellS6000Volt3(),
             DellS6000Temp8(),
             DellS6000Temp9(),
             DellS6000Temp10(),
             DellS6000Temp11(),
             DellS6000Fan1(),
             DellS6000Fan2(),
             DellS6000Fan3(),
             DellS6000Fan4(),
             DellS6000Fan5(),
             DellS6000Fan6(),
             DellS6000Fan7(),
             DellS6000Fan8(),
	     DellS6000T2Temp1(),
	     DellS6000T2Temp2(),
	     DellS6000T2Temp3(),
	     DellS6000T2Temp4(),
	     DellS6000T2Temp5(),
	     DellS6000T2Temp6(),
	     DellS6000T2Temp7(),
	     DellS6000T2Temp8(),
	     DellS6000T2Temp9(),
             )
    f2b_temps = (DellS6000Temp1F2B(),
                 DellS6000Temp2F2B(),
                 DellS6000Temp3F2B(),
                 DellS6000Temp4F2B(),
                 DellS6000Temp5F2B(),
                 DellS6000Temp6F2B(),
                 DellS6000Temp7F2B(),
                 )

    b2f_temps = (DellS6000Temp1B2F(),
                 DellS6000Temp2B2F(),
                 DellS6000Temp3B2F(),
                 DellS6000Temp4B2F(),
                 DellS6000Temp5B2F(),
                 DellS6000Temp6B2F(),
                 DellS6000Temp7B2F(),
                 )
    # Default to F2B if airflow cant be figured out.
    if dell_s6000_airflow() == 'B2F':
        units = units + b2f_temps
    else:
        units = units + f2b_temps
        
class DellS6000PWMs():
    units = (DellS6000PWM1(),
             )

class DellS6000LEDs():
    units = (DellS6000PSULED(),
             DellS6000SystemLED(),
             DellS6000FanLED(),
             DellS6000Fan1LED(),
             DellS6000Fan2LED(),
             DellS6000Fan3LED(),
        )

class DellS6000Switch(cumulus.platform.BCMSwitch):
    ledup = DellS6000LED
    _qsfp = DellS6000QSFPp

    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _qsfp(label="1", serdes=(  24,   25,   26,   27), leds=(0, 4),
              rx_lane_map=0x0213, tx_lane_map=0x2031,
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000e,
              pre_emphasis=(0xcad0, 0xc6e0, 0xc6e0, 0xd2b0),
              drv_current=(0x5, 0x5, 0x5, 0x5),
              pre_drv_current=(0x5, 0x5, 0x5, 0x5)),
        _qsfp(label="2", serdes=(  28,   29,   30,   31), leds=(0, 0),
              rx_lane_map=0x0213, tx_lane_map=0x0213,
              tx_polarity_flip=0x0009, rx_polarity_flip=0x000c,
              pre_emphasis=(0xc2f0, 0xd2b0, 0xc6e0, 0xc2f0),
              drv_current=(0x6, 0x7, 0x6, 0x6),
              pre_drv_current=(0x6, 0x7, 0x6, 0x6)),
        _qsfp(label="3", serdes=( 32,  33,  34,  35), leds=(0, 12),
              rx_lane_map=0x0213, tx_lane_map=0x0132,
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000e,
              pre_emphasis=(0xc6e0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="4", serdes=(  36,   37,  38,  39), leds=(0, 8),
              rx_lane_map=0x0213, tx_lane_map=0x1203,
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0003,
              pre_emphasis=(0xcad0, 0xcad0, 0xc2f0, 0xc2f0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="5", serdes=( 44,  45,  46,  47), leds=(0, 20),
              rx_lane_map=0x0213, tx_lane_map=0x0213,
              tx_polarity_flip=0x0008, rx_polarity_flip=0x000e,
              pre_emphasis=(0xc2f0, 0xc2f0, 0xc2f0, 0xc2f0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="6", serdes=( 40,  41,  42,  43), leds=(0, 16),
              rx_lane_map=0x0213, tx_lane_map=0x3021,
              tx_polarity_flip=0x000b, rx_polarity_flip=0x0003,
              pre_emphasis=(0xc6e0, 0xc2f0, 0xc2f0, 0xcad0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="7", serdes=( 0,  1,  2,  3), leds=(0, 28),
              rx_lane_map=0x0213, tx_lane_map=0x2031,
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000e,
              pre_emphasis=(0xc6e0, 0xcad0, 0xc6e0, 0xcad0),
              drv_current=(0x5, 0x5, 0x5, 0x5),
              pre_drv_current=(0x5, 0x5, 0x5, 0x5)),
        _qsfp(label="8", serdes=( 4,  5,  6,  7), leds=(0, 24),
              rx_lane_map=0x0213, tx_lane_map=0x1203,
              tx_polarity_flip=0x0001, rx_polarity_flip=0x000c,
              pre_emphasis=(0xc6e0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="9", serdes=( 12,  13,  14,  15), leds=(0, 36),
              rx_lane_map=0x0213, tx_lane_map=0x0132,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000e,
              pre_emphasis=(0xb270, 0xbb10, 0xb720, 0xb720),
              drv_current=(0x2, 0x3, 0x2, 0x2),
              pre_drv_current=(0x2, 0x3, 0x2, 0x2)),
        _qsfp(label="10", serdes=( 8,  9,  10,  11), leds=(0, 32),
              rx_lane_map=0x3120, tx_lane_map=0x3021,
              tx_polarity_flip=0x0004, rx_polarity_flip=0x0000,
              pre_emphasis=(0xc2f0, 0xc6e0, 0xbf00, 0xc2f0),
              drv_current=(0x3, 0x3, 0x3, 0x3),
              pre_drv_current=(0x3, 0x3, 0x3, 0x3)),
        _qsfp(label="11", serdes=( 16,  17,  18,  19), leds=(0, 44),
              rx_lane_map=0x0213, tx_lane_map=0x0132,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000e,
              pre_emphasis=(0xb330, 0xbb10, 0xbb10, 0xbb10),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="12", serdes=( 20,  21,  22,  23), leds=(0, 40),
              rx_lane_map=0x0123, tx_lane_map=0x1203,
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000c,
              pre_emphasis=(0xb330, 0xb330, 0xb330, 0xb330),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="13", serdes=( 52,  53,  54,  55), leds=(0, 52),
              rx_lane_map=0x0213, tx_lane_map=0x0231,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0001,
              pre_emphasis=(0xaf40, 0xaf40, 0xaf40, 0xaf40),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="14", serdes=( 48,  49,  50,  51), leds=(0, 48),
              rx_lane_map=0x1302, tx_lane_map=0x2031,
              tx_polarity_flip=0x0003, rx_polarity_flip=0x000b,
              pre_emphasis=(0xa760, 0xa760, 0xa760, 0xa760),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 60),
              rx_lane_map=0x0213, tx_lane_map=0x2031,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0001,
              pre_emphasis=(0xa760, 0xa760, 0xa760, 0xa760),
              drv_current=(0x1, 0x1, 0x1, 0x1),
              pre_drv_current=(0x1, 0x1, 0x1, 0x1)),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 56),
              rx_lane_map=0x0132, tx_lane_map=0x0213,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              pre_emphasis=(0xa760, 0xa760, 0xa760, 0xa760),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="17", serdes=( 68,  69,  70, 71), leds=(1, 4),
              rx_lane_map=0x0213, tx_lane_map=0x2130,
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0001,
              pre_emphasis=(0xa760, 0xa760, 0xa760, 0xa760),
              drv_current=(0x1, 0x1, 0x1, 0x1),
              pre_drv_current=(0x1, 0x1, 0x1, 0x1)),
        _qsfp(label="18", serdes=( 64,  65,  66,  67), leds=(1, 0),
              rx_lane_map=0x0132, tx_lane_map=0x2031,
              tx_polarity_flip=0x0009, rx_polarity_flip=0x0003,
              pre_emphasis=(0xa370, 0xa370, 0xa370, 0xa370),
              drv_current=(0x1, 0x1, 0x1, 0x1),
              pre_drv_current=(0x1, 0x1, 0x1, 0x1)),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 12),
              rx_lane_map=0x0213, tx_lane_map=0x2031,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0001,
              pre_emphasis=(0xa760, 0xa760, 0xa760, 0xa760),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 8),
              rx_lane_map=0x0123, tx_lane_map=0x1203,
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0003,
              pre_emphasis=(0xaf40, 0xaf40, 0xaf40, 0xaf40),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="21", serdes=( 108,  109,  110,  111), leds=(1, 20),
              rx_lane_map=0x0132, tx_lane_map=0x0132,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              pre_emphasis=(0xb330, 0xb330, 0xb330, 0xbff0),
              drv_current=(0x1, 0x1, 0x1, 0x2),
              pre_drv_current=(0x1, 0x1, 0x1, 0x2)),
        _qsfp(label="22", serdes=( 104,  105,  106,  107), leds=(1, 16),
              rx_lane_map=0x1320, tx_lane_map=0x3021,
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000d,
              pre_emphasis=(0xb330, 0xb330, 0xb330, 0xb330),
              drv_current=(0x1, 0x1, 0x1, 0x1),
              pre_drv_current=(0x1, 0x1, 0x1, 0x1)),
        _qsfp(label="23", serdes=( 112,  113,  114,  115), leds=(1, 28),
              rx_lane_map=0x0132, tx_lane_map=0x0132,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              pre_emphasis=(0xbb10, 0xbb10, 0xbb10, 0xc2f0),
              drv_current=(0x1, 0x1, 0x1, 0x1),
              pre_drv_current=(0x1, 0x1, 0x1, 0x1)),
        _qsfp(label="24", serdes=( 116,  117,  118,  119), leds=(1, 24),
              rx_lane_map=0x0231, tx_lane_map=0x1203,
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0003,
              pre_emphasis=(0xc6e0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x3, 0x5, 0x3, 0x3),
              pre_drv_current=(0x3, 0x5, 0x3, 0x3)),
        _qsfp(label="25", serdes=(124, 125, 126, 127), leds=(1, 36),
              rx_lane_map=0x0132, tx_lane_map=0x0132,
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0008,
              pre_emphasis=(0xc6e0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="26", serdes=( 120,  121,  122,  123), leds=(1, 32),
              rx_lane_map=0x1320, tx_lane_map=0x3021,
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000d,
              pre_emphasis=(0xc6e0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="27", serdes=(80, 81, 82, 83), leds=(1, 44),
              rx_lane_map=0x1320, tx_lane_map=0x2031,
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0001,
              pre_emphasis=(0xbb10, 0xbb10, 0xbf00, 0xbb10),
              drv_current=(0x2, 0x2, 0x2, 0x2),
              pre_drv_current=(0x2, 0x2, 0x2, 0x2)),
        _qsfp(label="28", serdes=(84, 85, 86, 87), leds=(1, 40),
              rx_lane_map=0x0213, tx_lane_map=0x1203,
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000c,
              pre_emphasis=(0xc2f0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x5, 0x4, 0x5),
              pre_drv_current=(0x4, 0x5, 0x4, 0x5)),
        _qsfp(label="29", serdes=(92, 93, 94, 95), leds=(1, 52),
              rx_lane_map=0x1320, tx_lane_map=0x2031,
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0001,
              pre_emphasis=(0xc2f0, 0xc2f0, 0xc2f0, 0xc2f0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="30", serdes=(88, 89, 90, 91), leds=(1, 48),
              rx_lane_map=0x1320, tx_lane_map=0x3021,
              tx_polarity_flip=0x000b, rx_polarity_flip=0x0002,
              pre_emphasis=(0xcad0, 0xc6e0, 0xc6e0, 0xc6e0),
              drv_current=(0x4, 0x4, 0x4, 0x4),
              pre_drv_current=(0x4, 0x4, 0x4, 0x4)),
        _qsfp(label="31", serdes=(100, 101, 102, 103), leds=(1, 60),
              rx_lane_map=0x1320, tx_lane_map=0x1203,
              tx_polarity_flip=0x0006, rx_polarity_flip=0x0001,
              pre_emphasis=(0xcec0, 0xcec0, 0xcad0, 0xc6e0),
              drv_current=(0x6, 0x6, 0x6, 0x7),
              pre_drv_current=(0x6, 0x6, 0x6, 0x7)),
        _qsfp(label="32", serdes=(96, 97, 98, 99), leds=(1, 56),
              rx_lane_map=0x0213, tx_lane_map=0x2031,
              tx_polarity_flip=0x0003, rx_polarity_flip=0x000c,
              pre_emphasis=(0xcad0, 0xcad0, 0xcad0, 0xcad0),
              drv_current=(0x5, 0x5, 0x5, 0x5),
              pre_drv_current=(0x5, 0x5, 0x5, 0x5)),
    )

class DellS6000(cumulus.platform.Platform):
    name = 'dell,s6000_s1220'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = DellS6000Switch(chip)
    cpld = DellS6000CPLD()
    pwms = DellS6000PWMs()
    sensors = DellS6000Sensors()
    statusleds = DellS6000LEDs()

Platform = DellS6000
