#
# Copyright 2019.  Cumulus Networks, Inc.
#
# dell_n3048ep.py --
#     Dell-EMC 3048EP platform
#
from cumulus.platforms.dni_3048up import *

class DellEMC3048EPSFPp(cumulus.platform.Helix410GPort):
    _set_led = '''
;;; DellEMC3048EPSFPp
;;;
;;; 3 bits  per port, 1: off, 0: on
;;;
;;; 1st bit: Link Yellow (not used)
;;; 2nd bit: Link Green
;;; 3rd bit: Activity LED
;;;
set_led:
        push    1                   ; Yellow Link LED off
        pack

        port    a                   ; Green link LED
        pushst  PORTSTATUS_LINKEN
        tinv
        pack

        port    a                   ; Activity LED
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pack

        ret

'''
    def __init__(self, label, hw_port, leds, phy_addr, rx_polarity_flip=None,
                        tx_polarity_flip=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type='84752',
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addr, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None

        return set_calls

class DellEMC3048EPGigE(cumulus.platform.Helix41GPort):
    #
    #   Since all ports are on the same LED chain, the amount of code produced
    #   when each port LED is handled individually exceeds the amount of memory
    #   for code. Therefore, all 46 RJ45 GE ports are handled in one shot here
    #   in this routine. Only one of the 46 GE ports should enable the "leds"
    #   attribute, port 46. All others must have the "leds" attribute set to
    #   None.
    #
    _set_led = '''
;;; DellEMC3048EPGigE
;;;
;;; 1 bits  per port, 1: off, 0: on
;;;
;;; 1st bit: activity
;;;
set_led:
set_led_loop:
        port    a                   ; Activity LED
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pack
        dec     a
        jnz     set_led_loop

        ret

'''
    def __init__(self, label, hw_port, phy_addr, prim_offset, leds):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        for index in range(led_index+1, led_index+46):
            set_calls[index] = None

        return set_calls

class DellEMC3048EPComboGigE(cumulus.platform.Helix41GPort):
    _set_led = '''
;;; DellEMC3048EPComboGigE
;;;
;;; 4 bits  per port
;;;
;;; 1st bit: Link Yellow (not used)
;;; 2nd bit: Link Green
;;; 3rd bit: POE         (Orange, used to indicate activity to match other GE ports)
;;; 4th bit: Activity    (not used)
;;;
set_led:
        push    1                   ; Yellow Link LED off
        pack

        port a                      ; Green link LED
        pushst  PORTSTATUS_LINKEN
        tinv
        pack

        port a                      ; Toggle the POE bit for activity.
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pack

        push    1                   ; Green activity LED is always off
        pack

        ret

'''
    def __init__(self, label, hw_port, phy_addr, prim_offset, leds):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None
        set_calls[led_index + 3] = None

        return set_calls

class DellEMC3048EPLED(cumulus.ledup.Helix4LEDController):
    num_leds = 76

class DellEMC3048EPSwitch(cumulus.platform.BCMSwitch):
    ledup = DellEMC3048EPLED
    _gige = DellEMC3048EPGigE
    _combogige = DellEMC3048EPComboGigE
    _sfp = DellEMC3048EPSFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True
    
    ports = (
        _gige(label="1",   hw_port='ge0',   phy_addr=0x01, prim_offset=0x0100, leds=None),
        _gige(label="2",   hw_port='ge1',   phy_addr=0x02, prim_offset=0x0101, leds=None),
        _gige(label="3",   hw_port='ge2',   phy_addr=0x03, prim_offset=0x0102, leds=None),
        _gige(label="4",   hw_port='ge3',   phy_addr=0x04, prim_offset=0x0103, leds=None),
        _gige(label="5",   hw_port='ge4',   phy_addr=0x05, prim_offset=0x0104, leds=None),
        _gige(label="6",   hw_port='ge5',   phy_addr=0x06, prim_offset=0x0105, leds=None),
        _gige(label="7",   hw_port='ge6',   phy_addr=0x07, prim_offset=0x0106, leds=None),
        _gige(label="8",   hw_port='ge7',   phy_addr=0x08, prim_offset=0x0107, leds=None),

        _gige(label="9",   hw_port='ge8',   phy_addr=0x0a, prim_offset=0x0900, leds=None),
        _gige(label="10",  hw_port='ge9',   phy_addr=0x0b, prim_offset=0x0901, leds=None),
        _gige(label="11",  hw_port='ge10',  phy_addr=0x0c, prim_offset=0x0902, leds=None),
        _gige(label="12",  hw_port='ge11',  phy_addr=0x0d, prim_offset=0x0903, leds=None),
        _gige(label="13",  hw_port='ge12',  phy_addr=0x0e, prim_offset=0x0904, leds=None),
        _gige(label="14",  hw_port='ge13',  phy_addr=0x0f, prim_offset=0x0905, leds=None),
        _gige(label="15",  hw_port='ge14',  phy_addr=0x10, prim_offset=0x0906, leds=None),
        _gige(label="16",  hw_port='ge15',  phy_addr=0x11, prim_offset=0x0907, leds=None),

        _gige(label="17",  hw_port='ge16',  phy_addr=0x13, prim_offset=0x1100, leds=None),
        _gige(label="18",  hw_port='ge17',  phy_addr=0x14, prim_offset=0x1101, leds=None),
        _gige(label="19",  hw_port='ge18',  phy_addr=0x15, prim_offset=0x1102, leds=None),
        _gige(label="20",  hw_port='ge19',  phy_addr=0x16, prim_offset=0x1103, leds=None),
        _gige(label="21",  hw_port='ge20',  phy_addr=0x17, prim_offset=0x1104, leds=None),
        _gige(label="22",  hw_port='ge21',  phy_addr=0x18, prim_offset=0x1105, leds=None),
        _gige(label="23",  hw_port='ge22',  phy_addr=0x19, prim_offset=0x1106, leds=None),
        _gige(label="24",  hw_port='ge23',  phy_addr=0x1a, prim_offset=0x1107, leds=None),

        _gige(label="25",  hw_port='ge24',  phy_addr=0x21, prim_offset=0x1900, leds=None),
        _gige(label="26",  hw_port='ge25',  phy_addr=0x22, prim_offset=0x1901, leds=None),
        _gige(label="27",  hw_port='ge26',  phy_addr=0x23, prim_offset=0x1902, leds=None),
        _gige(label="28",  hw_port='ge27',  phy_addr=0x24, prim_offset=0x1903, leds=None),
        _gige(label="29",  hw_port='ge28',  phy_addr=0x25, prim_offset=0x1904, leds=None),
        _gige(label="30",  hw_port='ge29',  phy_addr=0x26, prim_offset=0x1905, leds=None),
        _gige(label="31",  hw_port='ge30',  phy_addr=0x27, prim_offset=0x1906, leds=None),
        _gige(label="32",  hw_port='ge31',  phy_addr=0x28, prim_offset=0x1907, leds=None),

        _gige(label="33",  hw_port='ge32',  phy_addr=0x2a, prim_offset=0x2100, leds=None),
        _gige(label="34",  hw_port='ge33',  phy_addr=0x2b, prim_offset=0x2101, leds=None),
        _gige(label="35",  hw_port='ge34',  phy_addr=0x2c, prim_offset=0x2102, leds=None),
        _gige(label="36",  hw_port='ge35',  phy_addr=0x2d, prim_offset=0x2103, leds=None),
        _gige(label="37",  hw_port='ge36',  phy_addr=0x2e, prim_offset=0x2104, leds=None),
        _gige(label="38",  hw_port='ge37',  phy_addr=0x2f, prim_offset=0x2105, leds=None),
        _gige(label="39",  hw_port='ge38',  phy_addr=0x30, prim_offset=0x2106, leds=None),
        _gige(label="40",  hw_port='ge39',  phy_addr=0x31, prim_offset=0x2107, leds=None),

        _gige(label="41",  hw_port='ge40',  phy_addr=0x33, prim_offset=0x2900, leds=None),
        _gige(label="42",  hw_port='ge41',  phy_addr=0x34, prim_offset=0x2901, leds=None),
        _gige(label="43",  hw_port='ge42',  phy_addr=0x35, prim_offset=0x2902, leds=None),
        _gige(label="44",  hw_port='ge43',  phy_addr=0x36, prim_offset=0x2903, leds=None),
        _gige(label="45",  hw_port='ge44',  phy_addr=0x37, prim_offset=0x2904, leds=None),
        _gige(label="46",  hw_port='ge45',  phy_addr=0x38, prim_offset=0x2905, leds=(0, 30)),
        _combogige(label="47",  hw_port='ge46',  phy_addr=0x39, prim_offset=0x2906, leds=(0, 20)),
        _combogige(label="48",  hw_port='ge47',  phy_addr=0x3a, prim_offset=0x2907, leds=(0, 16)),
        _sfp(label="49",   hw_port="hg0",   phy_addr=0x48, leds=(0, 13), tx_polarity_flip=0x0),
        _sfp(label="50",   hw_port="hg1",   phy_addr=0x49, leds=(0, 10), tx_polarity_flip=0x0),
    ) 
    
class DellEMC3048EP(DNI3048UP):
    name = 'dellemc,n3048ep'
    switch = DellEMC3048EPSwitch(DNI3048UP.chip)

Platform = DellEMC3048EP
