#
# Copyright (c) 2019, 2020 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellN3248PXEMG(cumulus.platform.TridentThreeMGPort):
    def __init__(self, label, serdes, phy_addrs, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.TridentThreeMGPort.__init__(self,
                                                     label=label,
                                                     serdes=serdes,
                                                     phy_types=('AQR813',),
                                                     phy_addrs=(phy_addrs,),
                                                     phy_layer='10GBaseT',
                                                     leds=None,
                                                     rx_lane_map=rx_lane_map,
                                                     tx_lane_map=tx_lane_map,
                                                     rx_polarity_flip=rx_polarity_flip,
                                                     tx_polarity_flip=tx_polarity_flip,
                                                     unit=0,
                                                     oversub=True)

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    # Normally we just use the physical port number as the logical port number.
    # In this case we must remap that physical port number (on the left) to
    # a new logical port number (on the right):
    # physical port numbers 1 to 40 must map to logical port numbers 0 to 32.
    # physical port numbers 41 to 80 must map to logical port numbers 33 to 64.
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 61: 33,
                  62: 34,
                  63: 35,
                  64: 36,
                  65: 37,
                  66: 38,
                  67: 39,
                  68: 40,
                  69: 41,
                  70: 42,
                  71: 43,
                  72: 44,
                  73: 45,
                  74: 46,
                  75: 47,
                  76: 48,
                  77: 49,
                  78: 50,
                  79: 51,
                  80: 52,
                  }
        lport = reMap.get(lport, lport)
        return lport

class DellN3248PXESFP28(cumulus.platform.TridentThree25GPort):
    def __init__(self, label, serdes, phy_addrs, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                      label=label,
                                                      serdes=serdes,
                                                      leds=None,
                                                      phy_types=('82399',),
                                                      phy_addrs=(phy_addrs,),
                                                      phy_layer=sfp.type.upper(),
                                                      rx_lane_map=rx_lane_map,
                                                      rx_polarity_flip=rx_polarity_flip,
                                                      tx_lane_map=tx_lane_map,
                                                      tx_polarity_flip=tx_polarity_flip,
                                                      pre_emphasis=pre_emphasis,
                                                      oversub=True,
                                                      unit=0)
        
        self.sfp = sfp

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    # Normally we just use the physical port number as the logical port number.
    # In this case we must remap that physical port number (on the left) to
    # a new logical port number (on the right):
    # physical port numbers 1 to 40 must map to logical port numbers 0 to 32.
    # physical port numbers 41 to 80 must map to logical port numbers 33 to 64.
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 37: 29,
                  38: 30,
                  39: 31,
                  40: 32,
                  }
        lport = reMap.get(lport, lport)
        return lport

class DellN3248PXEQSFP28(cumulus.platform.TridentThree100GPort):
    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=None,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    # Normally we just use the physical port number as the logical port number.
    # In this case we must remap that physical port number (on the left) to
    # a new logical port number (on the right):
    # physical port numbers 1 to 40 must map to logical port numbers 0 to 32.
    # physical port numbers 41 to 80 must map to logical port numbers 33 to 64.
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 41: 53,
                  42: 54,
                  43: 55,
                  44: 56,
                  45: 57,
                  46: 58,
                  47: 59,
                  48: 60,
                  }
        lport = reMap.get(lport, lport)
        return lport

class DellN3248PXESwitch(cumulus.platform.BCMSwitch):
    _mg = DellN3248PXEMG
    _sfp28 = DellN3248PXESFP28
    _qsfp28 = DellN3248PXEQSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True
    
    ports = (
        _mg(label="1",   serdes=( 61,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x21)),
        _mg(label="2",   serdes=( 60,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x20)),
        _mg(label="3",   serdes=( 63,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x23)),
        _mg(label="4",   serdes=( 62,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x22)),
        
        _mg(label="5",   serdes=( 65,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x25)),
        _mg(label="6",   serdes=( 64,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x24)),
        _mg(label="7",   serdes=( 67,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x27)),
        _mg(label="8",   serdes=( 66,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x26)),

        _mg(label="9",   serdes=( 69,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x29)),
        _mg(label="10",  serdes=( 68,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x28)),
        _mg(label="11",  serdes=( 71,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x2b)),
        _mg(label="12",  serdes=( 70,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x2a)),

        _mg(label="13",  serdes=( 73,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x2d)),
        _mg(label="14",  serdes=( 72,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x2c)),
        _mg(label="15",  serdes=( 75,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x2f)),
        _mg(label="16",  serdes=( 74,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x2e)),

        _mg(label="17",  serdes=( 77,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x41)),
        _mg(label="18",  serdes=( 76,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x40)),
        _mg(label="19",  serdes=( 79,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x43)),
        _mg(label="20",  serdes=( 78,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x42)),

        _mg(label="21",  serdes=(  2,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x45)),
        _mg(label="22",  serdes=(  3,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x44)),
        _mg(label="23",  serdes=(  0,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x47)),
        _mg(label="24",  serdes=(  1,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x46)),

        _mg(label="25",  serdes=(  6,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x49)),
        _mg(label="26",  serdes=(  7,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x48)),
        _mg(label="27",  serdes=(  4,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x4b)),
        _mg(label="28",  serdes=(  5,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x4a)),

        _mg(label="29",  serdes=( 10,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x4d)),
        _mg(label="30",  serdes=( 11,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x4c)),
        _mg(label="31",  serdes=(  8,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x4f)),
        _mg(label="32",  serdes=(  9,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x4e)),

        _mg(label="33",  serdes=( 14,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x01)),
        _mg(label="34",  serdes=( 15,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x00)),
        _mg(label="35",  serdes=( 12,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x03)),
        _mg(label="36",  serdes=( 13,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x02)),

        _mg(label="37",  serdes=( 18,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x05)),
        _mg(label="38",  serdes=( 19,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x04)),
        _mg(label="39",  serdes=( 16,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x07)),
        _mg(label="40",  serdes=( 17,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x06)),

        _mg(label="41",  serdes=( 22,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x09)),
        _mg(label="42",  serdes=( 23,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x08)),
        _mg(label="43",  serdes=( 20,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x0b)),
        _mg(label="44",  serdes=( 21,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=(0x0a)),

        _mg(label="45",  serdes=( 26,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x0d)),
        _mg(label="46",  serdes=( 27,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x0c)),
        _mg(label="47",  serdes=( 24,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x0f)),
        _mg(label="48",  serdes=( 25,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x0e)),

        _sfp28(label="49", serdes=( 39,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x60)),
        _sfp28(label="50", serdes=( 38,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x60)),
        _sfp28(label="51", serdes=( 37,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x60)),
        _sfp28(label="52", serdes=( 36,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x60)),

        _qsfp28(label="53", serdes=(  40,  41,   42,   43),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x0006,
              tx_lane_map=0x2130, rx_lane_map=0x0312),
        _qsfp28(label="54", serdes=(  44,  45,   46,   47),
              tx_polarity_flip=0x0006, rx_polarity_flip=0x0006,
              tx_lane_map=0x2130, rx_lane_map=0x0312)
    ) 

# CPLD path

class DellN3248PXECPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/1-0032'

# PSUs

class DellN3248PXEPSU(cumulus.sensors.PSU_Unit):
    cpld_path = DellN3248PXECPLD.path
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "pmbus"])
            subprocess.call(["/sbin/modprobe", "pmbus"])
            self.init_done = True

class DellN3248PXEPSU1(DellN3248PXEPSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class DellN3248PXEPSU2(DellN3248PXEPSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# Intel Denverton CPU

class DellN3248PXECoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class DellN3248PXECPUTemp1(DellN3248PXECoreTemp):
    name = 'CPUTemp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class DellN3248PXECPUTemp2(DellN3248PXECoreTemp):
    name = 'CPUTemp2'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class DellN3248PXECPUTemp3(DellN3248PXECoreTemp):
    name = 'CPUTemp3'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Trident3 X5 sensors
#   alarm threshold = 100 C
#   shutdown threshold = 110 C

class DellN3248PXEASICTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/cumulus/switchd/run'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    temp_max    = 100
    temp_crit   = 110

class DellN3248PXEASICTemp1(DellN3248PXEASICTemp):
    name = 'ASICTemp1'
    driver_hwmon = ['temp1']

class DellN3248PXEASICTemp2(DellN3248PXEASICTemp):
    name = 'ASICTemp2'
    driver_hwmon = ['temp2']

class DellN3248PXEASICTemp3(DellN3248PXEASICTemp):
    name = 'ASICTemp3'
    driver_hwmon = ['temp3']

class DellN3248PXEASICTemp4(DellN3248PXEASICTemp):
    name = 'ASICTemp4'
    driver_hwmon = ['temp4']

class DellN3248PXEASICTemp5(DellN3248PXEASICTemp):
    name = 'ASICTemp5'
    driver_hwmon = ['temp5']

class DellN3248PXEASICTemp6(DellN3248PXEASICTemp):
    name = 'ASICTemp6'
    driver_hwmon = ['temp6']

class DellN3248PXEASICTemp7(DellN3248PXEASICTemp):
    name = 'ASICTemp7'
    driver_hwmon = ['temp7']

class DellN3248PXEASICTemp8(DellN3248PXEASICTemp):
    name = 'ASICTemp8'
    driver_hwmon = ['temp8']

class DellN3248PXEASICTemp9(DellN3248PXEASICTemp):
    name = 'ASICTemp9'
    driver_hwmon = ['temp9']

class DellN3248PXEASICTemp10(DellN3248PXEASICTemp):
    name = 'ASICTemp10'
    driver_hwmon = ['temp10']

# Board temp sensors

class DellN3248PXETemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    no_shutdown = True    

# U16
class DellN3248PXETemp1(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-0048'
    name = 'Temp1'
    description = 'CPU board temp sensor'
    temp_max = 70
    temp_crit = 80
    temp_fan_min = 58
    temp_fan_max = 65

# U344
class DellN3248PXETemp2(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-004f'
    name = 'Temp2'
    description = 'Fan board temp sensor'
    temp_max = 65
    temp_crit = 75
    temp_fan_min = 40
    temp_fan_max = 60

#U14
class DellN3248PXETemp3(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-004c'
    name = 'Temp3'
    description = 'Main board temp sensor 1'
    temp_max = 70
    temp_crit = 80
    temp_fan_min = 56
    temp_fan_max = 64

# U15
class DellN3248PXETemp4(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-0049'
    name = 'Temp4'
    description = 'Main board temp sensor 2'
    temp_max = 85
    temp_crit = 95
    temp_fan_min = 75
    temp_fan_max = 81

#U17
class DellN3248PXETemp5(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-004a'
    name = 'Temp5'
    description = 'Main board temp sensor 3'
    temp_max = 82
    temp_crit = 92
    temp_fan_min = 73
    temp_fan_max = 78

# U18
class DellN3248PXETemp6(DellN3248PXETemp):
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-004b'
    name = 'Temp6'
    description = 'Main board temp sensor 4'
    temp_max = 68
    temp_crit = 78
    temp_fan_min = 54
    temp_fan_max = 63

# POE temp sensors

class DellN3248PXEPOETemp(cumulus.sensors.Temp_Unit):
    driver_path = '/var/run/cumulus/poe/sensors'
    hwmon_search = False
    temp_max = 140
    temp_crit = 150

class DellN3248PXEPOETemp1(DellN3248PXEPOETemp):
    name = 'Temp7'
    description = 'POE device 1'
    driver_hwmon = ['temp1']

class DellN3248PXEPOETemp2(DellN3248PXEPOETemp):
    name = 'Temp8'
    description = 'POE device 2'
    driver_hwmon = ['temp2']

class DellN3248PXEPOETemp3(DellN3248PXEPOETemp):
    name = 'Temp9'
    description = 'POE device 3'
    driver_hwmon = ['temp3']

class DellN3248PXEPOETemp4(DellN3248PXEPOETemp):
    name = 'Temp10'
    description = 'POE device 4'
    driver_hwmon = ['temp4']

class DellN3248PXEPOETemp5(DellN3248PXEPOETemp):
    name = 'Temp11'
    description = 'POE device 5'
    driver_hwmon = ['temp5']

class DellN3248PXEPOETemp6(DellN3248PXEPOETemp):
    name = 'Temp12'
    description = 'POE device 6'
    driver_hwmon = ['temp6']

class DellN3248PXEPOETemp7(DellN3248PXEPOETemp):
    name = 'Temp13'
    description = 'POE device 7'
    driver_hwmon = ['temp7']

class DellN3248PXEPOETemp8(DellN3248PXEPOETemp):
    name = 'Temp14'
    description = 'POE device 8'
    driver_hwmon = ['temp8']

class DellN3248PXEPOETemp9(DellN3248PXEPOETemp):
    name = 'Temp15'
    description = 'POE device 9'
    driver_hwmon = ['temp9']

class DellN3248PXEPOETemp10(DellN3248PXEPOETemp):
    name = 'Temp16'
    description = 'POE device 10'
    driver_hwmon = ['temp10']

class DellN3248PXEPOETemp11(DellN3248PXEPOETemp):
    name = 'Temp17'
    description = 'POE device 11'
    driver_hwmon = ['temp11']

class DellN3248PXEPOETemp12(DellN3248PXEPOETemp):
    name = 'Temp18'
    description = 'POE device 12'
    driver_hwmon = ['temp12']

# PSU temp sensors

class DellN3248PXEPSUTemp(cumulus.sensors.Temp_Unit):
    pass
#    temp_fan_min = XXX
#    temp_max = XXX
#    temp_crit = XXX
#    temp_otp_hw = True???

class DellN3248PXEPSU1Temp1(DellN3248PXEPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-13/i2c-30/30-005e'
    driver_hwmon = ['temp1']
    psu = 'PSU1'

class DellN3248PXEPSU2Temp1(DellN3248PXEPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-13/i2c-31/31-005e'
    driver_hwmon = ['temp1']
    psu = 'PSU2'

# Fans

class DellN3248PXEFan(cumulus.sensors.Fan_Unit):
    cpld_path = DellN3248PXECPLD.path
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-15/15-002c'
    pwm_path = driver_path
    hwmon_search = False
    fan_min = 5720
    fan_max = 28600
    minpwm = 51
    maxpwm = 255

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellN3248PXEFan1(DellN3248PXEFan):
    fan_present = 'fan_tray1_present'
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_tray1']
    pwm_hwmon = ['pwm1']

class DellN3248PXEFan2(DellN3248PXEFan):
    fan_present = 'fan_tray2_present'
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_tray2']
    pwm_hwmon = ['pwm2']

class DellN3248PXEFan3(DellN3248PXEFan):
    fan_present = 'fan_tray3_present'
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_tray3']
    pwm_hwmon = ['pwm3']

# PSU fans

class DellN3248PXEPSUFan(cumulus.sensors.Fan_Unit):
    fan_min = 7950
    fan_max = 26500
    minpwm = 77
    maxpwm = 255

class DellN3248PXEPSU1Fan1(DellN3248PXEPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-13/i2c-30/30-005e'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'

class DellN3248PXEPSU2Fan1(DellN3248PXEPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = '/sys/devices/pci0000:00/0000:00:12.0/i2c-1/i2c-13/i2c-31/31-005e'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'

# PWMs

class DellN3248PXEPWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    temps = (DellN3248PXECPUTemp1(),
             DellN3248PXECPUTemp2(),
             DellN3248PXECPUTemp3(),
             DellN3248PXEASICTemp1(),
             DellN3248PXEASICTemp2(),
             DellN3248PXEASICTemp3(),
             DellN3248PXEASICTemp4(),
             DellN3248PXEASICTemp5(),
             DellN3248PXEASICTemp6(),
             DellN3248PXEASICTemp7(),
             DellN3248PXEASICTemp8(),
             DellN3248PXEASICTemp9(),
             DellN3248PXEASICTemp10(),
             DellN3248PXETemp1(),
             DellN3248PXETemp2(),
             DellN3248PXETemp3(),
             DellN3248PXETemp4(),
             DellN3248PXETemp5(),
             DellN3248PXETemp6(),
             )
    fans = (DellN3248PXEFan1(),
            DellN3248PXEFan2(),
            DellN3248PXEFan3(),
            DellN3248PXEPSU1Fan1(),
            DellN3248PXEPSU2Fan1(),
            )

# LEDs

'''
   Dell N3248PXE LEDs

   Front:
       Stacking:
           off     - not used
       System:
           green   - all systems OK
           amber   - service required
       Power:
           green   - PSU OK
           amber   - PSU failure
       Fan:
           green   - fans OK
           amber   - fan failure
       Locator:
           off     - not used

   Rear:
       FanTray1,
       FanTray2,
       Fantray3:
           green   - fans OK
           amber   - fan failure

'''
class DellN3248PXELED(cumulus.sensors.StatusLED_Unit):
    led_path = DellN3248PXECPLD.path
    good_led_color = 'green'
    bad_led_color = 'amber'

class DellN3248PXESystemLED(DellN3248PXELED):
    name = 'System LED'
    led_location = 'led_system'

class DellN3248PXEPSULED(DellN3248PXELED):
    name = 'PSU LED'
    led_location = 'led_power'
    sources = ['PSU1', 'PSU2']

class DellN3248PXEFanLED(DellN3248PXELED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3']

class DellN3248PXEFanTray1LED(DellN3248PXELED):
    name = 'Fan Tray 1 LED'
    led_location = 'led_fan_tray1'
    sources = ['Fan1']

class DellN3248PXEFanTray2LED(DellN3248PXELED):
    name = 'Fan Tray 2 LED'
    led_location = 'led_fan_tray2'
    sources = ['Fan2']

class DellN3248PXEFanTray3LED(DellN3248PXELED):
    name = 'Fan Tray 3 LED'
    led_location = 'led_fan_tray3'
    sources = ['Fan3']

class DellN3248PXESensors():
    units = (DellN3248PXEPSU1(),
             DellN3248PXEPSU2(),
             DellN3248PXECPUTemp1(),
             DellN3248PXECPUTemp2(),
             DellN3248PXECPUTemp3(),
             DellN3248PXEASICTemp1(),
             DellN3248PXEASICTemp2(),
             DellN3248PXEASICTemp3(),
             DellN3248PXEASICTemp4(),
             DellN3248PXEASICTemp5(),
             DellN3248PXEASICTemp6(),
             DellN3248PXEASICTemp7(),
             DellN3248PXEASICTemp8(),
             DellN3248PXEASICTemp9(),
             DellN3248PXEASICTemp10(),
             DellN3248PXETemp1(),
             DellN3248PXETemp2(),
             DellN3248PXETemp3(),
             DellN3248PXETemp4(),
             DellN3248PXETemp5(),
             DellN3248PXETemp6(),
             DellN3248PXEPOETemp1(),
             DellN3248PXEPOETemp2(),
             DellN3248PXEPOETemp3(),
             DellN3248PXEPOETemp4(),
             DellN3248PXEPOETemp5(),
             DellN3248PXEPOETemp6(),
             DellN3248PXEPOETemp7(),
             DellN3248PXEPOETemp8(),
             DellN3248PXEPOETemp9(),
             DellN3248PXEPOETemp10(),
             DellN3248PXEPOETemp11(),
             DellN3248PXEPOETemp12(),
             DellN3248PXEPSU1Temp1(),
             DellN3248PXEPSU2Temp1(),
             DellN3248PXEFan1(),
             DellN3248PXEFan2(),
             DellN3248PXEFan3(),
             DellN3248PXEPSU1Fan1(),
             DellN3248PXEPSU2Fan1(),
             )

class DellN3248PXEPWMs():
    units = (DellN3248PXEPWM(),
             )

class DellN3248PXELEDs():
    units = (DellN3248PXESystemLED(),
             DellN3248PXEPSULED(),
             DellN3248PXEFanLED(),
             DellN3248PXEFanTray1LED(),
             DellN3248PXEFanTray2LED(),
             DellN3248PXEFanTray3LED(),
             )

class DellN3248PXE(cumulus.platform.Platform):
    name = 'dellemc,n3248pxe_c3338'
    chip = cumulus.platform.TridentThreeX5_56771_Chip()
    switch = DellN3248PXESwitch(chip)
    cpld = DellN3248PXECPLD()
    bmc = None
    sensors = DellN3248PXESensors()
    statusleds = DellN3248PXELEDs()
    pwms = DellN3248PXEPWMs()
    portleds = None

Platform = DellN3248PXE
