#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import syslog

class DellS4128T10GT(cumulus.platform.Maverick10GPort):
    _set_led = '''
;;; DellS4128T10GT
;;
;;; Each 10G port has one tri-color LED: green, amber, and blue.
;;;
;;; We use:
;;;   green = ON with link; flashes with activity
;;;   amber = always OFF
;;;   blue  = always OFF
;;;
set_led:
        ld      b, 2
        call    set_port_led
        ret
'''

    _set_led_common = '''
set_port_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

pack_loop:
        pushst  PORTSTATUS_ZERO
        pack
        sub     b, 1
        jnz     pack_loop
        ret
'''
    def __init__(self, label, serdes, phy_addrs, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Maverick10GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=("84858",),
                                                  phy_addrs=(phy_addrs,),
                                                  phy_layer='10GBaseT',
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  unit=0,
                                                  oversub=True)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls

class DellS4128TQSFP28p(cumulus.platform.Maverick100GPort):
    _set_led = '''
;;; DellS4128TQSFP28p
;;;
;;; Each 100G port has four bi-color LEDs: green and amber
;;;
;;; In 100G mode, we use only the first LED:
;;;   green = ON with link; flashes with activity
;;;   amber = always OFF
;;; The other three LEDs are always OFF
;;;
set_led:
        ld      b, 7
        call    set_port_led
        ret
'''
    _set_led_2x = '''
;;; DellS4148QSFP28p_2x
;;;
;;;  For 2x50G mode, we use an LED for each port. Each port has
;;;  a single green/amber LED:
;;;    0: link_green   (active high)
;;;    1: link_amber  (active high)
;;;  We use the link_green led for link and activity
;;;  link_amber is always off
;;
set_led:
        ld      b, 3
        call    set_port_led

        inc     a
        ld      b, 3
        call    set_port_led

        ret
'''
    _set_led_4x = '''
;;; DellS4128TQSFP28p_4x
;;;
;;; In 4x25G mode, we use one LED for each port.
;;;   green = ON with link; flashes with activity
;;;   amber = always OFF
;;
set_led:
        ld      b, 1
        call    set_port_led

        inc     a
        ld      b, 1
        call    set_port_led

        inc     a
        ld      b, 1
        call    set_port_led

        inc     a
        ld      b, 1
        call    set_port_led

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None,
                 tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Maverick100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if ((mode in ('4x10G', '4x25G') and (sub != 0)) or
            ((mode == "2x50G") and (sub == 1 or sub == 3))):
            # For 4x modes, only update LEDs for the first sub-interface,
            # and for 2x mode, only update LEDs for the first and third
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode in ('4x10G', '4x25G'):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif mode in ('2x50G'):
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        elif mode in ('40G', '50G', '100G'):
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class DellS4128TLED(cumulus.ledup.MaverickLEDController):
    led_functions = (DellS4128T10GT._set_led_common)    
    num_leds = 50

# Dell S4128T switch ports

class DellS4128TSwitch(cumulus.platform.BCMSwitch):
    ledup = DellS4128TLED
    _10GT = DellS4128T10GT
    _qsfp28 = DellS4128TQSFP28p
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = True

    ports = (
        # TSCE0 - 0-3
        _10GT(label="1",  serdes=( 0,), phy_addrs=(0x01), leds=(0,  0), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="2",  serdes=( 1,), phy_addrs=(0x00), leds=(0,  3), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="3",  serdes=( 2,), phy_addrs=(0x03), leds=(0,  6), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="4",  serdes=( 3,), phy_addrs=(0x02), leds=(0,  9), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE1 - 4-7
        _10GT(label="5",  serdes=( 4,), phy_addrs=(0x05), leds=(0, 12), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="6",  serdes=( 5,), phy_addrs=(0x04), leds=(0, 15), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="7",  serdes=( 6,), phy_addrs=(0x07), leds=(0, 18), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="8",  serdes=( 7,), phy_addrs=(0x06), leds=(0, 21), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE2 - 8-11
        _10GT(label="9",  serdes=( 8,), phy_addrs=(0x09), leds=(0, 24), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="10", serdes=( 9,), phy_addrs=(0x08), leds=(0, 27), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="11", serdes=(10,), phy_addrs=(0x0b), leds=(0, 30), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="12", serdes=(11,), phy_addrs=(0x0a), leds=(0, 33), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE3 - 12-15
        _10GT(label="13", serdes=(12,), phy_addrs=(0x21), leds=(0, 36), tx_lane_map=0x2310, rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="14", serdes=(13,), phy_addrs=(0x20), leds=(0, 39), tx_lane_map=0x2310, rx_lane_map=0x2310, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE7 - 36-39
        _10GT(label="15", serdes=(36,), phy_addrs=(0x23), leds=(1, 39), tx_lane_map=0x2310, rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=True ),
        _10GT(label="16", serdes=(37,), phy_addrs=(0x22), leds=(1, 36), tx_lane_map=0x2310, rx_lane_map=0x2310, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCE8 - 40-43
        _10GT(label="17", serdes=(40,), phy_addrs=(0x25), leds=(1, 33), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="18", serdes=(41,), phy_addrs=(0x24), leds=(1, 30), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="19", serdes=(42,), phy_addrs=(0x27), leds=(1, 27), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="20", serdes=(43,), phy_addrs=(0x26), leds=(1, 24), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE9 - 44-47
        _10GT(label="21", serdes=(44,), phy_addrs=(0x29), leds=(1, 21), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="22", serdes=(45,), phy_addrs=(0x28), leds=(1, 18), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="23", serdes=(46,), phy_addrs=(0x2b), leds=(1, 15), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="24", serdes=(47,), phy_addrs=(0x2a), leds=(1, 12), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # Ports 25 and 26 are 100G
        # TSCF1 - 28-31
        _qsfp28(label="25", serdes=( 32, 33, 34, 35), leds=(0, 42), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x2, rx_polarity_flip=0x4),
        # TSCF2 - 32-35
        _qsfp28(label="26", serdes=( 64, 65, 66, 67), leds=(1, 42), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0xf, rx_polarity_flip=0xf),

        # TSCE10 - 40-43
        _10GT(label="27", serdes=(48,), phy_addrs=(0x41), leds=(1,  9), tx_lane_map=0x1032,rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="28", serdes=(49,), phy_addrs=(0x40), leds=(1,  6), tx_lane_map=0x1032,rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _10GT(label="29", serdes=(50,), phy_addrs=(0x43), leds=(1,  3), tx_lane_map=0x1032,rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="30", serdes=(51,), phy_addrs=(0x42), leds=(1,  0), tx_lane_map=0x1032,rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
    )

# sysfs paths

class DellS4128TCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dellemc_s4128t_cpld.0'

DellS4128T_cpld_path = '/sys/bus/platform/devices/dellemc_s4128t_cpld.0'

DellS4128T_i2c_path  = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 12 '''
DellS4128T_i2c_13_4b_path = DellS4128T_i2c_path + '13-004b'

''' tmp75: temp sensor 13 '''
DellS4128T_i2c_13_4e_path = DellS4128T_i2c_path + '13-004e'

''' tmp75: temp sensor 14 '''
DellS4128T_i2c_13_4a_path = DellS4128T_i2c_path + '13-004a'

''' tmp75: temp sensor 15 '''
DellS4128T_i2c_13_4c_path = DellS4128T_i2c_path + '13-004c'

''' tmp75: temp sensor 16 '''
DellS4128T_i2c_13_49_path = DellS4128T_i2c_path + '13-0049'

''' tmp75: temp sensor 17 '''
DellS4128T_i2c_13_4f_path = DellS4128T_i2c_path + '13-004f'

''' tmp75: temp sensor 18 '''
DellS4128T_i2c_10_4d_path = DellS4128T_i2c_path + '10-004d'

''' emc2305: fan controller 1'''
DellS4128T_i2c_11_4d_path = DellS4128T_i2c_path + '11-004d'

''' dps460: PSU1 Temp '''
DellS4128T_i2c_14_58_path = DellS4128T_i2c_path + '14-0058'

''' dps460: PSU2 Temp '''
DellS4128T_i2c_14_59_path = DellS4128T_i2c_path + '14-0059'

''' lm25066: Hot swap controller 1 '''
DellS4128T_i2c_11_40_path = DellS4128T_i2c_path + '11-0040'

''' lm25066: Hot swap controller 2 '''
DellS4128T_i2c_11_42_path = DellS4128T_i2c_path + '11-0042'

# U16 - Intel Rangeley CPU
class DellS4128TCT(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98

class DellS4128TCPUTemp1(DellS4128TCT):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DellS4128TCPUTemp2(DellS4128TCT):
    name = 'Temp2'
    driver_hwmon = ['temp3']

# Maverick - U43 Internal Temp Sensors
class DellS4128TBCMTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 100
    temp_crit    = 105
    temp_fan_min = 53
    temp_fan_max = 75

class DellS4128TBCMTemp1(DellS4128TBCMTemp):
    name = 'Temp3'
    driver_hwmon = ['temp1']

class DellS4128TBCMTemp2(DellS4128TBCMTemp):
    name = 'Temp4'
    driver_hwmon = ['temp2']

class DellS4128TBCMTemp3(DellS4128TBCMTemp):
    name = 'Temp5'
    driver_hwmon = ['temp3']

class DellS4128TBCMTemp4(DellS4128TBCMTemp):
    name = 'Temp6'
    driver_hwmon = ['temp4']

class DellS4128TBCMTemp5(DellS4128TBCMTemp):
    name = 'Temp7'
    driver_hwmon = ['temp5']

class DellS4128TBCMTemp6(DellS4128TBCMTemp):
    name = 'Temp8'
    driver_hwmon = ['temp6']

class DellS4128TBCMTemp7(DellS4128TBCMTemp):
    name = 'Temp9'
    driver_hwmon = ['temp7']

class DellS4128TBCMTemp8(DellS4128TBCMTemp):
    name = 'Temp10'
    driver_hwmon = ['temp8']

# U278 - Board Sensor Near Maverick
class DellS4128TTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp11'
    description = 'Near switch chip'
    driver_path = DellS4128T_i2c_13_4b_path
    driver_hwmon = ['temp1']

class DellS4128TTemp1F2B(DellS4128TTemp1):
    temp_max = 80
    temp_crit = 85
    temp_fan_min = 43
    temp_fan_max = 65

class DellS4128TTemp1B2F(DellS4128TTemp1):
    temp_max = 80
    temp_crit = 85
    temp_fan_min = 42
    temp_fan_max = 60

# U245 Board Sensor Close to Left of Front Vents
class DellS4128TTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp12'
    description = 'Near left of front vents'
    driver_path = DellS4128T_i2c_13_4e_path
    driver_hwmon = ['temp1']

class DellS4128TTemp2F2B(DellS4128TTemp2):
    temp_max = 65
    temp_crit = 70
# Does not control fans

class DellS4128TTemp2B2F(DellS4128TTemp2):
    temp_max = 70
    temp_crit = 75
# Does not control fans

# U244 - Board Sensor Close to Right PHY
class DellS4128TTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp13'
    description = 'Near right PHY'
    driver_path = DellS4128T_i2c_13_4a_path
    driver_hwmon = ['temp1']

class DellS4128TTemp3F2B(DellS4128TTemp3):
    temp_max = 70
    temp_crit = 75
    temp_fan_min = 40
    temp_fan_max = 60

class DellS4128TTemp3B2F(DellS4128TTemp3):
    temp_max = 80
    temp_crit = 85
    temp_fan_min = 46
    temp_fan_max = 65

# U254 - Board Sensor Close to Left PHY
class DellS4128TTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp14'
    description = 'Near left PHY'
    driver_path = DellS4128T_i2c_13_4c_path
    driver_hwmon = ['temp1']

class DellS4128TTemp4F2B(DellS4128TTemp4):
    temp_max = 81
    temp_crit = 86
    temp_fan_min = 49
    temp_fan_max = 66

class DellS4128TTemp4B2F(DellS4128TTemp4):
    temp_max = 84
    temp_crit = 88
    temp_fan_min = 54
    temp_fan_max = 71

# U288 - Board Sensor Between Maverick and QSFP
class DellS4128TTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp15'
    description = 'Between switch chip and QSFP'
    driver_path = DellS4128T_i2c_13_49_path
    driver_hwmon = ['temp1']

class DellS4128TTemp5F2B(DellS4128TTemp5):
    temp_max = 63
    temp_crit = 68
# Does not control fans

class DellS4128TTemp5B2F(DellS4128TTemp5):
    temp_max = 63
    temp_crit = 68
# Does not control fans

# U81 Board Sensor Close to Fan
class DellS4128TTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp16'
    description = "Near fan trays"
    driver_path = DellS4128T_i2c_13_4f_path
    driver_hwmon = ['temp1']

class DellS4128TTemp6F2B(DellS4128TTemp6):
    temp_max = 57
    temp_crit = 62
# Does not control fans

class DellS4128TTemp6B2F(DellS4128TTemp6):
    temp_max = 51
    temp_crit = 55
# Does not control fans

# U57 Board Sensor Near CPU
class DellS4128TTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp17'
    description = "Near CPU"
    driver_path = DellS4128T_i2c_10_4d_path
    driver_hwmon = ['temp1']

class DellS4128TTemp7F2B(DellS4128TTemp7):
    temp_max = 60
    temp_crit = 65
    temp_fan_min = 34
    temp_fan_max = 56

class DellS4128TTemp7B2F(DellS4128TTemp7):
    temp_max = 63
    temp_crit = 68
    temp_fan_min = 34
    temp_fan_max = 56

# PSU1 and PSU2 - PSU Internal Sensors
class DellS4128TPSUTemp(cumulus.sensors.Temp_Unit):
    # Copied temp_fan_min from Dell S4000.  Not sure where it came from.
    temp_fan_min = 30
    temp_max = 50
    # Dell recommends 60 C.  Increasing this limit to 65 C.
    # See Dell_s4000_c2338.py and:
    #   "Dell Mt.Baker S4128TF-ON/S4128F-ON/S4128TFE-ON/S4128TU-ON System Thermal
    #    Management Design Guide Rev 0.5"
    # for more details.  Also see CM-7318 for a related issue on Dell S4048.
    temp_crit = 65
    psu = None
    temp_otp_hw = True

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS4128TPSU1Temp1(DellS4128TPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS4128T_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'

class DellS4128TPSU2Temp1(DellS4128TPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS4128T_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'

# Fans

class DellS4128TFan(cumulus.sensors.Fan_Unit):
    fan_max = 22000
    fan_min = 11000
    maxpwm = 255
    minpwm = 128
    driver_path = DellS4128T_i2c_11_4d_path
    cpld_path = DellS4128T_cpld_path
    hwmon_search = False

class DellS4128TFan1(DellS4128TFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_hwmon = ['fan_1']

class DellS4128TFan2(DellS4128TFan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_hwmon = ['fan_2']

class DellS4128TFan3(DellS4128TFan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    cpld_hwmon = ['fan_3']

class DellS4128TFan4(DellS4128TFan):
    name = 'Fan4'
    description = 'Fan Tray 4'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_hwmon = ['fan_4']

# PSU Fans

def dell_s4128t_psu( psu ):
    ac_type = ['AC']
    dc_type = ['DC']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dellemc,s4128t_c2338':
        return None

    try:
        FNULL = open(os.devnull, "w")
        b_info = subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom','-t',psu], stderr=subprocess.STDOUT)
        FNULL.close()
    except:
        b_info = ''
    # psu_type             : AC or DC
    psu_type = ''
    for l in b_info.split('\n'):
        if 'psu_type' in l:
            psu_type = l.split()[2]
            break
    if psu_type in ac_type:
        return 'AC'
    if psu_type in dc_type:
        return 'DC'
    return 'unknown'

class DellS4128TPSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 128 # Corresponds to 50% duty / 11,500 RPM

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS4128TPSU1Fan1(DellS4128TPSUFan):
    psu_type = dell_s4128t_psu('psu1')
    if psu_type == 'DC':
        fan_max = 18000
        # Fan is controlled by PWM.
        # Set to ~15% below min (9,000 RPM) to allow for tolerance.
        fan_min = 7700
    elif psu_type == 'AC':
        fan_max = 23000
        # Fan is controlled by PWM.
        # Set to ~15% below min (11,500 RPM) to allow for tolerance.
        fan_min = 10000
    else:
        syslog.syslog(syslog.LOG_WARNING, "Unable to determine if PSU1 is AC or DC; setting PSU1 fan speeds to maximum range")
        fan_max = 23000
        fan_min = 7700

    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS4128T_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'

class DellS4128TPSU2Fan1(DellS4128TPSUFan):
    psu_type = dell_s4128t_psu('psu2')
    if psu_type == 'DC':
        fan_max = 18000
        # Fan is controlled by PWM.
        # Set to ~15% below min (9,000 RPM) to allow for tolerance.
        fan_min = 7700
    elif psu_type == 'AC':
        fan_max = 23000
        # Fan is controlled by PWM.
        # Set to ~15% below min (11,500 RPM) to allow for tolerance.
        fan_min = 10000
    else:
        syslog.syslog(syslog.LOG_WARNING, "Unable to determine if PSU2 is AC or DC; setting PSU2 fan speeds to maximum range")
        fan_max = 23000
        fan_min = 7700

    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS4128T_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'

# PSU's

class DellS4128TPSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS4128TPSU1(DellS4128TPSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_path = DellS4128T_i2c_11_40_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DellS4128TPSU2(DellS4128TPSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_path = DellS4128T_i2c_11_42_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']


def dell_s4128t_airflow():
    f2b_part_nums = ['0T6FP0']
    b2f_part_nums = ['0R5W3X']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dellemc,s4128t_c2338':
        return None

    try:
        b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3
    part_num = ''
    for l in b_info.split('\n'):
        if 'Part Number' in l:
            part_num = l.split()[4]
            break
    if part_num in b2f_part_nums:
        return 'B2F'
    if part_num in f2b_part_nums:
        return 'F2B'
    return None

class DellS4128TPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    fans = (DellS4128TFan1(),
            DellS4128TFan2(),
            DellS4128TFan3(),
            DellS4128TFan4(),
            DellS4128TPSU1Fan1(),
            DellS4128TPSU2Fan1(),
            )

    temps = (DellS4128TCPUTemp1(),
             DellS4128TCPUTemp2(),
             DellS4128TBCMTemp1(),
             DellS4128TBCMTemp2(),
             DellS4128TBCMTemp3(),
             DellS4128TBCMTemp4(),
             DellS4128TBCMTemp5(),
             DellS4128TBCMTemp6(),
             DellS4128TBCMTemp7(),
             DellS4128TBCMTemp8(),
             DellS4128TPSU1Temp1(),
             DellS4128TPSU2Temp1(),
             )
    f2b_temps = (DellS4128TTemp1F2B(),
                 DellS4128TTemp2F2B(),
                 DellS4128TTemp3F2B(),
                 DellS4128TTemp4F2B(),
                 DellS4128TTemp5F2B(),
                 DellS4128TTemp6F2B(),
                 DellS4128TTemp7F2B(),
                )
    b2f_temps = (DellS4128TTemp1B2F(),
                 DellS4128TTemp2B2F(),
                 DellS4128TTemp3B2F(),
                 DellS4128TTemp4B2F(),
                 DellS4128TTemp5B2F(),
                 DellS4128TTemp6B2F(),
                 DellS4128TTemp7B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4128t_airflow() == 'F2B':
        temps = temps + f2b_temps
    else:
        temps = temps + b2f_temps

class DellS4128TSensors():
    units = (DellS4128TPSU1(),
             DellS4128TPSU2(),
             DellS4128TCPUTemp1(),
             DellS4128TCPUTemp2(),
             DellS4128TBCMTemp1(),
             DellS4128TBCMTemp2(),
             DellS4128TBCMTemp3(),
             DellS4128TBCMTemp4(),
             DellS4128TBCMTemp5(),
             DellS4128TBCMTemp6(),
             DellS4128TBCMTemp7(),
             DellS4128TBCMTemp8(),
             DellS4128TPSU1Temp1(),
             DellS4128TPSU2Temp1(),
             DellS4128TFan1(),
             DellS4128TFan2(),
             DellS4128TFan3(),
             DellS4128TFan4(),
             DellS4128TPSU1Fan1(),
             DellS4128TPSU2Fan1(),
             )
    f2b_temps = (DellS4128TTemp1F2B(),
                 DellS4128TTemp2F2B(),
                 DellS4128TTemp3F2B(),
                 DellS4128TTemp4F2B(),
                 DellS4128TTemp5F2B(),
                 DellS4128TTemp6F2B(),
                 DellS4128TTemp7F2B(),
                )

    b2f_temps = (DellS4128TTemp1B2F(),
                 DellS4128TTemp2B2F(),
                 DellS4128TTemp3B2F(),
                 DellS4128TTemp4B2F(),
                 DellS4128TTemp5B2F(),
                 DellS4128TTemp6B2F(),
                 DellS4128TTemp7B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4128t_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps

class DellS4128TPWMs():
    units = (DellS4128TPWM1(),
             )

# LEDs

'''
   Dell S4128T LEDS
     System:
       green - all systems ok
       amber blinking - fan missing or broken, psu missing or broken
     Power:
       green - both PSUs installed and working
       amber blinking - PSU missing or broken
     Fan:
       green - all four fans installed and working
       amber blinking - fan missing or malfunction
     Fan Tray 1-4:
       green - fan is installed and working
       amber - fan is missing or malfunction
     Locator:
       off - normal operation
       blue blinking - beacon
'''

class DellS4128TSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_system'
    bad_led_color = 'amber_blinking'

class DellS4128TPowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'amber_blinking'

class DellS4128TFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan LED'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = 'led_fan'
    bad_led_color = 'amber_blinking'

class DellS4128TFanTray1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1 LED'
    sources = ['Fan1']
    led_location = 'led_fan_tray1'
    bad_led_color = 'amber'

class DellS4128TFanTray2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2 LED'
    sources = ['Fan2']
    led_location = 'led_fan_tray2'
    bad_led_color = 'amber'

class DellS4128TFanTray3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3 LED'
    sources = ['Fan3']
    led_location = 'led_fan_tray3'
    bad_led_color = 'amber'

class DellS4128TFanTray4LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 4 LED'
    sources = ['Fan4']
    led_location = 'led_fan_tray4'
    bad_led_color = 'amber'

class DellS4128TLEDs():
    units = (
             DellS4128TSystemLED(),
             DellS4128TPowerLED(),
             DellS4128TFanLED(),
             DellS4128TFanTray1LED(),
             DellS4128TFanTray2LED(),
             DellS4128TFanTray3LED(),
             DellS4128TFanTray4LED(),
        )

# Dell S4128T class

class DellS4128T(cumulus.platform.Platform):
    name = 'dellemc,s4128t_c2338'
    chip = cumulus.platform.MaverickChip()
    switch = DellS4128TSwitch(chip)
    cpld = DellS4128TCPLD()
    pwms = DellS4128TPWMs()
    sensors = DellS4128TSensors()
    statusleds = DellS4128TLEDs()

Platform = DellS4128T
