#
# Copyright (c) 2017 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS4148SFPp(cumulus.platform.Maverick10GPort):
    _set_led = '''
;;; DellS4148SFPp
;;
;;; Each 10G port has two LEDs. One for link and one for activity.
;;; The link LED is dual-color green/amber
;;; The activity LED is just green
;;; Three bits to pack all active high:
;;;   0: link_green
;;;   1: link_amber
;;;   2: activity_green
;;; We use:
;;;   link_green on with link
;;;   link_amber always off
;;;   activity_green flashes on with activity
;;;
set_led:
        call    set_sfp_led
        ret
'''

    _set_led_common = '''
set_sfp_led:
        port a

        pushst  PORTSTATUS_LINKEN   ; link_green
        pack                        ;  - on with link

        pushst  PORTSTATUS_ZERO     ; link_amber
        pack                        ;  - off

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand                        ; activity_green
        pack                        ;  - flash on with activity

        ret

set_qsfp_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ld      a, 7
qsfp_pack_loop:
        port 1
        pushst  PORTSTATUS_ZERO
        pack
        sub     a, 1
        jnz     qsfp_pack_loop

        ret

set_qsfp_2x_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO ; link_amber off
        pack
        pushst  PORTSTATUS_ZERO ; next two LEDs are off
        pack
        pushst  PORTSTATUS_ZERO
        pack

        ret

set_qsfp_4x_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO ; link_amber off
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Maverick10GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer='SFP+',
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls

class DellS4148QSFPp(cumulus.platform.Maverick40GPort):
    _set_led = '''
;;; DellS4148QSFPp
;;;
;;; QSFP+ ports have 4 green/amber LEDs, two bits per LED.
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;; We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        call    set_qsfp_led
        ret
'''
    _set_led_4x = '''
;;; DellS4148QSFPp_4x
;;;
;;;  For 4x10G modes, we use an LED for each port. Each port has
;;;  a single green/amber LED:
;;;    0: link_green   (active high)
;;;    1: link_amber  (active high)
;;;  We use the link_green led for link and activity
;;;  link_amber is always off
;;
set_led:
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.Maverick40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_lane_map=tx_lane_map,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0,
                                                 oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode == "4x10G":
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class DellS4148QSFP28p(cumulus.platform.Maverick100GPort):
    _set_led = '''
;;; DellS4148QSFP28p
;;;
;;; QSFP28 ports have 4 green/amber LEDs, two bits per LED.
;;; For 100G, 50G, and 40G mode, we use only the first LED to indicate
;;; link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;; We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        call    set_qsfp_led
        ret
'''
    _set_led_2x = '''
;;; DellS4148QSFP28p_2x
;;;
;;;  For 2x50G mode, we use an LED for each port. Each port has
;;;  a single green/amber LED:
;;;    0: link_green   (active high)
;;;    1: link_amber  (active high)
;;;  We use the link_green led for link and activity
;;;  link_amber is always off
;;
set_led:
        call    set_qsfp_2x_led
        inc     a
        call    set_qsfp_2x_led
        ret
'''
    _set_led_4x = '''
;;; DellS4148QSFP28p_4x
;;;
;;;  For 4x25G and 4x10G mode, we use an LED for each port. Each port has
;;;  a single green/amber LED:
;;;    0: link_green   (active high)
;;;    1: link_amber  (active high)
;;;  We use the link_green led for link and activity
;;;  link_amber is always off
;;
set_led:
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led
        inc     a
        call    set_qsfp_4x_led
        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None,
                 tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Maverick100GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_lane_map=tx_lane_map,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0,
                                                 oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if ((mode in ('4x10G', '4x25G') and (sub != 0)) or
            ((mode == "2x50G") and (sub == 1 or sub == 3))):
            # For 4x modes, only update LEDs for the first sub-interface,
            # and for 2x mode, only update LEDs for the first and third
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode in ('4x10G', '4x25G'):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif mode in ('2x50G'):
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        elif mode in ('40G', '50G', '100G'):
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class DellS4148LED(cumulus.ledup.MaverickLEDController):
    led_functions = (DellS4148SFPp._set_led_common)
    num_leds = 96

# Dell S4148 switch ports

class DellS4148Switch(cumulus.platform.BCMSwitch):
    ledup = DellS4148LED
    _sfp = DellS4148SFPp
    _qsfp = DellS4148QSFPp
    _qsfp28 = DellS4148QSFP28p
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # TSCE0 - 0-3
        _sfp(label="1",  serdes=( 0,), leds=(0,  0), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="2",  serdes=( 1,), leds=(0,  3), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="3",  serdes=( 2,), leds=(0,  6), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="4",  serdes=( 3,), leds=(0,  9), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE1 - 4-7
        _sfp(label="5",  serdes=( 4,), leds=(0, 12), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="6",  serdes=( 5,), leds=(0, 15), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="7",  serdes=( 6,), leds=(0, 18), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="8",  serdes=( 7,), leds=(0, 21), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE2 - 8-11
        _sfp(label="9",  serdes=( 8,), leds=(0, 24), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="10", serdes=( 9,), leds=(0, 27), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="11", serdes=(10,), leds=(0, 30), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="12", serdes=(11,), leds=(0, 33), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE3 - 12-15
        _sfp(label="13", serdes=(12,), leds=(0, 36), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="14", serdes=(13,), leds=(0, 39), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="15", serdes=(14,), leds=(0, 42), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="16", serdes=(15,), leds=(0, 45), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE4 - 16-19
        _sfp(label="17", serdes=(16,), leds=(0, 48), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="18", serdes=(17,), leds=(0, 51), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="19", serdes=(18,), leds=(0, 54), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="20", serdes=(19,), leds=(0, 57), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCE5 - 20-23
        _sfp(label="21", serdes=(20,), leds=(0, 60), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="22", serdes=(21,), leds=(0, 63), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="23", serdes=(22,), leds=(0, 66), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="24", serdes=(23,), leds=(0, 69), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # Ports 25 and 26 are 100G
        # TSCF0 - 28-31
        _qsfp28(label="25", serdes=( 28, 29, 30, 31), leds=(0, 72), tx_lane_map=0x2031, rx_lane_map=0x1023, tx_polarity_flip=0xf, rx_polarity_flip=0xf),
        # TSCF1 - 32-35
        _qsfp28(label="26", serdes=( 32, 33, 34, 35), leds=(0, 80), tx_lane_map=0x0312, rx_lane_map=0x3210, tx_polarity_flip=0xe, rx_polarity_flip=0xe),

        # Ports 27 and 28 are 40G
        # TSCE6 - 24-27
        _qsfp(label="27", serdes=( 24, 25, 26, 27), leds=(0, 88), tx_lane_map=0x2031, rx_lane_map=0x2310, tx_polarity_flip=0xf, rx_polarity_flip=0xf),
        # TSCE7 - 36-40
        _qsfp(label="28", serdes=( 36, 37, 38, 39), leds=(1, 72), tx_lane_map=0x1320, rx_lane_map=0x0132, tx_polarity_flip=0x0, rx_polarity_flip=0xd),

        # Ports 29 and 30 are 100G
        # TSCF2 - 64-67
        _qsfp28(label="29", serdes=( 64, 65, 66, 67), leds=(1, 80), tx_lane_map=0x2031, rx_lane_map=0x1023, tx_polarity_flip=0xf, rx_polarity_flip=0x0),
        # TSCF3 - 68-71
        _qsfp28(label="30", serdes=( 68, 69, 70, 71), leds=(1, 88), tx_lane_map=0x0213, rx_lane_map=0x3210, tx_polarity_flip=0xe, rx_polarity_flip=0x1),

        # TSCE8 - 40-43
        _sfp(label="31", serdes=(40,), leds=(1,  0), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="32", serdes=(41,), leds=(1,  3), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="33", serdes=(42,), leds=(1,  6), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="34", serdes=(43,), leds=(1,  9), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE9 - 44-47
        _sfp(label="35", serdes=(44,), leds=(1, 12), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="36", serdes=(45,), leds=(1, 15), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="37", serdes=(46,), leds=(1, 18), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="38", serdes=(47,), leds=(1, 21), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE10 - 48-51
        _sfp(label="39", serdes=(48,), leds=(1, 24), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="40", serdes=(49,), leds=(1, 27), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="41", serdes=(50,), leds=(1, 30), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="42", serdes=(51,), leds=(1, 33), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE11 - 52-55
        _sfp(label="43", serdes=(52,), leds=(1, 36), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="44", serdes=(53,), leds=(1, 39), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="45", serdes=(54,), leds=(1, 42), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="46", serdes=(55,), leds=(1, 45), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE12 - 56-59
        _sfp(label="47", serdes=(56,), leds=(1, 48), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="48", serdes=(57,), leds=(1, 51), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="49", serdes=(58,), leds=(1, 54), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="50", serdes=(59,), leds=(1, 57), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE13 - 60-63
        _sfp(label="51", serdes=(60,), leds=(1, 60), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="52", serdes=(61,), leds=(1, 63), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="53", serdes=(62,), leds=(1, 66), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp(label="54", serdes=(63,), leds=(1, 69), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
    )

# sysfs paths

class DellS4148CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dellemc_s4148_cpld.0'

DellS4148_cpld_path = '/sys/bus/platform/devices/dellemc_s4148_cpld.0'

DellS4148_i2c_path  = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 12 '''
DellS4148_i2c_13_4b_path = DellS4148_i2c_path + '13-004b'

''' tmp75: temp sensor 13 '''
DellS4148_i2c_13_4a_path = DellS4148_i2c_path + '13-004a'

''' tmp75: temp sensor 14 '''
DellS4148_i2c_13_4c_path = DellS4148_i2c_path + '13-004c'

''' tmp75: temp sensor 15 '''
DellS4148_i2c_13_4e_path = DellS4148_i2c_path + '13-004e'

''' tmp75: temp sensor 16 '''
DellS4148_i2c_13_4f_path = DellS4148_i2c_path + '13-004f'

''' tmp75: temp sensor 17 '''
DellS4148_i2c_10_4d_path = DellS4148_i2c_path + '10-004d'

''' emc2305: fan controller'''
DellS4148_i2c_11_4d_path = DellS4148_i2c_path + '11-004d'

''' dps460: PSU1 Temp '''
DellS4148_i2c_14_58_path = DellS4148_i2c_path + '14-0058'

''' dps460: PSU2 Temp '''
DellS4148_i2c_14_59_path = DellS4148_i2c_path + '14-0059'

''' lm25066: Hot swap controller 1 '''
DellS4148_i2c_11_40_path = DellS4148_i2c_path + '11-0040'

''' lm25066: Hot swap controller 2 '''
DellS4148_i2c_11_42_path = DellS4148_i2c_path + '11-0042'

# U16 - Intel Rangeley CPU
class DellS4148CPUTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98

class DellS4148CPUTemp1(DellS4148CPUTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DellS4148CPUTemp2(DellS4148CPUTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

# U12 - Maverick Internal Temp Sensors
class DellS4148BCMTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 100
    temp_crit    = 105
    temp_fan_min = 42
    temp_fan_max = 69

class DellS4148BCMTemp1(DellS4148BCMTemp):
    name = 'Temp3'
    driver_hwmon = ['temp1']

class DellS4148BCMTemp2(DellS4148BCMTemp):
    name = 'Temp4'
    driver_hwmon = ['temp2']

class DellS4148BCMTemp3(DellS4148BCMTemp):
    name = 'Temp5'
    driver_hwmon = ['temp3']

class DellS4148BCMTemp4(DellS4148BCMTemp):
    name = 'Temp6'
    driver_hwmon = ['temp4']

class DellS4148BCMTemp5(DellS4148BCMTemp):
    name = 'Temp7'
    driver_hwmon = ['temp5']

class DellS4148BCMTemp6(DellS4148BCMTemp):
    name = 'Temp8'
    driver_hwmon = ['temp6']

class DellS4148BCMTemp7(DellS4148BCMTemp):
    name = 'Temp9'
    driver_hwmon = ['temp7']

class DellS4148BCMTemp8(DellS4148BCMTemp):
    name = 'Temp10'
    driver_hwmon = ['temp8']

# U359 - Board Sensor Near Left of Front Vents
class DellS4148Temp1(cumulus.sensors.Temp_Unit):
    name = 'Temp11'
    description = 'Near left of front vents'
    driver_path = DellS4148_i2c_13_4b_path
    driver_hwmon = ['temp1']

class DellS4148Temp1F2B(DellS4148Temp1):
    temp_max = 59
    temp_crit = 63
# Does not control fans

class DellS4148Temp1B2F(DellS4148Temp1):
    temp_max = 62
    temp_crit = 66
# Does not control fans

# U360 Board Sensor Near Maverick
class DellS4148Temp2(cumulus.sensors.Temp_Unit):
    name = 'Temp12'
    description = 'Near switch chip'
    driver_path = DellS4148_i2c_13_4a_path
    driver_hwmon = ['temp1']

class DellS4148Temp2F2B(DellS4148Temp2):
    temp_max = 62
    temp_crit = 66
    temp_fan_min = 30
    temp_fan_max = 56

class DellS4148Temp2B2F(DellS4148Temp2):
    temp_max = 62
    temp_crit = 79
    temp_fan_min = 34
    temp_fan_max = 56

# U54 - Board Sensor Near Right of Front Vents
class DellS4148Temp3(cumulus.sensors.Temp_Unit):
    name = 'Temp13'
    description = 'Near right of front vents'
    driver_path = DellS4148_i2c_13_4c_path
    driver_hwmon = ['temp1']

class DellS4148Temp3F2B(DellS4148Temp3):
    temp_max = 57
    temp_crit = 61
# Does not control fans

class DellS4148Temp3B2F(DellS4148Temp3):
    temp_max = 61
    temp_crit = 65
# Does not control fans

# U245 - Board Sensor Near QSFP
class DellS4148Temp4(cumulus.sensors.Temp_Unit):
    name = 'Temp14'
    description = 'Near QSFP'
    driver_path = DellS4148_i2c_13_4e_path
    driver_hwmon = ['temp1']

class DellS4148Temp4F2B(DellS4148Temp4):
    temp_max = 60
    temp_crit = 64
    temp_fan_min = 29
    temp_fan_max = 55

class DellS4148Temp4B2F(DellS4148Temp4):
    temp_max = 64
    temp_crit = 68
    temp_fan_min = 32
    temp_fan_max = 54

# U334 - Board Sensor Near DC Fan
class DellS4148Temp5(cumulus.sensors.Temp_Unit):
    name = 'Temp15'
    description = 'Near fan trays'
    driver_path = DellS4148_i2c_13_4f_path
    driver_hwmon = ['temp1']

class DellS4148Temp5F2B(DellS4148Temp5):
    temp_max = 58
    temp_crit = 62
# Does not control fans

class DellS4148Temp5B2F(DellS4148Temp5):
    temp_max = 58
    temp_crit = 62
# Does not control fans

# U57 Board Sensor Near CPU
class DellS4148Temp6(cumulus.sensors.Temp_Unit):
    name = 'Temp16'
    description = "Near CPU"
    driver_path = DellS4148_i2c_10_4d_path
    driver_hwmon = ['temp1']

class DellS4148Temp6F2B(DellS4148Temp6):
    temp_max = 61
    temp_crit = 66
    temp_fan_min = 30
    temp_fan_max = 56

class DellS4148Temp6B2F(DellS4148Temp6):
    temp_max = 59
    temp_crit = 63
    temp_fan_min = 32
    temp_fan_max = 55

# PSU1 and PSU2 - PSU Internal Sensors
class DellS4148PSUTemp(cumulus.sensors.Temp_Unit):
    # Copied temp_fan_min from Dell S4000.  Not sure where it came from.
    temp_fan_min = 30
    temp_max = 55
    # Dell recommends 60 C.  Increasing this limit to 65 C.
    # See Dell_s4000_c2338.py and:
    #   "Dell Mt.Baker S4148F-ON/S4128F-ON/S4148FE-ON/S4148U-ON System Thermal
    #    Management Design Guide Rev 0.5"
    # for more details.  Also see CM-7318 for a related issue on Dell S4048.
    temp_crit = 65
    psu = None
    temp_otp_hw = True

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS4148PSU1Temp1(DellS4148PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS4148_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'

class DellS4148PSU2Temp1(DellS4148PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS4148_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'

# Fans

class DellS4148Fan(cumulus.sensors.Fan_Unit):
    fan_max = 22000
    fan_min = 11000
    minpwm = 128
    maxpwm = 255
    driver_path = DellS4148_i2c_11_4d_path
    cpld_path = DellS4148_cpld_path
    hwmon_search = False

class DellS4148Fan1(DellS4148Fan):
    name = 'Fan1'
    description = 'Fan 1'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_hwmon = ['fan_1']

class DellS4148Fan2(DellS4148Fan):
    name = 'Fan2'
    description = 'Fan 2'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_hwmon = ['fan_2']

class DellS4148Fan3(DellS4148Fan):
    name = 'Fan3'
    description = 'Fan 3'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    cpld_hwmon = ['fan_3']

class DellS4148Fan4(DellS4148Fan):
    name = 'Fan4'
    description = 'Fan 4'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_hwmon = ['fan_4']

# PSU Fans

def dell_s4148_psu( psu ):
    ac_type = ['AC']
    dc_type = ['DC']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dellemc,s4148_c2338':
        return None

    try:
        FNULL = open(os.devnull, "w")
        b_info = subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom','-t',psu], stderr=subprocess.STDOUT)
        FNULL.close()
    except:
        b_info = ''
    # psu_type             : AC or DC
    psu_type = ''
    for l in b_info.split('\n'):
        if 'psu_type' in l:
            psu_type = l.split()[2]
            break
    if psu_type in ac_type:
        return 'AC'
    if psu_type in dc_type:
        return 'DC'
    return None

class DellS4148PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 128 # Corresponds to 50% duty / 11,500 RPM
    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS4148PSU1Fan1(DellS4148PSUFan):
    # Default to AC if PSU type can't be figured out.
    if dell_s4148_psu('psu1') == 'DC':
        fan_max = 18000
        # Fan is controlled by PWM.
        # Set to ~15% below min (9,000 RPM) to allow for tolerance.
        fan_min = 7700
    else:
        fan_max = 23000
        # Fan is controlled by PWM.
        # Set to ~15% below min (11,500 RPM) to allow for tolerance.
        fan_min = 10000

    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS4148_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'

class DellS4148PSU2Fan1(DellS4148PSUFan):
    # Default to AC if PSU type can't be figured out.
    if dell_s4148_psu('psu2') == 'DC':
        fan_max = 18000
        # Fan is controlled by PWM.
        # Set to ~15% below min (9,000 RPM) to allow for tolerance.
        fan_min = 7700
    else:
        fan_max = 23000
        # Fan is controlled by PWM.
        # Set to ~15% below min (11,500 RPM) to allow for tolerance.
        fan_min = 10000

    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS4148_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'

# PSU's

class DellS4148PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS4148PSU1(DellS4148PSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_path = DellS4148_i2c_11_40_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']

class DellS4148PSU2(DellS4148PSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_path = DellS4148_i2c_11_42_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']


def dell_s4148_airflow():
    f2b_part_nums = ['0264Y6', '09H9MN', '0WT75M']
    b2f_part_nums = ['0401J0', '0R2RKC', '04CGCX']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dellemc,s4148_c2338':
        return None

    try:
        b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3
    part_num = ''
    for l in b_info.split('\n'):
        if 'Part Number' in l:
            part_num = l.split()[4]
            break
    if part_num in b2f_part_nums:
        return 'B2F'
    if part_num in f2b_part_nums:
        return 'F2B'
    return None

class DellS4148PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    fans = (DellS4148Fan1(),
            DellS4148Fan2(),
            DellS4148Fan3(),
            DellS4148Fan4(),
            DellS4148PSU1Fan1(),
            DellS4148PSU2Fan1(),
            )

    temps = (DellS4148CPUTemp1(),
             DellS4148CPUTemp2(),
             DellS4148BCMTemp1(),
             DellS4148BCMTemp2(),
             DellS4148BCMTemp3(),
             DellS4148BCMTemp4(),
             DellS4148BCMTemp5(),
             DellS4148BCMTemp6(),
             DellS4148BCMTemp7(),
             DellS4148BCMTemp8(),
             DellS4148PSU1Temp1(),
             DellS4148PSU2Temp1(),
             )
    f2b_temps = (DellS4148Temp1F2B(),
                 DellS4148Temp2F2B(),
                 DellS4148Temp3F2B(),
                 DellS4148Temp4F2B(),
                 DellS4148Temp5F2B(),
                 DellS4148Temp6F2B(),
                )
    b2f_temps = (DellS4148Temp1B2F(),
                 DellS4148Temp2B2F(),
                 DellS4148Temp3B2F(),
                 DellS4148Temp4B2F(),
                 DellS4148Temp5B2F(),
                 DellS4148Temp6B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4148_airflow() == 'F2B':
        temps = temps + f2b_temps
    else:
        temps = temps + b2f_temps

class DellS4148Sensors():
    units = (DellS4148PSU1(),
             DellS4148PSU2(),
             DellS4148CPUTemp1(),
             DellS4148CPUTemp2(),
             DellS4148BCMTemp1(),
             DellS4148BCMTemp2(),
             DellS4148BCMTemp3(),
             DellS4148BCMTemp4(),
             DellS4148BCMTemp5(),
             DellS4148BCMTemp6(),
             DellS4148BCMTemp7(),
             DellS4148BCMTemp8(),
             DellS4148PSU1Temp1(),
             DellS4148PSU2Temp1(),
             DellS4148Fan1(),
             DellS4148Fan2(),
             DellS4148Fan3(),
             DellS4148Fan4(),
             DellS4148PSU1Fan1(),
             DellS4148PSU2Fan1(),
             )
    f2b_temps = (DellS4148Temp1F2B(),
                 DellS4148Temp2F2B(),
                 DellS4148Temp3F2B(),
                 DellS4148Temp4F2B(),
                 DellS4148Temp5F2B(),
                 DellS4148Temp6F2B(),
                 )

    b2f_temps = (DellS4148Temp1B2F(),
                 DellS4148Temp2B2F(),
                 DellS4148Temp3B2F(),
                 DellS4148Temp4B2F(),
                 DellS4148Temp5B2F(),
                 DellS4148Temp6B2F(),
                 )
    # Default to B2F if airflow cant be figured out.
    if dell_s4148_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps

class DellS4148PWMs():
    units = (DellS4148PWM1(),
             )

# LEDs

'''
   Dell S4148 LEDS
     System:
       green - all systems ok
       amber blinking - fan missing or broken, psu missing or broken
     Power:
       green - both PSUs installed and working
       amber blinking - PSU missing or broken
     Fan:
       green - all four fans installed and working
       amber blinking - fan missing or malfunction
     Fan Tray 1-4:
       green - fan is installed and working
       amber - fan is missing or malfunction
     Locator: (not using this led)
       off - normal operation
       blue blinking - beacon
'''

class DellS4148SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_system'
    bad_led_color = 'amber_blinking'

class DellS4148PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'amber_blinking'

class DellS4148FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan LED'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    led_location = 'led_fan'
    bad_led_color = 'amber_blinking'

class DellS4148FanTray1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1 LED'
    sources = ['Fan1']
    led_location = 'led_fan_tray1'
    bad_led_color = 'amber'

class DellS4148FanTray2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2 LED'
    sources = ['Fan2']
    led_location = 'led_fan_tray2'
    bad_led_color = 'amber'

class DellS4148FanTray3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3 LED'
    sources = ['Fan3']
    led_location = 'led_fan_tray3'
    bad_led_color = 'amber'

class DellS4148FanTray4LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 4 LED'
    sources = ['Fan4']
    led_location = 'led_fan_tray4'
    bad_led_color = 'amber'

class DellS4148LEDs():
    units = (
             DellS4148SystemLED(),
             DellS4148PowerLED(),
             DellS4148FanLED(),
             DellS4148FanTray1LED(),
             DellS4148FanTray2LED(),
             DellS4148FanTray3LED(),
             DellS4148FanTray4LED(),
        )

# Dell S4148 class

class DellS4148(cumulus.platform.Platform):
    name = 'dellemc,s4148_c2338'
    chip = cumulus.platform.MaverickChip()
    switch = DellS4148Switch(chip)
    cpld = DellS4148CPLD()
    pwms = DellS4148PWMs()
    sensors = DellS4148Sensors()
    statusleds = DellS4148LEDs()

Platform = DellS4148
