#
# Copyright (c) 2020 Cumulus Networks, Inc.  All rights reserved.
#

import os
import sys
import string
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors


class DellZ9264FQSFP28(cumulus.platform.TomahawkTwo100GPort):
    '''
        Each port has eight bits of LED data controlling four 
	bi-color green/amber
        LEDs.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_port_led:
; pack two bits
;  10 => green
        port  a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                       ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
	pack
	pushst PORTSTATUS_ZERO
	pack
	ret

set_led_off:
;  00 => off
	pushst  PORTSTATUS_ZERO
	pack
	pushst  PORTSTATUS_ZERO
	pack
	ret

'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        call    set_led_off
        call    set_led_off
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and third bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_led_off
        inc     a
        call    set_port_led
        inc     a
        call    set_led_off
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, adj_label, serdes, leds, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None, blocked_adj_split=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TomahawkTwo100GPort.__init__(self,
                                                   label=label,
                                                   adj_label=adj_label,
                                                   serdes=serdes,
                                                   leds=leds,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   blocked_adj_split=blocked_adj_split,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class DellZ9264FLED(cumulus.ledup.TomahawkLEDController):
    led_functions = (DellZ9264FQSFP28._set_led_common)
    num_leds = 128
    default_pad = 0
    has_phys_id = True


# Dell Z9264F switch ports
class DellZ9264FSwitch(cumulus.platform.BCMSwitch):
    ledup = DellZ9264FLED
    led_controllers = set([0, 1, 2, 3])
    _qsfp28 = DellZ9264FQSFP28
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        _qsfp28(label="1", adj_label=None, serdes=(48, 49,  50,  51), leds=(0, 0), tx_lane_map=0x2130, rx_lane_map=0x3201, tx_polarity_flip=0x000a, rx_polarity_flip=0x0007),
        _qsfp28(label="2", adj_label="1", serdes=(52,  53,  54,  55), leds=(0, 8),tx_lane_map=0x3012, rx_lane_map=0x0132, tx_polarity_flip=0x000c, rx_polarity_flip=0x0009, blocked_adj_split=4),
        _qsfp28(label="3", adj_label=None, serdes=(64,  65,  66,  67), leds=(1, 0),tx_lane_map=0x2130, rx_lane_map=0x2103, tx_polarity_flip=0x0003, rx_polarity_flip=0x0008),
        _qsfp28(label="4", adj_label="3", serdes=(68,  69,  70,  71), leds=(1, 8),tx_lane_map=0x3012, rx_lane_map=0x3210, tx_polarity_flip=0x000d, rx_polarity_flip=0x0004, blocked_adj_split=4),

        _qsfp28(label="5", adj_label=None, serdes=(80,  81,  82,  83), leds=(1, 16),tx_lane_map=0x2130, rx_lane_map=0x2103, tx_polarity_flip=0x0003, rx_polarity_flip=0x0008),
        _qsfp28(label="6", adj_label="5", serdes=(84,  85,  86,  87), leds=(1, 24),tx_lane_map=0x1032, rx_lane_map=0x3210, tx_polarity_flip=0x0003, rx_polarity_flip=0x0004, blocked_adj_split=4),
        _qsfp28(label="7", adj_label=None, serdes=(96,  97,  98,  99), leds=(1, 32),tx_lane_map=0x0123, rx_lane_map=0x1023, tx_polarity_flip=0x0008, rx_polarity_flip=0x000a),
        _qsfp28(label="8", adj_label="7", serdes=(100, 101, 102, 103), leds=(1, 40),tx_lane_map=0x1320, rx_lane_map=0x3120, tx_polarity_flip=0x0005, rx_polarity_flip=0x000c, blocked_adj_split=4),

        _qsfp28(label="9", adj_label=None, serdes=(0, 1, 2, 3), leds=(0, 16),tx_lane_map=0x3120, rx_lane_map=0x1320, tx_polarity_flip=0x000c, rx_polarity_flip=0x000a),
        _qsfp28(label="10", adj_label="9", serdes=(4, 5, 6, 7), leds=(0, 24),tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=0x000d, rx_polarity_flip=0x0001, blocked_adj_split=4),
        _qsfp28(label="11", adj_label=None, serdes=(16, 17, 18, 19), leds=(0, 32),tx_lane_map=0x3201, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0006),
        _qsfp28(label="12", adj_label="11", serdes=(20, 21, 22, 23), leds=(0, 40),tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=0x000a, rx_polarity_flip=0x0005, blocked_adj_split=4),

        _qsfp28(label="13", adj_label=None, serdes=(32,  33,  34,  35), leds=(0, 48),tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c),
        _qsfp28(label="14", adj_label="13", serdes=(36,  37,  38,  39), leds=(0, 56),tx_lane_map=0x3120, rx_lane_map=0x0132, tx_polarity_flip=0x000a, rx_polarity_flip=0x0009, blocked_adj_split=4),
        _qsfp28(label="15", adj_label=None, serdes=(112, 113, 114, 115), leds=(1, 48),tx_lane_map=0x0123, rx_lane_map=0x1320, tx_polarity_flip=0x0001, rx_polarity_flip=0x0006),
        _qsfp28(label="16", adj_label="15", serdes=(116, 117, 118, 119), leds=(1, 56),tx_lane_map=0x1023, rx_lane_map=0x3102, tx_polarity_flip=0x0002, rx_polarity_flip=0x0005, blocked_adj_split=4),

        _qsfp28(label="17", adj_label=None, serdes=(132, 133, 134, 135), leds=(2, 0),tx_lane_map=0x3120, rx_lane_map=0x0312, tx_polarity_flip=0x000c, rx_polarity_flip=0x0006),
        _qsfp28(label="18", adj_label="17", serdes=(128, 129, 130, 131), leds=(2, 8),tx_lane_map=0x0312, rx_lane_map=0x0321, tx_polarity_flip=0x000d, rx_polarity_flip=0x0004, blocked_adj_split=4),
        _qsfp28(label="19", adj_label=None, serdes=(212, 213, 214, 215), leds=(3, 0),tx_lane_map=0x1203, rx_lane_map=0x2031, tx_polarity_flip=0x0001, rx_polarity_flip=0x0006),
        _qsfp28(label="20", adj_label="19", serdes=(208, 209, 210, 211), leds=(3, 8),tx_lane_map=0x1023, rx_lane_map=0x3201, tx_polarity_flip=0x0009, rx_polarity_flip=0x000b, blocked_adj_split=4),

        _qsfp28(label="21", adj_label=None, serdes=(228, 229, 230, 231), leds=(3, 16),tx_lane_map=0x1023, rx_lane_map=0x2310, tx_polarity_flip=0x000f, rx_polarity_flip=0x0009),
        _qsfp28(label="22", adj_label="21", serdes=(224, 225, 226, 227), leds=(3, 24),tx_lane_map=0x2031, rx_lane_map=0x2103, tx_polarity_flip=0x0003, rx_polarity_flip=0x0007, blocked_adj_split=4),
        _qsfp28(label="23", adj_label=None, serdes=(244, 245, 246, 247), leds=(3, 32),tx_lane_map=0x1203, rx_lane_map=0x2301, tx_polarity_flip=0x0001, rx_polarity_flip=0x0009),
        _qsfp28(label="24", adj_label="23", serdes=(240, 241, 242, 243), leds=(3, 40),tx_lane_map=0x1023, rx_lane_map=0x0321, tx_polarity_flip=0x0002, rx_polarity_flip=0x000a, blocked_adj_split=4),

        _qsfp28(label="25", adj_label=None, serdes=(148, 149, 150, 151), leds=(2, 16),tx_lane_map=0x2310, rx_lane_map=0x0312, tx_polarity_flip=0x000e, rx_polarity_flip=0x0006),
        _qsfp28(label="26", adj_label="25", serdes=(144, 145, 146, 147), leds=(2, 24),tx_lane_map=0x0312, rx_lane_map=0x0321, tx_polarity_flip=0x0009, rx_polarity_flip=0x0004, blocked_adj_split=4),
        _qsfp28(label="27", adj_label=None, serdes=(164, 165, 166, 167), leds=(2, 32),tx_lane_map=0x1203, rx_lane_map=0x2031, tx_polarity_flip=0x0001, rx_polarity_flip=0x0006),
        _qsfp28(label="28", adj_label="27", serdes=(160, 161, 162, 163), leds=(2, 40),tx_lane_map=0x1023, rx_lane_map=0x1032, tx_polarity_flip=0x0009, rx_polarity_flip=0x000b, blocked_adj_split=4),

        _qsfp28(label="29", adj_label=None, serdes=(180, 181, 182, 183), leds=(2, 48),tx_lane_map=0x1203, rx_lane_map=0x1230, tx_polarity_flip=0x0001, rx_polarity_flip=0x000a),
        _qsfp28(label="30", adj_label="29", serdes=(176, 177, 178, 179), leds=(2, 56),tx_lane_map=0x2031, rx_lane_map=0x1032, tx_polarity_flip=0x0000, rx_polarity_flip=0x0009, blocked_adj_split=4),
        _qsfp28(label="31", adj_label=None, serdes=(196, 197, 198, 199), leds=(3, 48),tx_lane_map=0x1203, rx_lane_map=0x1230, tx_polarity_flip=0x0005, rx_polarity_flip=0x000b),
        _qsfp28(label="32", adj_label="31", serdes=(192, 193, 194, 195), leds=(3, 56),tx_lane_map=0x2031, rx_lane_map=0x1032, tx_polarity_flip=0x0000, rx_polarity_flip=0x0009, blocked_adj_split=4),

        _qsfp28(label="33", adj_label=None, serdes=(60, 61, 62, 63), leds=(0, 64),tx_lane_map=0x0231, rx_lane_map=0x1032, tx_polarity_flip=0x000b, rx_polarity_flip=0x0009),
        _qsfp28(label="34", adj_label="33", serdes=(56, 57, 58, 59), leds=(0, 72),tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=0x0002, rx_polarity_flip=0x0006, blocked_adj_split=4),
        _qsfp28(label="35", adj_label=None, serdes=(76, 77, 78, 79), leds=(1, 64),tx_lane_map=0x0231, rx_lane_map=0x1032, tx_polarity_flip=0x000f, rx_polarity_flip=0x000b),
        _qsfp28(label="36", adj_label="35", serdes=(72, 73, 74, 75), leds=(1, 72),tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=0x0006, rx_polarity_flip=0x0006, blocked_adj_split=4),

        _qsfp28(label="37", adj_label=None, serdes=(92,  93,  94,  95), leds=(1, 80),tx_lane_map=0x0231, rx_lane_map=0x1032, tx_polarity_flip=0x000f, rx_polarity_flip=0x0002),
        _qsfp28(label="38", adj_label="37", serdes=(88, 89,  90, 91), leds=(1, 88),tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=0x0006, rx_polarity_flip=0x0006, blocked_adj_split=4),
        _qsfp28(label="39", adj_label=None, serdes=(108, 109, 110, 111), leds=(1, 96),tx_lane_map=0x0213, rx_lane_map=0x0231, tx_polarity_flip=0x0002, rx_polarity_flip=0x000e),
        _qsfp28(label="40", adj_label="39", serdes=(104, 105, 106, 107), leds=(1, 104),tx_lane_map=0x2301, rx_lane_map=0x1023, tx_polarity_flip=0x0002, rx_polarity_flip=0x0009, blocked_adj_split=4),

        _qsfp28(label="41", adj_label=None, serdes=(12, 13, 14, 15), leds=(0, 80),tx_lane_map=0x3021, rx_lane_map=0x0321, tx_polarity_flip=0x000a, rx_polarity_flip=0x0006),
        _qsfp28(label="42", adj_label="41", serdes=(8, 9, 10, 11), leds=(0, 88),tx_lane_map=0x0213, rx_lane_map=0x2301, tx_polarity_flip=0x000a, rx_polarity_flip=0x0002, blocked_adj_split=4),
        _qsfp28(label="43", adj_label=None, serdes=(28, 29, 30, 31), leds=(0, 96),tx_lane_map=0x0231, rx_lane_map=0x2013, tx_polarity_flip=0x0009, rx_polarity_flip=0x0007),
        _qsfp28(label="44", adj_label="43", serdes=(24, 25, 26, 27), leds=(0, 104),tx_lane_map=0x1023, rx_lane_map=0x2310, tx_polarity_flip=0x000e, rx_polarity_flip=0x0009, blocked_adj_split=4),

        _qsfp28(label="45", adj_label=None, serdes=(44, 45, 46, 47), leds=(0, 112),tx_lane_map=0x2130, rx_lane_map=0x1032, tx_polarity_flip=0x0008, rx_polarity_flip=0x000b),
        _qsfp28(label="46", adj_label="45", serdes=(40, 41, 42, 43), leds=(0, 120),tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=0x0003, rx_polarity_flip=0x0006, blocked_adj_split=4),
        _qsfp28(label="47", adj_label=None, serdes=(124, 125, 126, 127), leds=(1, 112),tx_lane_map=0x1302, rx_lane_map=0x0231, tx_polarity_flip=0x000f, rx_polarity_flip=0x0006),
        _qsfp28(label="48", adj_label="47", serdes=(120, 121, 122, 123), leds=(1, 120),tx_lane_map=0x2130, rx_lane_map=0x0213, tx_polarity_flip=0x0005, rx_polarity_flip=0x0003, blocked_adj_split=4),

        _qsfp28(label="49", adj_label=None, serdes=(136, 137, 138, 139), leds=(2, 64),tx_lane_map=0x1203, rx_lane_map=0x2103, tx_polarity_flip=0x0005, rx_polarity_flip=0x000d),
        _qsfp28(label="50", adj_label="49", serdes=(140, 141, 142, 143), leds=(2, 72),tx_lane_map=0x0132, rx_lane_map=0x0231, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c, blocked_adj_split=4),
        _qsfp28(label="51", adj_label=None, serdes=(216, 217, 218, 219), leds=(3, 64),tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x000d),
        _qsfp28(label="52", adj_label="51", serdes=(220, 221, 222, 223), leds=(3, 72),tx_lane_map=0x3102, rx_lane_map=0x3201, tx_polarity_flip=0x0006, rx_polarity_flip=0x0003, blocked_adj_split=4),

        _qsfp28(label="53", adj_label=None, serdes=(232, 233, 234, 235), leds=(3, 80),tx_lane_map=0x1320, rx_lane_map=0x0213, tx_polarity_flip=0x000c, rx_polarity_flip=0x0007),
        _qsfp28(label="54", adj_label="53", serdes=(236, 237, 238, 239), leds=(3, 88),tx_lane_map=0x2013, rx_lane_map=0x1032, tx_polarity_flip=0x0003, rx_polarity_flip=0x0002, blocked_adj_split=4),
        _qsfp28(label="55", adj_label=None, serdes=(248, 249, 250, 251), leds=(3, 96),tx_lane_map=0x3120, rx_lane_map=0x2103, tx_polarity_flip=0x0000, rx_polarity_flip=0x0004),
        _qsfp28(label="56", adj_label="55", serdes=(252, 253, 254, 255), leds=(3, 104),tx_lane_map=0x3210, rx_lane_map=0x2013, tx_polarity_flip=0x0001, rx_polarity_flip=0x0001, blocked_adj_split=4),

        _qsfp28(label="57", adj_label=None, serdes=(152, 153, 154, 155), leds=(2, 80),tx_lane_map=0x1032, rx_lane_map=0x3120, tx_polarity_flip=0x0002, rx_polarity_flip=0x0002),
        _qsfp28(label="58", adj_label="57", serdes=(156, 157, 158, 159), leds=(2, 88),tx_lane_map=0x3120, rx_lane_map=0x2103, tx_polarity_flip=0x0004, rx_polarity_flip=0x0003, blocked_adj_split=4),
        _qsfp28(label="59", adj_label=None, serdes=(168, 169, 170, 171), leds=(2, 96),tx_lane_map=0x0213, rx_lane_map=0x0132, tx_polarity_flip=0x0000, rx_polarity_flip=0x000a),
        _qsfp28(label="60", adj_label="59", serdes=(172, 173, 174, 175), leds=(2, 104),tx_lane_map=0x0123, rx_lane_map=0x2103, tx_polarity_flip=0x000a, rx_polarity_flip=0x0002, blocked_adj_split=4),

        _qsfp28(label="61", adj_label=None, serdes=(184, 185, 186, 187), leds=(2, 112),tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x0005),
        _qsfp28(label="62", adj_label="61", serdes=(188, 189, 190, 191), leds=(2, 120),tx_lane_map=0x3210, rx_lane_map=0x3201, tx_polarity_flip=0x0005, rx_polarity_flip=0x0003, blocked_adj_split=4),
        _qsfp28(label="63", adj_label=None, serdes=(200, 201, 202, 203), leds=(3, 112),tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x000d),
        _qsfp28(label="64", adj_label="63", serdes=(204, 205, 206, 207), leds=(3, 120),tx_lane_map=0x3210, rx_lane_map=0x3201, tx_polarity_flip=0x0005, rx_polarity_flip=0x0003, blocked_adj_split=4),
)

vhwmon_path = '/sys/class/misc/vhwmon'

# PSUs
class DellZ9264PSU(cumulus.sensors.PSU_Unit):
    driver_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    psu_ok = 0


class DellZ9264PSU1(DellZ9264PSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_hwmon = ['psu_pwr1']

    def get_status(self):
        val = self.attrs.get('psu_pwr1_all_ok', 0)
        self.attrs['psu_pwr1_all_ok'] = val
        DellZ9264PSU1.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)


class DellZ9264PSU2(DellZ9264PSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_hwmon = ['psu_pwr2']

    def get_status(self):
        val = self.attrs.get('psu_pwr2_all_ok', 0)
        self.attrs['psu_pwr2_all_ok'] = val
        DellZ9264PSU2.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)


# Intel Denverton CPU
class DellZ9264CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

    hwmon_path = driver_path + "/hwmon/"
    hwmon_dirs = os.listdir(hwmon_path)
    driver_hwmons = []
    descriptions = []

    for dir in hwmon_dirs:
        files = os.listdir(hwmon_path + dir)
        for file in files:
            if str.endswith(file,'_label'):
                idx = string.find(file,"_")
                tempx = file[:idx]
                driver_hwmons.append(tempx)

                file_path = hwmon_path + dir + '/' + file
                f = open(file_path,'r')
                description = f.read()
                f.close()
                description = string.rstrip(description,'\n')
                descriptions.append(description)


class DellZ9264Temp1(DellZ9264CoreTemp):
    name = 'Temp1'
    driver_hwmon = []
    driver_hwmon.append(DellZ9264CoreTemp.driver_hwmons[0])
    description = DellZ9264CoreTemp.descriptions[0]


class DellZ9264Temp2(DellZ9264CoreTemp):
    name = 'Temp2'
    driver_hwmon = []
    driver_hwmon.append(DellZ9264CoreTemp.driver_hwmons[1])
    description = DellZ9264CoreTemp.descriptions[1]


class DellZ9264Temp3(DellZ9264CoreTemp):
    name = 'Temp3'
    driver_hwmon = []
    driver_hwmon.append(DellZ9264CoreTemp.driver_hwmons[2])
    description = DellZ9264CoreTemp.descriptions[2]


class DellZ9264Temp4(DellZ9264CoreTemp):
    name = 'Temp4'
    driver_hwmon = []
    driver_hwmon.append(DellZ9264CoreTemp.driver_hwmons[3])
    description = DellZ9264CoreTemp.descriptions[3]


class DellZ9264Temp5(DellZ9264CoreTemp):
    name = 'Temp5'
    driver_hwmon = []
    driver_hwmon.append(DellZ9264CoreTemp.driver_hwmons[4])
    description = DellZ9264CoreTemp.descriptions[4]


# BMC Sensors
class DellZ9264BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'


class DellZ9264Temp6(DellZ9264BMCTemp):
    name = 'Temp6'
    description = 'PT_Mid_temp'
    driver_hwmon = ['temp1']


class DellZ9264Temp7(DellZ9264BMCTemp):
    name = 'Temp7'
    description = 'NPU_Near_temp'
    driver_hwmon = ['temp2']


class DellZ9264Temp8(DellZ9264BMCTemp):
    name = 'Temp8'
    description = 'PT_Left_temp'
    driver_hwmon = ['temp3']


class DellZ9264Temp9(DellZ9264BMCTemp):
    name = 'Temp9'
    description = 'PT_Right_temp'
    driver_hwmon = ['temp4']


class DellZ9264Temp10(DellZ9264BMCTemp):
    name = 'Temp10'
    description = 'ILET_AF_temp'
    driver_hwmon = ['temp5']


class DellZ9264Temp11(DellZ9264BMCTemp):
    name = 'Temp11'
    description = 'CPU_Near_temp'
    driver_hwmon = ['temp6']

class DellZ9264Temp12(DellZ9264BMCTemp):
    name = 'Temp12'
    description = 'TC_Near_temp'
    driver_hwmon = ['temp7']


# PSU temp sensors
class DellZ9264PSUTemp(cumulus.sensors.Temp_Unit):
    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)


class DellZ9264Temp12(DellZ9264PSUTemp):
    driver_hwmon = ['temp1']
    name = 'PSU1Temp1'
    description = 'PSU1_AF_temp'
    psu = DellZ9264PSU1()
    driver_path = vhwmon_path + '/bmc:psu1'


class DellZ9264Temp13(DellZ9264PSUTemp):
    driver_hwmon = ['temp2']
    name = 'PSU1Temp2'
    description = 'PSU1_temp'
    psu = DellZ9264PSU1()
    driver_path = vhwmon_path + '/bmc:psu1'


class DellZ9264Temp14(DellZ9264PSUTemp):
    driver_hwmon = ['temp2']
    name = 'PSU2Temp1'
    description = 'PSU2_AF_temp'
    psu = DellZ9264PSU2()
    driver_path = vhwmon_path + '/bmc:psu2'


class DellZ9264Temp15(DellZ9264PSUTemp):
    driver_hwmon = ['temp2']
    name = 'PSU2Temp2'
    description = 'PSU2_temp'
    psu = DellZ9264PSU2()
    driver_path = vhwmon_path + '/bmc:psu2'


# Fans
class DellZ9264Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True


class DellZ9264Fan1(DellZ9264Fan):
    name = 'Fan1'
    description = 'FAN1_Rear_rpm'
    driver_hwmon = ['fan1']


class DellZ9264Fan2(DellZ9264Fan):
    name = 'Fan2'
    description = 'FAN2_Rear_rpm'
    driver_hwmon = ['fan2']


class DellZ9264Fan3(DellZ9264Fan):
    name = 'Fan3'
    description = 'FAN3_Rear_rpm'
    driver_hwmon = ['fan3']


class DellZ9264Fan4(DellZ9264Fan):
    name = 'Fan4'
    description = 'FAN4_Rear_rpm'
    driver_hwmon = ['fan4']


class DellZ9264Fan5(DellZ9264Fan):
    name = 'Fan5'
    description = 'FAN1_Front_rpm'
    driver_hwmon = ['fan5']


class DellZ9264Fan6(DellZ9264Fan):
    name = 'Fan6'
    description = 'FAN2_Front_rpm'
    driver_hwmon = ['fan6']


class DellZ9264Fan7(DellZ9264Fan):
    name = 'Fan7'
    description = 'FAN3_Front_rpm'
    driver_hwmon = ['fan7']


class DellZ9264Fan8(DellZ9264Fan):
    name = 'Fan8'
    description = 'FAN4_Front_rpm'
    driver_hwmon = ['fan8']


class DellZ9264Sensors():
    units = (DellZ9264PSU1(),
             DellZ9264PSU2(),
             DellZ9264Temp1(),
             DellZ9264Temp2(),
             DellZ9264Temp3(),
             DellZ9264Temp4(),
             DellZ9264Temp5(),
             DellZ9264Temp6(),
             DellZ9264Temp7(),
             DellZ9264Temp8(),
             DellZ9264Temp9(),
             DellZ9264Temp10(),
             DellZ9264Temp11(),
             DellZ9264Temp12(),
             DellZ9264Temp13(),
             DellZ9264Temp14(),
             DellZ9264Temp15(),
             DellZ9264Fan1(),
             DellZ9264Fan2(),
             DellZ9264Fan3(),
             DellZ9264Fan4(),
             DellZ9264Fan5(),
             DellZ9264Fan6(),
             DellZ9264Fan7(),
             DellZ9264Fan8(),
             )


class DellZ9264SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'amber'


class DellZ9264PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_power'
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'


class DellZ9264LEDs():
    units = (DellZ9264SystemLED(),
             DellZ9264PowerLED(),
             )


class DellZ9264BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'


class DellZ9264F(cumulus.platform.Platform):
    name = 'dellemc,z9264f_c3538'
    chip = cumulus.platform.TomahawkTwo_56970_Chip()
    switch = DellZ9264FSwitch(chip)
    cpld = None
    bmc = DellZ9264BMC()
    sensors = DellZ9264Sensors()
    statusleds = DellZ9264LEDs()
    pwms = None

Platform = DellZ9264F
