#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DeltaAG5648V1SFP28(cumulus.platform.Tomahawk25GPort):
    _set_led = '''
;;; DeltaAG5648V1SFP28
;;
;;; Each 25G port has two LEDs. One bi-color for link and one for activity.
;;; The bi-color link LED is green/red
;;; The activity LED is just green
;;; Three bits to pack all active high:
;;;   0: link_green
;;;   1: link_red
;;;   2: activity_green
;;; We use:
;;;   link_green = on with link
;;;   link_red = always off
;;;   activity_green = flashes on with activity
;;;
set_led:
        port a
        pushst  PORTSTATUS_LINKEN   ; link_green
        pack                        ;  - on with link
        pushst  PORTSTATUS_ZERO     ; link_red
        pack                        ;  - off
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand                        ; activity_green
        pack                        ;  - flash on with activity
        ret
'''

    _set_led_common = '''
set_qsfp_led:
        port a
        pushst  PORTSTATUS_ZERO     ; laneX_blue
        pack                        ;  - off
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pushst  PORTSTATUS_LINKEN   ; laneX_green
        tand                        ;  - on with link
        pack                        ;  - flash off with activity
        pushst  PORTSTATUS_ZERO     ; laneX_red
        pack                        ;  - off
        ret

set_qsfp_off:
        pushst  PORTSTATUS_ZERO     ; all three leds
        pack                        ;  - off
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls

class DeltaAG5648V1QSFP28(cumulus.platform.Tomahawk100GPort):
    _set_led = '''
;;; DeltaAG5648V1QSFP28
;;;
;;; Each QSFP28 port has four RGB LEDs.
;;; 12 bits to pack, all active high:
;;;   0: lane0_blue
;;;   1: lane0_green
;;;   2: lane0_red
;;;   3: lane1_blue
;;;   4: lane1_green
;;;   5: lane1_red
;;;   6: lane2_blue
;;;   7: lane2_green
;;;   8: lane2_red
;;;   9: lane3_blue
;;;  10: lane3_green
;;;  11: lane3_red
;;;
;;; For 100G, 50G, and 40G modes, we use only the first LED (green) to
;;; indicate link/activity.
;;;    0: lane0_blue = always off
;;;    1: lane0_green = on for link, flashes with activity
;;;    2: lane0_red = always off
;;;    3-11: all other LEDs are off
;;;
set_led:
        call    set_qsfp_led
        call    set_qsfp_off
        call    set_qsfp_off
        call    set_qsfp_off
        ret
'''
    _set_led_2x = '''
;;; DeltaAG5648V1QSFP28_2x
;;;
;;;  For 2x50G mode, we use two RGB LEDs, one for each port.
;;;    0: lane0_blue = always off
;;;    1: lane0_green = on for link, flashes with activity
;;;    2: lane0_red = always off
;;;    3-5: LED1 is off
;;;    6: lane2_blue = always off
;;;    7: lane2_green = on for link, flashes with activity
;;;    8: lane2_red = always off
;;;    9-11: LED3 is off
;;;
set_led:
        call    set_qsfp_led
        call    set_qsfp_off
        inc     a
        inc     a
        call    set_qsfp_led
        call    set_qsfp_off
        ret
'''
    _set_led_4x = '''
;;; DeltaAG5648V1QSFP28_4x
;;;
;;;  For 4x25G and 4x10G mode, we use all four RGB LEDs, one for each port.
;;;    laneX_blue = always off
;;;    laneX_green = on for link, flashes with activity
;;;    laneX_red = always off
;;;
set_led:
        call    set_qsfp_led
        inc     a
        call    set_qsfp_led
        inc     a
        call    set_qsfp_led
        inc     a
        call    set_qsfp_led
        ret
'''

    def __init__(self, label, serdes, leds,
                 rx_lane_map=None, tx_lane_map=None,
                 rx_polarity_flip=None, tx_polarity_flip=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if ((mode in ('4x10G', '4x25G') and (sub != 0)) or
            ((mode == "2x50G") and (sub == 1 or sub == 3))):
            # For 4x modes, only update LEDs for the first sub-interface,
            # and for 2x mode, only update LEDs for the first and third
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode in ('4x10G', '4x25G'):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif mode in ('2x50G'):
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        elif mode in ('40G', '50G', '100G'):
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        for x in range(1,12):
            set_calls[led_index_list[0]+x] = None

        return set_calls

class DeltaAG5648V1LED_37_54(cumulus.ledup.TomahawkLEDController):
    led_functions = (DeltaAG5648V1SFP28._set_led_common)
    num_leds = 114

class DeltaAG5648V1LED_1_36(cumulus.ledup.TomahawkLEDController):
    led_functions = (DeltaAG5648V1SFP28._set_led_common)
    num_leds = 112

# Delta AG5648V1 switch ports

class DeltaAG5648V1Switch(cumulus.platform.BCMSwitch):
    ledup = (DeltaAG5648V1LED_37_54, DeltaAG5648V1LED_1_36)
    _sfp28 = DeltaAG5648V1SFP28
    _qsfp28 = DeltaAG5648V1QSFP28

    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # TSCF16
        _sfp28(label="1",  serdes=( 64,), leds=(1,   0), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="2",  serdes=( 65,), leds=(1,   3), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="3",  serdes=( 66,), leds=(1,   6), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="4",  serdes=( 67,), leds=(1,   9), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF17
        _sfp28(label="5",  serdes=( 68,), leds=(1,  12), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="6",  serdes=( 69,), leds=(1,  15), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=( 70,), leds=(1,  18), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="8",  serdes=( 71,), leds=(1,  21), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF20
        _sfp28(label="9",  serdes=( 80,), leds=(1,  24), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10", serdes=( 81,), leds=(1,  27), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="11", serdes=( 82,), leds=(1,  30), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="12", serdes=( 83,), leds=(1,  33), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF21
        _sfp28(label="13", serdes=( 84,), leds=(1,  36), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="14", serdes=( 85,), leds=(1,  39), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="15", serdes=( 86,), leds=(1,  42), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="16", serdes=( 87,), leds=(1,  45), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF8
        _sfp28(label="17", serdes=( 32,), leds=(1,  48), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="18", serdes=( 33,), leds=(1,  51), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="19", serdes=( 34,), leds=(1,  54), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="20", serdes=( 35,), leds=(1,  57), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF9
        _sfp28(label="21", serdes=( 36,), leds=(1,  60), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="22", serdes=( 37,), leds=(1,  63), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="23", serdes=( 38,), leds=(1,  66), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="24", serdes=( 39,), leds=(1,  69), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF10
        _sfp28(label="25", serdes=( 40,), leds=(1,  72), tx_lane_map=0x0213, rx_lane_map=0x2031, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="26", serdes=( 41,), leds=(1,  75), tx_lane_map=0x0213, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="27", serdes=( 42,), leds=(1,  78), tx_lane_map=0x0213, rx_lane_map=0x2031, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="28", serdes=( 43,), leds=(1,  81), tx_lane_map=0x0213, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF12
        _sfp28(label="29", serdes=( 48,), leds=(1,  84), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="30", serdes=( 49,), leds=(1,  87), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="31", serdes=( 50,), leds=(1,  90), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="32", serdes=( 51,), leds=(1,  93), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF13
        _sfp28(label="33", serdes=( 52,), leds=(1,  96), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="34", serdes=( 53,), leds=(1,  99), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="35", serdes=( 54,), leds=(1, 102), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="36", serdes=( 55,), leds=(1, 105), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF24
        _sfp28(label="37", serdes=( 96,), leds=(0,   0), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="38", serdes=( 97,), leds=(0,   3), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="39", serdes=( 98,), leds=(0,   6), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="40", serdes=( 99,), leds=(0,   9), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF25
        _sfp28(label="41", serdes=(100,), leds=(0,  12), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="42", serdes=(101,), leds=(0,  15), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="43", serdes=(102,), leds=(0,  18), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="44", serdes=(103,), leds=(0,  21), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF26
        _sfp28(label="45", serdes=(104,), leds=(0,  24), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="46", serdes=(105,), leds=(0,  27), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="47", serdes=(106,), leds=(0,  30), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="48", serdes=(107,), leds=(0,  33), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # TSCF29
        _qsfp28(label="49", serdes=(116, 117, 118, 119), leds=(0, 36), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=0xf, rx_polarity_flip=0xf),

        # TSCF27
        _qsfp28(label="50", serdes=(108, 109, 110, 111), leds=(0, 48), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=0xf, rx_polarity_flip=0xf),

        # TSCF1
        _qsfp28(label="51", serdes=(  4,   5,   6,   7), leds=(0, 60), tx_lane_map=0x2301, rx_lane_map=0x2310, tx_polarity_flip=0xf, rx_polarity_flip=0xf),

        # TSCF0
        _qsfp28(label="52", serdes=(  0,   1,   2,   3), leds=(0, 72), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0, rx_polarity_flip=0x0),

        # TSCF5
        _qsfp28(label="53", serdes=( 20,  21,  22,  23), leds=(0, 84), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xf, rx_polarity_flip=0xf),

        # TSCF2
        _qsfp28(label="54", serdes=(  8,   9,  10,  11), leds=(0, 96), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0, rx_polarity_flip=0x0),
)

# sysfs paths

class DeltaAG5648V1CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/delta_ag5648v1_cpld.0'

DeltaAG5648V1_cpld_path = '/sys/devices/platform/delta_ag5648v1_cpld.0'

DeltaAG5648V1_i2c_path  = '/sys/bus/i2c/devices/'

''' u290 tmp75 '''
DeltaAG5648V1_u290_path = DeltaAG5648V1_i2c_path + '11-004b'

''' u288 tmp75 '''
DeltaAG5648V1_u288_path = DeltaAG5648V1_i2c_path + '11-004c'

''' u291 tmp75 '''
DeltaAG5648V1_u291_path = DeltaAG5648V1_i2c_path + '11-0049'

''' u289 tmp75 '''
DeltaAG5648V1_u289_path = DeltaAG5648V1_i2c_path + '11-004e'

''' u334 tmp75 '''
DeltaAG5648V1_u334_path = DeltaAG5648V1_i2c_path + '11-004f'

''' u57 tmp75 '''
DeltaAG5648V1_u57_path = DeltaAG5648V1_i2c_path + '10-004d'

''' fan controller 1 '''
DeltaAG5648V1_fan_controller_1_path = DeltaAG5648V1_i2c_path + '11-004d'

''' fan controller 2 '''
DeltaAG5648V1_fan_controller_2_path = DeltaAG5648V1_i2c_path + '13-004d'

''' PSU 1 '''
DeltaAG5648V1_psu1_path = DeltaAG5648V1_i2c_path + '14-0059'

''' PSU 2 '''
DeltaAG5648V1_psu2_path = DeltaAG5648V1_i2c_path + '14-0058'

# Temp sensors

# Intel Rangeley CPU
class DeltaAG5648V1CPUTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 93
    temp_crit = 98
    temp_fan_min = 50
    temp_fan_max = 70

class DeltaAG5648V1CPUTemp1(DeltaAG5648V1CPUTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class DeltaAG5648V1CPUTemp2(DeltaAG5648V1CPUTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class DeltaAG5648V1CPUTemp3(DeltaAG5648V1CPUTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class DeltaAG5648V1CPUTemp4(DeltaAG5648V1CPUTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

# Tomahawk+ Internal Temp Sensors
class DeltaAG5648V1BCMTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 100
    temp_crit    = 105
    temp_fan_min = 50
    temp_fan_max = 70

class DeltaAG5648V1BCMTemp1(DeltaAG5648V1BCMTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class DeltaAG5648V1BCMTemp2(DeltaAG5648V1BCMTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class DeltaAG5648V1BCMTemp3(DeltaAG5648V1BCMTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class DeltaAG5648V1BCMTemp4(DeltaAG5648V1BCMTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class DeltaAG5648V1BCMTemp5(DeltaAG5648V1BCMTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class DeltaAG5648V1BCMTemp6(DeltaAG5648V1BCMTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class DeltaAG5648V1BCMTemp7(DeltaAG5648V1BCMTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class DeltaAG5648V1BCMTemp8(DeltaAG5648V1BCMTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

# U290 - Board Sensor Near Left of Front Vents
class DeltaAG5648V1Temp1(cumulus.sensors.Temp_Unit):
    name = 'Temp13'
    description = 'Near left of front vents'
    driver_path = DeltaAG5648V1_u290_path
    driver_hwmon = ['temp1']
    temp_max = 55
    temp_crit = 60
    temp_fan_min = 50
    temp_fan_max = 70

# U288 - Board Sensor Near Tomahawk+
class DeltaAG5648V1Temp2(cumulus.sensors.Temp_Unit):
    name = 'Temp14'
    description = 'Near switch chip'
    driver_path = DeltaAG5648V1_u288_path
    driver_hwmon = ['temp1']
    temp_max = 80
    temp_crit = 85
    temp_fan_min = 50
    temp_fan_max = 70

# U291 - Board Sensor Near Middle of Front Vents
class DeltaAG5648V1Temp3(cumulus.sensors.Temp_Unit):
    name = 'Temp15'
    description = 'Near middle of front vents'
    driver_path = DeltaAG5648V1_u291_path
    driver_hwmon = ['temp1']
    temp_max = 65
    temp_crit = 70
    temp_fan_min = 50
    temp_fan_max = 70

# U289 - Board Sensor Near Right of Front Vents
class DeltaAG5648V1Temp4(cumulus.sensors.Temp_Unit):
    name = 'Temp16'
    description = 'Near right of front vents'
    driver_path = DeltaAG5648V1_u289_path
    driver_hwmon = ['temp1']
    temp_max = 60
    temp_crit = 65
    temp_fan_min = 50
    temp_fan_max = 70

# U334 - Board Sensor Near DC Fan
class DeltaAG5648V1Temp5(cumulus.sensors.Temp_Unit):
    name = 'Temp17'
    description = 'Near fan trays'
    driver_path = DeltaAG5648V1_u334_path
    driver_hwmon = ['temp1']
    temp_max = 55
    temp_crit = 60
    temp_fan_min = 50
    temp_fan_max = 70

# U57 Board Sensor Near CPU
class DeltaAG5648V1Temp6(cumulus.sensors.Temp_Unit):
    name = 'Temp18'
    description = "Near CPU"
    driver_path = DeltaAG5648V1_u57_path
    driver_hwmon = ['temp1']
    temp_max = 75
    temp_crit = 80
    temp_fan_min = 50
    temp_fan_max = 70

# Fans

class DeltaAG5648V1Fan(cumulus.sensors.Fan_Unit):
    minpwm = 99
    maxpwm = 255
    hwmon_search = False
    cpld_path = DeltaAG5648V1_cpld_path

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DeltaAG5648V1FanController1(DeltaAG5648V1Fan):
    # for front fans (1 - 4)
    fan_max = 23000
    fan_min = 8000
    driver_path = DeltaAG5648V1_fan_controller_1_path

class DeltaAG5648V1FanController2(DeltaAG5648V1Fan):
    # for rear fans (5 - 8)
    fan_max = 20500
    fan_min = 8000
    driver_path = DeltaAG5648V1_fan_controller_2_path

class DeltaAG5648V1Fan1(DeltaAG5648V1FanController1):
    name = 'Fan1'
    description = 'Fan Tray 1 Front'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan1_present'

class DeltaAG5648V1Fan2(DeltaAG5648V1FanController1):
    name = 'Fan2'
    description = 'Fan Tray 2 Front'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan2_present'

class DeltaAG5648V1Fan3(DeltaAG5648V1FanController1):
    name = 'Fan3'
    description = 'Fan Tray 3 Front'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan3_present'

class DeltaAG5648V1Fan4(DeltaAG5648V1FanController1):
    name = 'Fan4'
    description = 'Fan Tray 4 Front'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan4_present'

class DeltaAG5648V1Fan5(DeltaAG5648V1FanController2):
    name = 'Fan5'
    description = 'Fan Tray 1 Rear'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    fan_present = 'fan1_present'

class DeltaAG5648V1Fan6(DeltaAG5648V1FanController2):
    name = 'Fan6'
    description = 'Fan Tray 2 Rear'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    fan_present = 'fan2_present'

class DeltaAG5648V1Fan7(DeltaAG5648V1FanController2):
    name = 'Fan7'
    description = 'Fan Tray 3 Rear'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    fan_present = 'fan3_present'

class DeltaAG5648V1Fan8(DeltaAG5648V1FanController2):
    name = 'Fan8'
    description = 'Fan Tray 4 Rear'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    fan_present = 'fan4_present'

# PSUs

class DeltaAG5648V1PSU(cumulus.sensors.PSU_Unit):
    init_done = False

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.psu_present) == '1':
            self.attrs[self.psu_present] = 1
        else:
            self.attrs[self.psu_present] = 0
        if self.get_cpld_value(self.psu_all_ok) == '1':
            self.attrs[self.psu_all_ok] = 1
        else:
            self.attrs[self.psu_all_ok] = 0
        return 0

class DeltaAG5648V1PSU1(DeltaAG5648V1PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_all_ok = 'psu_pwr1_all_ok'

class DeltaAG5648V1PSU2(DeltaAG5648V1PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_all_ok = 'psu_pwr2_all_ok'

class DeltaAG5648V1PSUTemp(cumulus.sensors.Temp_Unit):
    temp_max = 85
    temp_crit = 90
    temp_fan_min = 50
    temp_fan_max = 70

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DeltaAG5648V1PSU1Temp1(DeltaAG5648V1PSUTemp):
    name = 'PSU1Temp1'
    driver_path = DeltaAG5648V1_psu1_path
    driver_hwmon = ['temp1']
    description = 'PSU1 Temp Sensor'
    psu = 'PSU1'

class DeltaAG5648V1PSU1Temp2(DeltaAG5648V1PSUTemp):
    name = 'PSU1Temp2'
    driver_path = DeltaAG5648V1_psu1_path
    driver_hwmon = ['temp2']
    description = 'PSU1 Temp Sensor'
    psu = 'PSU1'

class DeltaAG5648V1PSU2Temp1(DeltaAG5648V1PSUTemp):
    name = 'PSU2Temp1'
    driver_path = DeltaAG5648V1_psu2_path
    driver_hwmon = ['temp1']
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'

class DeltaAG5648V1PSU2Temp2(DeltaAG5648V1PSUTemp):
    name = 'PSU2Temp2'
    driver_path = DeltaAG5648V1_psu2_path
    driver_hwmon = ['temp2']
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'

class DeltaAG5648V1PSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 18000
    fan_min = 7200
    minpwm = 102
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if state == 'ABSENT':
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DeltaAG5648V1PSU1Fan1(DeltaAG5648V1PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DeltaAG5648V1_psu1_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'


class DeltaAG5648V1PSU2Fan1(DeltaAG5648V1PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DeltaAG5648V1_psu2_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'

# Sensors

class DeltaAG5648V1Sensors():
    units = (DeltaAG5648V1CPUTemp1(),
             DeltaAG5648V1CPUTemp2(),
             DeltaAG5648V1CPUTemp3(),
             DeltaAG5648V1CPUTemp4(),
             DeltaAG5648V1BCMTemp1(),
             DeltaAG5648V1BCMTemp2(),
             DeltaAG5648V1BCMTemp3(),
             DeltaAG5648V1BCMTemp4(),
             DeltaAG5648V1BCMTemp5(),
             DeltaAG5648V1BCMTemp6(),
             DeltaAG5648V1BCMTemp7(),
             DeltaAG5648V1BCMTemp8(),
             DeltaAG5648V1Temp1(),
             DeltaAG5648V1Temp2(),
             DeltaAG5648V1Temp3(),
             DeltaAG5648V1Temp4(),
             DeltaAG5648V1Temp5(),
             DeltaAG5648V1Temp6(),
             DeltaAG5648V1Fan1(),
             DeltaAG5648V1Fan2(),
             DeltaAG5648V1Fan3(),
             DeltaAG5648V1Fan4(),
             DeltaAG5648V1Fan5(),
             DeltaAG5648V1Fan6(),
             DeltaAG5648V1Fan7(),
             DeltaAG5648V1Fan8(),
             DeltaAG5648V1PSU1(),
             DeltaAG5648V1PSU2(),
             DeltaAG5648V1PSU1Temp1(),
             DeltaAG5648V1PSU1Temp2(),
             DeltaAG5648V1PSU2Temp1(),
             DeltaAG5648V1PSU2Temp2(),
             DeltaAG5648V1PSU1Fan1(),
             DeltaAG5648V1PSU2Fan1(),
             )

class DeltaAG5648V1PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    fans = (DeltaAG5648V1Fan1(),
            DeltaAG5648V1Fan2(),
            DeltaAG5648V1Fan3(),
            DeltaAG5648V1Fan4(),
            DeltaAG5648V1Fan5(),
            DeltaAG5648V1Fan6(),
            DeltaAG5648V1Fan7(),
            DeltaAG5648V1Fan8(),
            DeltaAG5648V1PSU1Fan1(),
            DeltaAG5648V1PSU2Fan1(),
            )

    temps = (DeltaAG5648V1CPUTemp1(),
             DeltaAG5648V1CPUTemp2(),
             DeltaAG5648V1CPUTemp3(),
             DeltaAG5648V1CPUTemp4(),
             DeltaAG5648V1BCMTemp1(),
             DeltaAG5648V1BCMTemp2(),
             DeltaAG5648V1BCMTemp3(),
             DeltaAG5648V1BCMTemp4(),
             DeltaAG5648V1BCMTemp5(),
             DeltaAG5648V1BCMTemp6(),
             DeltaAG5648V1BCMTemp7(),
             DeltaAG5648V1BCMTemp8(),
             DeltaAG5648V1Temp1(),
             DeltaAG5648V1Temp2(),
             DeltaAG5648V1Temp3(),
             DeltaAG5648V1Temp4(),
             DeltaAG5648V1Temp5(),
             DeltaAG5648V1Temp6(),
             DeltaAG5648V1PSU1Temp1(),
             DeltaAG5648V1PSU1Temp2(),
             DeltaAG5648V1PSU2Temp1(),
             DeltaAG5648V1PSU2Temp2(),
             )

class DeltaAG5648V1PWMs():
    units = (DeltaAG5648V1PWM1(),
             )

# LEDs

'''
   Delta AG5648V1 LEDS
     System:
       green - all systems ok
       amber blinking - fan missing or broken, psu missing or broken
     Power:
       green - both PSUs installed and working
       amber blinking - PSU missing or broken
     Fan:
       green - all four fans installed and working
       yellow blinking - fan missing or malfunction
     Fan Tray 1-4:
       green - fan is installed and working
       amber - fan is missing or malfunction
'''

class DeltaAG5648V1SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_system'
    bad_led_color = 'amber_blinking'

class DeltaAG5648V1PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'amber_blinking'

class DeltaAG5648V1FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan LED'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6', 'Fan7', 'Fan8']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'

class DeltaAG5648V1FanTray1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1 LED'
    sources = ['Fan1', 'Fan5']
    led_location = 'led_fan_tray1'
    bad_led_color = 'amber'

class DeltaAG5648V1FanTray2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2 LED'
    sources = ['Fan2', 'Fan6']
    led_location = 'led_fan_tray2'
    bad_led_color = 'amber'

class DeltaAG5648V1FanTray3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3 LED'
    sources = ['Fan3', 'Fan7']
    led_location = 'led_fan_tray3'
    bad_led_color = 'amber'

class DeltaAG5648V1FanTray4LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 4 LED'
    sources = ['Fan4', 'Fan8']
    led_location = 'led_fan_tray4'
    bad_led_color = 'amber'

class DeltaAG5648V1LEDs():
    units = (
             DeltaAG5648V1SystemLED(),
             DeltaAG5648V1PowerLED(),
             DeltaAG5648V1FanLED(),
             DeltaAG5648V1FanTray1LED(),
             DeltaAG5648V1FanTray2LED(),
             DeltaAG5648V1FanTray3LED(),
             DeltaAG5648V1FanTray4LED(),
        )

# Delta AG5648V1 class

class DeltaAG5648V1(cumulus.platform.Platform):
    name = 'delta,ag5648v1'
    chip = cumulus.platform.TomahawkPlus_56967_Chip()
    switch = DeltaAG5648V1Switch(chip)
    cpld = DeltaAG5648V1CPLD()
    pwms = DeltaAG5648V1PWMs()
    sensors = DeltaAG5648V1Sensors()
    statusleds = DeltaAG5648V1LEDs()

Platform = DeltaAG5648V1
