#
# Copyright (c) 2019 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import syslog

class GMSS422SW10GT(cumulus.platform.Maverick10GPort):
    def __init__(self, label, serdes, phy_addrs, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Maverick10GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=("84888",),
                                                  phy_addrs=(phy_addrs,),
                                                  phy_layer='10GBaseT',
                                                  leds=None,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  unit=0,
                                                  oversub=True)

class GMSS422SWQSFPp(cumulus.platform.Maverick100GPort):
    def __init__(self, label, serdes, rx_lane_map=None,
                 tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Maverick100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   leds=None,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# GMS S422-SW switch ports

class GMSS422SWSwitch(cumulus.platform.BCMSwitch):
    _10GT = GMSS422SW10GT
    _qsfp = GMSS422SWQSFPp
    _fab = cumulus.platform.Maverick40GFabricPort
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = True

    ports = (

        # Grouped by serdes cores:
        
        # TSCE0 - 0-3
        #_10GT(label="1",  serdes=( 0,), phy_addrs=(0x20), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="2",  serdes=( 1,), phy_addrs=(0x21), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="3",  serdes=( 2,), phy_addrs=(0x22), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="4",  serdes=( 3,), phy_addrs=(0x23), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE1 - 4-7
        #_10GT(label="5",  serdes=( 4,), phy_addrs=(0x24), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="6",  serdes=( 5,), phy_addrs=(0x25), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="7",  serdes=( 6,), phy_addrs=(0x26), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="8",  serdes=( 7,), phy_addrs=(0x27), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE2 - 8-11
        #_10GT(label="9",  serdes=( 8,), phy_addrs=(0x28), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="10", serdes=( 9,), phy_addrs=(0x29), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="11", serdes=(10,), phy_addrs=(0x2a), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="12", serdes=(11,), phy_addrs=(0x2b), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE3 - 12-15
        #_10GT(label="13", serdes=(12,), phy_addrs=(0x2c), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="14", serdes=(13,), phy_addrs=(0x2d), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="15", serdes=(14,), phy_addrs=(0x2e), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="16", serdes=(15,), phy_addrs=(0x2f), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE4 - 16-19
        #_10GT(label="17", serdes=(16,), phy_addrs=(0x30), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="18", serdes=(17,), phy_addrs=(0x31), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="19", serdes=(18,), phy_addrs=(0x32), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="20", serdes=(19,), phy_addrs=(0x33), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE5 - 20-23
        #_10GT(label="21", serdes=(20,), phy_addrs=(0x34), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="22", serdes=(21,), phy_addrs=(0x35), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="23", serdes=(22,), phy_addrs=(0x36), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="24", serdes=(23,), phy_addrs=(0x37), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCE6 - 24-27
        #_10GT(label="25", serdes=(24,), phy_addrs=(0x38), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="26", serdes=(25,), phy_addrs=(0x39), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="27", serdes=(26,), phy_addrs=(0x3a), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        #_10GT(label="28", serdes=(27,), phy_addrs=(0x3b), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),


        # Grouped by cable bundles:
        
        # J6(e1)-J6(e10)
        _10GT(label="1",  serdes=(12,), phy_addrs=(0x2c), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="2",  serdes=(16,), phy_addrs=(0x30), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="3",  serdes=(24,), phy_addrs=(0x38), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="4",  serdes=(13,), phy_addrs=(0x2d), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="5",  serdes=(17,), phy_addrs=(0x31), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="6",  serdes=(25,), phy_addrs=(0x39), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="7",  serdes=(18,), phy_addrs=(0x32), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="8",  serdes=(26,), phy_addrs=(0x3a), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="9",  serdes=(19,), phy_addrs=(0x33), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="10", serdes=(27,), phy_addrs=(0x3b), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # J8(e1)-J8(e3), J8(e6)-J8(e10)
        _10GT(label="11", serdes=( 4,), phy_addrs=(0x24), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="12", serdes=( 7,), phy_addrs=(0x27), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="13", serdes=( 6,), phy_addrs=(0x26), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        # J8(e4) is not used
        # J8(e5) is not used
        _10GT(label="14", serdes=( 0,), phy_addrs=(0x20), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="15", serdes=(14,), phy_addrs=(0x2e), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="16", serdes=( 5,), phy_addrs=(0x25), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="17", serdes=( 1,), phy_addrs=(0x21), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="18", serdes=(15,), phy_addrs=(0x2f), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # J9(e1)-J9(e10)
        _10GT(label="19", serdes=(21,), phy_addrs=(0x35), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="20", serdes=( 2,), phy_addrs=(0x22), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="21", serdes=( 9,), phy_addrs=(0x29), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="22", serdes=(20,), phy_addrs=(0x34), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="23", serdes=( 3,), phy_addrs=(0x23), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="24", serdes=(11,), phy_addrs=(0x2b), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="25", serdes=(23,), phy_addrs=(0x37), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="26", serdes=( 8,), phy_addrs=(0x28), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="27", serdes=(22,), phy_addrs=(0x36), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _10GT(label="28", serdes=(10,), phy_addrs=(0x2a), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF0 - 28-31
        _qsfp(label="29", serdes=( 28, 29, 30, 31), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0, rx_polarity_flip=0x0),
        # TSCF1 - 32-35
        _qsfp(label="30", serdes=( 32, 33, 34, 35), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0, rx_polarity_flip=0x0),
        # TSCF2 - 64-67
        _qsfp(label="31", serdes=( 64, 65, 66, 67), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0, rx_polarity_flip=0x0),
        # TSCF3 - 68-71
        _qsfp(label="32", serdes=( 68, 67, 68, 69), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0, rx_polarity_flip=0x0),

        # TSCE7 - 36-39
        _fab(label="0",   serdes=( 36, 37, 38, 39), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        # TSCE8 - 40-43
        _fab(label="1",   serdes=( 40, 41, 42, 43), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        # TSCE9 - 44-47
        _fab(label="2",   serdes=( 44, 45, 46, 47), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        # TSCE10 - 48-51
        _fab(label="3",   serdes=( 48, 49, 50, 51), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
    )

# Intel Rangeley CPU

class GMSS422SWCT(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98

class GMSS422SWCPUTemp1(GMSS422SWCT):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class GMSS422SWCPUTemp2(GMSS422SWCT):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class GMSS422SWCPUTemp3(GMSS422SWCT):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class GMSS422SWCPUTemp4(GMSS422SWCT):
    name = 'Temp4'
    driver_hwmon = ['temp5']

# Maverick Internal Temp Sensors

class GMSS422SWBCMTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 100
    temp_crit    = 105
    temp_fan_min = 53
    temp_fan_max = 75

class GMSS422SWBCMTemp1(GMSS422SWBCMTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class GMSS422SWBCMTemp2(GMSS422SWBCMTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class GMSS422SWBCMTemp3(GMSS422SWBCMTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class GMSS422SWBCMTemp4(GMSS422SWBCMTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class GMSS422SWBCMTemp5(GMSS422SWBCMTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class GMSS422SWBCMTemp6(GMSS422SWBCMTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class GMSS422SWBCMTemp7(GMSS422SWBCMTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class GMSS422SWBCMTemp8(GMSS422SWBCMTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

# Sensors

class GMSS422SWSensors():
    units = (GMSS422SWCPUTemp1(),
             GMSS422SWCPUTemp2(),
             GMSS422SWCPUTemp3(),
             GMSS422SWCPUTemp4(),
             GMSS422SWBCMTemp1(),
             GMSS422SWBCMTemp2(),
             GMSS422SWBCMTemp3(),
             GMSS422SWBCMTemp4(),
             GMSS422SWBCMTemp5(),
             GMSS422SWBCMTemp6(),
             GMSS422SWBCMTemp7(),
             GMSS422SWBCMTemp8(),
             )

# GMS S422-SW class

class GMSS422SW(cumulus.platform.Platform):
    name = 'gms,s422_e3845'
    chip = cumulus.platform.MaverickChip()
    switch = GMSS422SWSwitch(chip)
    cpld = None
    pwms = None
    sensors = GMSS422SWSensors()
    statusleds = None

Platform = GMSS422SW
