#
# Copyright (c) 2019 Cumulus Networks, Inc.  All Rights Reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import syslog

class LenovoNE1064T10GT(cumulus.platform.TridentThree10GPort):
    def __init__(self, label, serdes, phy_addrs, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        cumulus.platform.TridentThree10GPort.__init__(self,
                                                     label=label,
                                                     serdes=serdes,
                                                     phy_types=('84888'),
                                                     phy_addrs=(phy_addrs,),
                                                     phy_layer='10GBaseT',
                                                     leds=None,
                                                     tx_lane_map=tx_lane_map,
                                                     rx_lane_map=rx_lane_map,
                                                     tx_polarity_flip=tx_polarity_flip,
                                                     rx_polarity_flip=rx_polarity_flip,
                                                     unit=0,
                                                     oversub=True)

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 41: 33,
                  42: 34,
                  43: 35,
                  44: 36,
                  45: 37,
                  46: 38,
                  47: 39,
                  48: 40,
                  61: 41,
                  62: 42,
                  63: 43,
                  64: 44,
                  65: 45,
                  66: 46,
                  67: 47,
                  68: 48,
                  69: 49,
                  70: 50,
                  71: 51,
                  72: 52,
                  73: 53,
                  74: 54,
                  75: 55,
                  76: 56,
                  }
        lport = reMap.get(lport, lport)
        return lport

class LenovoNE1064TQSFP28(cumulus.platform.TridentThree100GPort):
    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                       label=label,
                                                       serdes=serdes,
                                                       leds=None,
                                                       phy_types=None,
                                                       phy_addrs=None,
                                                       phy_layer=sfp.type.upper(),
                                                       tx_lane_map=tx_lane_map,
                                                       rx_lane_map=rx_lane_map,
                                                       tx_polarity_flip=tx_polarity_flip,
                                                       rx_polarity_flip=rx_polarity_flip,
                                                       pre_emphasis=pre_emphasis,
                                                       unit=0,
                                                       oversub=True)
        self.sfp = sfp

    # Need to remap logical port numbers to satisfy Maverick2 restrictions
    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 33: 26,
                  29: 25,
                  49: 57,
                  37: 27,
                  57: 59,
                  53: 58,
                  }
        lport = reMap.get(lport, lport)
        return lport

# Switch Ports

class LenovoNE1064TSwitch(cumulus.platform.BCMSwitch):
    _10gt = LenovoNE1064T10GT
    _qsfp28 = LenovoNE1064TQSFP28
    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _10gt(label="1",  serdes=(0,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x01)),
        _10gt(label="2",  serdes=(1,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x00)),
        _10gt(label="3",  serdes=(2,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x03)),
        _10gt(label="4",  serdes=(3,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x02)),

        _10gt(label="5",  serdes=(4,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x05)),
        _10gt(label="6",  serdes=(5,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x04)),
        _10gt(label="7",  serdes=(6,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x07)),
        _10gt(label="8",  serdes=(7,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x06)),

        _10gt(label="9",  serdes=(8,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x09)),
        _10gt(label="10", serdes=(9,),  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x08)),
        _10gt(label="11", serdes=(10,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x0b)),
        _10gt(label="12", serdes=(11,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x0a)),

        _10gt(label="13", serdes=(12,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x21)),
        _10gt(label="14", serdes=(13,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x20)),
        _10gt(label="15", serdes=(14,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x23)),
        _10gt(label="16", serdes=(15,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x22)),

        _10gt(label="17", serdes=(16,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x25)),
        _10gt(label="18", serdes=(17,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x24)),
        _10gt(label="19", serdes=(18,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x27)),
        _10gt(label="20", serdes=(19,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x26)),

        _10gt(label="21", serdes=(20,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x29)),
        _10gt(label="22", serdes=(21,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x28)),
        _10gt(label="23", serdes=(22,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x2b)),
        _10gt(label="24", serdes=(23,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x2a)),

        _10gt(label="25", serdes=(40,), tx_lane_map=0x1302, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x41)),
        _10gt(label="26", serdes=(41,), tx_lane_map=0x1302, rx_lane_map=0x3120, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x40)),
        _10gt(label="27", serdes=(42,), tx_lane_map=0x1302, rx_lane_map=0x3120, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=(0x43)),
        _10gt(label="28", serdes=(43,), tx_lane_map=0x1302, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x42)),

        _10gt(label="29", serdes=(44,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x45)),
        _10gt(label="30", serdes=(45,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x44)),
        _10gt(label="31", serdes=(46,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x47)),
        _10gt(label="32", serdes=(47,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x46)),

        _10gt(label="33", serdes=(60,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x49)),
        _10gt(label="34", serdes=(61,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x48)),
        _10gt(label="35", serdes=(62,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x4b)),
        _10gt(label="36", serdes=(63,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x4a)),

        _10gt(label="37", serdes=(64,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x61)),
        _10gt(label="38", serdes=(65,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x60)),
        _10gt(label="39", serdes=(66,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x63)),
        _10gt(label="40", serdes=(67,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x62)),

        _10gt(label="41", serdes=(68,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x65)),
        _10gt(label="42", serdes=(69,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x64)),
        _10gt(label="43", serdes=(70,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x67)),
        _10gt(label="44", serdes=(71,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x66)),

        _10gt(label="45", serdes=(72,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x69)),
        _10gt(label="46", serdes=(73,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x68)),
        _10gt(label="47", serdes=(74,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x6b)),
        _10gt(label="48", serdes=(75,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True,  phy_addrs=(0x6a)),

        _qsfp28(label="49", serdes=( 32,  33,  34,  35), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="50", serdes=( 28,  29,  30,  31), tx_lane_map=0x2031, rx_lane_map=0x1302, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="51", serdes=( 48,  49,  50,  51), tx_lane_map=0x3120, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="52", serdes=( 36,  37,  38,  39), tx_lane_map=0x3210, rx_lane_map=0x0132, tx_polarity_flip=0x0005, rx_polarity_flip=0x0009),
        _qsfp28(label="53", serdes=( 56,  57,  58,  59), tx_lane_map=0x3120, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="54", serdes=( 52,  53,  54,  55), tx_lane_map=0x1302, rx_lane_map=0x1032, tx_polarity_flip=0x000c, rx_polarity_flip=0x000a),
    )

# CPLD paths

class LenovoNE1064TCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0077'

# PSUs

class LenovoNE1064TPSU(cumulus.sensors.PSU_Unit):
    cpld_path = LenovoNE1064TCPLD.path

class LenovoNE1064TPSU1(LenovoNE1064TPSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class LenovoNE1064TPSU2(LenovoNE1064TPSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# Intel CPU temp sensors

class LenovoNE1064TCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 82
    temp_crit = 104

class LenovoNE1064TCPUTemp1(LenovoNE1064TCoreTemp):
    name = 'CPUTemp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class LenovoNE1064TCPUTemp2(LenovoNE1064TCoreTemp):
    name = 'CPUTemp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class LenovoNE1064TCPUTemp3(LenovoNE1064TCoreTemp):
    name = 'CPUTemp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class LenovoNE1064TCPUTemp4(LenovoNE1064TCoreTemp):
    name = 'CPUTemp4'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class LenovoNE1064TCPUTemp5(LenovoNE1064TCoreTemp):
    name = 'CPUTemp5'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Trident3 internal temp sensors

class LenovoNE1064TASICTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class LenovoNE1064TASICTemp1(LenovoNE1064TASICTemp):
    name = 'ASICTemp1'
    driver_hwmon = ['temp1']

class LenovoNE1064TASICTemp2(LenovoNE1064TASICTemp):
    name = 'ASICTemp2'
    driver_hwmon = ['temp2']

class LenovoNE1064TASICTemp3(LenovoNE1064TASICTemp):
    name = 'ASICTemp3'
    driver_hwmon = ['temp3']

class LenovoNE1064TASICTemp4(LenovoNE1064TASICTemp):
    name = 'ASICTemp4'
    driver_hwmon = ['temp4']

class LenovoNE1064TASICTemp5(LenovoNE1064TASICTemp):
    name = 'ASICTemp5'
    driver_hwmon = ['temp5']

class LenovoNE1064TASICTemp6(LenovoNE1064TASICTemp):
    name = 'ASICTemp6'
    driver_hwmon = ['temp6']

class LenovoNE1064TASICTemp7(LenovoNE1064TASICTemp):
    name = 'ASICTemp7'
    driver_hwmon = ['temp7']

class LenovoNE1064TASICTemp8(LenovoNE1064TASICTemp):
    name = 'ASICTemp8'
    driver_hwmon = ['temp8']

class LenovoNE1064TASICTemp9(LenovoNE1064TASICTemp):
    name = 'ASICTemp9'
    driver_hwmon = ['temp9']

class LenovoNE1064TASICTemp10(LenovoNE1064TASICTemp):
    name = 'ASICTemp10'
    driver_hwmon = ['temp10']

# Board temp sensors

class LenovoNE1064TTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

# U29
class LenovoNE1064TTemp1F2B(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-0048'
    name = 'Temp1'
    description = 'CPU board'
    temp_fan_max = 62
    temp_fan_min = 25

class LenovoNE1064TTemp1B2F(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-0048'
    name = 'Temp1'
    description = 'CPU board'
    temp_fan_max = 53
    temp_fan_min = 23

# U21
class LenovoNE1064TTemp2F2B(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-004a'
    name = 'Temp2'
    description = 'Near PHYs'
    temp_fan_max = 60
    temp_fan_min = 0

class LenovoNE1064TTemp2B2F(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-004a'
    name = 'Temp2'
    description = 'Near PHYs'
    temp_fan_max = 58
    temp_fan_min = 33

# U20
class LenovoNE1064TTemp3(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-004d'
    name = 'Temp3'
    description = 'Near fans'

# U19
class LenovoNE1064TTemp4F2B(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-004e'
    name = 'Temp4'
    description = 'Near switch ASIC'
    temp_fan_max = 60
    temp_fan_min = 10

class LenovoNE1064TTemp4B2F(LenovoNE1064TTemp):
    driver_path = '/sys/bus/i2c/devices/11-004e'
    name = 'Temp4'
    description = 'Near switch ASIC'
    temp_fan_max = 59
    temp_fan_min = 20

# PSU temp sensors

class LenovoNE1064TPSUTemp(cumulus.sensors.Temp_Unit):
    psu = None
    
    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'BAD'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class LenovoNE1064TPSU1Temp(LenovoNE1064TPSUTemp):
    psu = 'PSU1'
    driver_path = '/sys/bus/i2c/devices/10-005a'
    
class LenovoNE1064TPSU1Temp1(LenovoNE1064TPSU1Temp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor 1'
    driver_hwmon = ['temp1']
    
class LenovoNE1064TPSU1Temp2(LenovoNE1064TPSU1Temp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp Sensor 2'
    driver_hwmon = ['temp2']
    
class LenovoNE1064TPSU1Temp3(LenovoNE1064TPSU1Temp):
    name = 'PSU1Temp3'
    description = 'PSU1 Temp Sensor 3'
    driver_hwmon = ['temp3']
    
class LenovoNE1064TPSU2Temp(LenovoNE1064TPSUTemp):
    psu = 'PSU2'
    driver_path = '/sys/bus/i2c/devices/10-005b'
    
class LenovoNE1064TPSU2Temp1(LenovoNE1064TPSU2Temp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor 1'
    driver_hwmon = ['temp1']
    
class LenovoNE1064TPSU2Temp2(LenovoNE1064TPSU2Temp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp Sensor 2'
    driver_hwmon = ['temp2']
    
class LenovoNE1064TPSU2Temp3(LenovoNE1064TPSU2Temp):
    name = 'PSU2Temp3'
    description = 'PSU2 Temp Sensor 3'
    driver_hwmon = ['temp3']
    
# Fans

class LenovoNE1064TFan(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    cpld_path = LenovoNE1064TCPLD.path
    driver_path = cpld_path
    maxpwm = 255
    minpwm = 50

class LenovoNE1064TInletFan(LenovoNE1064TFan):
    fan_max = 23000
    fan_min = 3000

class LenovoNE1064TOutletFan(LenovoNE1064TFan):
    fan_max = 20200
    fan_min = 2700

class LenovoNE1064TInletFan1(LenovoNE1064TInletFan):
    name = 'Fan1'
    description = 'Fan1 Inlet'
    cpld_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']

class LenovoNE1064TInletFan2(LenovoNE1064TInletFan):
    name = 'Fan2'
    description = 'Fan2 Inlet'
    cpld_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']

class LenovoNE1064TInletFan3(LenovoNE1064TInletFan):
    name = 'Fan3'
    description = 'Fan3 Inlet'
    cpld_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']

class LenovoNE1064TInletFan4(LenovoNE1064TInletFan):
    name = 'Fan4'
    description = 'Fan4 Inlet'
    cpld_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']

class LenovoNE1064TInletFan5(LenovoNE1064TInletFan):
    name = 'Fan5'
    description = 'Fan5 Inlet'
    cpld_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']

class LenovoNE1064TOutletFan1(LenovoNE1064TOutletFan):
    name = 'Fan6'
    description = 'Fan1 Outlet'
    cpld_hwmon = ['fan6']
    pwm_hwmon = ['pwm1']

class LenovoNE1064TOutletFan2(LenovoNE1064TOutletFan):
    name = 'Fan7'
    description = 'Fan2 Outlet'
    cpld_hwmon = ['fan7']
    pwm_hwmon = ['pwm2']

class LenovoNE1064TOutletFan3(LenovoNE1064TOutletFan):
    name = 'Fan8'
    description = 'Fan3 Outlet'
    cpld_hwmon = ['fan8']
    pwm_hwmon = ['pwm3']

class LenovoNE1064TOutletFan4(LenovoNE1064TOutletFan):
    name = 'Fan9'
    description = 'Fan4 Outlet'
    cpld_hwmon = ['fan9']
    pwm_hwmon = ['pwm4']

class LenovoNE1064TOutletFan5(LenovoNE1064TOutletFan):
    name = 'Fan10'
    description = 'Fan5 Outlet'
    cpld_hwmon = ['fan10']
    pwm_hwmon = ['pwm5']

# LEDs

'''
    Lenovo NE1064T LEDs

    System:
        blue             - all systems OK
        blue blinking    - service required
    Stacking:
        off              - not used
    Power:
        green            - both PSUs OK
        green blinking   - power supply failure
    Fan:
        green            - all fans OK
        green blinking   - fan failure
'''

class LenovoNE1064TLED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE1064TCPLD.path

class LenovoNE1064TSystemLED(LenovoNE1064TLED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'blue'
    bad_led_color = 'blue_blinking'

class LenovoNE1064TPowerLED(LenovoNE1064TLED):
    name = 'Power LED'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    good_led_color = 'green'
    bad_led_color = 'green_blinking'

class LenovoNE1064TFanLED(LenovoNE1064TLED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5']
    good_led_color = 'green'
    bad_led_color = 'green_blinking'

class LenovoNE1064TFanTrayLED(LenovoNE1064TLED):
    good_led_color = 'green'
    bad_led_color = 'red'

class LenovoNE1064TFan1LED(LenovoNE1064TFanTrayLED):
    name = 'Fan1 LED'
    led_location = 'led_fan1'
    sources = ['Fan1']

class LenovoNE1064TFan2LED(LenovoNE1064TFanTrayLED):
    name = 'Fan2 LED'
    led_location = 'led_fan2'
    sources = ['Fan2']

class LenovoNE1064TFan3LED(LenovoNE1064TFanTrayLED):
    name = 'Fan3 LED'
    led_location = 'led_fan3'
    sources = ['Fan3']

class LenovoNE1064TFan4LED(LenovoNE1064TFanTrayLED):
    name = 'Fan4 LED'
    led_location = 'led_fan4'
    sources = ['Fan4']

class LenovoNE1064TFan5LED(LenovoNE1064TFanTrayLED):
    name = 'Fan5 LED'
    led_location = 'led_fan5'
    sources = ['Fan5']

# detect airflow direction

def LenovoNE1064T_airflow():
    fan_type_count = {'F2B': 0, 'B2F': 0}
    # Default to F2B type
    fan_type = 1
    b_info = ''

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'lenovo,ne1064t':
        return None
    for fan in ['fan1', 'fan2', 'fan3', 'fan4', 'fan5']:
        fan_direction = '%s/%s_direction' % (LenovoNE1064TCPLD.path, fan)
        try:
            b_info = subprocess.check_output(['cat', fan_direction])
            if b_info != '':
                if b_info.strip() == 'F2B':
                    fan_type_count['F2B'] += 1
                if b_info.strip() == 'B2F':
                    fan_type_count['B2F'] += 1
            else:
                continue
        except Exception:
            b_info = ''

    if fan_type_count['F2B'] == 0 and fan_type_count['B2F'] == 0:
        syslog.syslog(syslog.LOG_WARNING, "Airflow types (Front-to-Back/Back-to-Front) for all the fans could "
                      "not be detected, defaulting to F2B")
    elif fan_type_count['F2B'] in xrange(1, 6) and fan_type_count['B2F'] == 0:
        fan_type = 1
    elif fan_type_count['B2F'] in xrange(1, 6) and fan_type_count['F2B'] == 0:
        fan_type = 2
    else:
        syslog.syslog(syslog.LOG_WARNING, "Fans with different airflow types (Front-to-Back/Back-to-Front) "
                      "detected, defaulting to F2B")

    if fan_type == 2:
        return 'B2F'
    else:
        return 'F2B'

class LenovoNE1064TSensors():
    units = (LenovoNE1064TPSU1(),
             LenovoNE1064TPSU2(),
             LenovoNE1064TCPUTemp1(),
             LenovoNE1064TCPUTemp2(),
             LenovoNE1064TCPUTemp3(),
             LenovoNE1064TCPUTemp4(),
             LenovoNE1064TCPUTemp5(),
             LenovoNE1064TASICTemp1(),
             LenovoNE1064TASICTemp2(),
             LenovoNE1064TASICTemp3(),
             LenovoNE1064TASICTemp4(),
             LenovoNE1064TASICTemp5(),
             LenovoNE1064TASICTemp6(),
             LenovoNE1064TASICTemp7(),
             LenovoNE1064TASICTemp8(),
             LenovoNE1064TASICTemp9(),
             LenovoNE1064TASICTemp10(),
             LenovoNE1064TPSU1Temp1(),
             LenovoNE1064TPSU1Temp2(),
             LenovoNE1064TPSU1Temp3(),
             LenovoNE1064TPSU2Temp1(),
             LenovoNE1064TPSU2Temp2(),
             LenovoNE1064TPSU2Temp3(),
             LenovoNE1064TInletFan1(),
             LenovoNE1064TInletFan2(),
             LenovoNE1064TInletFan3(),
             LenovoNE1064TInletFan4(),
             LenovoNE1064TInletFan5(),
             LenovoNE1064TOutletFan1(),
             LenovoNE1064TOutletFan2(),
             LenovoNE1064TOutletFan3(),
             LenovoNE1064TOutletFan4(),
             LenovoNE1064TOutletFan5(),
             )
    F2B_units = (LenovoNE1064TTemp1F2B(),
                 LenovoNE1064TTemp2F2B(),
                 LenovoNE1064TTemp3(),
                 LenovoNE1064TTemp4F2B(),
                 )
    B2F_units = (LenovoNE1064TTemp1B2F(),
                 LenovoNE1064TTemp2B2F(),
                 LenovoNE1064TTemp3(),
                 LenovoNE1064TTemp4B2F(),
                 )
    # default to F2B if airflow can't be determined
    if LenovoNE1064T_airflow() == 'B2F':
        units = units + B2F_units
    else:
        units = units + F2B_units

class LenovoNE1064TPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (LenovoNE1064TInletFan1(),
            LenovoNE1064TInletFan2(),
            LenovoNE1064TInletFan3(),
            LenovoNE1064TInletFan4(),
            LenovoNE1064TInletFan5(),
            LenovoNE1064TOutletFan1(),
            LenovoNE1064TOutletFan2(),
            LenovoNE1064TOutletFan3(),
            LenovoNE1064TOutletFan4(),
            LenovoNE1064TOutletFan5(),
)
    temps = (LenovoNE1064TCPUTemp1(),
             LenovoNE1064TCPUTemp2(),
             LenovoNE1064TCPUTemp3(),
             LenovoNE1064TCPUTemp4(),
             LenovoNE1064TCPUTemp5(),
             LenovoNE1064TASICTemp1(),
             LenovoNE1064TASICTemp2(),
             LenovoNE1064TASICTemp3(),
             LenovoNE1064TASICTemp4(),
             LenovoNE1064TASICTemp5(),
             LenovoNE1064TASICTemp6(),
             LenovoNE1064TASICTemp7(),
             LenovoNE1064TASICTemp8(),
             LenovoNE1064TASICTemp9(),
             LenovoNE1064TASICTemp10(),
             )
    F2B_temps = (LenovoNE1064TTemp1F2B(),
                 LenovoNE1064TTemp2F2B(),
                 LenovoNE1064TTemp3(),
                 LenovoNE1064TTemp4F2B(),
                 )
    B2F_temps = (LenovoNE1064TTemp1B2F(),
                 LenovoNE1064TTemp2B2F(),
                 LenovoNE1064TTemp3(),
                 LenovoNE1064TTemp4B2F(),
                 )
    # default to F2B if airflow can't be determined
    if LenovoNE1064T_airflow() == 'B2F':
        temps = temps + B2F_temps
    else:
        temps = temps + F2B_temps

class LenovoNE1064TPWMs():
    units = (LenovoNE1064TPWM1(),
             )

class LenovoNE1064TLEDs():
    units = (LenovoNE1064TSystemLED(),
             LenovoNE1064TPowerLED(),
             LenovoNE1064TFanLED(),
             LenovoNE1064TFan1LED(),
             LenovoNE1064TFan2LED(),
             LenovoNE1064TFan3LED(),
             LenovoNE1064TFan4LED(),
             LenovoNE1064TFan5LED(),
             )

class LenovoNE1064T(cumulus.platform.Platform):
    name = 'lenovo,ne1064t'
    chip = cumulus.platform.TridentThreeX5_56771_Chip()
    switch = LenovoNE1064TSwitch(chip)
    sensors = LenovoNE1064TSensors()
    statusleds = LenovoNE1064TLEDs()
    pwms = LenovoNE1064TPWMs()

Platform = LenovoNE1064T
