#
# Copyright (c) 2018,2019,2020 Cumulus Networks,  Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class LenovoNE2572SFP28(cumulus.platform.Tomahawk25GPort):
    _set_led = '''
;;; Lenovo NE2572
;;;
;;; Each 25G port has two bits of LED data controlling a bi-color green/amber
;;; LED.  Each LED behaves as follows:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 | both  | green |
;;;  2nd bit   +-------+-------+
;;;          1 | amber |  off  |
;;;            +-------+-------+
set_led:
        call    set_sfp_led
        ret
'''

    def __init__(self, label, serdes, leds, phy_addrs, phy_types, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=phy_types,
                                                   phy_addrs=phy_addrs,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None

        return set_calls

    def hw_intf_num_portmap(self, mode, sub):
        '''
        Return the physical port interface number to be used in portmap
        '''
        hwintf = self.hw_intf_num(mode, sub)
        # Need to remap first eight ports on lenovo,ne2572 to correct front
        # panel ports. See CM-22612.
        reMap = { 32 : 35,
                  33 : 33,
                  34 : 34,
                  35 : 32,
                  36 : 38,
                  37 : 39,
                  38 : 36,
                  39 : 37}
        hwintf = reMap.get(hwintf, hwintf)
        return int(hwintf)

class LenovoNE2572QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
;;; Lenovo NE2572
;;;
;;; Each 100G port has eight bits of LED data controlling four bi-color green/other
;;; LEDs.  Each LED behaves as follows:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 | both  | other |
;;;  2nd bit   +-------+-------+
;;;          1 | green |  off  |
;;;            +-------+-------+
;;;
;;; '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_sfp_led:

        pushst  PORTSTATUS_ONE
        pack

set_qsfp_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack
	ret

set_led_off:
; pack two '1' bits
;  11 => off
        pushst PORTSTATUS_ONE
        pack
        pushst PORTSTATUS_ONE
        pack
        ret

'''
    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity.
;;;
set_led:
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack
        call    set_led_off
        call    set_led_off
        call    set_led_off
        ret
'''
    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and third bi-color LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        call    set_led_off

        inc     a
        inc     a
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        call    set_led_off
        ret
'''
    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four bi-color LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        inc     a
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        inc     a
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        inc     a
        call    set_qsfp_led
        pushst  PORTSTATUS_ONE
        pack

        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class LenovoNE2572LED0(cumulus.ledup.TomahawkLEDController):
    led_functions = (LenovoNE2572QSFP28._set_led_common)
    num_leds = 88
    has_phys_id = True

class LenovoNE2572LED1(cumulus.ledup.TomahawkLEDController):
    led_functions = (LenovoNE2572QSFP28._set_led_common)
    num_leds = 72
    has_phys_id = True

# Switch Ports

class LenovoNE2572Switch(cumulus.platform.BCMSwitch):
    ledup = (LenovoNE2572LED0, LenovoNE2572LED1)
    _sfp28 = LenovoNE2572SFP28
    _qsfp28 = LenovoNE2572QSFP28

    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp28(label="1",  serdes=( 32,), leds=(1,  64), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x20,), phy_types=("82381",),),
        _sfp28(label="2",  serdes=( 33,), leds=(1,  68), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x20,), phy_types=("82381",),),
        _sfp28(label="3",  serdes=( 34,), leds=(1,  66), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x20,), phy_types=("82381",),),
        _sfp28(label="4",  serdes=( 35,), leds=(1,  70), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x20,), phy_types=("82381",),),

        _sfp28(label="5",  serdes=( 36,), leds=(1,  58), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x21,), phy_types=("82381",),),
        _sfp28(label="6",  serdes=( 37,), leds=(1,  56), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x21,), phy_types=("82381",),),
        _sfp28(label="7",  serdes=( 38,), leds=(1,  62), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x21,), phy_types=("82381",),),
        _sfp28(label="8",  serdes=( 39,), leds=(1,  60), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=(0x21,), phy_types=("82381",),),

        _sfp28(label="9",  serdes=( 40,), leds=(1,  54), tx_lane_map=0x2310, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=None, phy_types=None,),
        _sfp28(label="10", serdes=( 41,), leds=(1,  52), tx_lane_map=0x2310, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="11", serdes=( 42,), leds=(1,  50), tx_lane_map=0x2310, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True,  phy_addrs=None, phy_types=None,),
        _sfp28(label="12", serdes=( 43,), leds=(1,  48), tx_lane_map=0x2310, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="13", serdes=( 48,), leds=(1,  46), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="14", serdes=( 49,), leds=(1,  44), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="15", serdes=( 50,), leds=(1,  42), tx_lane_map=0x0123, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="16", serdes=( 51,), leds=(1,  40), tx_lane_map=0x0123, rx_lane_map=0x3120, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="17", serdes=( 52,), leds=(1,  38), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="18", serdes=( 53,), leds=(1,  36), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="19", serdes=( 54,), leds=(1,  34), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="20", serdes=( 55,), leds=(1,  32), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="21", serdes=( 64,), leds=(1,  30), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="22", serdes=( 65,), leds=(1,  28), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="23", serdes=( 66,), leds=(1,  26), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="24", serdes=( 67,), leds=(1,  24), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="25", serdes=( 68,), leds=(1,  22), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="26", serdes=( 69,), leds=(1,  20), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="27", serdes=( 70,), leds=(1,  18), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="28", serdes=( 71,), leds=(1,  16), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="29", serdes=( 80,), leds=(1,  14), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="30", serdes=( 81,), leds=(1,  12), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="31", serdes=( 82,), leds=(1,  10), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="32", serdes=( 83,), leds=(1,   8), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _sfp28(label="33", serdes=( 84,), leds=(1,   6), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="34", serdes=( 85,), leds=(1,   4), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="35", serdes=( 86,), leds=(1,   2), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="36", serdes=( 87,), leds=(1,   0), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),



        _sfp28(label="37", serdes=( 96,), leds=(0,  86), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="38", serdes=( 97,), leds=(0,  84), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="39", serdes=( 98,), leds=(0,  82), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="40", serdes=( 99,), leds=(0,  80), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="41", serdes=(100,), leds=(0,  78), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="42", serdes=(101,), leds=(0,  76), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="43", serdes=(102,), leds=(0,  74), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="44", serdes=(103,), leds=(0,  72), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="45", serdes=(104,), leds=(0,  70), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="46", serdes=(105,), leds=(0,  68), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="47", serdes=(106,), leds=(0,  66), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),
        _sfp28(label="48", serdes=(107,), leds=(0,  64), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False, phy_addrs=None, phy_types=None,),

        _qsfp28(label="49", serdes=(108, 109, 110, 111), leds=(0,   0), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0x0002, rx_polarity_flip=0x0000),
        _qsfp28(label="50", serdes=(116, 117, 118, 119), leds=(0,   8), tx_lane_map=0x1230, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0008),
        _qsfp28(label="51", serdes=(  0,   1,   2,   3), leds=(0,  16), tx_lane_map=0x1032, rx_lane_map=0x3210, tx_polarity_flip=0x0001, rx_polarity_flip=0x0008),
        _qsfp28(label="52", serdes=(  4,   5,   6,   7), leds=(0,  24), tx_lane_map=0x1230, rx_lane_map=0x1230, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="53", serdes=(  8,   9,  10,  11), leds=(0,  32), tx_lane_map=0x3210, rx_lane_map=0x3201, tx_polarity_flip=0x0000, rx_polarity_flip=0x0000),
        _qsfp28(label="54", serdes=( 20,  21,  22,  23), leds=(0,  40), tx_lane_map=0x3210, rx_lane_map=0x3012, tx_polarity_flip=0x0000, rx_polarity_flip=0x0005),
    )

class LenovoNE2572CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/lenovo_ne2572_cpld.0'

# Intel CPU temp sensors

class LenovoNE2572CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    # Need the official temp values
    temp_fan_min = 65
    temp_max = 85
    temp_crit = 98
            
class LenovoNE2572CPUTemp1(LenovoNE2572CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class LenovoNE2572CPUTemp2(LenovoNE2572CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class LenovoNE2572CPUTemp3(LenovoNE2572CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class LenovoNE2572CPUTemp4(LenovoNE2572CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Tomahawk internal temp sensors

class LenovoNE2572TomahawkTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    # Need the official temp values
    temp_max     = 105
    temp_crit    = 110

class LenovoNE2572TomahawkTemp1(LenovoNE2572TomahawkTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class LenovoNE2572TomahawkTemp2(LenovoNE2572TomahawkTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class LenovoNE2572TomahawkTemp3(LenovoNE2572TomahawkTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class LenovoNE2572TomahawkTemp4(LenovoNE2572TomahawkTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class LenovoNE2572TomahawkTemp5(LenovoNE2572TomahawkTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class LenovoNE2572TomahawkTemp6(LenovoNE2572TomahawkTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class LenovoNE2572TomahawkTemp7(LenovoNE2572TomahawkTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class LenovoNE2572TomahawkTemp8(LenovoNE2572TomahawkTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

# Board temp sensors

class LenovoNE2572Temp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    temp_max = 61
    temp_fan_max = 68
    temp_crit = 75

class LenovoNE2572Temp1(LenovoNE2572Temp):
    driver_path = '/sys/bus/i2c/devices/0-004f'
    name = 'Temp13'
    description = 'CPU board'
    
class LenovoNE2572Temp2(LenovoNE2572Temp):
    driver_path = '/sys/bus/i2c/devices/12-004d'
    name = 'Temp14'
    description = 'Hot spot'
    
class LenovoNE2572Temp3(LenovoNE2572Temp):
    driver_path = '/sys/bus/i2c/devices/13-004c'
    name = 'Temp15'
    description = 'Ambient'
    
# Fans

class LenovoNE2572FanBase(cumulus.sensors.Fan_Unit):
    cpld_path = LenovoNE2572CPLD.path
    driver_path = cpld_path
    minpwm = 0

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class LenovoNE2572Fan_0(LenovoNE2572FanBase):
    fan_min = 0
    fan_max = 19000
    
class LenovoNE2572Fan_1(LenovoNE2572FanBase):
    fan_min = 0
    fan_max = 21000
    
class LenovoNE2572Fan1(LenovoNE2572Fan_0):
    fan_present = 'fantray1_present'
    name = 'Fan1_0'
    description = 'Fan1_0'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan2(LenovoNE2572Fan_1):
    fan_present = 'fantray1_present'
    name = 'Fan1_1'
    description = 'Fan1_1'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan3(LenovoNE2572Fan_0):
    fan_present = 'fantray2_present'
    name = 'Fan2_0'
    description = 'Fan2_0'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan4(LenovoNE2572Fan_1):
    fan_present = 'fantray2_present'
    name = 'Fan2_1'
    description = 'Fan2_1'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan5(LenovoNE2572Fan_0):
    fan_present = 'fantray3_present'
    name = 'Fan3_0'
    description = 'Fan3_0'
    driver_hwmon = ['fan5']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan6(LenovoNE2572Fan_1):
    fan_present = 'fantray3_present'
    name = 'Fan3_1'
    description = 'Fan3_1'
    driver_hwmon = ['fan6']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan7(LenovoNE2572Fan_0):
    fan_present = 'fantray4_present'
    name = 'Fan4_0'
    description = 'Fan4_0'
    driver_hwmon = ['fan7']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan8(LenovoNE2572Fan_1):
    fan_present = 'fantray4_present'
    name = 'Fan4_1'
    description = 'Fan4_1'
    driver_hwmon = ['fan8']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan9(LenovoNE2572Fan_0):
    fan_present = 'fantray5_present'
    name = 'Fan5_0'
    description = 'Fan5_0'
    driver_hwmon = ['fan9']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan10(LenovoNE2572Fan_1):
    fan_present = 'fantray5_present'
    name = 'Fan5_1'
    description = 'Fan5_1'
    driver_hwmon = ['fan10']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan11(LenovoNE2572Fan_0):
    fan_present = 'fantray6_present'
    name = 'Fan6_0'
    description = 'Fan6_0'
    driver_hwmon = ['fan11']
    pwm_hwmon = ['fan_pwm']

class LenovoNE2572Fan12(LenovoNE2572Fan_1):
    fan_present = 'fantray6_present'
    name = 'Fan6_1'
    description = 'Fan6_1'
    driver_hwmon = ['fan12']
    pwm_hwmon = ['fan_pwm']

# PSUs

class LenovoNE2572PSU(cumulus.sensors.PSU_Unit):
    cpld_path = LenovoNE2572CPLD.path

class LenovoNE2572PSU1(LenovoNE2572PSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class LenovoNE2572PSU2(LenovoNE2572PSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# LEDs
'''
   Lenovo NE2572 LEDs

   System:
       off            - all systems OK
       blue blinking  - service required
   Stacking:
       off            - not used
   Power:
       green          - power OK
       green blinking - power supply failure
   Fan:
       green          - fans OK
       green blinking - fan failure
   6 x Rear Fan:
       green          - fans OK
       green blinking - fan failure
'''
class LenovoNE2572LED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE2572CPLD.path
    
class LenovoNE2572SystemLED(LenovoNE2572LED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'off'
    bad_led_color = 'blue_blinking'

class LenovoNE2572PowerLED(LenovoNE2572LED):
    name = 'Power LED'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_psu'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanLED(LenovoNE2572LED):
    name = 'Fan LED'
    sources = ['Fan1_0', 'Fan1_1',
               'Fan2_0', 'Fan2_1',
               'Fan3_0', 'Fan3_1',
               'Fan4_0', 'Fan4_1',
               'Fan5_0', 'Fan5_1',
               'Fan6_0', 'Fan6_1']
    led_location = 'led_fan'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray1RearLED(LenovoNE2572LED):
    name = 'Fan Tray 1 Rear LED'
    sources = ['Fan1_0', 'Fan1_1']
    led_location = 'led_fan1_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray2RearLED(LenovoNE2572LED):
    name = 'Fan Tray 2 Rear LED'
    sources = ['Fan2_0', 'Fan2_1']
    led_location = 'led_fan2_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray3RearLED(LenovoNE2572LED):
    name = 'Fan Tray 3 Rear LED'
    sources = ['Fan3_0', 'Fan3_1']
    led_location = 'led_fan3_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray4RearLED(LenovoNE2572LED):
    name = 'Fan Tray 4 Rear LED'
    sources = ['Fan4_0', 'Fan4_1']
    led_location = 'led_fan4_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray5RearLED(LenovoNE2572LED):
    name = 'Fan Tray 5 Rear LED'
    sources = ['Fan5_0', 'Fan5_1']
    led_location = 'led_fan5_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572FanTray6RearLED(LenovoNE2572LED):
    name = 'Fan Tray 6 Rear LED'
    sources = ['Fan6_0', 'Fan6_1']
    led_location = 'led_fan6_rear'
    bad_led_color = 'green_blinking'

class LenovoNE2572PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    fans = (LenovoNE2572Fan1(),
            LenovoNE2572Fan2(),
            LenovoNE2572Fan3(),
            LenovoNE2572Fan4(),
            LenovoNE2572Fan5(),
            LenovoNE2572Fan6(),
            LenovoNE2572Fan7(),
            LenovoNE2572Fan8(),
            LenovoNE2572Fan9(),
            LenovoNE2572Fan10(),
            LenovoNE2572Fan11(),
            LenovoNE2572Fan12(),
            )

    temps = (LenovoNE2572CPUTemp1(),
             LenovoNE2572CPUTemp2(),
             LenovoNE2572CPUTemp3(),
             LenovoNE2572CPUTemp4(),
             LenovoNE2572TomahawkTemp1(),
             LenovoNE2572TomahawkTemp2(),
             LenovoNE2572TomahawkTemp3(),
             LenovoNE2572TomahawkTemp4(),
             LenovoNE2572TomahawkTemp5(),
             LenovoNE2572TomahawkTemp6(),
             LenovoNE2572TomahawkTemp7(),
             LenovoNE2572TomahawkTemp8(),
             LenovoNE2572Temp1(),
             LenovoNE2572Temp2(),
             LenovoNE2572Temp3(),
             )

# Units

class LenovoNE2572PWMs():
    units = (LenovoNE2572PWM1(),
             )

class LenovoNE2572LEDs():
    units = (LenovoNE2572SystemLED(),
             LenovoNE2572PowerLED(),
             LenovoNE2572FanLED(),
             LenovoNE2572FanTray1RearLED(),
             LenovoNE2572FanTray2RearLED(),
             LenovoNE2572FanTray3RearLED(),
             LenovoNE2572FanTray4RearLED(),
             LenovoNE2572FanTray5RearLED(),
             LenovoNE2572FanTray6RearLED(),
             )
    
class LenovoNE2572Sensors():
    units = (LenovoNE2572CPUTemp1(),
             LenovoNE2572CPUTemp2(),
             LenovoNE2572CPUTemp3(),
             LenovoNE2572CPUTemp4(),
             LenovoNE2572TomahawkTemp1(),
             LenovoNE2572TomahawkTemp2(),
             LenovoNE2572TomahawkTemp3(),
             LenovoNE2572TomahawkTemp4(),
             LenovoNE2572TomahawkTemp5(),
             LenovoNE2572TomahawkTemp6(),
             LenovoNE2572TomahawkTemp7(),
             LenovoNE2572TomahawkTemp8(),
             LenovoNE2572Temp1(),
             LenovoNE2572Temp2(),
             LenovoNE2572Temp3(),
             LenovoNE2572Fan1(),
             LenovoNE2572Fan2(),
             LenovoNE2572Fan3(),
             LenovoNE2572Fan4(),
             LenovoNE2572Fan5(),
             LenovoNE2572Fan6(),
             LenovoNE2572Fan7(),
             LenovoNE2572Fan8(),
             LenovoNE2572Fan9(),
             LenovoNE2572Fan10(),
             LenovoNE2572Fan11(),
             LenovoNE2572Fan12(),
             LenovoNE2572PSU1(),
             LenovoNE2572PSU2(),
             )

class LenovoNE2572(cumulus.platform.Platform):
    name = 'lenovo,ne2572'
    chip = cumulus.platform.TomahawkPlus_56967_Chip()
    switch = LenovoNE2572Switch(chip)
    cpld = LenovoNE2572CPLD()
    statusleds = LenovoNE2572LEDs()
    pwms = LenovoNE2572PWMs()
    sensors = LenovoNE2572Sensors()

Platform = LenovoNE2572
