#
# Copyright (c) 2019-2020 Cumulus Networks,  Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import syslog

class LenovoNE2580SFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

class LenovoNE2580QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# Lenovo NE2580 switch ports

class LenovoNE2580Switch(cumulus.platform.BCMSwitch):
    _sfp28 = LenovoNE2580SFP28
    _qsfp28 = LenovoNE2580QSFP28

    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # TSCF0
        _sfp28(label="1",  serdes=(  0,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="2",  serdes=(  1,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="3",  serdes=(  2,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="4",  serdes=(  3,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF1
        _sfp28(label="5",  serdes=(  4,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="6",  serdes=(  5,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=(  6,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="8",  serdes=(  7,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF3
        _sfp28(label="9",  serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="10", serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="11", serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="12", serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF5
        _sfp28(label="13", serdes=( 20,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="14", serdes=( 21,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="15", serdes=( 22,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="16", serdes=( 23,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF7
        _sfp28(label="17", serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="18", serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="19", serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="20", serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF8
        _sfp28(label="21", serdes=( 32,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="22", serdes=( 33,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="23", serdes=( 34,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="24", serdes=( 35,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF10
        _sfp28(label="25", serdes=( 40,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="26", serdes=( 41,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="27", serdes=( 42,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="28", serdes=( 43,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF12
        _sfp28(label="29", serdes=( 48,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="30", serdes=( 49,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="31", serdes=( 50,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="32", serdes=( 51,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF14
        _sfp28(label="33", serdes=( 56,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="34", serdes=( 57,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="35", serdes=( 58,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="36", serdes=( 59,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF15
        _sfp28(label="37", serdes=( 60,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="38", serdes=( 61,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="39", serdes=( 62,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="40", serdes=( 63,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF16
        _sfp28(label="41", serdes=( 64,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="42", serdes=( 65,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="43", serdes=( 66,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="44", serdes=( 67,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF17
        _sfp28(label="45", serdes=( 68,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="46", serdes=( 69,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="47", serdes=( 70,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="48", serdes=( 71,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF21
        _qsfp28(label="49", serdes=( 84,  85,  86,  87), tx_lane_map=0x3120, rx_lane_map=0x0213, tx_polarity_flip=0xc, rx_polarity_flip=0x3),

        # TSCF19
        _qsfp28(label="50", serdes=( 76,  77,  78,  79), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xc, rx_polarity_flip=0x3),

        # TSCF23
        _qsfp28(label="51", serdes=( 92,  93,  94,  95), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xe, rx_polarity_flip=0x3),

        # TSCF24
        _qsfp28(label="52", serdes=( 96,  97,  98,  99), tx_lane_map=0x3120, rx_lane_map=0x0213, tx_polarity_flip=0xc, rx_polarity_flip=0xc),

        # TSCF28
        _qsfp28(label="53", serdes=(112, 113, 114, 115), tx_lane_map=0x3120, rx_lane_map=0x1203, tx_polarity_flip=0xc, rx_polarity_flip=0x1),

        # TSCF26
        _qsfp28(label="54", serdes=(104, 105, 106, 107), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xc, rx_polarity_flip=0x3),

        # TSCF30
        _qsfp28(label="55", serdes=(120, 121, 122, 123), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xe, rx_polarity_flip=0x3),

        # TSCF31
        _qsfp28(label="56", serdes=(124, 125, 126, 127), tx_lane_map=0x3120, rx_lane_map=0x0213, tx_polarity_flip=0xc, rx_polarity_flip=0xc),
        )

# CPLD paths

class LenovoNE2580CPLD1(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0077'

class LenovoNE2580CPLD2(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0033'

# PSUs

class LenovoNE2580PSU(cumulus.sensors.PSU_Unit):
    cpld_path = LenovoNE2580CPLD2.path

class LenovoNE2580PSU1(LenovoNE2580PSU):
    name = 'PSU1'
    description = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class LenovoNE2580PSU2(LenovoNE2580PSU):
    name = 'PSU2'
    description = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

# Intel CPU temp sensors

class LenovoNE2580CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 98
    temp_crit = 101

class LenovoNE2580CPUTemp1(LenovoNE2580CoreTemp):
    name = 'CPUTemp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class LenovoNE2580CPUTemp2(LenovoNE2580CoreTemp):
    name = 'CPUTemp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class LenovoNE2580CPUTemp3(LenovoNE2580CoreTemp):
    name = 'CPUTemp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class LenovoNE2580CPUTemp4(LenovoNE2580CoreTemp):
    name = 'CPUTemp4'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class LenovoNE2580CPUTemp5(LenovoNE2580CoreTemp):
    name = 'CPUTemp5'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Trident3 internal temp sensors

class LenovoNE2580ASICTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110

class LenovoNE2580ASICTemp1(LenovoNE2580ASICTemp):
    name = 'ASICTemp1'
    driver_hwmon = ['temp1']

class LenovoNE2580ASICTemp2(LenovoNE2580ASICTemp):
    name = 'ASICTemp2'
    driver_hwmon = ['temp2']

class LenovoNE2580ASICTemp3(LenovoNE2580ASICTemp):
    name = 'ASICTemp3'
    driver_hwmon = ['temp3']

class LenovoNE2580ASICTemp4(LenovoNE2580ASICTemp):
    name = 'ASICTemp4'
    driver_hwmon = ['temp4']

class LenovoNE2580ASICTemp5(LenovoNE2580ASICTemp):
    name = 'ASICTemp5'
    driver_hwmon = ['temp5']

class LenovoNE2580ASICTemp6(LenovoNE2580ASICTemp):
    name = 'ASICTemp6'
    driver_hwmon = ['temp6']

class LenovoNE2580ASICTemp7(LenovoNE2580ASICTemp):
    name = 'ASICTemp7'
    driver_hwmon = ['temp7']

class LenovoNE2580ASICTemp8(LenovoNE2580ASICTemp):
    name = 'ASICTemp8'
    driver_hwmon = ['temp8']

class LenovoNE2580ASICTemp9(LenovoNE2580ASICTemp):
    name = 'ASICTemp9'
    driver_hwmon = ['temp9']

class LenovoNE2580ASICTemp10(LenovoNE2580ASICTemp):
    name = 'ASICTemp10'
    driver_hwmon = ['temp10']

class LenovoNE2580ASICTemp11(LenovoNE2580ASICTemp):
    name = 'ASICTemp11'
    driver_hwmon = ['temp11']

class LenovoNE2580ASICTemp12(LenovoNE2580ASICTemp):
    name = 'ASICTemp12'
    driver_hwmon = ['temp12']

# Board temp sensors

class LenovoNE2580Temp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

# U29
class LenovoNE2580Temp1F2B(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-0048'
    name = 'Temp1'
    description = 'CPU board'
    temp_max     = 58
    temp_crit    = 62
    temp_fan_max = 62
    temp_fan_min = 37

class LenovoNE2580Temp1B2F(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-0048'
    name = 'Temp1'
    description = 'CPU board'
    temp_max     = 60
    temp_crit    = 64
    temp_fan_max = 63
    temp_fan_min = 27

# U14
class LenovoNE2580Temp2F2B(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-004a'
    name = 'Temp2'
    description = 'Near QSFP28 ports'
    temp_max     = 56
    temp_crit    = 60
    temp_fan_max = 60
    temp_fan_min = 15

class LenovoNE2580Temp2B2F(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-004a'
    name = 'Temp2'
    description = 'Near QSFP28 ports'
    temp_max     = 63
    temp_crit    = 67
    temp_fan_max = 65
    temp_fan_min = 41

# U13
class LenovoNE2580Temp3(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-004d'
    name = 'Temp3'
    description = 'Near fans'

# U12
class LenovoNE2580Temp4F2B(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-004e'
    name = 'Temp4'
    description = 'Near switch ASIC'
    temp_max     = 56
    temp_crit    = 60
    temp_fan_max = 60
    temp_fan_min = 22

class LenovoNE2580Temp4B2F(LenovoNE2580Temp):
    driver_path = '/sys/bus/i2c/devices/11-004e'
    name = 'Temp4'
    description = 'Near switch ASIC'
    temp_max     = 58
    temp_crit    = 64
    temp_fan_max = 60
    temp_fan_min = 22

# PSU temp sensors

class LenovoNE2580PSUTemp(cumulus.sensors.Temp_Unit):
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'BAD'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class LenovoNE2580PSU1Temp(LenovoNE2580PSUTemp):
    psu = 'PSU1'
    driver_path = '/sys/bus/i2c/devices/10-005a'

class LenovoNE2580PSU1Temp1(LenovoNE2580PSU1Temp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor 1'
    driver_hwmon = ['temp1']

class LenovoNE2580PSU1Temp2(LenovoNE2580PSU1Temp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp Sensor 2'
    driver_hwmon = ['temp2']

class LenovoNE2580PSU1Temp3(LenovoNE2580PSU1Temp):
    name = 'PSU1Temp3'
    description = 'PSU1 Temp Sensor 3'
    driver_hwmon = ['temp3']

class LenovoNE2580PSU2Temp(LenovoNE2580PSUTemp):
    psu = 'PSU2'
    driver_path = '/sys/bus/i2c/devices/10-005b'

class LenovoNE2580PSU2Temp1(LenovoNE2580PSU2Temp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor 1'
    driver_hwmon = ['temp1']

class LenovoNE2580PSU2Temp2(LenovoNE2580PSU2Temp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp Sensor 2'
    driver_hwmon = ['temp2']

class LenovoNE2580PSU2Temp3(LenovoNE2580PSU2Temp):
    name = 'PSU2Temp3'
    description = 'PSU2 Temp Sensor 3'
    driver_hwmon = ['temp3']

# Fans

class LenovoNE2580Fan(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    cpld_path = LenovoNE2580CPLD2.path
    driver_path = cpld_path
    maxpwm = 255
    minpwm = 50

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.fan_present) == '1':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class LenovoNE2580InletFan(LenovoNE2580Fan):
    fan_max = 23000
    fan_min = 3600

class LenovoNE2580OutletFan(LenovoNE2580Fan):
    fan_max = 20500
    fan_min = 3600

class LenovoNE2580InletFan1(LenovoNE2580InletFan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    description = 'Fan1 Inlet'
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']

class LenovoNE2580InletFan2(LenovoNE2580InletFan):
    fan_present = 'fan2_present'
    name = 'Fan2'
    description = 'Fan2 Inlet'
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']

class LenovoNE2580InletFan3(LenovoNE2580InletFan):
    fan_present = 'fan3_present'
    name = 'Fan3'
    description = 'Fan3 Inlet'
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']

class LenovoNE2580InletFan4(LenovoNE2580InletFan):
    fan_present = 'fan4_present'
    name = 'Fan4'
    description = 'Fan4 Inlet'
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']

class LenovoNE2580InletFan5(LenovoNE2580InletFan):
    fan_present = 'fan5_present'
    name = 'Fan5'
    description = 'Fan5 Inlet'
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']

class LenovoNE2580OutletFan1(LenovoNE2580OutletFan):
    fan_present = 'fan1_present'
    name = 'Fan6'
    description = 'Fan1 Outlet'
    driver_hwmon = ['fan6']
    pwm_hwmon = ['pwm1']

class LenovoNE2580OutletFan2(LenovoNE2580OutletFan):
    fan_present = 'fan2_present'
    name = 'Fan7'
    description = 'Fan2 Outlet'
    driver_hwmon = ['fan7']
    pwm_hwmon = ['pwm2']

class LenovoNE2580OutletFan3(LenovoNE2580OutletFan):
    fan_present = 'fan3_present'
    name = 'Fan8'
    description = 'Fan3 Outlet'
    driver_hwmon = ['fan8']
    pwm_hwmon = ['pwm3']

class LenovoNE2580OutletFan4(LenovoNE2580OutletFan):
    fan_present = 'fan4_present'
    name = 'Fan9'
    description = 'Fan4 Outlet'
    driver_hwmon = ['fan9']
    pwm_hwmon = ['pwm4']

class LenovoNE2580OutletFan5(LenovoNE2580OutletFan):
    fan_present = 'fan5_present'
    name = 'Fan10'
    description = 'Fan5 Outlet'
    driver_hwmon = ['fan10']
    pwm_hwmon = ['pwm5']

# LEDs

'''
    Lenovo NE2580 LEDs

    System:
        blue             - all systems OK
        blue blinking    - service required
    Stacking:
        off              - not used
    Power:
        green            - both PSUs OK
        green blinking   - power supply failure
    Fan:
        green            - all fans OK
        green blinking   - fan failure
'''

class LenovoNE2580LED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE2580CPLD1.path

class LenovoNE2580SystemLED(LenovoNE2580LED):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = 'blue'
    bad_led_color = 'blue_blinking'

class LenovoNE2580PowerLED(LenovoNE2580LED):
    name = 'Power LED'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    good_led_color = 'green'
    bad_led_color = 'green_blinking'

class LenovoNE2580FanLED(LenovoNE2580LED):
    name = 'Fan LED'
    led_location = 'led_fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5']
    good_led_color = 'green'
    bad_led_color = 'green_blinking'

class LenovoNE2580FanTrayLED(cumulus.sensors.StatusLED_Unit):
    led_path = LenovoNE2580CPLD2.path
    good_led_color = 'green'
    bad_led_color = 'red'

class LenovoNE2580Fan1LED(LenovoNE2580FanTrayLED):
    name = 'Fan1 LED'
    led_location = 'led_fan1'
    sources = ['Fan1']

class LenovoNE2580Fan2LED(LenovoNE2580FanTrayLED):
    name = 'Fan2 LED'
    led_location = 'led_fan2'
    sources = ['Fan2']

class LenovoNE2580Fan3LED(LenovoNE2580FanTrayLED):
    name = 'Fan3 LED'
    led_location = 'led_fan3'
    sources = ['Fan3']

class LenovoNE2580Fan4LED(LenovoNE2580FanTrayLED):
    name = 'Fan4 LED'
    led_location = 'led_fan4'
    sources = ['Fan4']

class LenovoNE2580Fan5LED(LenovoNE2580FanTrayLED):
    name = 'Fan5 LED'
    led_location = 'led_fan5'
    sources = ['Fan5']

# detect airflow direction

def LenovoNE2580_airflow():
    fan_type_count = {'F2B': 0, 'B2F': 0}
    # Default to F2B type
    fan_type = 1
    b_info = ''

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'lenovo,ne2580':
        return None
    for fan in ['fan1', 'fan2', 'fan3', 'fan4', 'fan5']:
        fan_direction = '%s/%s_direction' % (LenovoNE2580CPLD2.path, fan)
        try:
            b_info = subprocess.check_output(['cat', fan_direction])
            if b_info != '':
                if b_info.strip() == 'F2B':
                    fan_type_count['F2B'] += 1
                if b_info.strip() == 'B2F':
                    fan_type_count['B2F'] += 1
            else:
                continue
        except Exception:
            b_info = ''

    if fan_type_count['F2B'] == 0 and fan_type_count['B2F'] == 0:
        syslog.syslog(syslog.LOG_WARNING, "Airflow types (Front-to-Back/Back-to-Front) for all the fans could "
                      "not be detected, defaulting to F2B")
    elif fan_type_count['F2B'] in xrange(1, 6) and fan_type_count['B2F'] == 0:
        fan_type = 1
    elif fan_type_count['B2F'] in xrange(1, 6) and fan_type_count['F2B'] == 0:
        fan_type = 2
    else:
        syslog.syslog(syslog.LOG_WARNING, "Fans with different airflow types (Front-to-Back/Back-to-Front) "
                      "detected, defaulting to F2B")

    if fan_type == 2:
        return 'B2F'
    else:
        return 'F2B'

class LenovoNE2580Sensors():
    units = (LenovoNE2580PSU1(),
             LenovoNE2580PSU2(),
             LenovoNE2580CPUTemp1(),
             LenovoNE2580CPUTemp2(),
             LenovoNE2580CPUTemp3(),
             LenovoNE2580CPUTemp4(),
             LenovoNE2580CPUTemp5(),
             LenovoNE2580ASICTemp1(),
             LenovoNE2580ASICTemp2(),
             LenovoNE2580ASICTemp3(),
             LenovoNE2580ASICTemp4(),
             LenovoNE2580ASICTemp5(),
             LenovoNE2580ASICTemp6(),
             LenovoNE2580ASICTemp7(),
             LenovoNE2580ASICTemp8(),
             LenovoNE2580ASICTemp9(),
             LenovoNE2580ASICTemp10(),
             LenovoNE2580ASICTemp11(),
             LenovoNE2580ASICTemp12(),
             LenovoNE2580PSU1Temp1(),
             LenovoNE2580PSU1Temp2(),
             LenovoNE2580PSU1Temp3(),
             LenovoNE2580PSU2Temp1(),
             LenovoNE2580PSU2Temp2(),
             LenovoNE2580PSU2Temp3(),
             LenovoNE2580InletFan1(),
             LenovoNE2580InletFan2(),
             LenovoNE2580InletFan3(),
             LenovoNE2580InletFan4(),
             LenovoNE2580InletFan5(),
             LenovoNE2580OutletFan1(),
             LenovoNE2580OutletFan2(),
             LenovoNE2580OutletFan3(),
             LenovoNE2580OutletFan4(),
             LenovoNE2580OutletFan5(),
             )
    F2B_units = (LenovoNE2580Temp1F2B(),
                 LenovoNE2580Temp2F2B(),
                 LenovoNE2580Temp3(),
                 LenovoNE2580Temp4F2B(),
                 )
    B2F_units = (LenovoNE2580Temp1B2F(),
                 LenovoNE2580Temp2B2F(),
                 LenovoNE2580Temp3(),
                 LenovoNE2580Temp4B2F(),
                 )
    # default to F2B if airflow can't be figured out
    if LenovoNE2580_airflow() == 'B2F':
        units = units + B2F_units
    else:
        units = units + F2B_units

class LenovoNE2580PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (LenovoNE2580InletFan1(),
            LenovoNE2580InletFan2(),
            LenovoNE2580InletFan3(),
            LenovoNE2580InletFan4(),
            LenovoNE2580InletFan5(),
            LenovoNE2580OutletFan1(),
            LenovoNE2580OutletFan2(),
            LenovoNE2580OutletFan3(),
            LenovoNE2580OutletFan4(),
            LenovoNE2580OutletFan5(),
)
    temps = (LenovoNE2580CPUTemp1(),
             LenovoNE2580CPUTemp2(),
             LenovoNE2580CPUTemp3(),
             LenovoNE2580CPUTemp4(),
             LenovoNE2580CPUTemp5(),
             LenovoNE2580ASICTemp1(),
             LenovoNE2580ASICTemp2(),
             LenovoNE2580ASICTemp3(),
             LenovoNE2580ASICTemp4(),
             LenovoNE2580ASICTemp5(),
             LenovoNE2580ASICTemp6(),
             LenovoNE2580ASICTemp7(),
             LenovoNE2580ASICTemp8(),
             LenovoNE2580ASICTemp9(),
             LenovoNE2580ASICTemp10(),
             LenovoNE2580ASICTemp11(),
             LenovoNE2580ASICTemp12(),
             )
    F2B_temps = (LenovoNE2580Temp1F2B(),
                 LenovoNE2580Temp2F2B(),
                 LenovoNE2580Temp3(),
                 LenovoNE2580Temp4F2B(),
                 )
    B2F_temps = (LenovoNE2580Temp1B2F(),
                 LenovoNE2580Temp2B2F(),
                 LenovoNE2580Temp3(),
                 LenovoNE2580Temp4B2F(),
                 )
    # default to F2B if airflow can't be figured out
    if LenovoNE2580_airflow() == 'B2F':
        temps = temps + B2F_temps
    else:
        temps = temps + F2B_temps

class LenovoNE2580PWMs():
    units = (LenovoNE2580PWM1(),
             )

class LenovoNE2580LEDs():
    units = (LenovoNE2580SystemLED(),
             LenovoNE2580PowerLED(),
             LenovoNE2580FanLED(),
             LenovoNE2580Fan1LED(),
             LenovoNE2580Fan2LED(),
             LenovoNE2580Fan3LED(),
             LenovoNE2580Fan4LED(),
             LenovoNE2580Fan5LED(),
             )

class LenovoNE2580(cumulus.platform.Platform):
    name = 'lenovo,ne2580'
    chip = cumulus.platform.TridentThreeX7_56873_Chip()
    switch = LenovoNE2580Switch(chip)
    cpld = None
    bmc = None
    sensors = LenovoNE2580Sensors()
    statusleds = LenovoNE2580LEDs()
    pwms = LenovoNE2580PWMs()

Platform = LenovoNE2580
