#
# Copyright 2016, 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.cpld
import cumulus.sensors

LPCI2C_BUS_NAME = 'lpci2c'
def MlxSN2xxx_get_cpld_path():
    for i in range(3):
        try:
            data = open('/sys/bus/i2c/devices/i2c-%u/name' % i, 'r').read().strip()
            if data == LPCI2C_BUS_NAME:
                return '/sys/bus/i2c/devices/%u-0060' % i
        except:
            pass

#
# Items common to the 27xx and 24xx platforms
#

MlxSN2xxx_cpld_path = MlxSN2xxx_get_cpld_path()
MlxSN2xxx_asic_path = '/sys/bus/i2c/devices/2-0048'
MlxSN2xxx_swb_temp_path = '/sys/bus/i2c/devices/7-004a'
MlxSN2xxx_mnb_temp_path = '/sys/bus/i2c/devices/17-0049'
MlxSN2xxx_core_temp_path = '/sys/devices/platform/coretemp.0'
MlxSN2xxx_psu1_path = '/sys/bus/i2c/devices/10-0059'
MlxSN2xxx_psu2_path = '/sys/bus/i2c/devices/10-0058'

class Mlx2xxxCPLD(cumulus.cpld.SysFS):
    path = MlxSN2xxx_cpld_path


# System LEDs
#
class MlxSN2xxxStatusLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6',
               'Fan7', 'Fan8', 'PSU1', 'PSU1Fan1', 'PSU1Temp1',
               'PSU2', 'PSU2Fan1', 'PSU2Temp1', 'Temp1', 'Temp2',
               'Temp3', 'Temp4', 'Temp5', 'Temp6']
    led_location = 'led_status'
    bad_led_color = 'red'

class MlxSN2xxxPsuLED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    bad_led_color = 'red'

class MlxSN2xxxFanLED(cumulus.sensors.StatusLED_Unit):
    bad_led_color = 'red'

class MlxSN2xxxFan1LED(MlxSN2xxxFanLED):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan1'

class MlxSN2xxxFan2LED(MlxSN2xxxFanLED):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan2'

class MlxSN2xxxFan3LED(MlxSN2xxxFanLED):
    name = 'Fan Tray 3'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan3'

class MlxSN2xxxFan4LED(MlxSN2xxxFanLED):
    name = 'Fan Tray 4'
    sources = ['Fan7', 'Fan8']
    led_location = 'led_fan4'

# Base class for power supply unit

# System Fans
#
# Base class for the system fans
#
# we access fans (fanX_input, fanX_enable, pwmX) through the
# mlx-asic-drv driver. This driver relies on the sx_core module
# which takes a while to come up. We keep things in an UNKNOWN
# state until we are able to successfully load the asic driver
# and read status. Once the driver is up, we write '2' to the
# fanX_enable nodes which allows us to control the fans via the
# pwms.
#
# A few additional notes:
#  - writing to any pwm (pwm1 - pwm8) affects all fans equally.
#  - fan numbers are backwards such that fans 1 and 2 are in
#     fan tray 4, while fans 7 and 8 are in fan tray 1.
#
class MlxSN2xxxFan(cumulus.sensors.Fan_Unit):
    fan_max = 23500
    fan_min = 12500
    driver_path = MlxSN2xxx_asic_path

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(self.driver_path, ('%s_input' % self.driver_hwmon[0]))):
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        if not self.init_done:
            if self.asic_write('%s_enable' % self.driver_hwmon[0], '2'):
                self.init_done = True
        return 'OK'

    def asic_write(self, node, val):
        try:
            f = open(os.path.join(self.driver_path, node), 'w')
            f.write(val)
            f.close()
        except:
            return 0
        return 1

class MlxSN2xxxOddFan(MlxSN2xxxFan):
    fan_max = 18000
    fan_min = 10500

#
# system fan #1, Fan Tray #1
#
class MlxSN2xxxFan_1(MlxSN2xxxOddFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan1_ok']
    driver_hwmon = ['fan8']
    init_done = False

    def __init__(self, asic_drv_name):
        self.asic_drv_name = asic_drv_name
        MlxSN2xxxFan.__init__(self)

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2xxx_asic_path, 'hwmon')):
            if os.path.isdir(MlxSN2xxx_asic_path):
                f = open('/sys/bus/i2c/devices/i2c-2/delete_device', 'w')
                f.write('0x48')
                f.close()
            f = open('/sys/bus/i2c/devices/i2c-2/new_device', 'w')
            f.write('%s 0x48' % self.asic_drv_name)
            f.close()
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        try:
            open(os.path.join(self.driver_path, '%s_input' % self.name.lower()), 'r').read()
            if not self.init_done:
                if self.asic_write('%s_enable' % self.name.lower(), '2'):
                    self.init_done = True
        except:
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        return 'OK'

#
# system fan #2, Fan Tray #1
#
class MlxSN2xxxFan_2(MlxSN2xxxFan):
    name = 'Fan2'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan1_ok']
    driver_hwmon = ['fan7']
    init_done = False

#
# system fan #3, Fan Tray #2
#
class MlxSN2xxxFan_3(MlxSN2xxxOddFan):
    name = 'Fan3'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan6']
    init_done = False

#
# system fan #4, Fan Tray #2
#
class MlxSN2xxxFan_4(MlxSN2xxxFan):
    name = 'Fan4'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan5']
    init_done = False

#
# system fan #5, Fan Tray #3
#
class MlxSN2xxxFan_5(MlxSN2xxxOddFan):
    name = 'Fan5'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan4']
    init_done = False

#
# system fan #6, Fan Tray #3
#
class MlxSN2xxxFan_6(MlxSN2xxxFan):
    name = 'Fan6'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    init_done = False

#
# system fan #7, Fan Tray #4
#
class MlxSN2xxxFan_7(MlxSN2xxxOddFan):
    name = 'Fan7'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan2']
    init_done = False

#
# system fan #8, Fan Tray #4
#
class MlxSN2xxxFan_8(MlxSN2xxxFan):
    name = 'Fan8'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan1']
    init_done = False

#
# Base class for the PSU Fans
#
class MlxSN2xxxPSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 20000
    fan_min = 1000

#
# PSU #1 Fan
#
class MlxSN2xxxPSUFan_1(MlxSN2xxxPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = MlxSN2xxx_psu1_path
    driver_hwmon = ['fan1']

#
# PSU #2 Fan
#
class MlxSN2xxxPSUFan_2(MlxSN2xxxPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = MlxSN2xxx_psu2_path
    driver_hwmon = ['fan1']

#
# Base class for the system Temp Sensors
class MlxSN2xxxTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 75
    temp_max = 95
    temp_crit = 115

#
# system temp sensor #1, CPU Package
#
class MlxSN2xxxTemp_1(MlxSN2xxxTemp):
    name = 'Temp1'
    driver_path = MlxSN2xxx_core_temp_path
    driver_hwmon = ['temp1']
    description = 'CPU Package Sensor'

#
# system temp sensor #2, CPU Core 0
#
class MlxSN2xxxTemp_2(MlxSN2xxxTemp):
    name = 'Temp2'
    driver_path = MlxSN2xxx_core_temp_path
    driver_hwmon = ['temp2']
    description = 'CPU Core Sensor 0'

#
# system temp #3, CPU Core 1
#
class MlxSN2xxxTemp_3(MlxSN2xxxTemp):
    name = 'Temp3'
    driver_path = MlxSN2xxx_core_temp_path
    driver_hwmon = ['temp3']
    description = 'CPU Core Sensor 1'

#
# system temp #4, Port Ambient Sensor
#
class MlxSN2xxxTemp_4(MlxSN2xxxTemp):
    name = 'Temp4'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    driver_path = MlxSN2xxx_swb_temp_path
    driver_hwmon = ['temp1']
    description = 'Port Ambient Sensor'

#
# system temp #5, Main Board Ambient
#
class MlxSN2xxxTemp_5(MlxSN2xxxTemp):
    name = 'Temp5'
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 110
    driver_path = MlxSN2xxx_mnb_temp_path
    driver_hwmon = ['temp1']
    description = 'Main Board Ambient Sensor'

#
# system temp #6, Asic Temp
#
class MlxSN2xxxTemp_6(MlxSN2xxxTemp):
    name = 'Temp6'
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 120
    driver_path = MlxSN2xxx_asic_path
    driver_hwmon = ['temp1']
    description = 'Asic Temp Sensor'

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2xxx_asic_path, 'hwmon')):
            return 'UNKNOWN'
        return 'OK'

#
# Base class for the PSU temp sensors
#
class MlxSN2xxxPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 55
    temp_crit = 65
    driver_hwmon = ['temp1']
    cpld_path = MlxSN2xxx_cpld_path

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

#
# PSU1 temp sensor
#
class MlxSN2xxxPSUTemp_1(MlxSN2xxxPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = MlxSN2xxx_psu1_path
    psu = 'PSU1'

#
# PSU2 temp sensor
#
class MlxSN2xxxPSUTemp_2(MlxSN2xxxPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = MlxSN2xxx_psu2_path
    psu = 'PSU2'

#
# Base class for the PSUs
#
class MlxSN2xxxPSU(cumulus.sensors.PSU_Unit):
    cpld_path = MlxSN2xxx_cpld_path
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'pmbus']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'pmbus']
                subprocess.call(cmd)
            self.init_done = True

#
# PSU #1
#
class MlxSN2xxxPSU1(MlxSN2xxxPSU):
    name = 'PSU1'
    driver_path = MlxSN2xxx_psu1_path
    cpld_hwmon = ['psu_pwr1_present', 'psu_pwr1_all_ok']

#
# PSU #2
#
class MlxSN2xxxPSU2(MlxSN2xxxPSU):
    name = 'PSU2'
    driver_path = MlxSN2xxx_psu2_path
    cpld_hwmon = ['psu_pwr2_present', 'psu_pwr2_all_ok']

class MlxSN2xxxLEDs():
    units = (
        MlxSN2xxxStatusLED(),
        MlxSN2xxxPsuLED(),
        MlxSN2xxxFan1LED(),
        MlxSN2xxxFan2LED(),
        MlxSN2xxxFan3LED(),
        MlxSN2xxxFan4LED(),
    )

class MlxSN2xxxSensors():

    def __init__(self, asic_drv_name='mlnx-asic-drv'):
        self.units = (MlxSN2xxxFan_1(asic_drv_name),
                 MlxSN2xxxFan_2(),
                 MlxSN2xxxFan_3(),
                 MlxSN2xxxFan_4(),
                 MlxSN2xxxFan_5(),
                 MlxSN2xxxFan_6(),
                 MlxSN2xxxFan_7(),
                 MlxSN2xxxFan_8(),
                 MlxSN2xxxPSUFan_1(),
                 MlxSN2xxxPSUFan_2(),
                 MlxSN2xxxTemp_1(),
                 MlxSN2xxxTemp_2(),
                 MlxSN2xxxTemp_3(),
                 MlxSN2xxxTemp_4(),
                 MlxSN2xxxTemp_5(),
                 MlxSN2xxxTemp_6(),
                 MlxSN2xxxPSUTemp_1(),
                 MlxSN2xxxPSUTemp_2(),
                 MlxSN2xxxPSU1(),
                 MlxSN2xxxPSU2(),
        )

class MlxSN2xxxSystemPWM(cumulus.sensors.PWM_Unit):
    name = 'System PWM'

    def __init__(self, asic_drv_name):
        self.fans = (MlxSN2xxxFan_1(asic_drv_name),
                MlxSN2xxxFan_2(),
                MlxSN2xxxFan_3(),
                MlxSN2xxxFan_4(),
                MlxSN2xxxFan_5(),
                MlxSN2xxxFan_6(),
                MlxSN2xxxFan_7(),
                MlxSN2xxxFan_8(),
        )
        self.temps = (
                 MlxSN2xxxTemp_1(),
                 MlxSN2xxxTemp_2(),
                 MlxSN2xxxTemp_3(),
                 MlxSN2xxxTemp_4(),
                 MlxSN2xxxTemp_5(),
                 MlxSN2xxxTemp_6(),
        )
        cumulus.sensors.PWM_Unit.__init__(self)

class MlxSN2xxxPWMs():

    def __init__(self, asic_drv_name='mlnx-asic-drv'):
        self.units = (
            MlxSN2xxxSystemPWM(asic_drv_name),
        )
