#
# Copyright 2018.  Cumulus Networks, Inc.
#
import os
import cumulus.platform
import cumulus.cpld
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx
import cumulus.platforms.mlnx_x86_MSN2100

def MlxSN2010_get_paths():
    i801_1f_3_path = '/sys/bus/pci/devices/0000:00:1f.3'
    if os.path.exists(i801_1f_3_path):
        for x in os.listdir(i801_1f_3_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                i801_1f_3_path = os.path.join(i801_1f_3_path, x, b)
                break

    ismt_0_13_0_path = '/sys/bus/pci/devices/0000:00:13.0'
    ismt_0_13_0_full_path = ''
    ismt_0_13_0_base_path = ''
    if os.path.exists(ismt_0_13_0_path):
        for x in os.listdir(ismt_0_13_0_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                ismt_0_13_0_base_path = os.path.join(ismt_0_13_0_path, x)
                ismt_0_13_0_full_path = os.path.join(ismt_0_13_0_base_path, b)
                break

    return i801_1f_3_path, ismt_0_13_0_base_path, ismt_0_13_0_full_path

MLX_ASIC_ADDR = '0x48'
MlxSN2010_i801_path, MlxSN2010_ismt_base_path, MlxSN2010_ismt_path = MlxSN2010_get_paths()
MlxSN2010_dimm1_path = MlxSN2010_i801_path + '-0018'
MlxSN2010_cpld_path = '/sys/bus/i2c/devices/i2c-2/2-0060'
MlxSN2010_led_path = MlxSN2010_cpld_path
MlxSN2010_asic_path = MlxSN2010_ismt_path + '-0048'
MlxSN2010_port_temp_path = '/sys/bus/i2c/devices/6-004a'
MlxSN2010_board_temp_path = '/sys/bus/i2c/devices/6-004b'
MlxSN2010_core_temp_path = '/sys/devices/platform/coretemp.0'

#
# System LEDs
#
class MlxSN2010StatusLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4',
               'PSU1', 'PSU2',
               'Temp1', 'Temp2', 'Temp3', 'Temp4',
               'Temp5', 'Temp6', 'Temp7', 'Temp8']
    led_path = MlxSN2010_led_path
    led_location = 'led_status'
    bad_led_color = 'red'

class MlxSN2010PsuLED(cumulus.sensors.StatusLED_Unit):
    name = 'Psu'
    led_location = 'led_psu'
    sources = ['PSU1', 'PSU2']
    bad_led_color = 'red'

class MlxSN2010FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    led_location = 'led_fan1'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4']
    bad_led_color = 'red'

#
# System Fans
#
# Base class for the system fans
#
# we access fans (fanX_input, fanX_enable, pwmX) through the
# mlx-asic-drv driver. This driver relies on the sx_core module
# which takes a while to come up. We keep things in an UNKNOWN
# state until we are able to successfully load the asic driver
# and read status. Once the driver is up, we write '2' to the
# fanX_enable nodes which allows us to control the fans via the
# pwms.
#
# writing to any pwm (pwm1 - pwm4) affects all fans equally.
#
class MlxSN2010Fan(cumulus.sensors.Fan_Unit):
    fan_max = 23000
    fan_min = 15000
    minpwm = 77
    driver_path = MlxSN2010_asic_path
    pwm_path = driver_path

    # it takes some time for the asic driver to load and give us our fan paths,
    # we do this check so we prevent a transition to BAD before going to OK.
    # That is, we do: UNKNOWN => OK, rather than UNKNOWN => BAD => OK
    def detect_state_plat(self):
        if not os.path.exists(os.path.join(self.driver_path, ('%s_input' % self.driver_hwmon[0]))):
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        if not self.init_done:
            if self.asic_write('%s_enable' % self.driver_hwmon[0], '2'):
                self.init_done = True
        return 'OK'

    # asic driver might not be ready for our write yet so silently fail
    # if we get an error
    def asic_write(self, node, val):
        try:
            f = open(os.path.join(self.driver_path, node), 'w')
            f.write(val)
            f.close()
        except:
            return 0
        return 1

#
# system fan #1
#
class MlxSN2010Fan_1(MlxSN2010Fan):
    name = 'Fan1'
    description = 'Fan 1'
    cpld_hwmon = ['fan1_ok']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    init_done = False

    def __init__(self):
        self.asic_drv_name = 'mlnx-asic-drv-16'
        MlxSN2010Fan.__init__(self)

    def detect_state_plat(self):
        # we need to check to see if the asic driver loaded properly
        # (it is dependent on sx_core initializing)
        if not os.path.exists(os.path.join(MlxSN2010_asic_path, 'hwmon')):
            # asic driver did not create 'hwmon' path, remove and try again
            if os.path.isdir(MlxSN2010_asic_path):
                try:
                    f = open(os.path.join(MlxSN2010_ismt_base_path, 'delete_device'), 'w')
                    f.write(MLX_ASIC_ADDR)
                    f.close()
                except:
                    pass
            try:
                f = open(os.path.join(MlxSN2010_ismt_base_path, 'new_device'), 'w')
                f.write('%s %s' % (self.asic_drv_name, MLX_ASIC_ADDR))
                f.close()
            except:
                pass
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        try:
            open(os.path.join(self.driver_path, '%s_input' % self.name.lower()), 'r').read()
            if not self.init_done:
                if self.asic_write('%s_enable' % self.name.lower(), '2'):
                    self.init_done = True
        except:
            self.setup_hwmon_paths()
            return 'UNKNOWN'
        return 'OK'

#
# system fan #2
#
class MlxSN2010Fan_2(MlxSN2010Fan):
    name = 'Fan2'
    description = 'Fan 2'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #3
#
class MlxSN2010Fan_3(MlxSN2010Fan):
    name = 'Fan3'
    description = 'Fan 3'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# system fan #4
#
class MlxSN2010Fan_4(MlxSN2010Fan):
    name = 'Fan4'
    description = 'Fan 4'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm1']
    init_done = False

#
# Base class for the system Temp Sensors
class MlxSN2010Temp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 80
    temp_max = 100
    temp_crit = 120

class MlxSN2010AmbientTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 40
    temp_max = 100
    temp_crit = 100

class MlxSN2010CoreTemp(MlxSN2010Temp):
    driver_path = MlxSN2010_core_temp_path

#
# system temp sensor #1, DIMM1
#
class MlxSN2010Temp_1(MlxSN2010Temp):
    name = 'Temp1'
    temp_fan_min = 65
    temp_max = 85
    temp_crit = 95
    driver_path = MlxSN2010_dimm1_path
    driver_hwmon = ['temp1']
    description = 'DIMM1 Temperature Sensor'

#
# system temp sensor #2, MLNX ASIC
#
class MlxSN2010Temp_2(MlxSN2010Temp):
    name = 'Temp2'
    driver_path = MlxSN2010_asic_path
    driver_hwmon = ['temp1']
    description = 'ASIC Temperature Sensor'

    def detect_state_plat(self):
        if not os.path.exists(os.path.join(MlxSN2010_asic_path, 'hwmon')):
            return 'UNKNOWN'
        return 'OK'

#
# system temp #3, Port Ambient Sensor
#
class MlxSN2010Temp_3(MlxSN2010AmbientTemp):
    name = 'Temp3'
    driver_path = MlxSN2010_port_temp_path
    driver_hwmon = ['temp1']
    description = 'Port Ambient Sensor'

#
# system temp #4, Main Board Ambient Sensor
#
class MlxSN2010Temp_4(MlxSN2010AmbientTemp):
    name = 'Temp4'
    driver_path = MlxSN2010_board_temp_path
    driver_hwmon = ['temp1']
    description = 'Main Board Ambient Sensor'

#
# system temp #5, core0
#
class MlxSN2010Temp_5(MlxSN2010CoreTemp):
    name = 'Temp5'
    description = 'core0 Sensor'
    driver_hwmon = ['temp2']

#
# system temp #6, core1
#
class MlxSN2010Temp_6(MlxSN2010CoreTemp):
    name = 'Temp6'
    description = 'core1 Sensor'
    driver_hwmon = ['temp3']

#
# system temp #7, core2
#
class MlxSN2010Temp_7(MlxSN2010CoreTemp):
    name = 'Temp7'
    description = 'core2 Sensor'
    driver_hwmon = ['temp4']

#
# system temp #8, core3
#
class MlxSN2010Temp_8(MlxSN2010CoreTemp):
    name = 'Temp8'
    description = 'core3 Sensor'
    driver_hwmon = ['temp5']

#
# Base class for the PSUs
#
class MlxSN2010PSU(cumulus.sensors.PSU_Unit):
    cpld_path = MlxSN2010_cpld_path

#
# PSU #1
#
class MlxSN2010PSU1(MlxSN2010PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1', 'psu_pwr1']

#
# PSU #2
#
class MlxSN2010PSU2(MlxSN2010PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2', 'psu_pwr2']

class MlxSN2010LEDs():
    units = (
        MlxSN2010StatusLED(),
        MlxSN2010PsuLED(),
        MlxSN2010FanLED(),
    )

#class MlxSN2010SFP(cumulus.platform.Spectrum10GPort):
#    _set_led = '''
#;;; MlxSN2010SFP
#'''
#    def __init__(self, label, adj_label, phy_addr, hw_port):
#        hw_port_name = 'xe%d' % hw_port
#
#        cumulus.platform.Spectrum10GPort.__init__(self, label=label,
#                        adj_label=adj_label, hw_port=hw_port_name,
#                        phy_addr=phy_addr)
#        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp+')
#
#    def set_leds(self, mode, led_port, sub):
#        self._check_mode(mode)
#
#        set_calls = {}
#
#        return set_calls

class MlxSN2010SFP28(cumulus.platform.Spectrum25GPort):
    _set_led = '''
;;; MlxSN2010SFP28
'''
    def __init__(self, label, adj_label, phy_addr, hw_port):
        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.Spectrum25GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2010QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2010QSFP28
;;;  port LEDs controlled by HW
'''
    def __init__(self, label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=None, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2010Switch(cumulus.platform.SpectrumSwitch):
#    _sfp = MlxSN2010SFP
    _sfp28 = MlxSN2010SFP28
    _qsfp28 = MlxSN2010QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _sfp28(label="1",  adj_label=None, phy_addr=0,     hw_port=45),
        _sfp28(label="2",  adj_label=None, phy_addr=1,     hw_port=46),
        _sfp28(label="3",  adj_label=None, phy_addr=2,     hw_port=41),
        _sfp28(label="4",  adj_label=None, phy_addr=3,     hw_port=42),
        _sfp28(label="5",  adj_label=None, phy_addr=4,     hw_port=43),
        _sfp28(label="6",  adj_label=None, phy_addr=5,     hw_port=44),
        _sfp28(label="7",  adj_label=None, phy_addr=6,     hw_port=37),
        _sfp28(label="8",  adj_label=None, phy_addr=7,     hw_port=38),
        _sfp28(label="9",  adj_label=None, phy_addr=8,     hw_port=39),
        _sfp28(label="10", adj_label=None, phy_addr=9,     hw_port=40),
        _sfp28(label="11", adj_label=None, phy_addr=10,    hw_port=33),
        _sfp28(label="12", adj_label=None, phy_addr=11,    hw_port=34),
        _sfp28(label="13", adj_label=None, phy_addr=12,    hw_port=35),
        _sfp28(label="14", adj_label=None, phy_addr=13,    hw_port=36),
        _sfp28(label="15", adj_label=None, phy_addr=14,    hw_port=1),
        _sfp28(label="16", adj_label=None, phy_addr=15,    hw_port=2),
        _sfp28(label="17", adj_label=None, phy_addr=16,    hw_port=3),
        _sfp28(label="18", adj_label=None, phy_addr=17,    hw_port=4),
        _qsfp28(label="19", phy_addrs=(18,18,18,18), hw_ports=(9,10,11,12)),
        _qsfp28(label="20", phy_addrs=(19,19,19,19), hw_ports=(5,6,7,8)),
        _qsfp28(label="21", phy_addrs=(20,20,20,20), hw_ports=(17,18,19,20)),
        _qsfp28(label="22", phy_addrs=(21,21,21,21), hw_ports=(13,14,15,16)),
    )

class MlxSN2010(cumulus.platform.Platform):
    name = 'mlnx,msn2010'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2010Switch(cumulus.platform.SpectrumChip())
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100Sensors()
    pwms = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100PWMs()
    statusleds = MlxSN2010LEDs()

Platform = MlxSN2010
