#
# Copyright 2016, 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLY2QSFPp(cumulus.platform.Trident40GPort):
    _set_led = '''
; QuantaLY2QSFPp - green link, amber activity
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pack
        pushst  PORTSTATUS_LINKEN
        tinv                    ; LEDs are active low
        pack

        ret
'''

    def __init__(self, label, serdes, phy_addrs, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84740",) * 4,
                                                 phy_addrs=phy_addrs,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class QuantaLY2SFPp(cumulus.platform.Trident10GPort):
    # LEDs, set register 0xc808 devad 1 for green link, amber activity
    phy_init = { '1 0xc808' : '0x0860' }

    def __init__(self, label, serdes, phy_addr):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type="8754",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=None,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

class QuantaLY2LED(cumulus.ledup.TridentLEDController):
    num_leds = 8
    # the phy-connected port LEDs blink with a 120ms period, make the QSFP
    # ports match.
    blink_period = 120

class QuantaLY2Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY2LED
    _sfp = QuantaLY2SFPp
    _qsfp = QuantaLY2QSFPp
    has_phy_ucode_spi_rom = True
    has_phy_ucode_mdio = True
    ports = (
        _sfp(label="1",  serdes=4,  phy_addr=0x0),
        _sfp(label="2",  serdes=5,  phy_addr=0x1),
        _sfp(label="3",  serdes=6,  phy_addr=0x2),
        _sfp(label="4",  serdes=7,  phy_addr=0x3),
        _sfp(label="5",  serdes=8,  phy_addr=0x4),
        _sfp(label="6",  serdes=9,  phy_addr=0x5),
        _sfp(label="7",  serdes=10, phy_addr=0x6),
        _sfp(label="8",  serdes=11, phy_addr=0x7),
        _sfp(label="9",  serdes=12, phy_addr=0x8),
        _sfp(label="10", serdes=13, phy_addr=0x9),
        _sfp(label="11", serdes=14, phy_addr=0xa),
        _sfp(label="12", serdes=15, phy_addr=0xb),
        _sfp(label="13", serdes=16, phy_addr=0xc),
        _sfp(label="14", serdes=17, phy_addr=0xd),
        _sfp(label="15", serdes=18, phy_addr=0xe),
        _sfp(label="16", serdes=19, phy_addr=0xf),
        _sfp(label="17", serdes=20, phy_addr=0x20),
        _sfp(label="18", serdes=21, phy_addr=0x21),
        _sfp(label="19", serdes=22, phy_addr=0x22),
        _sfp(label="20", serdes=23, phy_addr=0x23),
        _sfp(label="21", serdes=24, phy_addr=0x24),
        _sfp(label="22", serdes=25, phy_addr=0x25),
        _sfp(label="23", serdes=26, phy_addr=0x26),
        _sfp(label="24", serdes=27, phy_addr=0x27),
        _sfp(label="25", serdes=28, phy_addr=0x28),
        _sfp(label="26", serdes=29, phy_addr=0x29),
        _sfp(label="27", serdes=30, phy_addr=0x2a),
        _sfp(label="28", serdes=31, phy_addr=0x2b),
        _sfp(label="29", serdes=32, phy_addr=0x2c),
        _sfp(label="30", serdes=33, phy_addr=0x2d),
        _sfp(label="31", serdes=34, phy_addr=0x2e),
        _sfp(label="32", serdes=35, phy_addr=0x2f),
        _sfp(label="33", serdes=40, phy_addr=0x30),
        _sfp(label="34", serdes=41, phy_addr=0x31),
        _sfp(label="35", serdes=42, phy_addr=0x32),
        _sfp(label="36", serdes=43, phy_addr=0x33),
        _sfp(label="37", serdes=44, phy_addr=0x34),
        _sfp(label="38", serdes=45, phy_addr=0x35),
        _sfp(label="39", serdes=46, phy_addr=0x36),
        _sfp(label="40", serdes=47, phy_addr=0x37),
        _sfp(label="41", serdes=48, phy_addr=0x38),
        _sfp(label="42", serdes=49, phy_addr=0x39),
        _sfp(label="43", serdes=50, phy_addr=0x3a),
        _sfp(label="44", serdes=51, phy_addr=0x3b),
        _sfp(label="45", serdes=52, phy_addr=0x3c),
        _sfp(label="46", serdes=53, phy_addr=0x3d),
        _sfp(label="47", serdes=54, phy_addr=0x3e),
        _sfp(label="48", serdes=55, phy_addr=0x3f),

        _qsfp(label="49", serdes=(56,57,58,59), phy_addrs=(0x40,0x41,0x42,0x43), leds=(1,6)),
        _qsfp(label="50", serdes=(60,61,62,63), phy_addrs=(0x44,0x45,0x46,0x47), leds=(1,4)),
        _qsfp(label="51", serdes=(64,65,66,67), phy_addrs=(0x48,0x49,0x4a,0x4b), leds=(1,2)),
        _qsfp(label="52", serdes=(68,69,70,71), phy_addrs=(0x4c,0x4d,0x4e,0x4f), leds=(1,0)),
    )

class QuantaLY2_PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if self.get_gpio_value(self.psu_all_ok) == '1':
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaLY2Fan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_min = 2500
    fan_max = 23000
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'

class QuantaLY2Fan1(QuantaLY2Fan):
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaLY2Fan2(QuantaLY2Fan):
    name = 'Fan2'
    driver_hwmon = ['fan2']

class QuantaLY2Fan3(QuantaLY2Fan):
    name = 'Fan3'
    driver_hwmon = ['fan3']

class QuantaLY2Fan4(QuantaLY2Fan):
    name = 'Fan4'
    driver_hwmon = ['fan4']

class QuantaLY2Fan5(QuantaLY2Fan):
    name = 'Fan5'
    driver_hwmon = ['fan5']

class QuantaLY2Fan6(QuantaLY2Fan):
    name = 'Fan6'
    driver_hwmon = ['fan6']

class QuantaLY2Fan7(QuantaLY2Fan):
    name = 'Fan7'
    driver_hwmon = ['fan7']

class QuantaLY2Fan8(QuantaLY2Fan):
    name = 'Fan8'
    driver_hwmon = ['fan8']

def QuantaLY2FindFans():
    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return ()
    if plat.strip() != 'quanta,ly2':
        return ()

    # Logic taken from /usr/share/platform-config/quanta/ly2/hw_init.d/S20fan_config.sh
    f2b_fans = (QuantaLY2Fan1(),
                QuantaLY2Fan2(),
                QuantaLY2Fan3(),
                QuantaLY2Fan4(),
    )
    b2f_fans = (QuantaLY2Fan5(),
                QuantaLY2Fan6(),
                QuantaLY2Fan7(),
                QuantaLY2Fan8(),
    )
    proc_decode = subprocess.Popen('/usr/cumulus/bin/decode-syseeprom',
                                   stdout=subprocess.PIPE)
    proc_awk = subprocess.Popen(['/usr/bin/awk', '/Part Number/ {print $5}'],
                                stdin=proc_decode.stdout,
                                stdout=subprocess.PIPE)
    f_name = proc_awk.communicate()[0]
    proc_decode.wait()
    f_name =f_name.rsplit('\n')[0]
    
    found = 0
    if f_name == "1LY2BZZ000R":
        return f2b_fans
    elif f_name == "1LY2BZZ000S":
        return b2f_fans

    b2f_count = 0
    f2b_count = 0
    for fan in b2f_fans:
        if fan.detect_state() != 'ABSENT':
            b2f_count = b2f_count + 1
    for fan in f2b_fans:
        if fan.detect_state() != 'ABSENT':
            f2b_count = f2b_count + 1
    if f2b_count > b2f_count:
        return f2b_fans

    return b2f_fans
    
class QuantaLY2PSU1(QuantaLY2_PSU):
    name = 'PSU1'
    psu_present = 'gpio192'
    psu_all_ok = 'gpio193'
    
class QuantaLY2PSU2(QuantaLY2_PSU):
    name = 'PSU2'
    psu_present = 'gpio208'
    psu_all_ok = 'gpio209'

class QuantaLY2SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'gpio235'
    good_led_color = '1'  # Off
    bad_led_color = '0'   # Yellow

    def led_update(self, color):
        self.set_gpio_value(self.led_location, color)
        #print 'loc: %s color %s' %(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'yellow'
        else:
            color = 'off'
        return self.name, color

class QuantaLY2Temp(cumulus.sensors.Temp_Unit):
    #These are approx numbers from data provided by Quanta.
    temp_max = 68
    temp_crit = 75

class QuantaLY2Temp1(QuantaLY2Temp):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'
    temp_max = 100
    temp_crit = 105

class QuantaLY2Temp2(QuantaLY2Temp):
    name = 'Temp2'
    description = "Right Side of Networking ASIC"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'
    driver_hwmon = ['temp1']

class QuantaLY2Temp3(QuantaLY2Temp):
    name = 'Temp3'
    description = "Near CPU"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'
    driver_hwmon = ['temp2']

class QuantaLY2Temp4(QuantaLY2Temp):
    name = 'Temp4'
    description = "Left Side of Networking ASIC"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'
    driver_hwmon = ['temp3']

class QuantaLY2Temp5(QuantaLY2Temp):
    name = 'Temp5'
    description = "Left Corner near PHY"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'
    driver_hwmon = ['temp4']

class QuantaLY2Temp6(QuantaLY2Temp):
    name = 'Temp6'
    description = "Fan Outlet sensor"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-4/4-002e'
    driver_hwmon = ['temp5']

class QuantaLY2PWM1(cumulus.sensors.PWM_Unit):
    fans = QuantaLY2FindFans()    
    temps = (QuantaLY2Temp1(),
             QuantaLY2Temp2(),
             QuantaLY2Temp3(),
             QuantaLY2Temp4(),
             QuantaLY2Temp5(),
             QuantaLY2Temp6(),
             )

class QuantaLY2Sensors():
    units = ( QuantaLY2PSU1(),
              QuantaLY2PSU2(),
              QuantaLY2Temp1(),
              QuantaLY2Temp2(),
              QuantaLY2Temp3(),
              QuantaLY2Temp4(),
              QuantaLY2Temp5(),
              QuantaLY2Temp6(),
              )

    fans = QuantaLY2FindFans()
    units = units + fans
        
class QuantaLY2PWMs():
    units = (QuantaLY2PWM1(),
             )

class QuantaLY2LEDs():
    units = (QuantaLY2SystemLED(),
             )

class QuantaLY2(cumulus.platform.Platform):
    name = 'quanta,ly2'
    chip = cumulus.platform.TridentPlusChip()
    switch = QuantaLY2Switch(chip)
    statusleds = QuantaLY2LEDs()
    pwms = QuantaLY2PWMs()
    sensors = QuantaLY2Sensors()

Platform = QuantaLY2
